%Copyright (c) 2009, urs (us) schwarz
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without 
% modification, are permitted provided that the following conditions are 
% met:
% 
%     * Redistributions of source code must retain the above copyright 
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright 
%       notice, this list of conditions and the following disclaimer in 
%       the documentation and/or other materials provided with the distribution
%       
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
% POSSIBILITY OF SUCH DAMAGE.
%
%
% ASHAPE	an ASLIB wrapper to compute ALPHA SHAPEs
%
%		ASHAPE is a convenient wrapper for the
%		ALPHA SHAPE library ASLIB
%		for instructions see:	HELP ASLIB
%
% SYNTAX
% -------------------------------------------------------------------------------
%		P = ASHAPE(X,Y,R,OPT)
%		P = ASHAPE(PO,OPT)
%
% INPUT
% -------------------------------------------------------------------------------
%  X/Y	:	input vectors|matrices of x/y data coordinates
%  R	:	radius of alpha circle template
%  OPT	:	options
%		for explanations see:	ASLIB -o
%  PO	:	previously computed ASLIB structure
%
% OUTPUT
% -------------------------------------------------------------------------------
%  P	:	ASLIB structure
%		for explanations see:	ASLIB -f
%  P.f	:	structure of handles to ASLIB functions
%		for explanations see:	ASLIB -s
%
% EXAMPLE
% -------------------------------------------------------------------------------
%		x=cosd(0:10:180);
%		y=sind(0:10:180);
%		p=ashape(x,y,.1);

% created:
%	us	17-Jan-2005
% modified:
%	us	24-Apr-2007 15:34:25

%--------------------------------------------------------------------------------
function	p=ashape(varargin)

	if	nargout
		p=[];
	end

	if	nargin					&&...
		isstruct(varargin{1})
		c=aslib(varargin{1},1);
	if	c.r < 2
		p=ashape(c.p.x,c.p.y,c.p.r,varargin{2:end});
		return
	else
		error(c.msg);
	end
	end

	if	nargin < 3
		help(mfilename);
		return;
	else
		x=varargin{1};
		y=varargin{2};
		r=varargin{3};
		o={};
	if	nargin > 3
		o=varargin(4:end);
	end
	end

		p.f=aslib;
		p=p.f.aini(x,y,r,o{:});
		p.runtime=clock;
		p=p.f.mseg(p);
		p=p.f.mcon(p);
		p=p.f.mpat(p);
		p.runtime=etime(clock,p.runtime);
		p=p.f.aini(p);

% 	if	p.flg.gflg
% 	if	p.flg.pflg
% 		p=p.f.gpat(p);
% 	else
% 		p=p.f.gall(p);
% 	end
% 	end
		return;
%--------------------------------------------------------------------------------
