function royalsampler_helper(hObject,eventdata)

%do NSGA-II (bounds version). Export a subset of all generations run back to workspace, then fit
%this data to a uniform grid or LHC. Result will be a uniform sampling of output space. Export
%results as a text file.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(hObject);
udata=get(handles.fakeax(handles.activeindex),'userdata');

tic

delete(findobj(handles.activegraph,'tag','crap'))

statustext=findobj('tag','statustext');
timetext=findobj('tag','timetext');
set(statustext,'string',{'Populating Space';'Lineage:1 Generation 0'});
drawnow;

genedit=findobj(handles.tabobjects,'tag','genedit');
popedit=findobj(handles.tabobjects,'tag','popedit');

paretotarget={[-inf -inf];[-inf inf];[ inf inf];[inf -inf]};

d=udata.view.dataset;
x=udata.view.x;
y=udata.view.y;

N=floor(str2double(get(popedit,'string'))); %population size
M=2; %always 2 objectives for 2-D scatterplot pareto front.

pcross=0.9;
pmutation=0.9;
nbits=8; %number of bits to discretize each input var


settings.animation=false;
settings.gif =false;
settings.fps = 0;
settings.loop = 0;
settings.pcross = .9;
settings.pmutation = .9;
settings.nbits = 8;
settings.method='nsgaii';
settings.pop= N;
settings.gens = 0;
settings.plotprogress = true;
settings.plotvars = 0;
settings.targets = paretotarget{1};
settings.d = d;
settings.a = udata.view.analysis;
settings.x = [x y];
settings.activeconstraints = 0;
settings.startbutton = 0;
settings.stopbuttonpos = 0;

handles.temp.activeconstraints=0;
handles.temp.activeconstraints=handles.temp.activeconstraints(logical(handles.temp.activeconstraints)); %remove 0 from list.
guidata(handles.fig,handles);

%get col numbers of input variables that will be changed (sorted)
indexx=union(handles.funvars{d}{x},handles.funvars{d}{y});

nvars=numel(indexx);%(number of input vars) not the same as size(d,2)

ranges=[min(handles.alldata{d}); max(handles.alldata{d})];
rangestep=(ranges(2,:)-ranges(1,:))/(2^nbits-1);

%get generations from text box:

Ttext=get(genedit,'string');
Ts=eval(['[',Ttext,']']); %Ts is vector of length = # of ga's, entries = # of generations for that ga.
% colorz=summer(numel(Ts));

saveddata=cell(numel(Ts),1);
P=cell(4,1);

for qqq=1:numel(Ts) %for each GA...
    T=Ts(qqq); %T=number of gens.

    %decide which gen numbers to store in data set:
    %keep 0th gen, first 2*nvars gens, every 4th gen up to T/3, 5 evenly spaced gens from T/3 to T,
    %including the last (Tth) gen.

    keeperz=0; %0th gen
    keeperz=[keeperz 1:3*nvars];  %#ok<AGROW> %first 3*nvars gens
    if round(T/3)>(keeperz(end)+1)
        keeperz=[keeperz (keeperz(end)+1):4:round(T/3)]; %#ok<AGROW> %every 4th gen up to T/3;
    end

    if T>(keeperz(end)+4)
        keeperz=[keeperz linspace((keeperz(end)+4),T,5)]; %#ok<AGROW> %5 evenly spaced gens from T/3 to T
    end
    keeperz=round(unique(keeperz)); %remove any duds


    exploreData=uint16(zeros(N*length(keeperz)*4,nvars*nbits)); %initialize storage space.
    s=1; %keeperz index counter

    %generate random initial populations:
    P{1}=uint16(round(rand(N,nvars*nbits))); %P is binary representation of population
    P{2}=uint16(round(rand(N,nvars*nbits)));
    P{3}=uint16(round(rand(N,nvars*nbits)));
    P{4}=uint16(round(rand(N,nvars*nbits)));

    Plast=P;

    objective=cell(4,1);
    %R=cell(4,1);
    F=cell(4,1);
    rank=cell(4,1);

    for D=1:4 %for each corner of design space:
        %% evaluate this population:
        [objective{D} cviolation{D}]=evaluate_population(P{D},M,nbits,nvars,paretotarget{D},ranges,rangestep,indexx,handles,udata,settings);

        %% calculate sorting/crowding of initial population:
        [F{D},rank{D}]=fast_non_dominated_sort(objective{D},cviolation{D});

        %% calculate crowding distance: (for all frontiers)
        distance=cell(4,1);
        crowding=cell(4,1);

        for i=1:numel(F{D})
            I=objective{D}(F{D}{i},:); %i think it's ok to keep this in max-the-best format... but make sure thats correct.
            distance{D}{i}=crowding_distance_assignment(I);
            crowding{D}(F{D}{i})=distance{D}{i};
        end

        Plast{D}=P{D};
        P{D}=make_new_population(P{D},crowding{D},rank{D},pcross,pmutation,cviolation{D},handles);
    end

    exploreData(1:N,:)=P{1};
    exploreData(1*N+1:2*N,:)=P{2};
    exploreData(2*N+1:3*N,:)=P{3};
    exploreData(3*N+1:4*N,:)=P{4};

    %% DO GENERATIONS:
    for t=1:T

        objective=cell(4,1);
        R=cell(4,1);
        F=cell(4,1);
        rank=cell(4,1);

        for D=1:4

            %% combine new/old population:
            R{D}=[P{D};Plast{D}]; %combine new/old popualtions

            %evaluate population at R:
            [objective{D} cviolation{D}]=evaluate_population(R{D},M,nbits,nvars,paretotarget{D},ranges,rangestep,indexx,handles,udata,settings);
            %% calculate sorting/crowding of initial population:
            [F{D},rank{D}]=fast_non_dominated_sort(objective{D},cviolation{D});
            %trim down to N members using rank/crowding sort:

            %% calculate crowding distance: (for all frontiers)
            distance=cell(4,1);
            crowding=cell(4,1);
            for i=1:numel(F{D})
                I=objective{D}(F{D}{i},:); %i think it's ok to keep this in max-the-best format... but make sure thats correct.
                distance{i}=crowding_distance_assignment(I);
                crowding{D}(F{D}{i})=distance{i};
            end

            survivors=[];
            i=1;
            while numel(survivors)<N && i<=N
                if numel(survivors)+numel(F{D}{i})<=N
                    survivors=union(survivors,F{D}{i}); %add all members of F to survivors
                else %add only enough members to make numel(survivors)==N
                    shortage=N-numel(survivors);

                    %add this many elements to survivors:
                    cdist=crowding{D}(F{D}{i});
                    index=F{D}{i}; %index of cdist points into R
                    data=[cdist' index'];
                    data=sortrows(data);
                    %take the last (shortage) rows of data (these are least crowded)
                    survivors=union(survivors,data(end-shortage+1:end,2));

                end
                i=i+1;
            end

            %pull out these rows of R:
            Plast{D}=R{D}(survivors,:);
            P{D}=make_new_population(Plast{D},crowding{D},rank{D},pcross,pmutation,cviolation{D},handles);

        end

        if any(keeperz==t) %permanently store these points, these are the ones that will generate grid.
            exploreData((s*4*N)+(0*N+1:1*N),:)=P{1};
            exploreData((s*4*N)+(1*N+1:2*N),:)=P{2};
            exploreData((s*4*N)+(2*N+1:3*N),:)=P{3};
            exploreData((s*4*N)+(3*N+1:4*N),:)=P{4};
            s=s+1;
        end

        set(statustext,'string',{'Populating Space';['Lineage: ',num2str(qqq),' Generation: ',num2str(t)]});
        set(timetext,'string',['Elapsed Time: ',num2str(toc),' sec']);
        drawnow;

    end

    %convert population back to real numbers:

    P3d=uint16(reshape(exploreData,[size(exploreData,1),nbits,nvars]));

    colnums=(size(P3d,2):-1:1)-1;
    colz=uint16(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));

    prodz=P3d.*colz;
    expoz=2.^prodz.*P3d;
    sumz=sum(expoz,2); %THIS LINE CONVERTES BACK TO DOUBLE
    Prow2D=permute(sumz,[1 3 2]);
    Prowindex=ones(size(Prow2D,1),size(ranges,2));
    Prowindex(:,indexx)=Prow2D;

    Preal=bsxfun(@times,Prowindex,rangestep);
    Preal=bsxfun(@plus,Preal,ranges(1,:)); %Preal is decimal representation of population

    objective=raveevaluateregression(handles.fig,udata.view.dataset,[udata.view.x udata.view.y],Preal);

    initialpop=line(objective(:,1),objective(:,2),'marker','.','linestyle','none','parent',handles.activegraph,...
        'tag','initialpop','color',[.5 0 .5],'markersize',1,'visible','off'); %#ok<NASGU>

    saveddata{qqq}=Preal(:,1:end-1);

    %save boundary from largest GA to use to generate grid:
    if T==max(Ts)
        vertices=Plast';
        vobjective=cell(4,1);
        for d=1:4
            %convert final population back to real numbers:
            %convert binary to decimal:
            P3d=uint16(reshape(vertices{d},[size(vertices{d},1),nbits,nvars]));

            colnums=(size(P3d,2):-1:1)-1;
            colz=uint16(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));

            prodz=P3d.*colz;
            expoz=2.^prodz.*P3d;
            sumz=sum(expoz,2); %THIS LINE CONVERTES BACK TO DOUBLE
            Prow2D=permute(sumz,[1 3 2]);
            Prowindex=ones(size(Prow2D,1),size(ranges,2));
            Prowindex(:,indexx)=Prow2D;

            Preal=bsxfun(@times,Prowindex,rangestep);
            Preal=bsxfun(@plus,Preal,ranges(1,:)); %Preal is decimal representation of population

            %free up memory:
            clear P P3d Plast Prow2D Prowindex colz expoz exploreData prodz sumz
            
            vobjective{d}=raveevaluateregression(handles.fig,udata.view.dataset,[udata.view.x udata.view.y],Preal);
        end

        vobjective{1}=sortrows(vobjective{1});
        vobjective{2}=flipud(sortrows(vobjective{2}));
        vobjective{3}=flipud(sortrows(vobjective{3}));
        vobjective{4}=sortrows(vobjective{4});
        vobjective=cell2mat(vobjective);

        if isempty(findobj('tag','pareto'))
            pfront=line(vobjective(:,1),vobjective(:,2),'tag','crap','parent',handles.activegraph,'color',[.5 1 0]);
        else
            pfront=[];

        end
    end



end

set(findobj(handles.fig,'tag','initialpop'),'visible','on')

set(statustext,'string','Generating Grid');
set(timetext,'string',['Elapsed Time: ',num2str(toc,3),' sec']);

drawnow;

%% HERE ENDS THE GA AND BEGINS THE GRID FITTING:

%evalute saved data:
saveddata=cell2mat(saveddata);

saveddata=unique(saveddata,'rows');


sobj=raveevaluateregression(handles.fig,udata.view.dataset,[udata.view.x udata.view.y],saveddata);

%normalize data to 0:1 range: (otherwise this aint gonna work, now is it?)


sObj=bsxfun(@minus,sobj,min(sobj));
sObj=bsxfun(@rdivide,sObj,(max(sobj)-min(sobj)));

vObj=bsxfun(@minus,vobjective,min(vobjective));
vObj=bsxfun(@rdivide,vObj,(max(vobjective)-min(vobjective)));


resoedit=findobj(handles.fig,'tag','resolutionedit');
rrr=str2double(get(resoedit,'string'));


%make grid:
x=linspace(min(vobjective(:,1)),max(vobjective(:,1)),rrr);
y=linspace(min(vobjective(:,2)),max(vobjective(:,2)),rrr);
X=linspace(0,1,rrr);
Y=linspace(0,1,rrr);

[X,Y]=meshgrid(X,Y);
[x,y]=meshgrid(x,y);
IN=inpolygon(X,Y,vObj(:,1),vObj(:,2));


% %make MC grid:
% X=rand(20000,1);
% Y=rand(20000,1);
% x=X*(max(vobjective(:,1))-min(vobjective(:,1)))+min(vobjective(:,1));
% y=Y*(max(vobjective(:,2))-min(vobjective(:,2)))+min(vobjective(:,2));
% IN=inpolygon(X,Y,vObj(:,1),vObj(:,2));

targetgrid=line(x(IN),y(IN),'marker','+','markersize',4,'parent',handles.activegraph,'linestyle','none',...
    'tag','crap');

set(statustext,'string','Fitting Data to Grid');
set(timetext,'string',['Elapsed Time: ',num2str(toc),' sec']);
drawnow;

targets=[X(IN) Y(IN)];

d=ipdm(targets,sObj,'subset','nearestneighbor');

[xxx,yyy]=find(d);

bestfits=unique(yyy);

finalgrid=line(sobj(bestfits,1),sobj(bestfits,2),'marker','+','markersize',2,'parent',handles.activegraph,...
    'tag','royalgrid','linestyle','none','color',[0 .5 1],'userdata',saveddata(bestfits,:));

set(statustext,'string','Finished');
set(timetext,'string',['Elapsed Time: ',num2str(toc),' sec']);
drawnow;

delete(findobj('tag','initialpop'));
delete([targetgrid pfront]);

handles.temp.royalinputs=saveddata(bestfits,:);
handles.temp.royaloutputs=sobj(bestfits,:);
handles.temp.finalgrid=finalgrid;

guidata(handles.fig,handles);

exportbutton=findobj('tag','exportbutton');
set(exportbutton,'enable','on');
