function raveupdateprofiler(src,eventdata)
%this is the buttondownfcn for derivative profiler axes, to update the x
%value of the selected graph and redraw all graphs in the new position.
%Also includes buttonmotionfcn version.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

thistype=get(src,'tag');
if strcmp(thistype,'points');
    points=src;
    src=get(src,'parent');
else
    points=[];
end
   
if strcmp(get(handles.fig,'selectiontype'),'alt') && udata.format.showpoints && ~udata.view.singleyaxis %ctrl click or right click = select point closest to clicked point. 
    %need to check if this graph is active. if not , return and do nothing.
    thisindex=get(src,'userdata');
    if thisindex~=handles.activeindex
        return
    end
    
    %if format tab is active, instead of doing this, make this graph the active subgraph to allow axes
    %changes and such:
    
    if strcmp(handles.activetab,'format')
        raveselectsubgraph(src,eventdata)
        return
    end
    
    cc=get(src,'currentpoint');
    xclick=cc(1,1);
    yclick=cc(1,2);
    
    try %gives error if you didnt click on a point (eg clicked on background)
        index=ravefindclosestpoint2D(xclick,yclick,points);
    catch
        return
    end
    
    %figure out
    
    oldselects=handles.isselected{udata.view.dataset}{udata.view.analysis};
    oldselects=false(size(oldselects));
    oldselects(index)=true;
    handles.isselected{udata.view.dataset}{udata.view.analysis}=oldselects;
    
    handles.xdials{udata.view.dataset}{udata.view.analysis}=handles.alldata{udata.view.dataset}(index,:);
    
    guidata(src,handles);
    
    raveupdateanalysis(src,[],'isselected',1);
    raveupdateanalysis(src,[],'xdials',1);
    
    return  
    
    

    
else
    %need to check if this graph is active. if not , return and do nothing.
    thisindex=get(src,'userdata');
    if thisindex~=handles.activeindex
        return
    end
    
    %if format tab is active, instead of doing this, make this graph the active subgraph to allow axes
    %changes and such:
    
    if strcmp(handles.activetab,'format')
        raveselectsubgraph(src,eventdata)
        return
    end
    
    raveslideprofiler(handles.fig,[]);
    %make sliding along graph update continuously:
    set(handles.fig,'windowbuttonupfcn',@raveprofdone)
    set(handles.fig,'windowbuttonmotionfcn',@raveslideprofiler)
    set(handles.fig,'interruptible','off')
end


function raveslideprofiler(src,ev)
%NOTE: SRC FOR THIS FUNCTION IS EQUAL TO HANDLES.FIG REGARDLESS OF WHAT WAS
%CLICKED.
%persistent handles
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

d=udata.view.dataset;
a=udata.view.analysis;

xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};

xcol=udata.view.x;
ycol=udata.view.y;

numy=numel(ycol);


%find this plot's location in the matrix:
[crap,index]=find(handles.subax{handles.activeindex}==gca);

if isempty(index);
    return
end



%if ~isempty(src) %if this function is called as a callback: (get mouse location and slide bars)
clicked=get(gca,'currentpoint');
val=clicked(1,1);

%move marker within bounds
if val>xmax(xcol(index))
    val=xmax(xcol(index));
elseif val<xmin(xcol(index))
    val=xmin(xcol(index));
end


%if this variable is non-continuous, round val to nearest allowable value:
thistype=handles.datatype{d}{xcol(index)};


if  strcmp(thistype,'constant')
    val=handles.minlimit{d}{a}{xcol(index)};
elseif ~strcmp(thistype,'continuous')
    %find nearest neighbor in allowable values:
    allowed=handles.discretevalues{d}{a}{xcol(index)};
    val=allowed(abs(allowed-val)==min(abs(allowed-val)));
end
    

%make text label:
if ~any(strcmp(thistype,{'string','logical'}))
   valtext=strtrim(sprintf('%5.5g',val));
elseif strcmp(thistype,'string')
    valtext=handles.discretevalues{d}{a}{xcol(index)}{val};
else %logical
    if val
        valtext=handles.prefs.logicaltrue;
    else
        valtext=handles.prefs.logicalfalse;
    end   
end



%set xlabel:
set(handles.profilers.xhair{handles.activeindex}(:,index),'xdata',[val val])
set(get(handles.subfakeax{handles.activeindex}(end,index),'xlabel'),'string',{' ';' ';['\color[rgb]{',udata.format.linecolorString,'} ',valtext]},'verticalalignment','top','interpreter','tex');


%if new value is the same as old value (because of discrete vars) don't
%waste time evaluating funcitons:
xdials=handles.xdials{d}{a};

if xdials(xcol(index))==val;
    return
end
    
xdials(xcol(index))=val;

handles.xdials{d}{a}=xdials;
guidata(handles.fig,handles);

%calculate profiler data for each graph:
if udata.view.singleyaxis
    rows=1;
else
    rows=length(udata.view.y);
end

cols=length(udata.view.x);
numpts=udata.view.numpts;




%get type of each variable:
datatypes=handles.datatype{d}(xcol);

totalrows=zeros(size(datatypes)); %totalrows lists the total number of rows in the input data that correspond to the graph of this input.
%thus final input data has sum(totalrows) rows. 


xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};

step=(xmax-xmin)/numpts;
xsteps=cell(1,numel(datatypes));


for j=1:numel(datatypes);
   switch datatypes{j}
       case 'constant'
           xsteps{j}=xdials(xcol(j));
           totalrows(j)=1; 
       case 'continuous'
           xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
           totalrows(j)=numel(xsteps{j});
       case 'integer'          
           if numel(handles.discretevalues{d}{a}{xcol(j)})<20
               xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
               totalrows(j)=numel(xsteps{j}); 
           else
               xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
               totalrows(j)=numel(xsteps{j});   
           end          
       case 'discrete'
               xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
               totalrows(j)=numel(xsteps{j});
       case 'string'
           xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
           totalrows(j)=numel(xsteps{j});
       case 'logical'
           xsteps{j}=[0 1];
           totalrows(j)=numel(xsteps{j});
   end
end


rowz=sum(totalrows);

xfixed=repmat(xdials,[rowz,1]);

%convert to cell if strings are present:
if any(strcmp('string',datatypes))
    xfixed=num2cell(xfixed);
    for k=1:cols
        [xfixed{sum(totalrows(1:k-1))+1:sum(totalrows(1:k)),xcol(k)}]=deal(xsteps{k});
    end
    
else
    for k=1:cols
        xfixed(sum(totalrows(1:k-1))+1:sum(totalrows(1:k)),xcol(k))=xsteps{k};
    end
end

%evaluate the functions: 
yvalues=raveevaluateregression(src,d,ycol,xfixed);

doupdate=0;
for g=1:numy
   if min(yvalues(:,g))<handles.minlimit{d}{a}(ycol(g));handles.minlimit{d}{a}(ycol(g))=min(yvalues(:,g));doupdate=1;end
   if max(yvalues(:,g))>handles.maxlimit{d}{a}(ycol(g));handles.maxlimit{d}{a}(ycol(g))=max(yvalues(:,g));doupdate=1;end
end

if doupdate;guidata(src,handles);end

ycurrent=raveevaluateregression(src,d,ycol,xdials);

%put results of each response in a seperate cell:
yvalues=mat2cell(yvalues,size(yvalues,1),ones(1,numy));



ycurrent=mat2cell(ycurrent,size(ycurrent,1),ones(1,numy));

%and put each x for each resposne in its own cell too: 
for zz=1:numy
    yvalues{zz}=mat2cell(yvalues{zz},totalrows,1);
end

%update current-value texts for responses:
for tt=1:rows
    %set y label:
    g=udata.view.ylabels(tt);
    if ~udata.view.singleyaxis
        set(g,'string',{['\color[rgb]{',udata.format.linecolorString,'} ',strtrim(sprintf('%5.5g',ycurrent{tt}))];'';''});    
    end
    
    
    for cc=1:cols
        if  ~any(strcmp(datatypes{cc},{'continuous';'constant'})) %if this variable is discrete
            allowed=handles.discretevalues{d}{a}{xcol(cc)};
            endind=numel(allowed);
            
            intsides=repmat(2:(endind-1),3,1);
            ymap=[1 1 intsides(:)' endind endind];
            
            xvals=xsteps{cc}(:); %force to be column.
            xintermediates=xvals+[diff(xvals)./2;0];
            
            xvals=sort([xvals;xintermediates(1:end-1)]);
            
            xmap=repmat(2:2:2*(endind-1),3,1);
            xmap(end,:)=xmap(end,:)+ones(size(xmap(end,:)));
            
            xmap=[1 xmap(:)'];
            
            set(handles.profilers.yhair{handles.activeindex}(tt,cc),'ydata',[ycurrent{tt} ycurrent{tt}])
            set(handles.profilers.main{handles.activeindex}(tt,cc),'ydata',yvalues{tt}{cc}(ymap),'xdata',xvals(xmap)) 
            
        else
            if ~udata.view.singleyaxis
                set(handles.profilers.yhair{handles.activeindex}(tt,cc),'ydata',[ycurrent{tt} ycurrent{tt}])
                set(handles.profilers.main{handles.activeindex}(tt,cc),'ydata',yvalues{tt}{cc},'xdata',xsteps{cc})
            else
                for R=1:numy
                    set(handles.profilers.yhair{handles.activeindex}(R,cc),'ydata',[ycurrent{R} ycurrent{R}])
                    set(handles.profilers.main{handles.activeindex}(R,cc),'ydata',yvalues{R}{cc},'xdata',xsteps{cc})
                end
            end
        end
    end
end

if any(udata.view.constraints~=0)
    ravedrawconstraint(handles.allgraphs(handles.activeindex),handles.activeindex,udata.view.constraints);
end

%update on slide if desired:
if handles.prefs.alwaysupdate && ~handles.prefs.alwaysrasterize
    raveupdateanalysis(src,[],'xdials')
end



function raveprofdone(src,ev) %#ok<INUSD>

handles=guidata(src);
set(handles.fig,'windowbuttonmotionfcn','')
set(handles.fig,'windowbuttonupfcn','')
 set(handles.fig,'interruptible','on')

%update other analyses to reflect new values:
raveupdateanalysis(handles.fig,[],'xdials')






















