function raveupdatecontourxdials(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
getudata;
ai=handles.activeindex;
xcol=udata.view.x;
ycol=udata.view.y;

d=udata.view.dataset;
a=udata.view.analysis;


%prepare data for gridding:
datatypes=handles.datatype{d}([xcol ycol]) ;
totalrows=zeros(size(datatypes));
numpts=udata.view.smooth;
xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};
step=(xmax-xmin)/numpts;
xsteps=cell(1,numel(datatypes));
xdials=handles.xdials{d}{a};

set(handles.profilers.xhair{ai},'xdata',[xdials(xcol) xdials(xcol)])
set(handles.profilers.yhair{ai},'ydata',[xdials(ycol) xdials(ycol)])

xcol=[xcol ycol]; %#ok<AGROW>
for j=1:numel(datatypes);
    switch datatypes{j}
        case 'constant'
            xsteps{j}=xdials(xcol(j));
            totalrows(j)=1;
        case 'continuous'
            xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
            totalrows(j)=numel(xsteps{j});
        case 'integer'
            if numel(handles.discretevalues{d}{a}{xcol(j)})<20
                xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
                totalrows(j)=numel(xsteps{j});
            else
                xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
                totalrows(j)=numel(xsteps{j});
            end
        case 'discrete'
            xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
            totalrows(j)=numel(xsteps{j});
        case 'string'
            xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
            totalrows(j)=numel(xsteps{j});
        case 'logical'
            xsteps{j}=[0 1];
            totalrows(j)=numel(xsteps{j});
    end
end



%precompute data so contour levels can be shared among all plots:
allthedata=cell(1,3);


thisxdata=xsteps{1};
thisydata=xsteps{2};

[xgrid,ygrid]=meshgrid(thisxdata,thisydata);

xcalldata=xgrid(:);
ycalldata=ygrid(:);

constants=repmat(xdials,[length(xcalldata),1]);
constants(:,xcol(1))=xcalldata;
constants(:,ycol)=ycalldata;

%evaluate the functions:

zvalues=raveevaluateregression(src,d,udata.view.z,constants);

allthedata{1}=reshape(xcalldata,numel(thisydata),numel(thisxdata));
allthedata{2}=reshape(ycalldata,numel(thisydata),numel(thisxdata));
allthedata{3}=reshape(zvalues,numel(thisydata),numel(thisxdata));

zdataz=allthedata{3};
zdataz=zdataz(:);


lowestvalue=min(zdataz);
highestvalue=max(zdataz);

levelz=linspace(1,128,udata.view.ncont)+128*(udata.format.mcmap-1);
udata.view.levels=levelz;

zplotdata=allthedata{3};
if ~udata.format.reversecolors
    zplotdata=(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
else
    zplotdata=128-(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
end

% contourf(ax(r,c),single(allthedata{r,c,1}),single(allthedata{r,c,2}),single(zplotdata),single(levelz),'linecolor','none');

C=contourc(allthedata{1}(1,:),allthedata{2}(:,1),zplotdata,levelz);
delete(handles.profilers.main{ai})
mainhandles=contourplot(C,'style','line','parent',handles.allgraphs(ai),'linewidth',udata.format.linesize,'hittest','off');
% uistack(mainhandles,'bottom');

handles.profilers.main{ai}=mainhandles;
guidata(src,handles); %save now in case an error happens later.


%udpate colorbar:
if udata.format.colorbar
    legdata=get(handles.fakerax(ai),'userdata');
    
    cbar=findobj(legdata,'tag','Colorbar');
    
    ylim=get(cbar,'ylim');
    clim=udata.view.clim;
    
    %determine current c value:
    thisvalue=raveevaluateregression(src,udata.view.dataset,udata.view.c,handles.xdials{udata.view.dataset}{udata.view.analysis});
    
    thiscoord=interp1(clim,ylim,thisvalue);
    
    %make triangle patch at this value:
    cvpatch=findobj(cbar,'tag','currentvalue');
    set(cvpatch,'ydata',[thiscoord-1.5 thiscoord+1.5 thiscoord]);
    cvtext=findobj(cbar,'tag','currentvaluetxt');
    pos=get(cvtext,'pos');
    set(cvtext,'pos',[pos(1) thiscoord],'string',sprintf('%5.5g',thisvalue));
end

% if ai~=handles.activeindex;raverasterfarian(handles.fig,[],ai);end;
% 
% handles=guidata(handles.fig);
% 

guidata(handles.fig,handles);

