function raveupdatehyperslice(src,eventdata)
%this is the buttondownfcn for hyperslice axes, to update the x
%value of the selected graph and redraw all graphs in the new position.
%Also includes buttonmotionfcn version.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

%need to check if this graph is active. if not , return and do nothing.
thisindex=get(src,'userdata');
if thisindex~=handles.activeindex
    return
end

%if format tab is active, instead of doing this, make this graph the active subgraph to allow axes
%changes and such:

if strcmp(handles.activetab,'format')
    raveselectsubgraph(src,eventdata)
    return
end

raveslidehyperslice(handles.fig,[]);
%make sliding along graph update continuously:
set(handles.fig,'windowbuttonupfcn',@raveslidedone)
set(handles.fig,'windowbuttonmotionfcn',@raveslidehyperslice)



function raveslidehyperslice(src,ev)
%NOTE: SRC FOR THIS FUNCTION IS EQUAL TO HANDLES.FIG REGARDLESS OF WHAT WAS
%CLICKED.
%persistent handles
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

d=udata.view.dataset;
a=udata.view.analysis;

mins=handles.minlimit{d}{a};
maxs=handles.maxlimit{d}{a};


xcol=udata.view.x;
ycol=udata.view.y;
zcol=udata.view.z;

numy=numel(ycol);


%find this plot's location in the matrix:
[yindex,xindex]=find(handles.subax{handles.activeindex}==gca);

%if ~isempty(src) %if this function is called as a callback: (get mouse location and slide bars)
clicked=get(gca,'currentpoint');
xval=clicked(1,1);
yval=clicked(1,2);

%move marker within bounds
if xval>maxs(xcol(xindex))
    xval=maxs(xcol(xindex));
elseif xval<mins(xcol(xindex))
    xval=mins(xcol(xindex));
end

if yval>maxs(ycol(yindex))
    yval=maxs(ycol(yindex));
elseif yval<mins(ycol(yindex))
    yval=mins(ycol(yindex));
end


%if this variable is non-continuous, round val to nearest allowable value:
thistype=handles.datatype{d}{xcol(xindex)};
if  strcmp(thistype,'constant')
    xval=handles.minlimit{d}{a}{xcol(xindex)};
elseif ~strcmp(thistype,'continuous')
    %find nearest neighbor in allowable values:
    allowed=handles.discretevalues{d}{a}{xcol(xindex)};
    xval=allowed(abs(allowed-xval)==min(abs(allowed-xval)));
end

thistype=handles.datatype{d}{ycol(yindex)};
if  strcmp(thistype,'constant')
    yval=handles.minlimit{d}{a}{ycol(yindex)};
elseif ~strcmp(thistype,'continuous')
    %find nearest neighbor in allowable values:
    allowed=handles.discretevalues{d}{a}{ycol(yindex)};
    yval=allowed(abs(allowed-yval)==min(abs(allowed-yval)));
end

xval=xval(1);
yval=yval(1);

set(cell2mat(handles.profilers.xhair{handles.activeindex}(:,xindex)),'xdata',[xval xval])
set(cell2mat(handles.profilers.yhair{handles.activeindex}(yindex,:)),'ydata',[yval yval])

%set(get(handles.subfakeax{handles.activeindex}(end,index),'xlabel'),'string',{' ';' ';['\color[rgb]{',udata.format.linecolorString,'} ',valtext]},'verticalalignment','top','interpreter','tex');

%if new value is the same as old value (because of discrete vars) don't
%waste time evaluating funcitons:
xdials=handles.xdials{d}{a};

if xdials(xcol(xindex))==xval && xdials(ycol(yindex))==yval
    return
end


xdials(xcol(xindex))=xval;
xdials(ycol(yindex))=yval;

handles.xdials{d}{a}=xdials;
guidata(handles.fig,handles);


%initialize handles

%h.main=zeros(rows,cols,10);

%prepare data for gridding:
datatypes=handles.datatype{d}(xcol);
totalrows=zeros(size(datatypes));
numpts=udata.view.smooth;
xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};
step=(xmax-xmin)/numpts;
xsteps=cell(1,numel(datatypes));
xdials=handles.xdials{d}{a};


for j=1:numel(datatypes);
    switch datatypes{j}
        case 'constant'
            xsteps{j}=xdials(xcol(j));
            totalrows(j)=1;
        case 'continuous'
            xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
            totalrows(j)=numel(xsteps{j});
        case 'integer'
            if numel(handles.discretevalues{d}{a}{xcol(j)})<20
                xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
                totalrows(j)=numel(xsteps{j});
            else
                xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
                totalrows(j)=numel(xsteps{j});
            end
        case 'discrete'
            xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
            totalrows(j)=numel(xsteps{j});
        case 'string'
            xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
            totalrows(j)=numel(xsteps{j});
        case 'logical'
            xsteps{j}=[0 1];
            totalrows(j)=numel(xsteps{j});
    end
end



%precompute data so contour levels can be shared among all plots:
allthedata=cell(numel(xcol),numel(xcol),3);
for r=numel(ycol):-1:1,
    for c=numel(xcol):-1:1,
        if r==c;continue;end

        thisxdata=xsteps{c};
        thisydata=xsteps{r};

        [xgrid,ygrid]=meshgrid(thisxdata,thisydata);

        xcalldata=xgrid(:);
        ycalldata=ygrid(:);

        constants=repmat(xdials,[length(xcalldata),1]);
        constants(:,xcol(c))=xcalldata;
        constants(:,xcol(r))=ycalldata;

        %evaluate the functions:

        zvalues=raveevaluateregression(src,d,udata.view.z,constants);

        allthedata{r,c,1}=reshape(xcalldata,numel(thisydata),numel(thisxdata));
        allthedata{r,c,2}=reshape(ycalldata,numel(thisydata),numel(thisxdata));
        allthedata{r,c,3}=reshape(zvalues,numel(thisydata),numel(thisxdata));
    end
end

zdataz=allthedata(:,:,3);
zdataz=zdataz(:);
zdataz=zdataz(~cellfun(@isempty,zdataz));

lowestvalue=min(zdataz{1}(:));
highestvalue=max(zdataz{1}(:));
for b=2:numel(zdataz)
    lowestvalue=min(lowestvalue,min(zdataz{b}(:)));
    highestvalue=max(highestvalue,max(zdataz{b}(:)));
end

%check against existing limits:
doupdate=false;
if lowestvalue<handles.minlimit{d}{a}(udata.view.z);handles.minlimit{d}{a}(udata.view.z)=lowestvalue;doupdate=true;end
if highestvalue<handles.maxlimit{d}{a}(udata.view.z);handles.maxlimit{d}{a}(udata.view.z)=highestvalue;doupdate=true;end
if doupdate
    guidata(src,handles);

end

lowestvalue=handles.minlimit{d}{a}(udata.view.z);
highestvalue=handles.maxlimit{d}{a}(udata.view.z);

levelz=linspace(1,128,udata.view.ncont)+128*(udata.format.mcmap-1);
udata.view.levels=levelz;

for r=numel(ycol):-1:1,
    for c=numel(xcol):-1:1,
        if r==c;continue;end
        zplotdata=allthedata{r,c,3};
            if ~udata.format.reversecolors
                zplotdata=(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
            else
                zplotdata=128-(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
            end

            % contourf(ax(r,c),single(allthedata{r,c,1}),single(allthedata{r,c,2}),single(zplotdata),single(levelz),'linecolor','none');        
            
            C=contourc(allthedata{r,c,1}(1,:),allthedata{r,c,2}(:,1),zplotdata,levelz);
            delete(cell2mat(handles.profilers.main{handles.activeindex}(r,c)))
            mainhandles{r,c}=contourplot(C,'style','line','parent',handles.subax{handles.activeindex}(r,c),'linewidth',udata.format.linesize,'hittest','off');
            uistack(mainhandles{r,c},'bottom');
    end
end

handles.profilers.main{handles.activeindex}=mainhandles;
guidata(src,handles);

%update colorbar:
if udata.format.colorbar
    legdata=get(handles.fakerax(handles.activeindex),'userdata');

    cbar=findobj(legdata,'tag','Colorbar');
    
    ylim=get(cbar,'ylim');
    clim=udata.view.clim;

    %determine current c value:
    thisvalue=raveevaluateregression(src,udata.view.dataset,udata.view.c,handles.xdials{udata.view.dataset}{udata.view.analysis});

    thiscoord=interp1(clim,ylim,thisvalue);

    %make triangle patch at this value:
    cvpatch=findobj(cbar,'tag','currentvalue');
    set(cvpatch,'ydata',[thiscoord-1.5 thiscoord+1.5 thiscoord]);
    cvtext=findobj(cbar,'tag','currentvaluetxt');
    pos=get(cvtext,'pos');
    set(cvtext,'pos',[pos(1) thiscoord],'string',sprintf('%5.5g',thisvalue));

end

%update on slide if desired:
if handles.prefs.alwaysupdate && ~handles.prefs.alwaysrasterize
    raveupdateanalysis(src,[],'xdials')
end

function raveslidedone(src,ev) %#ok<INUSD>

handles=guidata(src);
set(handles.fig,'windowbuttonmotionfcn','')
set(handles.fig,'windowbuttonupfcn','')

%update other analyses to reflect new values:
raveupdateanalysis(handles.fig,[],'xdials')

clear contourplot %clear persistent vars

