function viewtabhyperslice(src,eventdata)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

%move front tabs to front if necessary
if handles.fronttabsfront==0 %move swap fronttabs and backtabs
    backpos=get(handles.backtabaxes,'position');
    frontpos=get(handles.fronttabaxes,'position');
    set(handles.fronttabaxes,'position',backpos)
    set(handles.backtabaxes,'position',frontpos)
    [x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
    set(handles.backtablines,'xdata',x,'ydata',y);   
    set(handles.fronttabaxes,'xcolor',handles.colors.tabbg,'ycolor',handles.colors.tabbg);
    set(handles.backtabaxes,'xcolor',handles.colors.frame,'ycolor',handles.colors.frame);
end
handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,2,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');

%% clear existing tab
%delete(handles.tabobjects)

%% get input info from graph:
udata=get(handles.fakeax(handles.activeindex),'userdata');

d=udata.view.dataset;
a=udata.view.analysis;


%% text headers

projecttext=text(.15-.10317,.95*1.0317-.0317,'Project','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

dataseriestext=text(.15-.10317,.75*1.0317-.0317,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% text labels

analysistext=text(.25-.10317,.920*1.0317-.0317,'Choose Parent Analysis:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');

datasettext=text(.25-.10317,.85*1.0317-.0317,'Choose a Data Set:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

%% variable lists
xvars=handles.funvars{udata.view.dataset}{udata.view.z(1)};
for yy=1:length(udata.view.z)
    xvars=union(xvars,handles.funvars{udata.view.dataset}{udata.view.z(yy)});
end

xvars=find(~strcmp(handles.datatype{d},'constant') & ~handles.isfunction{d});

allnames=handles.allnames{d};
xvarnames=allnames(xvars);

xaxistext=text(.25-.10317,.72*1.0317-.0317,'Choose Variables:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

xaxismenu=uicontrol('position',[origin(1)+55 origin(2)+370 180 70],...
    'style','listbox','string',xvarnames,'value',find(ismember(xvarnames,allnames(udata.view.x))),...
    'backgroundcolor',[1 1 1],'tag','xaxismenu','max',50,'callback',@xaxismenu_callback);

zaxistext=text(.25-.10317,.58*1.0317-.0317,'Draw Contours of:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

zaxismenu=uicontrol('position',[origin(1)+55 origin(2)+315 180 30],...
    'style','popupmenu','string',handles.allnames{d}(logical(handles.isfunction{d})),...
    'backgroundcolor',[1 1 1],'tag','zaxismenu','callback',@zaxismenu_callback,'value',find(ismember(find(handles.isfunction{d}),udata.view.z)));

layouthandles=[xaxistext xaxismenu zaxistext zaxismenu];


constrainttext=text(.25-.10317,.51*1.0317-.0317,'Draw these constraints:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');



if isempty(handles.constrainttext{udata.view.dataset}{udata.view.analysis})
    conststring='No constraints have been created yet.';
else
    conststring=[cellstr('') handles.constrainttext{udata.view.dataset}{udata.view.analysis}];
end


constraintmenu=uicontrol('position',[origin(1)+40 origin(2)+220 215 80],...
    'style','listbox','string',conststring,...
    'callback',@constraintmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','constraintmenu','max',100,'value',udata.view.constraints+1);



if udata.view.ncont>20
    value=(udata.view.ncont-20)/2+20;
else
    value=udata.view.ncont;
end

nconttext=uicontrol('position',[origin(1)+25 origin(2)+280-85 150 20],...
    'style','text','string','Number of contours:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
ncont=uicontrol('position',[107+3.6*value origin(2)+235-85 50 20],...
    'style','text','string',num2str(udata.view.ncont),'backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',12,'horizontalalignment','left','fontweight','bold');


[jslider1 hslider1]=javacomponent('javax.swing.JSlider',[origin(1)+40 origin(2)+255-85 190 30]);
jslider1.setPaintTicks(1)

jslider1.setSnapToTicks(0)
jslider1.setMinimum(2)
jslider1.setMaximum(50)
jslider1.setMajorTickSpacing(2);
%jslider.setMinorTickSpacing(1);
jslider1.setValue(value);
set(jslider1,'background',java.awt.Color.white)
set(jslider1,'StateChangedCallback',{@ncont_Callback,ncont});


if udata.view.smooth>20
    value=(udata.view.smooth-20)/2+20;
else
    value=udata.view.smooth;
end


smoothtext=uicontrol('position',[origin(1)+25 origin(2)+210-85 150 20],...
    'style','text','string','Grid resolution:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
smooth=uicontrol('position',[107+3.6*value origin(2)+165-85 50 20],...
    'style','text','string',num2str(udata.view.smooth),'backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',12,'horizontalalignment','left','fontweight','bold');

%scale slider:
[jslider2 hslider2]=javacomponent('javax.swing.JSlider',[origin(1)+40 origin(2)+185-85 190 30]);
jslider2.setPaintTicks(1)

jslider2.setSnapToTicks(0)
jslider2.setMinimum(2)
jslider2.setMaximum(50)
jslider2.setMajorTickSpacing(2);
%jslider.setMinorTickSpacing(1);
jslider2.setValue(value);
set(jslider2,'background',java.awt.Color.white)
set(jslider2,'StateChangedCallback',{@smooth_Callback,smooth});

handles.javacomps={jslider1;jslider2};


%draw on top:
optionstext=text(.15-.10317,.15*1.0317-.0317,'Options','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);




%% variable lists

analysismenu=uicontrol('position',[origin(1)+55 origin(2)+535 180 30],...
    'style','popupmenu','string',handles.analysisnames,...
    'backgroundcolor',[1 1 1],'tag','analysismenu','callback',@analysismenu_callback,'value',a);

datasetmenu=uicontrol('position',[origin(1)+55 origin(2)+490 180 30],...
    'style','popupmenu','string',handles.datasetnames,...
    'backgroundcolor',[1 1 1],'tag','datasetmenu','callback',@datasetmenu_callback,'value',d);

layoutvalue=find(ismember({'square';'upper';'lower';'rectangle'},udata.view.layout));

layoutmenu=uicontrol('position',[origin(1)+55 origin(2)+405 180 30],...
    'style','popupmenu','string',{'Square';'Upper Triangle';'Lower Triangle';'Rectangle'},...
    'backgroundcolor',[1 1 1],'tag','layoutmenu','callback',@layoutmenu_callback,'value',layoutvalue,'visible','off');


%% buttons

scalebutton=uicontrol('position',[origin(1)+55 origin(2)+10 180 30],...
    'style','pushbutton','string','Scale/Transform Variables','backgroundcolor',handles.colors.buttonbg);

showpointscheck=uicontrol('position',[origin(1)+55 origin(2)+45 200 26],...
    'style','checkbox','string','Show Data Set',...
    'backgroundcolor',handles.colors.tabbg,'value',udata.format.showpoints,'callback',@showpoints_callback);

%% group items and finish up
handles.tabobjects=[datasettext datasetmenu layoutmenu layouthandles,...
    scalebutton analysistext projecttext analysismenu dataseriestext optionstext showpointscheck,...
    smoothtext smooth hslider1 hslider2 ncont nconttext constrainttext constraintmenu];

handles.activetab='view';

guidata(src,handles);

%% Here be callbacks:%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function datasetmenu_callback(src,eventdata)

handles=guidata(src);
%update other menus to show correct variables

datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');

d=get(datasetmenu,'value');

%get initial columns for this data set: (need at least 2 x's for a
%scatterplot matrix)
if ~all(handles.isresponse{d}==0)
    x=find((~handles.isconstant{d} & ~handles.isresponse{d}),2);
else
    x=find((~handles.isconstant{d}),2);
end

%set fakeax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.dataset=get(datasetmenu,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);
handles.graphdataset(handles.activeindex)=get(src,'value');
guidata(src,handles);

viewtabmenufilter(src,[])

ravecreatehyperslice(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function analysismenu_callback(src,eventdata)
handles=guidata(src);
handles.graphanalysis(handles.activeindex)=get(src,'value');
guidata(src,handles);

udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.analysis=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatehyperslice(src,[]);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xaxismenu_callback(src,eventdata)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');


xvars=find(~strcmp(handles.datatype{udata.view.dataset},'constant') & ~handles.isfunction{udata.view.dataset});


udata.view.x=xvars(get(src,'value'));
udata.view.y=udata.view.x; %must be a square matrix

%ensure at least 2 vars are selected:
if length(udata.view.x)==1
    udata.view.x=xvars(1,2); %select first 2 vars
    set(handles.infobar,'string','You need to select at least two variables for this graph.')
else
    set(handles.infobar,'string','');
end

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatehyperslice(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function useresponsedata_callback(src,eventdata)
handles=guidata(src);

useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');

%set fakeax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.response=get(useresponsedata,'value'); %response check box value
set(handles.fakeax(handles.activeindex),'userdata',udata);

viewtabmenufilter(src,[])

function layoutmenu_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

optionz={'square';'upper';'lower';'rectangle'};

udata.view.layout=optionz{get(src,'value')};

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatehyperslice(src,[]);

viewtab(src,[]);

function zaxismenu_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

zvars=find(handles.isfunction{udata.view.dataset});

udata.view.z=zvars(get(src,'value'));

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatehyperslice(src,[]);



function ncont_Callback(src,ev,hObject) %#ok<INUSL>
%hObject = nbars 
numbars=src.getValue;
if numbars>20;numbars=20+(numbars-20)*2;end
pos=get(hObject,'position');
set(hObject,'string',sprintf('%3.0f',numbars),'position',[49+58+3.6*src.getValue pos(2:4)]);

if src.getValueIsAdjusting;return;end

handles=guidata(hObject);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.ncont=numbars;
set(handles.fakeax(handles.activeindex),'userdata',udata)
guidata(hObject,handles);
%update histogram:
ravecreatehyperslice(handles.fig,[]);




function smooth_Callback(src,ev,hObject) %#ok<INUSL>
%hObject = nbars 
numbars=src.getValue;
if numbars>20;numbars=20+(numbars-20)*2;end
pos=get(hObject,'position');
set(hObject,'string',sprintf('%3.0f',numbars),'position',[49+58+3.6*src.getValue pos(2:4)]);

if src.getValueIsAdjusting;return;end

handles=guidata(hObject);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.smooth=numbars;
set(handles.fakeax(handles.activeindex),'userdata',udata)
guidata(hObject,handles);
%update histogram:
ravecreatehyperslice(handles.fig,[]);


function constraintmenu_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.constraints=get(src,'value')-1;
set(handles.fakeax(handles.activeindex),'userdata',udata)
guidata(src,handles);
%update histogram:
ravecreatehyperslice(handles.fig,[]);


