function formattabsurfaceprofiler(src,eventdata)
%rave format tab for line graphs.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format; %get format tab structure.

%move front tabs to front if necessary
if handles.fronttabsfront==0 %move swap fronttabs and backtabs
    backpos=get(handles.backtabaxes,'position');
    frontpos=get(handles.fronttabaxes,'position');
    set(handles.fronttabaxes,'position',backpos)
    set(handles.backtabaxes,'position',frontpos)
    [x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
    set(handles.backtablines,'xdata',x,'ydata',y);       
end
handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,3,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');


%% clear existing tab
delete(handles.tabobjects)

%% text headers 
graphtext=text(.15-.10317,.97*1.0317-0317,'Graph','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
axestext=text(.15-.10317,.85*1.0317-0317,'Axes','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
dataseriestext=text(.15-.10317,.40*1.0317-0317,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
legendtext=text(.15-.10317,.085*1.0317-0317,'Legends','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% GRAPH UICONTROLS

bgtext=uicontrol('position',[origin(1)+25 origin(2)+575 180 20],...
    'style','text','string','Background Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
gridtext=uicontrol('position',[origin(1)+25 origin(2)+550 180 20],...
    'style','text','string','Grid Lines:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

%bgcolor=get(handles.fakerax(handles.activeindex),'color');

bgindex=eye(4);
bgin=format.bg;

% color blobs: (userdata = gridcolor; textcolor; index)
bgwhite=uicontrol('position',[origin(1)+155 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',[1 1 1],'value',bgindex(1,bgin),...
    'callback',@bgblob_callback,'userdata',[.6 .6 .6;0 0 0;1 0 0]);
bglight=uicontrol('position',[origin(1)+180 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',[.753 .753 .753],'value',bgindex(2,bgin),...
    'callback',@bgblob_callback,'userdata',[1 1 1;0 0 0;2 0 0]);
bgdark=uicontrol('position',[origin(1)+205 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',[.502 .502 .502],'value',bgindex(3,bgin),...
    'callback',@bgblob_callback,'userdata',[.6 .6 .6;1 1 1;3 0 0]);
bgblack=uicontrol('position',[origin(1)+230 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',[0 0 0],'value',bgindex(4,bgin),...
    'callback',@bgblob_callback,'userdata',[.3 .3 .3;1 1 1;4 0 0]);

handles.bgblobs=[bgwhite bglight bgdark bgblack];


gridindex=eye(3);
gridin=format.grid;


%grid radios
nogrid=uicontrol('position',[origin(1)+100 origin(2)+551 60 20],...
    'style','radiobutton','string','None','backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(1,gridin),'callback',@nogrid_callback);
magrid=uicontrol('position',[origin(1)+152 origin(2)+551 60 20],...
    'style','radiobutton','string','Major','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(2,gridin),'callback',@magrid_callback);
migrid=uicontrol('position',[origin(1)+204 origin(2)+551 55 20],...
    'style','radiobutton','string','Minor','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(3,gridin),'callback',@migrid_callback);

handles.gridchecks=[nogrid magrid migrid];

%% Axes UICONTROLS

xlim=get(handles.activegraph,'xlim');
ylim=get(handles.activegraph,'ylim');
zlim=get(handles.activegraph,'zlim');

xlabeltext=uicontrol('position',[origin(1)+25 origin(2)+498 180 20],...
    'style','text','string','X Label:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xlimtext=uicontrol('position',[origin(1)+35 origin(2)+474 180 20],...
    'style','text','string','Limits:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xto=uicontrol('position',[origin(1)+158 origin(2)+474 20 20],...
    'style','text','string','to','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xlabelbox=uicontrol('position',[origin(1)+85 origin(2)+500 160 20],...
    'style','edit','string',get(get(handles.activegraph,'xlabel'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@xlabel_callback);
xminbox=uicontrol('position',[origin(1)+85 origin(2)+476 70 20],...
    'style','edit','string',xlim(1),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@xmin_callback,...
    'tag','xminbox');
xmaxbox=uicontrol('position',[origin(1)+185 origin(2)+476 70 20],...
    'style','edit','string',xlim(2),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@xmax_callback,...
    'tag','xmaxbox');
xresetbutton=uicontrol('position',[origin(1)+85 origin(2)+452 160 20],...
    'style','pushbutton','string','Reset X Axis Limits','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','callback',@xresetbutton_callback);

ylabeltext=uicontrol('position',[origin(1)+25 origin(2)+410+4 180 20],...
    'style','text','string','Y Label:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
ylimtext=uicontrol('position',[origin(1)+35 origin(2)+386+4 180 20],...
    'style','text','string','Limits:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
yto=uicontrol('position',[origin(1)+158 origin(2)+386+4 20 20],...
    'style','text','string','to','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
ylabelbox=uicontrol('position',[origin(1)+85 origin(2)+412+4 160 20],...
    'style','edit','string',get(get(handles.activegraph,'ylabel'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@ylabel_callback);
yminbox=uicontrol('position',[origin(1)+85 origin(2)+388+4 70 20],...
    'style','edit','string',ylim(1),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@ymin_callback,...
    'tag','yminbox');
ymaxbox=uicontrol('position',[origin(1)+175 origin(2)+388+4 70 20],...
    'style','edit','string',ylim(2),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@ymax_callback,...
    'tag','ymaxbox');
yresetbutton=uicontrol('position',[origin(1)+85 origin(2)+364+4 160 20],...
    'style','pushbutton','string','Reset Y Axis Limits','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','callback',@yresetbutton_callback);

zlabeltext=uicontrol('position',[origin(1)+25 origin(2)+410-80 180 20],...
    'style','text','string','Z Label:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
zlimtext=uicontrol('position',[origin(1)+35 origin(2)+386-80 180 20],...
    'style','text','string','Limits:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
zto=uicontrol('position',[origin(1)+158 origin(2)+406-100 20 20],...
    'style','text','string','to','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
zlabelbox=uicontrol('position',[origin(1)+85 origin(2)+432-100 160 20],...
    'style','edit','string',get(get(handles.activegraph,'zlabel'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@zlabel_callback);
zminbox=uicontrol('position',[origin(1)+85 origin(2)+408-100 70 20],...
    'style','edit','string',zlim(1),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@zmin_callback,...
    'tag','yminbox');
zmaxbox=uicontrol('position',[origin(1)+175 origin(2)+408-100 70 20],...
    'style','edit','string',zlim(2),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@zmax_callback,...
    'tag','ymaxbox');
zresetbutton=uicontrol('position',[origin(1)+85 origin(2)+384-100 160 20],...
    'style','pushbutton','string','Reset Z Axis Limits','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','callback',@zresetbutton_callback);

titletext=uicontrol('position',[origin(1)+25 origin(2)+332-80 180 20],...
    'style','text','string','Title:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
titlebox=uicontrol('position',[origin(1)+65 origin(2)+333-80 180 20],...
    'style','edit','string',get(get(handles.activegraph,'title'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@title_callback);

%%  marker DATA SERIES UICONTROLS


colortext=uicontrol('position',[origin(1)+30 origin(2)+200+5 180 20],...
    'style','text','string','Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

cdata=ravemakecdata(handles.fig,[],handles.cmapnames{udata.format.mcmap});

markercolor=uicontrol('position',[origin(1)+85 origin(2)+207 120 20],...
    'style','pushbutton','string','','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','cdata',cdata,'callback',@mcolor_callback);



edgetext=uicontrol('position',[origin(1)+30 origin(2)+170 180 20],...
    'style','text','string','Edges:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

solidedge=uicontrol('position',[origin(1)+85 origin(2)+170 24 24],...
    'style','togglebutton','value',0,'backgroundcolor',udata.format.edgecolor,...
    'tag','solidedge','userdata',1,'callback',@edgeblob_callback);
flatedge=uicontrol('position',[origin(1)+85+30 origin(2)+170 24 24],...
    'style','togglebutton','value',0,'backgroundcolor',handles.colors.togglebuttonbg,...
    'cdata',imread('flat.bmp'),'tag','flatedge','userdata',2,'callback',@edgeblob_callback);
interpedge=uicontrol('position',[origin(1)+85+90 origin(2)+170 24 24],...
    'style','togglebutton','value',0,'backgroundcolor',handles.colors.togglebuttonbg,...
    'cdata',imread('interp.bmp'),'tag','interpedge','userdata',3,'callback',@edgeblob_callback,'visible','off');
noedge=uicontrol('position',[origin(1)+85+60 origin(2)+170 24 24],...
    'style','togglebutton','value',0,'backgroundcolor',handles.colors.togglebuttonbg,...
    'cdata',imread('none.bmp'),'tag','noedge','userdata',4,'callback',@edgeblob_callback);

edgeblobs=[solidedge flatedge interpedge noedge];
set(edgeblobs(udata.format.edgestyle),'value',1);


facetext=uicontrol('position',[origin(1)+30 origin(2)+135-32 180 20],...
    'style','text','string','Face:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

solidface=uicontrol('position',[origin(1)+85 origin(2)+137-32 24 24],...
    'style','togglebutton','value',0,'backgroundcolor',udata.format.facecolor,...
    'tag','solidface','userdata',1,'callback',@faceblob_callback);
flatface=uicontrol('position',[origin(1)+85+30 origin(2)+137-32 24 24],...
    'style','togglebutton','value',0,'backgroundcolor',handles.colors.togglebuttonbg,...
    'cdata',imread('flat.bmp'),'tag','flatface','userdata',2,'callback',@faceblob_callback);
interpface=uicontrol('position',[origin(1)+85+90 origin(2)+137-32 24 24],...
    'style','togglebutton','value',0,'backgroundcolor',handles.colors.togglebuttonbg,...
    'cdata',imread('interp.bmp'),'tag','interpface','userdata',3,'callback',@faceblob_callback,'visible','off');
noface=uicontrol('position',[origin(1)+85+60 origin(2)+137-32 24 24],...
    'style','togglebutton','value',0,'backgroundcolor',handles.colors.togglebuttonbg,...
    'cdata',imread('none.bmp'),'tag','noface','userdata',4,'callback',@faceblob_callback);

faceblobs=[solidface flatface interpface noface];
set(faceblobs(udata.format.facestyle),'value',1);


smoothtext=uicontrol('position',[origin(1)+25 origin(2)+68 180 20],...
    'style','text','string','Resolution:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

%smoothness slider:
[jslider hslider]=javacomponent('javax.swing.JSlider',[origin(1)+110 origin(2)+56 140 40]);
jslider.setPaintTicks(1)

jslider.setSnapToTicks(1)
jslider.setMinimum(2)
jslider.setMaximum(30)
jslider.setMajorTickSpacing(2);
jslider.setValue(udata.view.numpts);
set(jslider,'background',java.awt.Color.white)
set(jslider,'StateChangedCallback',{@jslider_Callback,handles.fig});

smoothtext2=uicontrol('position',[origin(1)+107 origin(2)+42 150 20],...
    'style','text','string','Low                                  High','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',8,'horizontalalignment','left','fontweight','normal');




%width slider:
[jslider2 wslider]=javacomponent('javax.swing.JSlider',[origin(1)+85 origin(2)+167-28 100 27]);
jslider2.setPaintTicks(1)
jslider2.setSnapToTicks(1)
jslider2.setMinimum(1)
jslider2.setMaximum(5)
jslider2.setMajorTickSpacing(1);
jslider2.setValue(format.linesize);
set(jslider2,'background',java.awt.Color.white)
set(jslider2,'StateChangedCallback',{@linewidth_callback,handles.fig});


widthaxes=axes('units','pixels','position',[origin(1)+80 origin(2)+167-33 110 5],'visible','off');
gauge=imread('widthgauge.bmp');
image(gauge,'parent',widthaxes)
set(widthaxes,'visible','off');







%% LEGENDS
boxoncheck=uicontrol('position',[origin(1)+135 origin(2)+02 120 20],...
    'style','checkbox','string','Show Cube Edges','value',format.boxon,'backgroundcolor',handles.colors.tabbg,...
    'fontname','arial narrow','fontsize',10,'horizontalalignment','left','callback',@boxon_callback);



colorbarcheck=uicontrol('position',[origin(1)+25 origin(2)+02 100 20],...
    'style','checkbox','string','Show Colorbar','fontname','arial narrow','fontsize',10,...
    'backgroundcolor',handles.colors.tabbg,'callback',@colorbarcheck_callback,...
    'fontweight','normal','horizontalalignment','left','value',udata.format.colorbar);



%% group items and finish up
handles.tabobjects=[graphtext axestext dataseriestext bgtext gridtext bgwhite...
    bglight bgdark bgblack nogrid magrid migrid xlabeltext ylabeltext...
    xlimtext ylimtext xlabelbox ylabelbox xminbox yminbox xto xmaxbox...
    xresetbutton yto ymaxbox yresetbutton colortext...
    markercolor edgetext facetext faceblobs edgeblobs...
    legendtext titletext titlebox boxoncheck colorbarcheck...
    zlimtext zlabelbox zminbox zmaxbox zto zresetbutton zlabeltext smoothtext smoothtext2 hslider wslider widthaxes];

handles.activetab='format';

set(handles.infobar,'string','');

guidata(src,handles);

%% HERE BE CALLBACKS
%
%
%
%
%
%
%
%
%% 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function bgblob_callback(src,eventdata)

handles=guidata(src);

uicontrol(handles.dummyobj)

colors=get(src,'userdata');
gridcolor=colors(1,:);
textcolor=colors(2,:);

plotcolor=get(src,'backgroundcolor');

%update fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.bg=colors(3,1);
set(handles.fakeax(handles.activeindex),'userdata',udata);

%unpop blobs:
set(handles.bgblobs,'value',0);
set(src,'value',1)

%change colors
set(handles.fakerax(handles.activeindex),'color',plotcolor,'xcolor',gridcolor,'ycolor',gridcolor,'zcolor',gridcolor)
set(handles.fakeax(handles.activeindex),'xcolor',gridcolor,'ycolor',gridcolor,'zcolor',gridcolor);

%fix tick marks
if all(plotcolor==[0 0 0])
    set(handles.activegraph,'ticklength',[0 0])
else
    set(handles.activegraph,'tickdir','in','ticklength',[.01 .025])
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function magrid_callback(src, eventdata)
handles=guidata(src);
%inputs=get(handles.fakerax{handles.activeindex},'userdata');

set(handles.gridchecks,'value',0)
set(src,'value',1)

%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=2;
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on','zgrid','on')
set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off','zminorgrid','off')
set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')

function migrid_callback(src, eventdata)
handles=guidata(src);

set(handles.gridchecks,'value',0)
set(src,'value',1)


%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=3;
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on','ZminorGrid','on')
set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off','ZGrid','off')
set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on','zgrid','on')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function nogrid_callback(src, eventdata)
handles=guidata(src);

set(handles.gridchecks,'value',0)
set(src,'value',1)

%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=1;
set(handles.fakeax(handles.activeindex),'userdata',udata);


set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off','zminorgrid','off')
set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function xmin_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
xlim=get(handles.activegraph,'xlim');
xlim(1)=newval;
set(handles.activegraph,'xlim',xlim)
set(handles.fakeax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'),'zlim',get(handles.activegraph,'zlim'))
set(handles.fakerax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'),'zlim',get(handles.activegraph,'zlim'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xmax_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
xlim=get(handles.activegraph,'xlim');
xlim(2)=newval;
set(handles.activegraph,'xlim',xlim)
set(handles.fakeax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'),'zlim',get(handles.activegraph,'zlim'))
set(handles.fakerax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'),'zlim',get(handles.activegraph,'zlim'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ymin_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
ylim=get(handles.activegraph,'ylim');
ylim(1)=newval;
set(handles.activegraph,'ylim',ylim)
set(handles.fakeax(handles.activeindex),'ylim',ylim,'xlim',get(handles.activegraph,'xlim'),'zlim',get(handles.activegraph,'zlim'))
set(handles.fakerax(handles.activeindex),'ylim',ylim,'xlim',get(handles.activegraph,'xlim'),'zlim',get(handles.activegraph,'zlim'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ymax_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
ylim=get(handles.activegraph,'ylim');
ylim(2)=newval;
set(handles.activegraph,'ylim',ylim)
set(handles.fakeax(handles.activeindex),'ylim',ylim,'xlim',get(handles.activegraph,'xlim'),'zlim',get(handles.activegraph,'zlim'))
set(handles.fakerax(handles.activeindex),'ylim',ylim,'xlim',get(handles.activegraph,'xlim'),'zlim',get(handles.activegraph,'zlim'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function zmin_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
zlim=get(handles.activegraph,'zlim');
zlim(1)=newval;
set(handles.activegraph,'zlim',zlim)
set(handles.fakeax(handles.activeindex),'zlim',zlim,'ylim',get(handles.activegraph,'ylim'),'xlim',get(handles.activegraph,'xlim'))
set(handles.fakerax(handles.activeindex),'zlim',zlim,'ylim',get(handles.activegraph,'ylim'),'xlim',get(handles.activegraph,'xlim'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function zmax_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
zlim=get(handles.activegraph,'zlim');
zlim(2)=newval;
set(handles.activegraph,'zlim',zlim)
set(handles.fakeax(handles.activeindex),'zlim',zlim,'ylim',get(handles.activegraph,'ylim'),'xlim',get(handles.activegraph,'xlim'))
set(handles.fakerax(handles.activeindex),'zlim',zlim,'ylim',get(handles.activegraph,'ylim'),'xlim',get(handles.activegraph,'xlim'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xresetbutton_callback(src,eventdata)

handles=guidata(src);
set(handles.activegraph,'xlimmode','auto')
xlim=get(handles.activegraph,'xlim');

set(handles.fakeax(handles.activeindex),'xlim',xlim)
set(handles.fakerax(handles.activeindex),'xlim',xlim)

xminbox=findobj(handles.tabobjects,'tag','xminbox');
xmaxbox=findobj(handles.tabobjects,'tag','xmaxbox');

set(xminbox,'string',num2str(xlim(1)))
set(xmaxbox,'string',num2str(xlim(2)))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function yresetbutton_callback(src,eventdata)

handles=guidata(src);
set(handles.activegraph,'ylimmode','auto')
ylim=get(handles.activegraph,'ylim');

set(handles.fakeax(handles.activeindex),'ylim',ylim)
set(handles.fakerax(handles.activeindex),'ylim',ylim)

yminbox=findobj(handles.tabobjects,'tag','yminbox');
ymaxbox=findobj(handles.tabobjects,'tag','ymaxbox');

set(yminbox,'string',num2str(ylim(1)))
set(ymaxbox,'string',num2str(ylim(2)))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function zresetbutton_callback(src,eventdata)

handles=guidata(src);
set(handles.activegraph,'zlimmode','auto')
zlim=get(handles.activegraph,'zlim');

set(handles.fakeax(handles.activeindex),'zlim',zlim)
set(handles.fakerax(handles.activeindex),'zlim',zlim)

zminbox=findobj(handles.tabobjects,'tag','zminbox');
zmaxbox=findobj(handles.tabobjects,'tag','zmaxbox');

set(zminbox,'string',num2str(zlim(1)))
set(zmaxbox,'string',num2str(zlim(2)))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xlabel_callback(src,eventdata)

handles=guidata(src);
set(get(handles.activegraph,'xlabel'),'string',get(src,'string'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ylabel_callback(src,eventdata)

handles=guidata(src);
set(get(handles.activegraph,'ylabel'),'string',get(src,'string'))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function zlabel_callback(src,eventdata)

handles=guidata(src);
set(get(handles.activegraph,'zlabel'),'string',get(src,'string'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function mcolor_callback(src,eventdata)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

newcmap=ravechoosecolormap(src,[],udata.format.mcmap);


if newcmap==udata.format.mcmap;return;end

udata.format.mcmap=newcmap;


colorz=handles.cmapnames;
newcolormap=colorz{newcmap};

cdata=ravemakecdata(handles.fig,[],newcolormap);
set(src,'cdata',cdata);


set(handles.fakeax(handles.activeindex),'userdata',udata);

if udata.view.c~=0
    raveupdatesurfaceprofiler(handles.fig,[]);
end


if udata.format.colorbar;
    if udata.view.c
        ravefixcolorbar(handles.fig,[]);
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function title_callback(src,eventdata)
handles=guidata(src);
newtitle=get(src,'string');
set(get(handles.activegraph,'title'),'string',newtitle,'fontsize',14);
raveresizeaxes(src,[],handles.activeindex);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
function formatlegend_callback(src,eventdata)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
function boxon_callback(src,eventdata)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.boxon=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);
targets=[handles.activegraph handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)];

if udata.format.boxon
    set(targets,'box','on')
else
    set(targets,'box','off')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function colorbarcheck_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.format.colorbar=get(src,'value');

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatesurfaceprofiler(src,[]);


function faceblob_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');


faceblobs=findobj(handles.tabobjects,'tag','noface','-or','tag','flatface','-or','tag','solidface','-or','tag','interpface');
set(faceblobs,'value',0);
set(src,'value',1);


main=findobj(handles.activegraph,'tag','main');

if strcmp(get(src,'tag'),'solidface')

    udata.format.facecolor=ravecolorchooser(handles.fig,udata.format.facecolor,'Pick a Color');

    set(src,'backgroundcolor',udata.format.facecolor);
    set(main,'facecolor',udata.format.facecolor);
    udata.format.facestyle=1;


elseif strcmp(get(src,'tag'),'noface')
    set(main,'facecolor','none');
    udata.format.facestyle=4;

elseif strcmp(get(src,'tag'),'flatface')
    set(main,'facecolor','flat');
    udata.format.facestyle=2;
else
    set(main,'facecolor','interp','cdatamapping','direct');
    udata.format.facestyle=3;

end

set(handles.fakeax(handles.activeindex),'userdata',udata);



function edgeblob_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

edgeblobs=findobj(handles.tabobjects,'tag','noedge','-or','tag','flatedge','-or','tag','solidedge','-or','tag','interpedge');
set(edgeblobs,'value',0);
set(src,'value',1);

main=findobj(handles.activegraph,'tag','main');

if strcmp(get(src,'tag'),'solidedge')

    udata.format.edgecolor=ravecolorchooser(handles.fig,udata.format.edgecolor,'Pick a Color');

    set(src,'backgroundcolor',udata.format.edgecolor);
    set(main,'edgecolor',udata.format.edgecolor);
    
    udata.format.edgestyle=1;


elseif strcmp(get(src,'tag'),'noedge')
    set(main,'edgecolor','none');
    udata.format.edgestyle=4;

elseif strcmp(get(src,'tag'),'flatedge')
    set(main,'edgecolor','flat');
    udata.format.edgestyle=2;
else
    set(main,'edgecolor','interp');
udata.format.edgestyle=3;
end


set(handles.fakeax(handles.activeindex),'userdata',udata);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function jslider_Callback(src,ev,fighandle) %#ok<INUSL> %reslution slider 

moving = get(src,'ValueIsAdjusting');
if strcmp(moving,'off')
    return
end

handles=guidata(fighandle);
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.numpts=src.getValue;

set(handles.fakeax(handles.activeindex),'userdata',udata)

ravecreatesurfaceprofiler(handles.fig,[]);




function linewidth_callback(src,ev,fighandle) %#ok<INUSL>


moving = get(src,'ValueIsAdjusting');
if strcmp(moving,'off')
    return
end

handles=guidata(fighandle);
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.format.linesize=src.getValue;

set(handles.fakeax(handles.activeindex),'userdata',udata)

ravecreatesurfaceprofiler(handles.fig,[]);












