function ravecreatesurfaceprofiler(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;


%inputs=varargin{3};
targetaxes=handles.activegraph;
xcol=udata.view.x; %ones are used to take first var in case 2 vars have same name.
ycol=udata.view.y;
zcol=udata.view.z;
ccol=udata.view.c;


d=udata.view.dataset;
a=udata.view.analysis;

xdials=handles.xdials{d}{a};
funvars=udata.view.funvars;
funvarnames=handles.allnames{d}(funvars);

if isempty(zcol)
    set(handles.infobar,'string','Error: No variables in this dataset are modeled by Equations.')
    ravedeletegraph(src,[],0)
    return
end

%clear existing graph
cla(targetaxes);
%legend(targetaxes,'off');


%font formatting
fweight='normal';
labelsize=12;

%get data


%get type of each variable:
datatypes=handles.datatype{d}([xcol ycol]);

totalrows=zeros(size(datatypes)); %totalrows lists the total number of rows in the input data that correspond to the graph of this input.
%thus final input data has sum(totalrows) rows.


xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};

numpts=udata.view.numpts;

step=(xmax-xmin)/numpts;
xsteps=cell(1,numel(datatypes));

colz=[xcol ycol];



for j=1:numel(datatypes);
    switch datatypes{j}
        case 'constant'
            xsteps{j}=xdials(colz(j));
            totalrows(j)=1;
        case 'continuous'
            xsteps{j}=sort((xmin(colz(j)):step(colz(j)):xmax(colz(j))))'; %do not include xdials for surface profiler, resolution isnt good enough to notice.
            totalrows(j)=numel(xsteps{j});
        case 'integer'
            if numel(handles.discretevalues{d}{a}{colz(j)})<20
                xsteps{j}=handles.discretevalues{d}{a}{colz(j)};
                totalrows(j)=numel(xsteps{j});
            else
                xsteps{j}=sort([xmin(colz(j)):step(colz(j)):xmax(colz(j)) xdials(colz(j))]);
                totalrows(j)=numel(xsteps{j});
            end
        case 'discrete'
            xsteps{j}=handles.discretevalues{d}{a}{colz(j)};
            totalrows(j)=numel(xsteps{j});
        case 'string'
            xsteps{j}=handles.discretevalues{d}{a}{colz(j)};
            totalrows(j)=numel(xsteps{j});
        case 'logical'
            xsteps{j}=[0 1];
            totalrows(j)=numel(xsteps{j});
    end
end

%evaluation grid will be numel(xsteps{2}) by numel(xsteps{1}) elements.
yevals=repmat(xsteps{2}(:),[numel(xsteps{1}) 1]);
index=repmat(1:numel(xsteps{1}),[numel(xsteps{2}) 1]);
xevals=xsteps{1}(index(:));

%make whole matrix of poitns to eval. this is fixed array of xdials, with
%xeval/yeval subbed in:
indata=repmat(xdials,[numel(xevals) 1]);
indata(:,xcol)=xevals;
indata(:,ycol)=yevals;

%eval z at indata:

zvalues=raveevaluateregression(src,d,zcol,indata);

%reshape to square matrices:
xdata=reshape(xevals,[numel(xsteps{2}) numel(xsteps{1})]);
ydata=reshape(yevals,[numel(xsteps{2}) numel(xsteps{1})]);
zdata=reshape(zvalues,[numel(xsteps{2}) numel(xsteps{1})]);

if ccol~=0
    c=raveevaluateregression(src,d,ccol,indata);
    %normalize cdata to desired colormap:
    c=(c-min(c))/(max(c)-min(c))*127+128*(udata.format.mcmap-1)+1;  
    cdata=reshape(c,[numel(xsteps{2}) numel(xsteps{1})]);
else    
        cdata=[0 .5 1]; %non-noisy colors cdata=0.5      
end

set(targetaxes,'climmode','manual','clim',[1 size(get(handles.fig,'colormap'),1)]);

%fold if xvar is discrete:
if  ~any(strcmp(datatypes{1},{'continuous';'constant'})) %if this variable is discrete
    allowed=handles.discretevalues{d}{a}{xcol};
    endind=numel(allowed);

    xvals=xsteps{1}(:); %force to be column.
    xintermediates=xvals+[diff(xvals)./2;0];
    xvals=sort([xvals;xintermediates(1:end-1)]);
    
    xmap=repmat(2:2:2*(endind-1),2,1);
    xmap=[1 xmap(:)' xmap(end)+1];
    
    zmap=repmat(1:numel(xmap)/2,2,1);
    zmap=zmap(:)';
    
    xdata=repmat(xvals(xmap)',size(xdata,1),1);
    ydata=repmat(ydata(:,1),1,numel(xmap));
    zdata=zdata(:,zmap);
    cdata=cdata(:,zmap);   
end

%fold if yvar is discrete:
if  ~any(strcmp(datatypes{2},{'continuous';'constant'})) %if this variable is discrete
    allowed=handles.discretevalues{d}{a}{ycol};
    endind=numel(allowed);

    yvals=xsteps{2}(:); %force to be column.
    yintermediates=yvals+[diff(yvals)./2;0];
    yvals=sort([yvals;yintermediates(1:end-1)]);

    ymap=repmat(2:2:2*(endind-1),2,1);
    ymap=[1 ymap(:)' ymap(end)+1];
    
    zmap=repmat(1:numel(ymap)/2,2,1);
    zmap=zmap(:)';
    
    ydata=repmat(yvals(ymap),1,size(ydata,2));
    xdata=repmat(xdata(1,:),numel(ymap),1);
    zdata=zdata(zmap,:);
    cdata=cdata(zmap,:);   
end

facestyles={udata.format.facecolor,'flat','interp','none'};
edgestyles={udata.format.edgecolor,'flat','interp','none'};

if isscalar(zdata) %in case both x and y are constants:
    xdata=[xdata xdata;xdata+eps(xdata) xdata+eps];
    ydata=[ydata ydata+eps(ydata);ydata ydata+eps(ydata)];
    zdata=[zdata zdata;zdata zdata];
    cdata=[cdata cdata;cdata cdata];
end

H=surface(xdata,ydata,zdata,cdata,'parent',targetaxes,'tag','main','linewidth',udata.format.linesize,...
    'edgecolor',edgestyles{udata.format.edgestyle},'facecolor',facestyles{udata.format.facestyle});

if xcol==ycol
    set(H,'visible','off')
end

%set axes labels
set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter','none','fontname','arial')
set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname','arial')
set(get(targetaxes,'ZLabel'),'String',handles.allnames{d}{zcol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname','arial')
    
    
set(get(handles.fakerax(handles.activeindex),'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter','none','fontname','arial')
set(get(handles.fakerax(handles.activeindex),'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname','arial')
set(get(handles.fakerax(handles.activeindex),'ZLabel'),'String',handles.allnames{d}{zcol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname','arial')  
    
set(get(handles.fakeax(handles.activeindex),'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter','none','fontname','arial')
set(get(handles.fakeax(handles.activeindex),'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname','arial')
set(get(handles.fakeax(handles.activeindex),'ZLabel'),'String',handles.allnames{d}{zcol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname','arial')   

%resize axes so axes labels/ticks fit inside the bounding box.
targetindex=find(handles.allgraphs==targetaxes);

%adjust gridlines to match settings:
if format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off','zminorgrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
elseif format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on','zgrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off','zminorgrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
elseif format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on','zminorgrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on','zgrid','on')
end

%make reasonable axes limits:

zlims=[min(handles.alldata{d}(:,zcol)) max(handles.alldata{d}(:,zcol))];
xlims=[handles.minlimit{d}{a}(xcol) handles.maxlimit{d}{a}(xcol)];
ylims=[handles.minlimit{d}{a}(ycol) handles.maxlimit{d}{a}(ycol)];

try %#ok<TRYNC> %gives error if x/y is constant. just let it use auto lims in that case. 
    set(targetaxes,'xlim',xlims,'ylim',ylims,'zlim',zlims);
end
view(targetaxes,3);
view(handles.fakeax(handles.activeindex),3);
view(handles.fakerax(handles.activeindex),3);


%make all axes have same limits
Q=[handles.fakerax(handles.activeindex) handles.fakeax(handles.activeindex)];

set(Q,'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'),'zlim',get(targetaxes,'zlim'),...
    'xtickmode','manual','xtick',get(targetaxes,'xtick'),...
    'ytickmode','manual','ytick',get(targetaxes,'ytick'),...
    'ztickmode','manual','ztick',get(targetaxes,'ztick'));

set(Q,...
    'xticklabel',get(targetaxes,'xticklabel'),...
    'yticklabel',get(targetaxes,'yticklabel'),...
    'zticklabel',get(targetaxes,'zticklabel'));

set(Q,'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'),'zlim',get(targetaxes,'zlim'))
set(targetaxes,'nextplot','add')

set(Q,'ticklength',[0 0]);


%update da handles
guidata(handles.fig,handles)

ravemaketextticks(targetaxes,[],xcol,ycol,[],[])

raveresizeaxes(src,[],targetindex)

ravemakecolorbar(handles.fig,[]);     




set(Q,...
    'xticklabel','',...
    'yticklabel','',...
    'zticklabel','');


set(get(handles.fakerax(handles.activeindex),'XLabel'),'String','')
set(get(handles.fakerax(handles.activeindex),'YLabel'),'String','')
set(get(handles.fakerax(handles.activeindex),'ZLabel'),'String','')  
    
set(get(handles.fakeax(handles.activeindex),'XLabel'),'String','')
set(get(handles.fakeax(handles.activeindex),'YLabel'),'String','')
set(get(handles.fakeax(handles.activeindex),'ZLabel'),'String','')   



xxx=get(handles.activegraph,'xlabel');
set(xxx,'buttondownfcn',@ravemove3dlabels,'hittest','on');
yyy=get(handles.activegraph,'ylabel');
set(yyy,'buttondownfcn',@ravemove3dlabels,'hittest','on');





