function ravecreategreatness(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');

targetaxes=handles.activegraph; %=handles.activegraph
ycol=udata.view.y;
xcol=udata.view.x;
a=udata.view.analysis; %#ok<NASGU>
d=udata.view.dataset; %data set index

numy=length(ycol);
numx=length(xcol);

if numx==0 %this can happen when switching to a new yvar that shares no vars with previous xvar selection.
    numx=1;
    xcol=1;
end


if numy==0
    set(handles.infobar,'string','Error: No variables in this dataset are modeled by functions.')
    ravedeletegraph(src,[],0)
    return
end

%clear existing graph
cla(targetaxes);
delete(handles.subax{handles.activeindex})
delete(handles.subfakeax{handles.activeindex})
delete(handles.subfakerax{handles.activeindex})


%get data
xnames=handles.allnames{d}(xcol);
ynames=handles.allnames{d}(ycol);


set(targetaxes,'visible','off')
set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'visible','off')

% Create and plot into small axes

xyax=zeros(numy,numx);
xxax=zeros(numx,numx);
yyax=zeros(numy,numy);
xyfakeax=zeros(numy,numx);
xxfakeax=zeros(numx,numx);
yyfakeax=zeros(numy,numy);
xyfakerax=zeros(numy,numx);
xxfakerax=zeros(numx,numx);
yyfakerax=zeros(numy,numy);



h.xymain=[]; 
h.xyxhair=[];
h.xyyhair=[];
h.xxmain=[]; 
h.xxxhair=[];
h.xxyhair=[];
h.yymain=[]; 
h.yyxhair=[];
h.yyyhair=[];


rows=numy;
cols=numx;

pos = get(targetaxes,'Position');%+[52 60 -60 -75];
origin=get(targetaxes,'pos');

totalwidth=pos(3);
totalheight=pos(4);

space=handles.prefs.matrixspacing;
space=0;

sizecoefs=[numy numx/2;numy/2 numx];
outdims=sizecoefs\[totalheight-(numx+numy)*(1+space);totalwidth-(numx+numy)*(1+space)]; %subtract 1px per graph for rounding error

height=ceil(outdims(1));
width=ceil(outdims(2));


numpts=udata.view.numpts;

BigAxParent = get(targetaxes,'Parent');

%% calculate profiler data for each graph:

%get type of each variable:
datatypes=handles.datatype{d}(xcol);

totalrows=zeros(size(datatypes)); %totalrows lists the total number of rows in the input data that correspond to the graph of this input.
%thus final input data has sum(totalrows) rows.

xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};

xdials=handles.xdials{d}{a};


step=(xmax-xmin)/numpts;
xsteps=cell(1,numel(datatypes));

for j=1:numel(datatypes);
    switch datatypes{j}
        case 'constant'
            xsteps{j}=xdials(xcol(j));
            totalrows(j)=1;
        case 'continuous'
            xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
            totalrows(j)=numel(xsteps{j});
        case 'integer'
            if numel(handles.discretevalues{d}{a}{xcol(j)})<20
                xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
                totalrows(j)=numel(xsteps{j});
            else
                xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
                totalrows(j)=numel(xsteps{j});
            end
        case 'discrete'
            xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
            totalrows(j)=numel(xsteps{j});
        case 'string'
            xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
            totalrows(j)=numel(xsteps{j});
        case 'logical'
            xsteps{j}=[0 1];
            totalrows(j)=numel(xsteps{j});
    end
end

rowz=sum(totalrows);



xfixed=repmat(xdials,[rowz,1]);

%convert to cell if strings are present:
if any(strcmp('string',datatypes))
    xfixed=num2cell(xfixed);
    for k=1:cols
        [xfixed{sum(totalrows(1:k-1))+1:sum(totalrows(1:k)),xcol(k)}]=deal(xsteps{k});
    end

else
    for k=1:cols
        xfixed(sum(totalrows(1:k-1))+1:sum(totalrows(1:k)),xcol(k))=xsteps{k};
    end
end

%evaluate the functions:

yvalues=raveevaluateregression(src,d,udata.view.y,xfixed);
ycurrent=raveevaluateregression(src,d,udata.view.y,xdials);

%put results of each response in a seperate cell:
yvalues=mat2cell(yvalues,size(yvalues,1),ones(1,numy));
ycurrent=mat2cell(ycurrent,size(ycurrent,1),ones(1,numy));

%and put each x for each resposne in its own cell too:
for zz=1:numy
    yvalues{zz}=mat2cell(yvalues{zz},totalrows,1);
end

%set up prediction profiler grid of axes.
for r=1:1:rows
    for c=1:1:cols
        axPos = [round(pos(1))+(c-1)*(width+space) pos(2)+(rows-r)*(height+space) ...
            width height];

        xyfakerax(r,c) = axes('units','pixels','Position',axPos,'parent',handles.wspanel,...
            'hittest','off','gridlinestyle',':','xcolor',[.5 .5 .5],'ycolor',[.5 .5 .5],'ticklength',[.02 .02],'visible','off');
        xyfakeax(r,c) = axes('units','pixels','Position',axPos,'parent',handles.wspanel,...
            'hittest','off','gridlinestyle','-','color','none','xcolor',[.5 .5 .5],'ycolor',[.5 .5 .5],'ticklength',[.02 .02],'visible','off');
        xyax(r,c) = axes('units','pixels','Position',axPos,'parent',handles.wspanel,...
            'visible','on','nextplot','add','box','on','hittest','on','buttondownfcn',@raveupdategreatness,...
            'userdata',handles.activeindex,'color','none','tag','pprof','ticklength',[.02 .02]); %userdata contains the index of the bigax for this subgraph.

        % set([fakerax(r,c) fakeax(r,c)],'units','pixels','visible','on','XColor',[.5 .5 .5],'YColor',[.5 .5 .5]);


        %make graphs here.
        
        if udata.format.usevariablecolors
            thisxcolor=handles.variablecolors{d}(udata.view.x(c),:);
            thisycolor=handles.variablecolors{d}(udata.view.y(r),:);
            maincolor=thisycolor;
        else
            
            thisxcolor=udata.format.haircolor;
            thisycolor=udata.format.haircolor;
            maincolor=udata.format.linecolor;
        end
        

        if udata.format.smarthaircolors
            if handles.isconstant{udata.view.dataset}(xcol(c))
                thisxcolor=get(handles.fakeax(handles.activeindex),'xcolor');
                set(xyax(r,c),'buttondownfcn','') %this var is constant so shouldnt update profiler when clicking it.
            end

            if ~ismember(xcol(c),handles.funvars{udata.view.dataset}{ycol(r)})
                thisycolor=get(handles.fakeax(handles.activeindex),'xcolor');
            end
        end
        %hairline data series:
        h.xyxhair(r,c)=line([xdials(xcol(c)) xdials(xcol(c))],[0 1],'visible','on','linestyle',':','color',thisxcolor,...
            'parent',xyax(r,c),'hittest','off','tag','hair');
        %response hairline series:

        h.xyyhair(r,c)=line([-10000000 1000000000],[ycurrent{r} ycurrent{r}],'visible','on','linestyle',':','color',thisycolor,...
            'parent',xyax(r,c),'hittest','off','tag','hair');

        %main data series:

        if  ~any(strcmp(datatypes{c},{'continuous';'constant'})) %if this variable is discrete
            allowed=handles.discretevalues{d}{a}{xcol(c)};
            endind=numel(allowed);

            intsides=repmat(2:(endind-1),3,1);
            ymap=[1 1 intsides(:)' endind endind];

            xvals=xsteps{c}(:); %force to be column.
            xintermediates=xvals+[diff(xvals)./2;0];

            xvals=sort([xvals;xintermediates(1:end-1)]);

            xmap=repmat(2:2:2*(endind-1),3,1);
            xmap(end,:)=xmap(end,:)+ones(size(xmap(end,:)));

            xmap=[1 xmap(:)'];

            h.xymain(r,c)=line(xvals(xmap),yvalues{r}{c}(ymap),'linewidth',udata.format.linesize,'linestyle','-','marker','none',...
                'color',maincolor,'hittest','off','tag','keeper','parent',xyax(r,c));

        else
            h.xymain(r,c)=line(xsteps{c},yvalues{r}{c},'linewidth',udata.format.linesize,'linestyle','-','marker','none',...
                'color',maincolor,'hittest','off','tag','keeper','parent',xyax(r,c));
        end

        if udata.format.showpoints
            nothidden=handles.isvisible{udata.view.dataset}{udata.view.analysis};
            xdata=handles.alldata{udata.view.dataset}(nothidden,udata.view.x(c));
            ydata=handles.alldata{udata.view.dataset}(nothidden,udata.view.y(r));
            h.points(r,c)=line(xdata,ydata,'parent',xyax(r,c),'tag','points','linestyle','none','markeredgecolor','none','markerfacecolor',udata.format.pointcolor,...
                'marker','o','markersize',4,'hittest','on','buttondownfcn',@raveupdateprofiler);
            uistack(h.points(r,c),'bottom');
        end


        set(xyax(r,c),'xlimmode','auto','ylimmode','auto','xgrid','off','ygrid','off')

    end
end


ylabels=get(xyfakeax(:,1),'ylabel');
if iscell(ylabels)
    ylabels=cell2mat(ylabels);
end
udata.view.ylabels=ylabels;
set(handles.fakeax(handles.activeindex),'userdata',udata);


%turn on fakeax that display current values:
set(xyfakeax(:,1),'visible','on');
set(xyfakeax(end,:),'visible','on');


xlimmin = xmin(xcol);
xlimmax = xmax(xcol);
ylimmin=min(handles.alldata{d}(:,ycol));
ylimmax=max(handles.alldata{d}(:,ycol));

for u=1:cols
    if xlimmin(u)==xlimmax(u)
        xlimmin(u)=.9999*xlimmin(u);
        xlimmax(u)=1.0001*xlimmax(u);
    end
end


inset = .10;
%fix y limits
for r=1:rows

    set([xyax(r,1) xyfakeax(r,1) xyfakerax(r,1)],'ylim',[ylimmin(r) ylimmax(r)])
    dy = diff(get(xyax(r,1),'ylim'))*inset;
    set([xyax(r,:) xyfakeax(r,:) xyfakerax(r,:)],'ylim',[ylimmin(r)-dy ylimmax(r)+dy])
    set(h.xyxhair(r,:),'ydata',[ylimmin(r)-dy ylimmax(r)+dy])

    %force non exponential labels:
    set(xyax(r,1),'YTickLabel',get(xyax(r,1),'YTick'))
    set(xyfakeax(r,1),'YTickLabel','')
    set(xyfakerax(r,1),'YTickLabel','')

    %make axes labels:

    set(get(xyax(r,1),'ylabel'),'string',{ynames{r};'';'';''},'hittest','off','interpreter','none');
    set(get(xyfakeax(r,1),'ylabel'),'string',{['\color[rgb]{',num2str(udata.format.linecolor),'} ',num2str(ycurrent{r},5)];'';''},'interpreter','tex','hittest','off');
end
dx = zeros(1,cols);


%fix x limits
for c=1:cols,

    set([xyax(1,c) xyfakeax(1,c) xyfakerax(1,c)],'xlim',[xlimmin(c) xlimmax(c)])
    dx(c) = diff(get(xyax(1,c),'xlim'))*inset;
    set([xyax(:,c) xyfakeax(:,c) xyfakerax(:,c)],'xlim',[xlimmin(c)-dx(c) xlimmax(c)+dx(c)])

    %force non exponential labels:
    set(xyax(end,c),'XTickLabel',get(xyax(end,c),'XTick'))
    set(xyfakeax(end,c),'XTickLabel','')
    set(xyfakerax(end,c),'XTickLabel','')

    %make axes labels:
    set(get(xyax(1,c),'title'),'string',xnames{c},'interpreter','none');
    set(get(xyax(end,c),'xlabel'),'string',{'';'';'';xnames{c}},'verticalalignment','top','hittest','off','interpreter','none');
    set(get(xyfakeax(end,c),'xlabel'),'string',{' ';' ';['\color[rgb]{',num2str(udata.format.linecolor),'} ',num2str(xdials(xcol(c)),5)]},'verticalalignment','top','interpreter','tex','hittest','off');

end

set(xyax(1:rows-1,:),'xticklabel','')
set(xyax(:,2:cols),'yticklabel','')

set(xyfakeax(1:rows-1,:),'xticklabel','')
set(xyfakeax(:,2:cols),'yticklabel','')

set(xyfakerax(1:rows-1,:),'xticklabel','')
set(xyfakerax(:,2:cols),'yticklabel','')

set(xyax,'xlimmode','manual','ylimmode','manual')




%if more than 4 variables, rotate x axis labels 90 degrees:

%if less than 250 pixels per variable, rotate x axis labels 90 degrees: (use this for all labels so hittest can
%bet set to off.


if pos(3)/cols<250
    rotationz=90;
    halign='right';
    valign='default';
else
    rotationz=0;
    halign='center';
    valign='top';
end

for y=1:numx

    thistype=datatypes{y};
    switch thistype
        case {'continuous','constant','integer'}
            putthemhere=get(xyax(end,y),'XTick');
            usethesevalues=get(xyax(end,y),'XTickLabel');
        case 'discrete'
            discretez=handles.discretevalues{d}{a}{y};
            if numel(discretez)<=12;
                putthemhere=discretez;
                usethesevalues=discretez;
            else %too many, just treat as continuous
                putthemhere=get(xyax(end,y),'XTick');
                usethesevalues=get(xyax(end,y),'XTickLabel');
            end
        case 'string'
            usethesevalues=handles.discretevalues{d}{a}{y};
            putthemhere=unique(handles.alldata{d}(:,y));

        case 'logical'
            usethesevalues={handles.prefs.logicalfalse;handles.prefs.logicaltrue};
            putthemhere=[0 1];
    end

    set(xyax(end,y),'XTickLabel',[]);
    set(xyfakeax(end,y),'XTickLabel',[]);
    set(xyfakerax(end,y),'XTickLabel',[]);


    ylimits=get(xyax(end,1),'Ylim');
    ypos=ylimits(1)-.02*(ylimits(2)-ylimits(1));

    text(putthemhere,ypos*ones(1,length(putthemhere)),usethesevalues,...
        'HorizontalAlignment',halign,'rotation',rotationz,'parent',xyax(end,y),'hittest','off',...
        'verticalalignment',valign,'handlevisibility','off','tag','xticklabel');
end


%also replace yticklabels with strings, this allows y (axis) labels to be aligned with each other.
for y=1:rows

    usethesevalues=get(xyax(y,1),'YTickLabel');
    set(xyax(y,1),'YTickLabel',[]);
    set(xyfakeax(y,1),'YTickLabel',[]);
    set(xyfakerax(y,1),'YTickLabel',[]);

    putthemhere=get(xyax(y,1),'YTick');

    xlimits=get(xyax(end,1),'Xlim');
    xpos=xlimits(1)-.02*(xlimits(2)-xlimits(1));

    text(xpos*ones(1,length(putthemhere)),putthemhere,usethesevalues,...
        'HorizontalAlignment','right','rotation',0,'parent',xyax(y,1),'hittest','off',...
        'verticalalignment','middle','handlevisibility','off','tag','yticklabel');
end



xyax=xyax';
xyax=xyax(:);

xyfakeax=xyfakeax';
xyfakeax=xyfakeax(:);

xyfakerax=xyfakerax';
xyfakerax=xyfakerax(:);





%% make response profilers
xlim = zeros([numy-1 numy-1 2]);
ylim = zeros([numy-1 numy-1 2]);



for r=1:1:(numy-1),
    %for c=(numx-1):-1:r,
    for c=r+1:1:(numy)
        axPos = [origin(1)+(width+space)*numx+height/2*(c-r-1)   origin(2)+(height+space)*numy-height/2*(r+c) ...
            height height]; %note, don't actualy need to shrink these, that is handled by view command.

        yyfakerax(r,c) = axes('units','pixels','Position',axPos,'parent',BigAxParent,...
            'hittest','off','gridlinestyle',':','xcolor',[.5 .5 .5],'ycolor',[.5 .5 .5],'visible','off');
        yyfakeax(r,c) = axes('units','pixels','Position',axPos,'parent',BigAxParent,...
            'hittest','off','gridlinestyle','-','color','none','xcolor',[.5 .5 .5],'ycolor',[.5 .5 .5],'visible','off');
        yyax(r,c) = axes('units','pixels','Position',axPos,'parent',BigAxParent,...
            'visible','on','nextplot','add','box','on','hittest','on','buttondownfcn','',...
            'userdata',handles.activeindex,'color','none','tag','pprof','xdir','reverse'); %userdata contains the index of the bigax for this subgraph.

        %         set([fakeax(r,c)],'ticklength',[.02 .02],'xticklabel','','yticklabel','')




        %make graphs here.



        
        if udata.format.usevariablecolors
            thisxcolor=handles.variablecolors{d}(udata.view.y(c),:);
            thisycolor=handles.variablecolors{d}(udata.view.y(r),:);
        else
            thisxcolor=udata.format.haircolor;
            thisycolor=udata.format.haircolor;
        end
        
        %hairline data series:
        h.yyxhair(r,c-1)=line([ycurrent{c} ycurrent{c}],[ylimmin(r) ylimmax(r)],'visible','on','linestyle',':','color',thisxcolor,...
            'parent',yyax(r,c),'hittest','off','tag','hair');
        %response hairline series:


        h.yyyhair(r,c-1)=line([ylimmin(c) ylimmax(c)],[ycurrent{r} ycurrent{r}],'visible','on','linestyle',':','color',thisycolor,...
            'parent',yyax(r,c),'hittest','off','tag','hair');
      

        %main data series:

        for x=1:numx
            h.yymain{r,c-1}(x)=line(yvalues{c}{x},yvalues{r}{x},'linewidth',udata.format.linesize,'linestyle','-','marker','none',...
                'color',handles.variablecolors{d}(xcol(x),:),'hittest','off','tag','keeper','parent',yyax(r,c));
        end

        %         if udata.format.showpoints
        %             nothidden=handles.isvisible{udata.view.dataset}{udata.view.analysis};
        %             xdata=handles.alldata{udata.view.dataset}(nothidden,udata.view.y(c));
        %             ydata=handles.alldata{udata.view.dataset}(nothidden,udata.view.y(r));
        %
        %             h.points(r,c)=line(xdata,ydata,'parent',yyax(r,c),'tag','points','linestyle','none','markeredgecolor','none','markerfacecolor',udata.format.pointcolor,...
        %                 'marker','o','markersize',6,'hittest','on','buttondownfcn',@raveupdateprofiler);
        %             uistack(h.points(r,c),'bottom');
        %         end
        
        if udata.format.showpoints
            nothidden=handles.isvisible{udata.view.dataset}{udata.view.analysis};
            xdata=handles.alldata{udata.view.dataset}(nothidden,udata.view.y(c));
            ydata=handles.alldata{udata.view.dataset}(nothidden,udata.view.y(r));
            h.points(r,c)=line(xdata,ydata,'parent',yyax(r,c),'tag','points','linestyle','none','markeredgecolor','none','markerfacecolor',udata.format.pointcolor,...
                'marker','o','markersize',4,'hittest','on','buttondownfcn',@raveupdateprofiler);
            uistack(h.points(r,c),'bottom');
        end
        


        newxlim=[ylimmin(c) ylimmax(c)];
        newylim=[ylimmin(r) ylimmax(r)];

        set([yyax(r,c) yyfakeax(r,c) yyfakerax(r,c)],'xlim',newxlim,'ylim',newylim,...
            'xgrid','off','ygrid','off','cameraupvector',[sqrt(2)*diff(newxlim) sqrt(2)*diff(newylim) 0])


        xlim(r,c-1,:) = get(yyax(r,c),'xlim');
        ylim(r,c-1,:) = get(yyax(r,c),'ylim');


    end
end


yyax=yyax(1:end-1,2:end);
yyfakeax=yyfakeax(1:end-1,2:end);
yyfakerax=yyfakerax(1:end-1,2:end);

% xlim=xlim(1:end,2:end,:);
% ylim=ylim(1:end,2:end,:);

set(yyax(yyax~=0),'xticklabel','')
set(yyax(yyax~=0),'yticklabel','')

set(yyfakeax(yyax~=0),'xticklabel','')
set(yyfakeax(yyax~=0),'yticklabel','')

set(yyfakerax(yyax~=0),'xticklabel','')
set(yyfakerax(yyax~=0),'yticklabel','')


% X TICKS:
xlegendmultipliertext=cell(1,numy-1);

for q=1:numy-1
    ticks=get(yyax(1,q),'XTick');
    omag=round(log10(abs(ticks)));
    %set really small numbers=0
    ticks(omag<-15)=0;
    omag(omag<-15)=0;
    %only reformat if ANY omag is 10^3 or greater.
    % if sum(omag>2)>=(numel(omag)-1)
    if any(omag>2)
        omag=max(omag);
        ticks=ticks/10^max(omag);
        if strcmp(handles.prefs.interpreter,'tex')
            xlegendmultipliertext{q} = [' x10^{',num2str(-max(omag)),'}'];
        else
            xlegendmultipliertext{q} = [' x10^',num2str(-max(omag))];
        end
    elseif  sum(omag<-2)>=(numel(omag)-1)
        omag=min(omag);
        ticks=ticks/10^min(omag);
        if strcmp(handles.prefs.interpreter,'tex')
            xlegendmultipliertext{q} = [' x10^{',num2str(-min(omag)),'}'];
        else
            xlegendmultipliertext{q} = [' x10^',num2str(-min(omag))];
        end
    else %no scaling necessary
        xlegendmultipliertext{q} ='';
        omag=0;
    end

    %     set(xxax(1,q),'XTickLabel',[]);
    %     set(xxfakeax(1,q),'XTickLabel',[]);
    %     set(xxfakerax(1,q),'XTickLabel',[]);

    switch handles.datatype{d}{ycol(q+1)}
        case {'continuous','constant','integer'}
            putthemhere=get(yyax(1,q),'xtick');
            usethesevalues=cell(numel(ticks),1);
            for w=1:numel(ticks)
                if ticks(w)*10^omag > (xlim(1,q,2)-0.1*(xlim(1,q,2)-xlim(1,q,1))) %within 10% of endpoint...
                    usethesevalues{w}='';
                else
                    usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                end
            end

        case 'discrete'
            discretez=handles.discretevalues{d}{a}{ycol(q)};
            if numel(discretez)<=12;
                putthemhere=discretez;
                usethesevalues=cell(numel(discretez),1);
                for w=1:numel(discretez)
                    usethesevalues{w}=strtrim(sprintf('%-6.4g',discretez(w)));
                end
            else %too many, just treat as continuous
                putthemhere=get(yyax(1,q),'xtick');
                usethesevalues=cell(numel(ticks),1);
                for w=1:numel(ticks)
                    usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                end
            end
        case 'string'
            usethesevalues=handles.discretevalues{d}{a}{ycol(q)};
            putthemhere=unique(handles.alldata{d}(:,ycol(q)));

        case 'logical'
            usethesevalues={handles.prefs.logicalfalse;handles.prefs.logicaltrue};
            putthemhere=[0 1];
    end
    ylimits=get(yyax(1,1),'Ylim');
    ypos=ylimits(2)+.02*(ylimits(2)-ylimits(1));
    th=text(putthemhere,ypos*ones(1,length(putthemhere)),usethesevalues,...
        'HorizontalAlignment','left','rotation',0,'parent',yyax(1,q),'hittest','off',...
        'verticalalignment','baseline','handlevisibility','off','tag','xticklabel'); %#ok<NASGU>
end


% Y TICKS:
ylegendmultipliertext=cell(1,numy-1); %first row is xcol, 2nd is ycol.

for q=1:numy-1
    ticks=get(yyax(q,end),'YTick');
    omag=round(log10(abs(ticks)));
    %set really small numbers=0
    ticks(omag<-15)=0;
    omag(omag<-15)=0;
    %only reformat if all omag is 10^3 or greater.
    % if sum(omag>2)>=(numel(omag)-1)%         all(omag>2)
    if any(omag>2)
        omag=max(omag);
        ticks=ticks/10^max(omag);
        if strcmp(handles.prefs.interpreter,'tex')
            ylegendmultipliertext{q} = [' x10^{',num2str(-max(omag)),'}'];
        else
            ylegendmultipliertext{q} = [' x10^',num2str(-max(omag))];
        end
    elseif sum(omag<-2)>=(numel(omag)-1)%   all(omag<-2)
        omag=min(omag);
        ticks=ticks/10^min(omag);
        if strcmp(handles.prefs.interpreter,'tex')
            ylegendmultipliertext{q} = [' x10^{',num2str(-min(omag)),'}'];
        else
            ylegendmultipliertext{q} = [' x10^',num2str(-min(omag))];
        end
    else %no scaling necessary
        omag=0;
        ylegendmultipliertext{q} ='';
    end

    switch handles.datatype{d}{ycol(q+1)}
        case {'continuous','constant','integer'}
            putthemhere=get(yyax(q,end),'ytick');
            usethesevalues=cell(numel(ticks),1);
            for w=1:numel(ticks)
                if ticks(w)*10^omag > (ylim(q,end,2)-0.1*(ylim(q,end,2)-ylim(q,end,1))) %within 10% of endpoint...
                    usethesevalues{w}='';
                else
                    usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                end

            end
        case 'discrete'
            discretez=handles.discretevalues{d}{a}{ycol(q+1)};
            if numel(discretez)<=12;
                putthemhere=discretez;
                usethesevalues=cell(numel(discretez),1);
                for w=1:numel(discretez)
                    usethesevalues{w}=strtrim(sprintf('%-6.4g',discretez(w)));
                end
            else %too many, just treat as continuous
                putthemhere=get(yyax(q,end),'ytick');
                usethesevalues=cell(numel(ticks),1);
                for w=1:numel(ticks)
                    usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                end
            end
        case 'string'
            usethesevalues=handles.discretevalues{d}{a}{ycol(q+1)};
            putthemhere=unique(handles.alldata{d}(:,ycol(q+1)));

        case 'logical'
            usethesevalues={handles.prefs.logicalfalse;handles.prefs.logicaltrue};
            putthemhere=[0 1];
    end

    xlimits=get(yyax(q,end),'xlim');
    xpos=xlimits(1)-.02*(xlimits(2)-xlimits(1));

    th=text(xpos*ones(1,length(putthemhere)),putthemhere,usethesevalues,...
        'HorizontalAlignment','left','rotation',0,'parent',yyax(q,end),'hittest','off',...
        'verticalalignment','cap','handlevisibility','off','tag','yticklabel'); %#ok<NASGU>

end

yyax=yyax';
yyax=yyax(yyax~=0);

yyfakeax=yyfakeax';
yyfakeax=yyfakeax(yyfakeax~=0);

yyfakerax=yyfakerax';
yyfakerax=yyfakerax(yyfakerax~=0);


%% make contour plots:
xlim = zeros([numx-1 numx-1 2]);
ylim = zeros([numx-1 numx-1 2]);


%precompute data so contour levels can be shared among all plots:
allthedata=cell(numel(xcol),numel(xcol),3);
for r=1:1:(numx-1)
    for c=(r+1):1:(numx)
        if r==c;continue;end

        thisxdata=xsteps{c};
        thisydata=xsteps{r};

        [xgrid,ygrid]=meshgrid(thisxdata,thisydata);

        xcalldata=xgrid(:);
        ycalldata=ygrid(:);

        constants=repmat(xdials,[length(xcalldata),1]);
        constants(:,xcol(c))=xcalldata;
        constants(:,xcol(r))=ycalldata;

        %evaluate the functions:

        zvalues=raveevaluateregression(src,d,udata.view.z,constants);

        allthedata{r,c,1}=reshape(xcalldata,numel(thisydata),numel(thisxdata));
        allthedata{r,c,2}=reshape(ycalldata,numel(thisydata),numel(thisxdata));
        allthedata{r,c,3}=reshape(zvalues,numel(thisydata),numel(thisxdata));
    end
end

%detemrine which levels to draw contours at:
zdataz=allthedata(:,:,3);
zdataz=zdataz(:);
zdataz=cell2mat(zdataz);
zdataz=zdataz(:);
lowestvalue=min(zdataz);
highestvalue=max(zdataz);
levelz=linspace(1,128,20)+128*(udata.format.mcmap-1);


for r=1:1:(numx-1), %first variable
    %for c=(numx-1):-1:r,
    for c=(r+1):1:(numx) %second variable
        axPos = [origin(1)+(width/2)*(r+c-2)  origin(2)+(height+space)*numy + width/2*(c-r-1) width width]; %note, don't actualy need to shrink these, that is handled by view command.

        xxfakerax(r,c) = axes('units','pixels','Position',axPos,'parent',BigAxParent,...
            'hittest','off','gridlinestyle',':','xcolor',[.5 .5 .5],'ycolor',[.5 .5 .5],'visible','off');
        xxfakeax(r,c) = axes('units','pixels','Position',axPos,'parent',BigAxParent,...
            'hittest','off','gridlinestyle','-','color','none','xcolor',[.5 .5 .5],'ycolor',[.5 .5 .5],'visible','off');
        xxax(r,c) = axes('units','pixels','Position',axPos,'parent',BigAxParent,...
            'visible','on','nextplot','add','box','on','hittest','on','buttondownfcn','',...
            'userdata',handles.activeindex,'color','none','tag','pprof','xdir','reverse'); %userdata contains the index of the bigax for this subgraph.

       %set([fakeax(r,c)],'ticklength',[.02 .02],'xticklabel','','yticklabel','')



        zplotdata=allthedata{r,c,3};
        if ~udata.format.reversecolors
            zplotdata=(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
        else
            zplotdata=128-(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
        end

        % contourf(ax(r,c),single(allthedata{r,c,1}),single(allthedata{r,c,2}),single(zplotdata),single(levelz),'linecolor','none');

        C=contourc(allthedata{r,c,1}(1,:),allthedata{r,c,2}(:,1),zplotdata,levelz);
        h.xxmain{r,c}=contourplot(C,'style','line','parent',xxax(r,c),'linewidth',udata.format.linesize,'hittest','off');

        %use single precision axes limits for correct positioning on screen:
        xlimz=single([allthedata{r,c,1}(1,1) allthedata{r,c,1}(1,end)]);
        ylimz=single([allthedata{r,c,2}(1,1) allthedata{r,c,2}(end,1)]);

        set([xxax(r,c) xxfakeax(r,c) xxfakerax(r,c)],'xlim',xlimz,'ylim',ylimz,'xlimmode','manual','ylimmode','manual',...
            'xgrid','off','ygrid','off','layer','top','cameraupvector',double([sqrt(2)*diff(xlimz) -sqrt(2)*diff(ylimz) 0]))

        %make tracers
        if udata.format.usevariablecolors
            h.xxxhair(r,c-1)=line([xdials(xcol(c)) xdials(xcol(c))],[xmin(xcol(r)) xmax(xcol(r))],'parent',xxax(r,c),'hittest','off','linestyle',udata.format.linestyle,'linestyle',':','linewidth',1+udata.format.bold,'color',handles.variablecolors{d}(xcol(c),:));
            h.xxyhair(r,c-1)=line([xmin(xcol(c)) xmax(xcol(c))],[xdials(xcol(r)) xdials(xcol(r))],'parent',xxax(r,c),'hittest','off','linestyle',udata.format.linestyle,'linestyle',':','linewidth',1+udata.format.bold,'color',handles.variablecolors{d}(xcol(r),:));
        else
            h.xxxhair(r,c-1)=line([xdials(xcol(c)) xdials(xcol(c))],[xmin(xcol(r)) xmax(xcol(r))],'parent',xxax(r,c),'hittest','off','linestyle',udata.format.linestyle,'linestyle',':','linewidth',1+udata.format.bold,'color',udata.format.haircolor);
            h.xxyhair(r,c-1)=line([xmin(xcol(c)) xmax(xcol(c))],[xdials(xcol(r)) xdials(xcol(r))],'parent',xxax(r,c),'hittest','off','linestyle',udata.format.linestyle,'linestyle',':','linewidth',1+udata.format.bold,'color',udata.format.haircolor);
            %
        end
        
        if udata.format.showpoints
            nothidden=handles.isvisible{udata.view.dataset}{udata.view.analysis};
            xdata=handles.alldata{udata.view.dataset}(nothidden,udata.view.x(c));
            ydata=handles.alldata{udata.view.dataset}(nothidden,udata.view.x(r));
            h.points(r,c)=line(xdata,ydata,'parent',xxax(r,c),'tag','points','linestyle','none','markeredgecolor','none','markerfacecolor',udata.format.pointcolor,...
                'marker','o','markersize',4,'hittest','on','buttondownfcn',@raveupdateprofiler);
            uistack(h.points(r,c),'bottom');
        end
        
        

        xlim(r,c-1,:) = get(xxax(r,c),'xlim');
        ylim(r,c-1,:) = get(xxax(r,c),'ylim');



    end
end

clear contourplot %clear persistent vars;

xxax=xxax(1:end-1,2:end);
xxfakeax=xxfakeax(1:end-1,2:end);
xxfakerax=xxfakerax(1:end-1,2:end);

% xlim=xlim(1:end,2:end,:);
% ylim=ylim(1:end,2:end,:);

set(xxax(xxax~=0),'xticklabel','')
set(xxax(xxax~=0),'yticklabel','')

set(xxfakeax(xxax~=0),'xticklabel','')
set(xxfakeax(xxax~=0),'yticklabel','')

set(xxfakerax(xxax~=0),'xticklabel','')
set(xxfakerax(xxax~=0),'yticklabel','')



%
% for r=1:cols-1,
%     %force non exponential labels:
%     set(xxax(r,end),'YTickLabel',get(xxax(r,end),'YTick'))
%     set(xxfakeax(r,end),'YTickLabel','')
%     set(xxfakerax(r,end),'YTickLabel','')
% end
%
%
% for c=1:cols-1,
%     %force non exponential labels:
%     set(xxax(1,c),'XTickLabel',get(xxax(1,c),'XTick'))
%     set(xxfakeax(1,c),'XTickLabel','')
%     set(xxfakerax(1,c),'XTickLabel','')
% end
%




%
%
% set(targetaxes,'XTick',get(xxax(cols,1),'xtick'),'YTick',get(xxax(cols,1),'ytick'), ...
%     'userdata','greatness','tag','');

% set(xxax,'xlimmode','manual','ylimmode','manual')

if numx>1
    % X TICKS:
    xlegendmultipliertext=cell(1,numx);

    for q=1:numx-1
        ticks=get(xxax(1,q),'XTick');
        omag=round(log10(abs(ticks)));
        %set really small numbers=0
        ticks(omag<-15)=0;
        omag(omag<-15)=0;
        %only reformat if ANY omag is 10^3 or greater.
        % if sum(omag>2)>=(numel(omag)-1)
        if any(omag>2)
            omag=max(omag);
            ticks=ticks/10^max(omag);
            if strcmp(handles.prefs.interpreter,'tex')
                xlegendmultipliertext{q} = [' x10^{',num2str(-max(omag)),'}'];
            else
                xlegendmultipliertext{q} = [' x10^',num2str(-max(omag))];
            end
        elseif  sum(omag<-2)>=(numel(omag)-1)
            omag=min(omag);
            ticks=ticks/10^min(omag);
            if strcmp(handles.prefs.interpreter,'tex')
                xlegendmultipliertext{q} = [' x10^{',num2str(-min(omag)),'}'];
            else
                xlegendmultipliertext{q} = [' x10^',num2str(-min(omag))];
            end
        else %no scaling necessary
            xlegendmultipliertext{q} ='';
            omag=0;
        end

        %     set(xxax(1,q),'XTickLabel',[]);
        %     set(xxfakeax(1,q),'XTickLabel',[]);
        %     set(xxfakerax(1,q),'XTickLabel',[]);

        switch handles.datatype{d}{xcol(q+1)}
            case {'continuous','constant','integer'}
                putthemhere=get(xxax(1,q),'xtick');
                usethesevalues=cell(numel(ticks),1);
                for w=1:numel(ticks)
                    if ticks(w)*10^omag > (xlim(1,q,2)-0.1*(xlim(1,q,2)-xlim(1,q,1))) %within 10% of endpoint...
                        usethesevalues{w}='';
                    else
                        usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                    end
                end

            case 'discrete'
                discretez=handles.discretevalues{d}{a}{xcol(q)};
                if numel(discretez)<=12;
                    putthemhere=discretez;
                    usethesevalues=cell(numel(discretez),1);
                    for w=1:numel(discretez)
                        usethesevalues{w}=strtrim(sprintf('%-6.4g',discretez(w)));
                    end
                else %too many, just treat as continuous
                    putthemhere=get(xxax(1,q),'xtick');
                    usethesevalues=cell(numel(ticks),1);
                    for w=1:numel(ticks)
                        usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                    end
                end
            case 'string'
                usethesevalues=handles.discretevalues{d}{a}{xcol(q)};
                putthemhere=unique(handles.alldata{d}(:,xcol(q)));

            case 'logical'
                usethesevalues={handles.prefs.logicalfalse;handles.prefs.logicaltrue};
                putthemhere=[0 1];
        end
        ylimits=get(xxax(1,1),'Ylim');
        ypos=ylimits(1)-.01*(ylimits(2)-ylimits(1));
        th=text(putthemhere,ypos*ones(1,length(putthemhere)),usethesevalues,...
            'HorizontalAlignment','right','rotation',0,'parent',xxax(1,q),'hittest','off',...
            'verticalalignment','baseline','handlevisibility','off','tag','xticklabel'); %#ok<NASGU>
    end


    % Y TICKS:
    ylegendmultipliertext=cell(1,numel(ycol)); %first row is xcol, 2nd is ycol.


    for q=1:numy-1
        ticks=get(xxax(q,end),'YTick');
        omag=round(log10(abs(ticks)));
        %set really small numbers=0
        ticks(omag<-15)=0;
        omag(omag<-15)=0;
        %only reformat if all omag is 10^3 or greater.
        % if sum(omag>2)>=(numel(omag)-1)%         all(omag>2)
        if any(omag>2)
            omag=max(omag);
            ticks=ticks/10^max(omag);
            if strcmp(handles.prefs.interpreter,'tex')
                ylegendmultipliertext{q} = [' x10^{',num2str(-max(omag)),'}'];
            else
                ylegendmultipliertext{q} = [' x10^',num2str(-max(omag))];
            end
        elseif sum(omag<-2)>=(numel(omag)-1)%   all(omag<-2)
            omag=min(omag);
            ticks=ticks/10^min(omag);
            if strcmp(handles.prefs.interpreter,'tex')
                ylegendmultipliertext{q} = [' x10^{',num2str(-min(omag)),'}'];
            else
                ylegendmultipliertext{q} = [' x10^',num2str(-min(omag))];
            end
        else %no scaling necessary
            omag=0;
            ylegendmultipliertext{q} ='';
        end
        %     set(ax(q,1),'YTickLabel',[]);
        %     set(fakeax(q,1),'YTickLabel',[]);
        %     set(fakerax(q,1),'YTickLabel',[]);


        switch handles.datatype{d}{xcol(q+1)}
            case {'continuous','constant','integer'}
                putthemhere=get(xxax(q,end),'ytick');
                usethesevalues=cell(numel(ticks),1);
                for w=1:numel(ticks)
                    if ticks(w)*10^omag > (ylim(q,end,2)-0.1*(ylim(q,end,2)-ylim(q,end,1))) %within 10% of endpoint...
                        usethesevalues{w}='';
                    else
                        usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                    end

                end
            case 'discrete'
                discretez=handles.discretevalues{d}{a}{xcol(q+1)};
                if numel(discretez)<=12;
                    putthemhere=discretez;
                    usethesevalues=cell(numel(discretez),1);
                    for w=1:numel(discretez)
                        usethesevalues{w}=strtrim(sprintf('%-6.4g',discretez(w)));
                    end
                else %too many, just treat as continuous
                    putthemhere=get(xxax(q,end),'ytick');
                    usethesevalues=cell(numel(ticks),1);
                    for w=1:numel(ticks)
                        usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                    end
                end
            case 'string'
                usethesevalues=handles.discretevalues{d}{a}{xcol(q+1)};
                putthemhere=unique(handles.alldata{d}(:,xcol(q+1)));

            case 'logical'
                usethesevalues={handles.prefs.logicalfalse;handles.prefs.logicaltrue};
                putthemhere=[0 1];
        end

        xlimits=get(xxax(q,end),'xlim');
        xpos=xlimits(2)+.02*(xlimits(2)-xlimits(1));

        th=text(xpos*ones(1,length(putthemhere)),putthemhere,usethesevalues,...
            'HorizontalAlignment','left','rotation',0,'parent',xxax(q,end),'hittest','off',...
            'verticalalignment','baseline','handlevisibility','off','tag','yticklabel'); %#ok<NASGU>

    end
end



xxax=xxax';
xxax=xxax(xxax~=0);

xxfakeax=xxfakeax';
xxfakeax=xxfakeax(xxfakeax~=0);

xxfakerax=xxfakerax';
xxfakerax=xxfakerax(xxfakerax~=0);



%% finish up

%interpreting the subax vectors for house of plots:
%vector length = [numx*numy+numx^2-numx + numy^2-numy]
%first [numx*numy] elements are the prediction profiler graphs, in the order y1x1 y1x2...y1xn y2x1 y2x2..
%next [numx^2-numx] elements are the roof (contour plots) in the order x1x2 x1x3 ... x1xn x2x3 x2x4...x2xn ... x(n-1)xn
%last [numy^2-numy] elements are the side roof (response profilers in the order y1y2 y1y3 ...y1yn y2y3 y2y4... y2yn ... y(n-1)yn

ax=[xyax;xxax;yyax];
fakeax=[xyfakeax;xxfakeax;yyfakeax];
fakerax=[xyfakerax;xxfakerax;yyfakerax];

handles.subax{handles.activeindex}=ax;
handles.subfakeax{handles.activeindex}=fakeax;
handles.subfakerax{handles.activeindex}=fakerax;

handles.profilers.main{handles.activeindex}{1}=h.xymain; %1 = xy
handles.profilers.xhair{handles.activeindex}{1}=h.xyxhair;
handles.profilers.yhair{handles.activeindex}{1}=h.xyyhair;
handles.profilers.main{handles.activeindex}{2}=h.xxmain; %2 = xx
handles.profilers.xhair{handles.activeindex}{2}=h.xxxhair;
handles.profilers.yhair{handles.activeindex}{2}=h.xxyhair;
handles.profilers.main{handles.activeindex}{3}=h.yymain; %3 = yy
handles.profilers.xhair{handles.activeindex}{3}=h.yyxhair;
handles.profilers.yhair{handles.activeindex}{3}=h.yyyhair;

guidata(src,handles);


%adjust background and text labels to match settings:
%make background color data matrix:
bgcolordata={[1 1 1;.6 .6 .6;0 0 0]; %white
    [.753 .753 .753;1 1 1;0 0 0]; %light
    [.502 .502 .502;.6 .6 .6;1 1 1]; %dark
    [0 0 0;.3 .3 .3;1 1 1]}; %black

set(handles.subax{handles.activeindex},'color',bgcolordata{udata.format.bg}(1,:));
% set(handles.subfakerax{handles.activeindex},'xcolor',bgcolordata{udata.format.bg}(2,:),'ycolor',bgcolordata{udata.format.bg}(2,:));
% set(handles.subfakeax{handles.activeindex},'xcolor',bgcolordata{udata.format.bg}(2,:),'ycolor',bgcolordata{udata.format.bg}(2,:));






