function viewtabgreatness(src,eventdata)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

%move front tabs to front if necessary
if handles.fronttabsfront==0 %move swap fronttabs and backtabs
    backpos=get(handles.backtabaxes,'position');
    frontpos=get(handles.fronttabaxes,'position');
    set(handles.fronttabaxes,'position',backpos)
    set(handles.backtabaxes,'position',frontpos)
    [x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
    set(handles.backtablines,'xdata',x,'ydata',y);   
    set(handles.fronttabaxes,'xcolor',handles.colors.tabbg,'ycolor',handles.colors.tabbg);
    set(handles.backtabaxes,'xcolor',handles.colors.frame,'ycolor',handles.colors.frame);
end
handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,2,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');

%% clear existing tab
%delete(handles.tabobjects)

%% get input info from graph:
udata=get(handles.fakeax(handles.activeindex),'userdata');
%view=udata.view; %get view tab structure.
d=udata.view.dataset;

%% text headers

projecttext=text(.15-.10317,.95*1.0317-.0317,'Project','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

dataseriestext=text(.15-.10317,.75*1.0317-.0317,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

optionstext=text(.15-.10317,.20*1.0317-.0317,'Options','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% text labels

analysistext=text(.25-.10317,.920*1.0317-.0317,'Choose Parent Analysis:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');

datasettext=text(.25-.10317,.85*1.0317-.0317,'Choose a Data Set:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

xaxistext=text(.25-.10317,.7111,'Choose Functions:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

yaxistext=text(.25-.10317,.48,'Choose Variables:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

zaxistext=text(.25-.10317,.255,'Draw Contours of:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');


%% variable lists
xvars=handles.funvars{udata.view.dataset}{udata.view.z(1)};
for yy=1:length(udata.view.z)
    xvars=union(xvars,handles.funvars{udata.view.dataset}{udata.view.z(yy)});
end

xvars=find(~strcmp(handles.datatype{d},'constant') & ~handles.isfunction{d});
allnames=handles.allnames{d};
xvarnames=allnames(xvars);

analysismenu=uicontrol('position',[origin(1)+55 origin(2)+535 180 30],...
    'style','popupmenu','string','Default',...
    'backgroundcolor',[1 1 1],'tag','analysismenu','callback',@analysismenu_callback,'value',a);

datasetmenu=uicontrol('position',[origin(1)+55 origin(2)+490 180 30],...
    'style','popupmenu','string',handles.datasetnames,...
    'backgroundcolor',[1 1 1],'tag','datasetmenu','callback',@datasetmenu_callback,'value',d);

xaxismenu=uicontrol('position',[origin(1)+55 origin(2)+315 180 120],...
    'style','listbox','string',xvarnames,'value',find(ismember(xvarnames,allnames(udata.view.x))),...
    'backgroundcolor',[1 1 1],'tag','xaxismenu','max',50,'callback',@xaxismenu_callback,'listboxtop',1);

yaxismenu=uicontrol('position',[origin(1)+55 origin(2)+170 180 120],...
    'style','listbox','string',handles.allnames{d}(logical(handles.isfunction{d})),'value',find(ismember(find(handles.isfunction{d}),udata.view.y)),...
    'backgroundcolor',[1 1 1],'tag','yaxismenu','max',50,'callback',@yaxismenu_callback);

zaxismenu=uicontrol('position',[origin(1)+55 origin(2)+126 180 22],...
    'style','popupmenu','string',handles.allnames{d}(logical(handles.isfunction{d})),'value',find(ismember(find(handles.isfunction{d}),udata.view.z)),...
    'backgroundcolor',[1 1 1],'tag','yaxismenu','max',50,'callback',@zaxismenu_callback);

%% buttons

scalebutton=uicontrol('position',[origin(1)+55 origin(2)+40 180 30],...
    'style','pushbutton','string','Scale/Transform Variables','backgroundcolor',handles.colors.buttonbg);

showpointscheck=uicontrol('position',[origin(1)+55 origin(2)+75 200 26],...
    'style','checkbox','string','Show Data Set',...
    'backgroundcolor',handles.colors.tabbg,'value',udata.format.showpoints,'callback',@showpoints_callback);
% excludecheck=uicontrol('position',[origin(1)+55 origin(2)+30 200 25],...
%     'style','checkbox','string','Hide constant columns in menus',...
%     'backgroundcolor',handles.colors.tabbg,'value',view.exclude,'callback',@excludecheck_callback,'tag','excludecheck');

%% group items and finish up
handles.tabobjects=[datasettext xaxistext datasetmenu xaxismenu,...
    scalebutton analysistext projecttext analysismenu dataseriestext optionstext ...
    yaxistext yaxismenu zaxistext zaxismenu showpointscheck];

handles.activetab='view';


guidata(src,handles);



%% Here be callbacks:%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function datasetmenu_callback(src,eventdata)

handles=guidata(src);
%update other menus to show correct variables
udata=get(handles.fakeax(handles.activeindex),'userdata');

% xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
yaxismenu=findobj(handles.tabobjects,'tag','yaxismenu');
% analysismenu=findobj(handles.tabobjects,'tag','analysismenu');
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
% useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
% excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

d=get(datasetmenu,'value');

if d==udata.view.dataset
    return %same data set, do nothing to avoid resetting graph.
end

%get initial columns for this data set: 
if all(handles.isfunction{d}==0)
   %no functions in this data set... can't use it.
   set(datasetmenu,'value',udata.view.dataset)
   set(handles.infobar,'string','Selected data set has no variables modeled by functions.')
   return
else
   %data set is valid, make y = first function, x = variables of that
   %function.
   set(handles.infobar,'string','');
   y=find(handles.isfunction{d},1);
   udata.view.y=y;
   udata.view.dataset=d;
   udata.view.x=handles.funvars{d}{y}; %note, these are names, need to convert them to numbers
%    allnames=handles.allnames{d};
% %    remove spaces from these to match varnames format (ravejmp2m removes
% %    spaces in variable names)
%    for j=1:length(allnames)
%        allnames{j}=allnames{j}(allnames{j}~=char(32));
%    end
% %    find the index into allnames of sortnames.
%    for jj=1:numel(udata.view.x)
%        index(jj)=find(strcmp(allnames,udata.view.x{jj})); %#ok<AGROW>
%    end
%    
%    udata.view.x=sort(index); %transform x from names to numbers
%    
   udata.view.isvalid=1;   
%    calculate starting x values:
   xmax=max(handles.alldata{d}(:,:));
   xmin=min(handles.alldata{d}(:,:));
   udata.view.xdials=mean([xmax;xmin]);
end

%set fakeax userdata

udata.view.dataset=d;
set(handles.fakeax(handles.activeindex),'userdata',udata);
handles.graphdataset(handles.activeindex)=get(src,'value');
guidata(src,handles);

viewtabmenufilter(src,[])

%update y menu:
set(yaxismenu,'string',handles.allnames{d}(logical(handles.isfunction{d})),'value',1)


%update plots
% 
% inputs.targetaxes=handles.activegraph;
% inputs.pscale=[]; %project scale factors
% inputs.gscale=[]; %graph scale factors
% inputs.analysis=get(analysismenu,'value');
% inputs.dataset=get(datasetmenu,'value');

ravecreategreatness(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function analysismenu_callback(src,eventdata)
handles=guidata(src);
handles.graphanalysis(handles.activeindex)=get(src,'value');
guidata(src,handles);

udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.analysis=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreategreatness(src,[]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function yaxismenu_callback(src,eventdata)

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
d=udata.view.dataset;

yvars=find(handles.isfunction{d});

udata.view.y=yvars(get(src,'value'));

%save udata
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreategreatness(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xaxismenu_callback(src,eventdata)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');


xvars=find(~strcmp(handles.datatype{udata.view.dataset},'constant') & ~handles.isfunction{udata.view.dataset});

udata.view.x=xvars(get(src,'value'));

%ensure at least 2 vars are selected:
% if length(udata.view.x)==1
%     udata.view.x=xvars(1,2); %select first 2 vars
%     set(handles.infobar,'string','You need to select at least two variables for this graph.')
% else
%     set(handles.infobar,'string','');
% end

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreategreatness(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function zaxismenu_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

zvars=find(handles.isfunction{udata.view.dataset});

udata.view.z=zvars(get(src,'value'));

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreategreatness(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function showpoints_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.format.showpoints=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreategreatness(src,[]);




