function raveupdateonevarsweep(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

%this is the buttondownfcn for hyperslice axes, to update the x
%value of the selected graph and redraw all graphs in the new position.
%Also includes buttonmotionfcn version.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

%need to check if this graph is active. if not , return and do nothing.
thisindex=find(handles.allgraphs==src);
if thisindex~=handles.activeindex
    return
end


raveslidecontour(handles.fig,[]);
%make sliding along graph update continuously:
set(handles.fig,'windowbuttonupfcn',@raveslidedone)
set(handles.fig,'windowbuttonmotionfcn',@raveslidecontour)



function raveslidecontour(src,ev)
%NOTE: SRC FOR THIS FUNCTION IS EQUAL TO HANDLES.FIG REGARDLESS OF WHAT WAS
%CLICKED.
%persistent handles
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

d=udata.view.dataset;
a=udata.view.analysis;

mins=handles.minlimit{d}{a};
maxs=handles.maxlimit{d}{a};


xcol=udata.view.x;
ycol=udata.view.y;
zcol=udata.view.z;

numy=numel(ycol);

%if ~isempty(src) %if this function is called as a callback: (get mouse location and slide bars)
clicked=get(gca,'currentpoint');
xval=clicked(1,1);
yval=clicked(1,2);

%move marker within bounds
if xval>maxs(xcol)
    xval=maxs(xcol);
elseif xval<mins(xcol)
    xval=mins(xcol);
end

if yval>maxs(ycol)
    yval=maxs(ycol);
elseif yval<mins(ycol)
    yval=mins(ycol);
end


%if this variable is non-continuous, round val to nearest allowable value:
thistype=handles.datatype{d}{xcol};
if  strcmp(thistype,'constant')
    xval=handles.minlimit{d}{a}{xcol};
elseif ~strcmp(thistype,'continuous')
    %find nearest neighbor in allowable values:
    allowed=handles.discretevalues{d}{a}{xcol};
    xval=allowed(abs(allowed-xval)==min(abs(allowed-xval)));
end

thistype=handles.datatype{d}{ycol};
if  strcmp(thistype,'constant')
    yval=handles.minlimit{d}{a}{ycol};
elseif ~strcmp(thistype,'continuous')
    %find nearest neighbor in allowable values:
    allowed=handles.discretevalues{d}{a}{ycol};
    yval=allowed(abs(allowed-yval)==min(abs(allowed-yval)));
end


xval=xval(1); %sometimes weird rounding can give two values.
yval=yval(1);

set(handles.profilers.xhair{handles.activeindex},'xdata',[xval xval])
set(handles.profilers.yhair{handles.activeindex},'ydata',[yval yval])

handles.xdials{d}{a}(xcol)=xval;
handles.xdials{d}{a}(ycol)=yval;
guidata(src,handles);

%if colorbar is displayed, show current value:

if udata.format.colorbar
    legdata=get(handles.fakerax(handles.activeindex),'userdata');

    cbar=findobj(legdata,'tag','Colorbar');
    
    ylim=get(cbar,'ylim');
    clim=udata.view.clim;

    %determine current c value:
    thisvalue=raveevaluateregression(src,udata.view.dataset,udata.view.c,handles.xdials{udata.view.dataset}{udata.view.analysis});

    thiscoord=interp1(clim,ylim,thisvalue);

    %make triangle patch at this value:
    cvpatch=findobj(cbar,'tag','currentvalue');
    set(cvpatch,'ydata',[thiscoord-1.5 thiscoord+1.5 thiscoord]);
    cvtext=findobj(cbar,'tag','currentvaluetxt');
    pos=get(cvtext,'pos');
    set(cvtext,'pos',[pos(1) thiscoord],'string',sprintf('%5.5g',thisvalue));
    

end




%set(get(handles.subfakeax{handles.activeindex}(end,index),'xlabel'),'string',{' ';' ';['\color[rgb]{',udata.format.linecolorString,'} ',valtext]},'verticalalignment','top','interpreter','tex');

%if new value is the same as old value (because of discrete vars) don't
%waste time evaluating funcitons:

%{

xdials=handles.xdials{d}{a};

if xdials(xcol)==xval && xdials(ycol)==yval
    return
end


xdials(xcol)=xval;
xdials(ycol)=yval;

handles.xdials{d}{a}=xdials;
guidata(handles.fig,handles);

%initialize handles

%h.main=zeros(rows,cols,10);

%prepare data for gridding:
datatypes=handles.datatype{d}([xcol ycol]) ;
totalrows=zeros(size(datatypes));
numpts=20;
xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};
step=(xmax-xmin)/numpts;
xsteps=cell(1,numel(datatypes));
xdials=handles.xdials{d}{a};

xcol=[xcol ycol];
for j=1:numel(datatypes);
    switch datatypes{j}
        case 'constant'
            xsteps{j}=xdials(xcol(j));
            totalrows(j)=1;
        case 'continuous'
            xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
            totalrows(j)=numel(xsteps{j});
        case 'integer'
            if numel(handles.discretevalues{d}{a}{xcol(j)})<20
                xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
                totalrows(j)=numel(xsteps{j});
            else
                xsteps{j}=sort([xmin(xcol(j)):step(xcol(j)):xmax(xcol(j)) xdials(xcol(j))]);
                totalrows(j)=numel(xsteps{j});
            end
        case 'discrete'
            xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
            totalrows(j)=numel(xsteps{j});
        case 'string'
            xsteps{j}=handles.discretevalues{d}{a}{xcol(j)};
            totalrows(j)=numel(xsteps{j});
        case 'logical'
            xsteps{j}=[0 1];
            totalrows(j)=numel(xsteps{j});
    end
end



%precompute data so contour levels can be shared among all plots:
allthedata=cell(1,3);


        thisxdata=xsteps{1};
        thisydata=xsteps{2};

        [xgrid,ygrid]=meshgrid(thisxdata,thisydata);

        xcalldata=xgrid(:);
        ycalldata=ygrid(:);

        constants=repmat(xdials,[length(xcalldata),1]);
        constants(:,xcol(1))=xcalldata;
        constants(:,ycol)=ycalldata;

        %evaluate the functions:

        zvalues=raveevaluateregression(src,d,udata.view.z,constants);

        allthedata{1}=reshape(xcalldata,numel(thisydata),numel(thisxdata));
        allthedata{2}=reshape(ycalldata,numel(thisydata),numel(thisxdata));
        allthedata{3}=reshape(zvalues,numel(thisydata),numel(thisxdata));

zdataz=allthedata{3};
zdataz=zdataz(:);


lowestvalue=min(zdataz);
highestvalue=max(zdataz);

levelz=linspace(1,128,20)+128*(udata.format.mcmap-1);
udata.view.levels=levelz;

zplotdata=allthedata{3};
if ~udata.format.reversecolors
    zplotdata=(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
else
    zplotdata=128-(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
end

% contourf(ax(r,c),single(allthedata{r,c,1}),single(allthedata{r,c,2}),single(zplotdata),single(levelz),'linecolor','none');

C=contourc(allthedata{1}(1,:),allthedata{2}(:,1),zplotdata,levelz);
delete(handles.profilers.main{handles.activeindex})
mainhandles=contourplot(C,'style','line','parent',handles.activegraph,'linewidth',udata.format.linesize,'hittest','off');
uistack(mainhandles,'bottom');



handles.profilers.main{handles.activeindex}=mainhandles;
guidata(src,handles);
%}


%update on slide if desired:
if handles.prefs.alwaysupdate && ~handles.prefs.alwaysrasterize
    raveupdateanalysis(src,[],'xdials')
end



function raveslidedone(src,ev) %#ok<INUSD>

handles=guidata(src);
set(handles.fig,'windowbuttonmotionfcn','')
set(handles.fig,'windowbuttonupfcn','')

%update other analyses to reflect new values:
raveupdateanalysis(handles.fig,[],'xdials')

clear contourplot %clear persistent vars




















