function formattabline(src,eventdata) %#ok<INUSD>
%rave format tab for line graphs.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata'); %i think the (1) is for if multiple graphs are selected.
format=udata.format; %get format tab structure.

%move front tabs to front if necessary
if handles.fronttabsfront==0 %move swap fronttabs and backtabs
    backpos=get(handles.backtabaxes,'position');
    frontpos=get(handles.fronttabaxes,'position');
    set(handles.fronttabaxes,'position',backpos)
    set(handles.backtabaxes,'position',frontpos)
    [x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
    set(handles.backtablines,'xdata',x,'ydata',y);
end
handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,3,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');


%% clear existing tab
delete(handles.tabobjects)

%% text headers
graphtext=text(.15-.10317,.97*1.0317-.0317,'Graph','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
axestext=text(.15-.10317,.85*1.0317-.0317,'Axes','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
dataseriestext=text(.15-.10317,.515*1.0317-.0317,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
legendtext=text(.15-.10317,.11*1.0317-.0317,'Legends','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% GRAPH UICONTROLS

bgtext=uicontrol('position',[origin(1)+25 origin(2)+575 180 20],...
    'style','text','string','Background Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
gridtext=uicontrol('position',[origin(1)+25 origin(2)+550 180 20],...
    'style','text','string','Grid Lines:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

%bgcolor=get(handles.fakerax(handles.activeindex),'color');

bgindex=eye(4);
bgin=format.bg;

% color blobs: (userdata = gridcolor; textcolor; index)
bgwhite=uicontrol('position',[origin(1)+155 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',[1 1 1],'value',bgindex(1,bgin),...
    'callback',@bgblob_callback,'userdata',[.6 .6 .6;0 0 0;1 0 0]);
bglight=uicontrol('position',[origin(1)+180 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',[.753 .753 .753],'value',bgindex(2,bgin),...
    'callback',@bgblob_callback,'userdata',[1 1 1;0 0 0;2 0 0]);
bgdark=uicontrol('position',[origin(1)+205 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',[.502 .502 .502],'value',bgindex(3,bgin),...
    'callback',@bgblob_callback,'userdata',[.6 .6 .6;1 1 1;3 0 0]);
bgblack=uicontrol('position',[origin(1)+230 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',[0 0 0],'value',bgindex(4,bgin),...
    'callback',@bgblob_callback,'userdata',[.3 .3 .3;1 1 1;4 0 0]);

handles.bgblobs=[bgwhite bglight bgdark bgblack];


gridindex=eye(3);
gridin=format.grid;


%grid radios
nogrid=uicontrol('position',[origin(1)+100 origin(2)+551 60 20],...
    'style','radiobutton','string','None','backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(1,gridin),'callback',@nogrid_callback);
magrid=uicontrol('position',[origin(1)+152 origin(2)+551 60 20],...
    'style','radiobutton','string','Major','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(2,gridin),'callback',@magrid_callback);
migrid=uicontrol('position',[origin(1)+204 origin(2)+551 55 20],...
    'style','radiobutton','string','Minor','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(3,gridin),'callback',@migrid_callback);

handles.gridchecks=[nogrid magrid migrid];

%% Axes UICONTROLS

xlim=get(handles.activegraph,'xlim');
ylim=get(handles.activegraph,'ylim');

xlabeltext=uicontrol('position',[origin(1)+25 origin(2)+498 180 20],...
    'style','text','string','X Label:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xlimtext=uicontrol('position',[origin(1)+35 origin(2)+474 180 20],...
    'style','text','string','Limits:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xto=uicontrol('position',[origin(1)+158 origin(2)+474 20 20],...
    'style','text','string','to','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xlabelbox=uicontrol('position',[origin(1)+85 origin(2)+500 160 20],...
    'style','edit','string',get(get(handles.activegraph,'xlabel'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@xlabel_callback);
xminbox=uicontrol('position',[origin(1)+85 origin(2)+476 70 20],...
    'style','edit','string',xlim(1),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@xmin_callback,...
    'tag','xminbox');
xmaxbox=uicontrol('position',[origin(1)+175 origin(2)+476 70 20],...
    'style','edit','string',xlim(2),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@xmax_callback,...
    'tag','xmaxbox');
xresetbutton=uicontrol('position',[origin(1)+85 origin(2)+452 160 20],...
    'style','pushbutton','string','Reset X Axis Limits','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','callback',@xresetbutton_callback);

ylabeltext=uicontrol('position',[origin(1)+25 origin(2)+410 180 20],...
    'style','text','string','Y Label:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
ylimtext=uicontrol('position',[origin(1)+35 origin(2)+386 180 20],...
    'style','text','string','Limits:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
yto=uicontrol('position',[origin(1)+158 origin(2)+386 20 20],...
    'style','text','string','to','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
ylabelbox=uicontrol('position',[origin(1)+85 origin(2)+412 160 20],...
    'style','edit','string',get(get(handles.activegraph,'ylabel'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@ylabel_callback);
yminbox=uicontrol('position',[origin(1)+85 origin(2)+388 70 20],...
    'style','edit','string',ylim(1),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@ymin_callback,...
    'tag','yminbox');
ymaxbox=uicontrol('position',[origin(1)+175 origin(2)+388 70 20],...
    'style','edit','string',ylim(2),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@ymax_callback,...
    'tag','ymaxbox');
yresetbutton=uicontrol('position',[origin(1)+85 origin(2)+364 160 20],...
    'style','pushbutton','string','Reset Y Axis Limits','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','callback',@yresetbutton_callback);


titletext=uicontrol('position',[origin(1)+25 origin(2)+332 180 20],...
    'style','text','string','Title:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
titlebox=uicontrol('position',[origin(1)+65 origin(2)+333 180 20],...
    'style','edit','string',get(get(handles.activegraph,'title'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@title_callback);

%%  marker DATA SERIES UICONTROLS

showmarkercheck=uicontrol('position',[origin(1)+30 origin(2)+309-25 18 18],...
    'style','checkbox','string','','backgroundcolor',handles.colors.tabbg,...
    'value',format.mcheck,'callback',@mcheck_callback);
markerstyletext=uicontrol('position',[origin(1)+50 origin(2)+305-25 49 20],...
    'style','text','string','Marker:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
markersizetext=uicontrol('position',[origin(1)+66 origin(2)+275-25 36 20],...
    'style','text','string','Size:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
markercolortext=uicontrol('position',[origin(1)+60 origin(2)+240-25 180 20],...
    'style','text','string','Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
selecttext=uicontrol('position',[origin(1)+21 origin(2)+64 180 20],...
    'style','text','string','Select Data:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

markerstyleaxes=axes('units','pixels','position',[origin(1)+102 origin(2)+307-25 144 20],...
    'xlimmode','manual','ylimmode','manual','visible','off','nextplot','add','xcolor',[1 1 1],...
    'ycolor',[1 1 1],'zcolor',[1 1 1]);
gap=.9/7;
%markerstyleoptions:
offcolor=[.9 .9 .9];
oncolor=handles.colors.text;

line(.05,.5,'marker','s','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.05+gap,.5,'marker','o','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.05+2*gap,.5,'marker','d','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.05+3*gap,.5,'marker','^','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.05+4*gap,.5,'marker','h','parent',markerstyleaxes,'markersize',12,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.05+5*gap,.5,'marker','p','parent',markerstyleaxes,'markersize',12,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.05+6*gap,.5,'marker','*','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.95,.5,'marker','+','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback,'tag','styleblob')

%handles.styleblobs=get(markerstyleaxes,'children');
%color selected style:

hhh=findobj(markerstyleaxes,'tag','styleblob','marker',format.marker);
set(hhh,'markerfacecolor',oncolor);


markersizeaxes=axes('units','pixels','position',[origin(1)+100 origin(2)+277-25 144 20],...
    'xlimmode','manual','ylimmode','manual','visible','off','nextplot','add','xcolor',[1 1 1],...
    'ycolor',[1 1 1],'zcolor',[1 1 1]);
%gap=.9/7;
%markersizeoptions:
line(.05,.5,'marker','s','parent',markersizeaxes,'markersize',2,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.12,.5,'marker','s','parent',markersizeaxes,'markersize',4,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.22,.5,'marker','s','parent',markersizeaxes,'markersize',5,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.33,.5,'marker','s','parent',markersizeaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.45,.5,'marker','s','parent',markersizeaxes,'markersize',10,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.59,.5,'marker','s','parent',markersizeaxes,'markersize',12,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.75,.5,'marker','s','parent',markersizeaxes,'markersize',14,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.95,.5,'marker','s','parent',markersizeaxes,'markersize',18,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')

%handles.sizeblobs=get(markersizeaxes,'children');

%make size blobs the right style
set(get(markersizeaxes,'children'),'marker',format.marker)

%color the selected blob
hhh=findobj(markersizeaxes,'tag','sizeblob','markersize',format.markersize);
set(hhh,'markerfacecolor',oncolor);


%cdata=ravemakecdata(handles.fig,[],'hsv');

cdata=ravemakecdata(handles.fig,[],handles.cmapnames{udata.format.mcmap});

markercolor=uicontrol('position',[origin(1)+115 origin(2)+242-25 120 20],...
    'style','pushbutton','string','','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','cdata',cdata,'callback',@mcolor_callback);

%%  line DATA SERIES UICONTROLS

showlinecheck=uicontrol('position',[origin(1)+30 origin(2)+199-25 18 18],...
    'style','checkbox','string','','backgroundcolor',handles.colors.tabbg,...
    'value',format.lcheck,'callback',@lcheck_callback);
linestyletext=uicontrol('position',[origin(1)+67 origin(2)+195-25 35 20],...
    'style','text','string','Line:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
linesizetext=uicontrol('position',[origin(1)+58 origin(2)+163-30 44 20],...
    'style','text','string','Width:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
linecolortext=uicontrol('position',[origin(1)+60 origin(2)+130-30 180 20],...
    'style','text','string','Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

%linestyleoptions:
offcolor=[.5 .5 .5];
oncolor=handles.colors.text;

linestyleaxes=axes('units','pixels','position',[origin(1)+105 origin(2)+195-25 140 26],...
    'xlimmode','manual','ylimmode','manual','visible','off','nextplot','add','xcolor',[1 1 1],...
    'ycolor',[1 1 1],'zcolor',[1 1 1]);

line([0 1],[.9 .9],'linestyle','-','parent',linestyleaxes,'linewidth',2.5,'color',offcolor,'buttondownfcn',@lineblob_callback,'tag','lineblob')
line([0 1],[.62 .62],'linestyle','--','parent',linestyleaxes,'linewidth',2.5,'color',offcolor,'buttondownfcn',@lineblob_callback,'tag','lineblob')
line([0 1],[.38 .38],'linestyle',':','parent',linestyleaxes,'linewidth',2.5,'color',offcolor,'buttondownfcn',@lineblob_callback,'tag','lineblob')
line([0 1],[.1 .1],'linestyle','-.','parent',linestyleaxes,'linewidth',2.5,'color',offcolor,'buttondownfcn',@lineblob_callback,'tag','lineblob')

hhh=findobj(linestyleaxes,'tag','lineblob','linestyle',format.linestyle);
set(hhh,'color',oncolor)


%width slider:
[jslider hslider]=javacomponent('javax.swing.JSlider',[origin(1)+130 origin(2)+167-25 100 27]);
jslider.setPaintTicks(1)
jslider.setSnapToTicks(1)
jslider.setMinimum(1)
jslider.setMaximum(5)
jslider.setMajorTickSpacing(1);
jslider.setValue(format.linesize);
set(jslider,'background',java.awt.Color.white)
set(jslider,'StateChangedCallback',{@linewidth_callback,handles.fig});

handles.javacomps=jslider;

widthaxes=axes('units','pixels','position',[origin(1)+125 origin(2)+167-35 110 5],'visible','off');
gauge=imread('widthgauge.bmp');
image(gauge,'parent',widthaxes)
set(widthaxes,'visible','off');

%cdata=ravemakecdata(handles.fig,[],'hsv');

cdata=ravemakecdata(handles.fig,[],handles.cmapnames{udata.format.mcmap});

linecolor=uicontrol('position',[origin(1)+115 origin(2)+105+1-5 120 20],...
    'style','pushbutton','string','','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','cdata',cdata,'callback',@lcolor_callback);


%% legend uicontrols
legendbutton=uicontrol('position',[origin(1)+85 origin(2)+18 160 20],...
    'style','pushbutton','string','Format Legend','fontname','arial','fontsize',10,...
    'backgroundcolor',handles.colors.buttonbg,'callback',@formatlegend_callback,...
    'fontweight','bold');

pointbutton=uicontrol('position',[origin(1)+115 origin(2)+64 25 25],...
    'style','togglebutton','string','Poi','value',0,'backgroundcolor',handles.colors.togglebuttonbg,...
    'fontname','arial narrow','fontsize',6,'horizontalalignment','left','callback',@pointbutton_callback,...
    'cdata',imread('singlepoint.bmp'),'tag','pointbutton');

boxbutton=uicontrol('position',[origin(1)+145 origin(2)+64 25 25],...
    'style','togglebutton','string','','value',0,'backgroundcolor',handles.colors.togglebuttonbg,...
    'fontname','arial narrow','fontsize',6,'horizontalalignment','left','callback',@boxbutton_callback,...
    'cdata',imread('box.bmp'),'tag','boxbutton');

%lasso button is disabled... it's more trouble than it's worth.
lassobutton=uicontrol('position',[origin(1)+175 origin(2)+64 25 25],...
    'style','togglebutton','string','','value',0,'backgroundcolor',handles.colors.togglebuttonbg,...
    'fontname','arial narrow','fontsize',6,'horizontalalignment','left','callback',@lassobutton_callback,...
    'cdata',imread('lasso.bmp'),'tag','lassobutton','visible','off');


%% group items and finish up
handles.tabobjects=[graphtext axestext dataseriestext bgtext gridtext bgwhite...
    bglight bgdark bgblack nogrid magrid migrid xlabeltext ylabeltext...
    xlimtext ylimtext xlabelbox ylabelbox xminbox yminbox xto xmaxbox...
    xresetbutton yto ymaxbox yresetbutton markerstyletext markersizetext markercolortext...
    markerstyleaxes markersizeaxes showmarkercheck markercolor linestyletext linesizetext linecolortext...
    linestyleaxes widthaxes showlinecheck linecolor legendtext legendbutton titletext titlebox...
    pointbutton lassobutton boxbutton selecttext hslider];

handles.activetab='format';

set(handles.infobar,'string','');

guidata(src,handles);

%% HERE BE CALLBACKS
%
%
%
%
%
%
%
%
%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function styleblob_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);

offcolor=[.9 .9 .9];

styleblobs=findobj(handles.tabobjects,'tag','styleblob');
sizeblobs=findobj(handles.tabobjects,'tag','sizeblob');

set(styleblobs,'markerfacecolor',offcolor)
set(src,'markerfacecolor',handles.colors.text)
%update fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.marker=get(src,'marker');
set(handles.fakeax(handles.activeindex),'userdata',udata);


%get target data series
targets=get(handles.activegraph,'children');
%filter out line series with no marker
nottargets=findobj(targets,'marker','none');
targets=setdiff(targets,nottargets);
%update graph
set(targets,'marker',get(src,'marker'));
%update sizeblobs
set(sizeblobs,'marker',get(src,'marker'));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function sizeblob_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);

offcolor=[.9 .9 .9];

sizeblobs=findobj(handles.tabobjects,'tag','sizeblob');

set(sizeblobs,'markerfacecolor',offcolor)
set(src,'markerfacecolor',handles.colors.text)
%update fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.markersize=get(src,'markersize');
set(handles.fakeax(handles.activeindex),'userdata',udata);

%get target data series
targets=get(handles.activegraph,'children');

%filter out line series with no marker
nottargets=findobj(targets,'marker','none');

targets=setdiff(targets,nottargets);

%update graph
set(targets,'markersize',get(src,'markersize'));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function bgblob_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);

uicontrol(handles.dummyobj)

colors=get(src,'userdata');
gridcolor=colors(1,:);
%textcolor=colors(2,:);

plotcolor=get(src,'backgroundcolor');

%update fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.bg=colors(3,1);
set(handles.fakeax(handles.activeindex),'userdata',udata);



%unpop blobs:
set(handles.bgblobs,'value',0);
set(src,'value',1)

%change colors
set(handles.fakerax(handles.activeindex),'color',plotcolor,'xcolor',gridcolor,'ycolor',gridcolor)
set(handles.fakeax(handles.activeindex),'xcolor',gridcolor,'ycolor',gridcolor);

%fix tick marks
if all(plotcolor==[0 0 0])
    set(handles.activegraph,'ticklength',[0 0])
else
    set(handles.activegraph,'tickdir','in','ticklength',[.01 .025])
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function magrid_callback(src, eventdata) %#ok<INUSD>
handles=guidata(src);
%inputs=get(handles.fakerax{handles.activeindex},'userdata');

set(handles.gridchecks,'value',0)
set(src,'value',1)

%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=2;
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')

function migrid_callback(src, eventdata) %#ok<INUSD>
handles=guidata(src);

set(handles.gridchecks,'value',0)
set(src,'value',1)


%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=3;
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on')
set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on')

function nogrid_callback(src, eventdata) %#ok<INUSD>
handles=guidata(src);

set(handles.gridchecks,'value',0)
set(src,'value',1)

%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=1;
set(handles.fakeax(handles.activeindex),'userdata',udata);


set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function xmin_callback(src, eventdata) %#ok<INUSD>

handles=guidata(src);
newval=eval(get(src,'string'));
xlim=get(handles.activegraph,'xlim');
xlim(1)=newval;
set(handles.activegraph,'xlim',xlim)
set(handles.fakeax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'))
set(handles.fakerax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'))

function xmax_callback(src, eventdata) %#ok<INUSD>

handles=guidata(src);
newval=eval(get(src,'string'));
xlim=get(handles.activegraph,'xlim');
xlim(2)=newval;
set(handles.activegraph,'xlim',xlim)
set(handles.fakeax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'))
set(handles.fakerax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'))

function ymin_callback(src, eventdata) %#ok<INUSD>

handles=guidata(src);
newval=eval(get(src,'string'));
ylim=get(handles.activegraph,'ylim');
ylim(1)=newval;
set(handles.activegraph,'ylim',ylim)
set(handles.fakeax(handles.activeindex),'ylim',ylim,'xlim',get(handles.activegraph,'xlim'))
set(handles.fakerax(handles.activeindex),'ylim',ylim,'xlim',get(handles.activegraph,'xlim'))

function ymax_callback(src, eventdata) %#ok<INUSD>

handles=guidata(src);
newval=eval(get(src,'string'));
ylim=get(handles.activegraph,'ylim');
ylim(2)=newval;
set(handles.activegraph,'ylim',ylim)
set(handles.fakeax(handles.activeindex),'ylim',ylim,'xlim',get(handles.activegraph,'xlim'))
set(handles.fakerax(handles.activeindex),'ylim',ylim,'xlim',get(handles.activegraph,'xlim'))

function xresetbutton_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
set(handles.activegraph,'xlimmode','auto')
xlim=get(handles.activegraph,'xlim');

set(handles.fakeax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'))
set(handles.fakerax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'))

xminbox=findobj(handles.tabobjects,'tag','xminbox');
xmaxbox=findobj(handles.tabobjects,'tag','xmaxbox');

set(xminbox,'string',num2str(xlim(1)))
set(xmaxbox,'string',num2str(xlim(2)))

function yresetbutton_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
set(handles.activegraph,'ylimmode','auto')
ylim=get(handles.activegraph,'ylim');

set(handles.fakeax(handles.activeindex),'ylim',ylim,'xlim',get(handles.activegraph,'xlim'))
set(handles.fakerax(handles.activeindex),'ylim',ylim,'xlim',get(handles.activegraph,'xlim'))

yminbox=findobj(handles.tabobjects,'tag','yminbox');
ymaxbox=findobj(handles.tabobjects,'tag','ymaxbox');

set(yminbox,'string',num2str(ylim(1)))
set(ymaxbox,'string',num2str(ylim(2)))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function lineblob_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);

lineblobs=findobj(handles.tabobjects,'tag','lineblob');

offcolor=[.5 .5 .5];
set(lineblobs,'color',offcolor)
set(src,'color',handles.colors.text)

%update inputs
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.linestyle=get(src,'linestyle');
set(handles.fakeax(handles.activeindex),'userdata',udata);


%get target data series
targets=get(handles.activegraph,'children');

%filter out line series with no marker
nottargets=findobj(targets,'linestyle','none');

targets=setdiff(targets,nottargets);

%update graph
set(targets,'linestyle',get(src,'linestyle'));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function linewidth_callback(src,eventdata,fighandle) %#ok<INUSL>

handles=guidata(fighandle);

%update inputs

udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.linesize=src.getValue;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%get target data series
targets=get(handles.activegraph,'children');


%filter out line series with no marker
nottargets=findobj(targets,'linestyle','none');

targets=setdiff(targets,nottargets);

%update graph
set(targets,'linewidth',udata.format.linesize);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xlabel_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
set(get(handles.activegraph,'xlabel'),'string',get(src,'string'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ylabel_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
set(get(handles.activegraph,'ylabel'),'string',get(src,'string'))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function mcolor_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);

uicontrol(handles.dummyobj)


handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
uicontrol(handles.dummyobj)


newcmap=ravechoosecolormap(src,[],udata.format.mcmap);

if newcmap==udata.format.mcmap;return;end
udata.format.mcmap=newcmap;

colorz=handles.cmapnames;
newcolormap=colorz{newcmap};
% 
cdata=ravemakecdata(handles.fig,[],newcolormap);
set(src,'cdata',cdata);
% 
% main=findobj(handles.activegraph,'tag','marker');
% selected=findobj(handles.activegraph,'tag','selected');

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreateline(src,[]);

%{
if ~udata.view.usevariablecolors
    [linecolors,markercolors]=ravemakelinecolors(handles.fig,[],6,numel(marker)); %#ok<NASGU>
 
   for g=1:length(main)
       set(main(g),'color',linecolors(g,:))
   end
 
end

%}



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function lcolor_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
uicontrol(handles.dummyobj)


newcmap=ravechoosecolormap(src,[],udata.format.lcmap);

if newcmap==udata.format.lcmap;return;end
udata.format.lcmap=newcmap;

colorz=handles.cmapnames;
newcolormap=colorz{newcmap};

cdata=ravemakecdata(handles.fig,[],newcolormap);
set(src,'cdata',cdata);


main=findobj(handles.activegraph,'tag','line');
%selected=findobj(handles.activegraph,'tag','selected');

set(handles.fakeax(handles.activeindex),'userdata',udata);


if ~udata.view.usevariablecolors
    [linecolors,markercolors]=ravemakelinecolors(handles.fig,[],numel(main),6); %#ok<NASGU>
 
   for g=1:length(main)
       set(main(g),'color',linecolors(g,:))
   end
 
end








%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function mcheck_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);

%get target data series
targets=findobj(handles.activegraph,'tag','marker');

if get(src,'value') %turn markers on
    set(targets,'visible','on');
else
    set(targets,'visible','off');
end

%update userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.mcheck=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function lcheck_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);

%get target data series
targets=get(handles.activegraph,'children');
%filter out line series with no line
nottargets=findobj(targets,'linestyle','none');
targets=setdiff(targets,nottargets);

if get(src,'value') %turn line on
    set(targets,'visible','on');
else
    set(targets,'visible','off');
end

udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.lcheck=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function title_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);
set(get(handles.activegraph,'title'),'string',get(src,'string'),'fontsize',14);
raveresizeaxes(src,[],handles.activeindex);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
function formatlegend_callback(src,eventdata)




function lassobutton_callback(src,ev)
handles=guidata(src);

otherbuttons=findobj(handles.tabobjects,'tag','boxbutton','-or','tag','pointbutton');
set(otherbuttons,'value',0);

if get(src,'value')
    handles.selectionmode='lasso';
else
    handles.selectionmode='none';
end
guidata(src,handles);


function boxbutton_callback(src,ev)
handles=guidata(src);

otherbuttons=findobj(handles.tabobjects,'tag','lassobutton','-or','tag','pointbutton');
set(otherbuttons,'value',0);

if get(src,'value')
    handles.selectionmode='box';
else
    handles.selectionmode='none';
end
guidata(src,handles);


function pointbutton_callback(src,ev)
handles=guidata(src);

otherbuttons=findobj(handles.tabobjects,'tag','lassobutton','-or','tag','boxbutton');
set(otherbuttons,'value',0);

if get(src,'value')
    handles.selectionmode='point';
else
    handles.selectionmode='none';
end
guidata(src,handles);



