function ravecreateline(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);

udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;

targetaxes=handles.activegraph;
xcol=udata.view.x;
ycol=udata.view.y;
constcol=udata.view.c;
% pscales=udata.view.pscale;
% gscales=udata.view.gscale;
a=udata.view.analysis;
d=udata.view.dataset;

%clear existing graph
labels=findall(handles.activegraph,'tag','xticklabel','-or','tag','yticklabel');
delete(labels);
cla(targetaxes);
legend(targetaxes,'off');

%font formatting
fweight='normal';
labelsize=12;

%get data
x=handles.alldata{d}(:,xcol);
y1=handles.alldata{d}(:,ycol);


isselected=handles.isselected{udata.view.dataset}{udata.view.analysis};


%maintain hidden points:
nothidden=handles.isvisible{udata.view.dataset}{udata.view.analysis};


%do scaling on y1 if necessary:


legendmultipliertext=cell(1,size(y1,2));

if udata.view.autoscale && numel(ycol)>1
    %determine order of magnitude of each response:
    ordersofmagnitude=10.^(round(log10(abs(mean(y1)))));
    y1=bsxfun(@rdivide,y1,ordersofmagnitude);
    legendmultipliers=log10(ordersofmagnitude);
    
    for ggg=1:length(legendmultipliers)
        if legendmultipliers(ggg)==-1
            legendmultipliertext{ggg} = ' x10';
        else
            if strcmp(handles.prefs.interpreter,'tex')
                legendmultipliertext{ggg} = [' x10^{',num2str(-legendmultipliers(ggg)),'}'];
            else
                legendmultipliertext{ggg} = [' x10^',num2str(-legendmultipliers(ggg))];
            end
        end
    end
    [legendmultipliertext{legendmultipliers==0}]=deal('');
else
    [legendmultipliertext{:}]=deal('');
end





%% get constantvalues:
% if constcol is blank, use a column of ones of same length as xdata.
% Otherwise use the actual data from the constcol column.


%append a column of ones on the end for when constcol="none selected"...
%might need to move this line somewhere else in code.

if constcol~=0
    constcolvalues=handles.alldata{d}(:,constcol);
    constantvalues=unique(constcolvalues);
else
    constcolvalues=ones(length(x),1);
    constantvalues=1;
end


%%
%some counters
V=length(ycol); %number of responses being plotted;
J=length(constantvalues);%number of constant contours being plotted;


if isempty(x) %empty data set
    J=0;
    V=0;
end

%run colormap creation script

if udata.view.usevariablecolors
    [linecolors,markercolors]=ravemakelinecolors(handles.fig,[],V,2*J); %still need these markercolors
    linecolors=handles.variablecolors{d}(ycol,:);

else
    [linecolors,markercolors]=ravemakelinecolors(handles.fig,[],V,2*J);

end

set(handles.activegraph,'drawmode','fast');

linehandles=ones(J,V);
markerhandles=ones(J,V);
selectedhandles=ones(J,V);
%make the graph.
for j=1:J %j is counter for number of constant values lines being plotted for each response.
    thisvalue=constantvalues(j);
    plotdeezpoints=constcolvalues==thisvalue;

    for v=1:V %v is counter for number of responses being plotted
        linehandles(j,v)=line(x(plotdeezpoints),y1(plotdeezpoints,v),'linewidth',format.linesize,...
            'userdata',ycol(v),'parent',targetaxes,'color',linecolors(v,:),'hittest','on',...
            'linestyle',format.linestyle,'marker','none','buttondownfcn',@pointselectline,'tag','line',...
            'markersize',ycol(v)); %NOTE: markersize property is essentially being used as another userdata property. can be used to 
            %determine which varible this line is.

        set(targetaxes,'nextplot','add');

        if J>1 %color markers according to 3rd variable
            markerhandles(j,v)=line(x(plotdeezpoints),y1(plotdeezpoints,v),'markersize',format.markersize,...
                'userdata',ycol(v),'parent',targetaxes,'hittest','on',...
                'linestyle','none','marker',format.marker,'markerfacecolor',markercolors(2*j,:),...
                'markeredgecolor',[0 0 0],'buttondownfcn',@pointselectline,'tag','marker');

        else %make color match line color
            markerhandles(j,v)=line(x(plotdeezpoints),y1(plotdeezpoints,v),'markersize',format.markersize,...
                'userdata',ycol(v),'parent',targetaxes,'hittest','on',...
                'linestyle','none','marker',format.marker,'markerfacecolor',linecolors(v,:),...
                'markeredgecolor',[0 0 0],'buttondownfcn',@pointselectline,'tag','marker');

        end

        if any(nothidden & isselected)
            selectedhandles(j,v)=line(x(plotdeezpoints & isselected),y1(plotdeezpoints & isselected,v),'markersize',format.markersize,...
                'userdata',ycol(v),'parent',targetaxes,'hittest','on',...
                'linestyle','none','marker',format.marker,'markerfacecolor','none',...
                'markeredgecolor',[1 0 0],'buttondownfcn','','tag','selected','linewidth',2);
        else
             selectedhandles(j,v)=line(x(plotdeezpoints),y1(plotdeezpoints,v),'markersize',format.markersize,...
                'userdata',ycol(v),'parent',targetaxes,'hittest','on',...
                'linestyle','none','marker',format.marker,'markerfacecolor','none',...
                'markeredgecolor',[1 0 0],'buttondownfcn','','tag','selected','linewidth',2,'visible','off');
            
        end
    end
end


if size(linehandles,1)==1 %not drawing lines of constant anything
    for g=1:numel(linehandles)
        linedata.delete=0;
        linedata.handles=linehandles(g);
        linedata.ai=handles.activeindex;
        linedata.variable=ycol(g);
        %set(linehandles(g),'userdata',linedata,'buttondownfcn',@lineobject_callback)
        set(linehandles(g),'userdata',linedata)
    end
 
end

% %make selected data series:
% selectedhandles=copyobj(markerhandles,targetaxes);
% set(selectedhandles,'markeredgecolor',[1 0 0])
% set(selectedhandles,'visible','on','tag','selected','linewidth',2)
uistack(selectedhandles(:),'top')

%make all axes have same limits
set(handles.fakerax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'))
set(handles.fakeax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'))
set(targetaxes,'nextplot','add')

%set axes labels
set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter','none','fontname','arial')

if numel(ycol)==1 %if plotting 1 response
    set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname','arial')
else
    set(get(targetaxes,'YLabel'),'String','','fontsize',labelsize,...
        'fontweight',fweight,'interpreter','none','fontname','arial')
end

%set line series visibility depending on format settings:
if ~format.mcheck
    set(markerhandles,'visible','off')
end

if ~format.lcheck
    set(linehandles,'visible','off')
end

%resize axes so axes labels/ticks fit inside the bounding box.
targetindex=find(handles.allgraphs==targetaxes);

%adjust gridlines to match settings:
if format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on')
end

%if more than one response variable, make legend:

 set(handles.fakerax(handles.activeindex),'userdata',[]) %clear existing legdata

if numel(ycol)>1 && J>0
    leg1=legend(handles.activegraph,linehandles(1,:),strcat(handles.allnames{d}(ycol),legendmultipliertext),'interpreter',handles.prefs.interpreter);
    set(leg1,'color',[1 1 1])
    set(handles.fakerax(handles.activeindex),'userdata',leg1)
else
    leg1=[];
end


if numel(constantvalues)>1 && numel(constantvalues)<=8 %if more than 8 just use color bar
    leg2=legend(handles.fakeax(handles.activeindex),flipud(markerhandles(:,1)),cellstr(num2str(flipud(constantvalues))),'interpreter','none');
    set(leg2,'color',[1 1 1]);
    titul=get(leg2,'title');
    set(titul,'string',handles.allnames{d}{constcol},'fontsize',12,'fontweight','bold','ver','middle')
    set(handles.fakerax(handles.activeindex),'userdata',[leg1;leg2])  
   
end

%update da handles
guidata(src,handles)

if numel(ycol)==1
    ravemaketextticks(targetaxes,[],xcol,ycol,[],[])
else
    ravemaketextticks(targetaxes,[],xcol,[],[],[])
end

set(targetaxes,'hittest','on','buttondownfcn',@buttondownfcnline)


raveresizeaxes(src,[],targetindex)


set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'xlim',get(handles.activegraph,'xlim'),...
    'ylim',get(handles.activegraph,'ylim'),'xtick',get(handles.activegraph,'xtick'),'ytick',get(handles.activegraph,'ytick'));














