function ravecreatescatter(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);

udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;


raveresizeaxes(src,[],handles.activeindex) %make appropriate margins for this graph type.
%inputs=varargin{3};

targetaxes=handles.activegraph;
xcol=udata.view.x;
ycol=udata.view.y;
ccol=udata.view.c;
scol=udata.view.s;
% pscales=udata.view.pscale;
% gscales=udata.view.gscale;
analysis=udata.view.analysis;
d=udata.view.dataset; %data set index

%clear existing graph

labels=findall(handles.activegraph,'tag','xticklabel','-or','tag','yticklabel');
delete(labels);

cla(targetaxes);
legend(targetaxes,'off');


set(targetaxes,'xtickmode','auto','ytickmode','auto')


%font formatting
fweight='normal';
labelsize=12;

%get data
x=handles.alldata{d}(:,xcol);
y=handles.alldata{d}(:,ycol);

isselected=handles.isselected{udata.view.dataset}{udata.view.analysis}; 

%maintain hidden points:
nothidden=handles.isvisible{udata.view.dataset}{udata.view.analysis};


%% get constantvalues:
% if color/sizecol is blank, use a column of ones of same length as xdata.
% Otherwise use the actual data from the column. 

if ccol~=0
    c=handles.alldata{d}(:,ccol);
    %normalize cdata to desired colormap:
    if ~udata.format.reversecolors
        c=(c-min(c))/(max(c)-min(c))*127+1+128*(udata.format.mcmap-1);
    else
        c=-(c-min(c))/(max(c)-min(c))*127+128*(udata.format.mcmap);        
    end
   
    
else    
    %c=zeros(length(x),1);
    if format.ncolor
        c=handles.prefs.rowcolors(handles.rowcolors{udata.view.dataset}{udata.view.analysis})'+10*rand(length(x),1)-.5; %noisy colors
    else
        c=handles.prefs.rowcolors(handles.rowcolors{udata.view.dataset}{udata.view.analysis})'; %non-noisy colors cdata=0.5      
    end
end

set(targetaxes,'climmode','manual','clim',[1 size(get(handles.fig,'colormap'),1)]);

%set(handles.activegraph,'drawmode','fast')

%% normalize sdata to fit a reasonable range of marker sizes:
if scol~=0
    s=handles.alldata{d}(:,scol);
    isallonesize=0;
    minsize=format.minmarkersize;
    maxsize=format.maxmarkersize;
    s=(((s-min(s))/(max(s)-min(s))).^2)*maxsize + minsize + rand(length(x),1)/100;

else
    s=ones(length(x),1);
    isallonesize=1;
    
end




%format for fillstyle:
fillstyle=format.markerfill;
if fillstyle==1
    pvpairs={'markerfacecolor','none','markeredgecolor','flat'};
elseif fillstyle==2
    pvpairs={'markerfacecolor','flat','markeredgecolor','none'};
else %fillstyle=3
    pvpairs={'markerfacecolor','flat','markeredgecolor',[0 0 0]};
end

h=specgraph.scattergroup('parent',targetaxes,'xdata',x(nothidden),'ydata',y(nothidden),...
    'sizedata',s(nothidden),'cdata',c(nothidden),'hittest','off','marker',format.marker,pvpairs{:},'clipping','on');
set(h,'refreshmode','auto'); %i have no idea what this line does, but i think it's important.
%h.refresh
plotdoneevent(targetaxes,h); %i have no idea what this line does, but i think it's important.
h=double(h); %i have no idea what this line does, but i think it's important.
set(h,'tag','main','hittest','off','buttondownfcn',@pointselectscatter)


if any(nothidden & isselected)
    H=specgraph.scattergroup('parent',targetaxes,'xdata',x(isselected & nothidden),'ydata',y(isselected & nothidden),...
        'sizedata',s(isselected & nothidden),'cdata',c(isselected & nothidden),'hittest','off','marker',format.marker,'markerfacecolor','none','markeredgecolor',[1 0 0],'clipping','on','linewidth',2);
    set(H,'refreshmode','auto'); %i have no idea what this line does, but i think it's important.
    plotdoneevent(targetaxes,h); %i have no idea what this line does, but i think it's important.
    H=double(H); %i have no idea what this line does, but i think it's important.
    set(H,'tag','selected','markeredgecolor',[1 0 0],'visible','on','userdata',isselected)
    %NOTE: userdata for selected, is vector of which points are currently selected.
else
    H=specgraph.scattergroup('parent',targetaxes,'xdata',x(nothidden),'ydata',y(nothidden),...
        'sizedata',s(nothidden),'cdata',c(nothidden),'hittest','off','marker',format.marker,'markerfacecolor','none','markeredgecolor',[1 0 0],'clipping','on','linewidth',2);
    set(H,'refreshmode','auto'); %i have no idea what this line does, but i think it's important.
    plotdoneevent(targetaxes,h); %i have no idea what this line does, but i think it's important.
    H=double(H); %i have no idea what this line does, but i think it's important.
    set(H,'tag','selected','markeredgecolor',[1 0 0],'visible','off','userdata',false(length(x),1))
    %NOTE: userdata for selected, is vector of which points are currently selected.
end

%check markersize:
minsize=format.minmarkersize;
if isallonesize
   set(get(h,'children'),'markersize',sqrt(minsize)) ;
   set(get(H,'children'),'markersize',sqrt(minsize)) ; 
end

%make all axes have same limits
%set(targetaxes,'nextplot','add','xlimmode','manual','ylimmode','manual','zlimmode','manual')

set(handles.fakerax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'))
set(handles.fakeax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'))


%% make tick marks:

%set axes labels
set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter','none','fontname','arial')

if numel(ycol)==1 %if plotting 1 response
    set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname','arial')
else
    set(get(targetaxes,'YLabel'),'String','','fontsize',labelsize,...
        'fontweight',fweight,'interpreter','none','fontname','arial')
end

%resize axes so axes labels/ticks fit inside the bounding box.
targetindex=find(handles.allgraphs==targetaxes);

%adjust gridlines to match settings:
if format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on')
end


%update da handles
guidata(src,handles)


set(targetaxes,'buttondownfcn',@buttondownfcnscatter,'hittest','on')


%if there is a colorbar, fix it:
%NOTE: shrinking graph to make room for colorbar is handled by raveresizeaxes.
%if udata.format.colorbar && udata.view.c;         
        ravemakecolorbar(handles.fig,[]);     
%end

ravemaketextticks(targetaxes,[],xcol,ycol,[],[])


raveresizeaxes(src,[],targetindex)


%set(targetaxes,'xlimmode','manual','ylimmode','manual')








