function ravecreatescatter3D(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);

udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;


%inputs=varargin{3};
targetaxes=handles.activegraph;
xcol=udata.view.x(1); %ones are used to take first var in case 2 vars have same name.
ycol=udata.view.y(1);
zcol=udata.view.z(1);
ccol=udata.view.c(1);
scol=udata.view.s(1);
%pscales=udata.view.pscale;
%gscales=udata.view.gscale;
analysis=udata.view.analysis;
d=udata.view.dataset; %data set index


set(targetaxes,'drawmode','fast');

%clear existing graph
cla(targetaxes);
legend(targetaxes,'off');


%font formatting
fweight='normal';
labelsize=12;

%get data
x=handles.alldata{d}(:,xcol);
y=handles.alldata{d}(:,ycol);
z=handles.alldata{d}(:,zcol);


isselected=handles.isselected{udata.view.dataset}{udata.view.analysis};

%maintain hidden points:
nothidden=handles.isvisible{udata.view.dataset}{udata.view.analysis};

if ccol~=0
    c=handles.alldata{d}(:,ccol);
    %normalize cdata to desired colormap:
    c=(c-min(c))/(max(c)-min(c))*127+128*(udata.format.mcmap-1)+1;    
else    
    %c=zeros(length(x),1);
    if format.ncolor
        c=handles.prefs.rowcolors(handles.rowcolors{udata.view.dataset}{udata.view.analysis})'+10*rand(length(x),1)-.5; %noisy colors
    else
        c=handles.prefs.rowcolors(handles.rowcolors{udata.view.dataset}{udata.view.analysis})'; %non-noisy colors cdata=0.5      
    end
end

set(targetaxes,'climmode','manual','clim',[1 size(get(handles.fig,'colormap'),1)]);



if scol~=0 && ~isempty(x)
    s=handles.alldata{d}(:,scol);
    isallonesize=0;
    minsize=format.minmarkersize;
    maxsize=format.maxmarkersize;
    s=(((s-min(s))/(max(s)-min(s))).^2)*maxsize + minsize + rand(length(x),1)/100;
else
    s=ones(length(x),1);
    isallonesize=1;
end


%% normalize sdata to fit a reasonable range of marker sizes:

%format for fillstyle:
fillstyle=format.markerfill;
if fillstyle==1
    pvpairs={'markerfacecolor','none','markeredgecolor','flat'};
elseif fillstyle==2
    pvpairs={'markerfacecolor','flat','markeredgecolor','flat'};
else %fillstyle=3
    pvpairs={'markerfacecolor','flat','markeredgecolor',[0 0 0]};
end

h=specgraph.scattergroup('parent',targetaxes,'xdata',x(nothidden),'ydata',y(nothidden),'zdata',z(nothidden),...
    'sizedata',s(nothidden),'cdata',c(nothidden),'hittest','off','marker',format.marker,pvpairs{:});
set(h,'refreshmode','auto'); %i have no idea what this line does, but i think it's important.
plotdoneevent(targetaxes,h); %i have no idea what this line does, but i think it's important.
h=double(h); %i have no idea what this line does, but i think it's important.
set(h,'tag','main');


if any(nothidden & isselected)
    H=specgraph.scattergroup('parent',targetaxes,'xdata',x(isselected & nothidden),'ydata',y(isselected & nothidden),...
        'zdata',z(isselected & nothidden),...
        'sizedata',s(isselected & nothidden),'cdata',c(isselected & nothidden),'hittest','off',...
        'marker',format.marker,'markerfacecolor','none','markeredgecolor',[1 0 0],'clipping','on',...
        'linewidth',1);
    set(H,'refreshmode','auto'); %i have no idea what this line does, but i think it's important.
    plotdoneevent(targetaxes,h); %i have no idea what this line does, but i think it's important.
    H=double(H); %i have no idea what this line does, but i think it's important.
    set(H,'tag','selected','markeredgecolor',[1 0 0],'visible','on','userdata',isselected)
    %NOTE: userdata for selected, is vector of which points are currently selected.
else
    H=specgraph.scattergroup('parent',targetaxes,'xdata',x(nothidden),'ydata',y(nothidden),...
        'zdata',z(nothidden),...
        'sizedata',s(nothidden),'cdata',c(nothidden),'hittest','off','marker',format.marker,...
        'markerfacecolor','none','markeredgecolor',[1 0 0],'clipping','on',...
        'linewidth',1);
    set(H,'refreshmode','auto'); %i have no idea what this line does, but i think it's important.
    plotdoneevent(targetaxes,h); %i have no idea what this line does, but i think it's important.
    H=double(H); %i have no idea what this line does, but i think it's important.
    set(H,'tag','selected','markeredgecolor',[1 0 0],'visible','off','userdata',false(length(x),1))
    %NOTE: userdata for selected, is vector of which points are currently selected.
end



%check markersize:
minsize=format.minmarkersize;
if isallonesize
   set(get(h,'children'),'markersize',sqrt(minsize)) ;
   set(get(H,'children'),'markersize',sqrt(minsize)) ; 
end




%link axes so subsequent limit changes affect all axes:
% linkaxes([targetaxes handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'xy');
% set(targetaxes,'xlimmode','auto','ylimmode','auto');

%set axes labels
set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter','none','fontname','arial')
set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname','arial')
set(get(targetaxes,'ZLabel'),'String',handles.allnames{d}{zcol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname','arial')
    
    
set(get(handles.fakerax(handles.activeindex),'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter','none','fontname','arial')
set(get(handles.fakerax(handles.activeindex),'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname','arial')
set(get(handles.fakerax(handles.activeindex),'ZLabel'),'String',handles.allnames{d}{zcol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname','arial')  
    
set(get(handles.fakeax(handles.activeindex),'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter','none','fontname','arial')
set(get(handles.fakeax(handles.activeindex),'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname','arial')
set(get(handles.fakeax(handles.activeindex),'ZLabel'),'String',handles.allnames{d}{zcol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname','arial')   
    
%resize axes so axes labels/ticks fit inside the bounding box.
targetindex=find(handles.allgraphs==targetaxes);

%adjust gridlines to match settings:
if format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off','zminorgrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
elseif format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on','zgrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off','zminorgrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
elseif format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on','zminorgrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on','zgrid','on')
end

%reposition camera for iso view:

% set([targetaxes handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],...
%     'cameraposition',[1 1 1]);

view(targetaxes,3);
view(handles.fakeax(handles.activeindex),3);
view(handles.fakerax(handles.activeindex),3);


%make all axes have same limits
Q=[handles.fakerax(handles.activeindex) handles.fakeax(handles.activeindex)];

set(Q,'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'),'zlim',get(targetaxes,'zlim'),...
    'xtickmode','manual','xtick',get(targetaxes,'xtick'),...
    'ytickmode','manual','ytick',get(targetaxes,'ytick'),...
    'ztickmode','manual','ztick',get(targetaxes,'ztick'));

set(Q,...
    'xticklabel',get(targetaxes,'xticklabel'),...
    'yticklabel',get(targetaxes,'yticklabel'),...
    'zticklabel',get(targetaxes,'zticklabel'));

set(Q,'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'),'zlim',get(targetaxes,'zlim'))
set(targetaxes,'nextplot','add')

set(Q,'ticklength',[0 0]);


%update da handles
guidata(handles.fig,handles)


%if there is a colorbar, fix it:
%NOTE: shrinking graph to make room for colorbar is handled by raveresizeaxes.


raveresizeaxes(src,[],targetindex) %call this before and after maketexticks to avoid a problem with zaxis not updating. 

ravemaketextticks(targetaxes,[],xcol,ycol,zcol,[])

raveresizeaxes(src,[],targetindex)
         
ravemakecolorbar(handles.fig,[]);     


%{
if udata.format.colorbar
        cbar=findobj(handles.fig,'tag','Colorbar','userdata',handles.activeindex);

        properYrange=get(cbar,'ylim');
        
        theimage=get(cbar,'children');
        realcmap=get(handles.fig,'colormap');
        newcdata=permute(realcmap,[1 3 2]);
        

        %drawnow;

        cbarpos=get(cbar,'position');
        cbarpos(1)=cbarpos(1)+30;
        set(cbar,'position',cbarpos,'ylim',properYrange);
        set(theimage,'cdata',newcdata);
end
%}
set(Q,...
    'xticklabel','',...
    'yticklabel','',...
    'zticklabel','');


set(get(handles.fakerax(handles.activeindex),'XLabel'),'String','')
set(get(handles.fakerax(handles.activeindex),'YLabel'),'String','')
set(get(handles.fakerax(handles.activeindex),'ZLabel'),'String','')  
    
set(get(handles.fakeax(handles.activeindex),'XLabel'),'String','')
set(get(handles.fakeax(handles.activeindex),'YLabel'),'String','')
set(get(handles.fakeax(handles.activeindex),'ZLabel'),'String','')   


%fix rotation if necessary:
% if ~isempty(udata.view.campos)
%     set(targetaxes,'cameraposition',udata.view.campos,'cameratarget',udata.view.camtarg);
%     ravesynchaxes;
% end
% 

xxx=get(handles.activegraph,'xlabel');
set(xxx,'buttondownfcn',@ravemove3dlabels,'hittest','on');
yyy=get(handles.activegraph,'ylabel');
set(yyy,'buttondownfcn',@ravemove3dlabels,'hittest','on');












