function ravecreatescattermatrix(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

raveresizeaxes(src,[],handles.activeindex) %make appropriate margins for this graph type.

%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;

%inputs=varargin{3};

targetaxes=handles.activegraph; %=handles.activegraph

switch udata.view.layout
    case {'square';'upper';'lower'}
        xcol=udata.view.x;
        ycol=udata.view.x;
        ccol=udata.view.c;
    otherwise %rectangle
        xcol=udata.view.x;
        ycol=udata.view.y;
        ccol=udata.view.c;
end

% pscales=udata.view.pscale;
% gscales=udata.view.gscale;
a=udata.view.analysis;
d=udata.view.dataset; %data set index

%clear existing graph
cla(targetaxes);

delete(handles.subax{handles.activeindex})
delete(handles.subfakeax{handles.activeindex})
delete(handles.subfakerax{handles.activeindex})


%maintain hidden points:
nothidden=logical(handles.isvisible{udata.view.dataset}{udata.view.analysis});

%get data
xdata=handles.alldata{d}(:,xcol);
xnames=handles.allnames{d}(xcol);

ydata=handles.alldata{d}(:,ycol);
ynames=handles.allnames{d}(ycol);

set(targetaxes,'Visible','off','color','none')

titlestring=udata.format.title;
%make overall title
text(.5,1,titlestring,'parent',targetaxes,'horizontalalignment','center','verticalalignment','bottom',...
    'interpreter',handles.prefs.interpreter,'fontsize',14,'tag','title')

if isempty(titlestring)
    titoffset=0;
else
    titoffset=10;
end

% Create and plot into small axes
ax = zeros(numel(ycol),numel(xcol));
fakeax = zeros(numel(ycol),numel(xcol));
fakerax = zeros(numel(ycol),numel(xcol));
selected= zeros(numel(ycol),numel(xcol));

rows=numel(ycol);
cols=numel(xcol);

pos = get(targetaxes,'Position');%+[30 30 -45 -30]; %shrink a little to fit tick labels. (now handled by raveresizeaxes)
space = handles.prefs.matrixspacing; % 3 percent space between subaxes

width = floor((pos(3)-numel(xcol)*space)/numel(xcol));
height = floor((pos(4)-titoffset-numel(ycol)*space)/numel(ycol));
%pos(1:2) = pos(1:2) + space*[width height];

xlim = zeros([rows cols 2]);
ylim = zeros([rows cols 2]);

%format for fillstyle:
fillstyle=format.markerfill;

if udata.view.c
    
    if fillstyle==1
        pvpairs={'markerfacecolor','none','markeredgecolor','flat'};
    elseif fillstyle==2
        pvpairs={'markerfacecolor','flat','markeredgecolor','none'};
    else %fillstyle=3
        pvpairs={'markerfacecolor','flat','markeredgecolor',[0 0 0]};
    end
    %override for dots:
    if format.marker=='.' || format.marker=='*' || format.marker=='+'
        pvpairs={'markerfacecolor','none','markeredgecolor','flat'};
    end
else
    thecolors=get(handles.fig,'colormap');
    index=handles.prefs.rowcolors;
    if fillstyle==1       
        facecolors=['none';'none';'none';'none';'none';'none';'none';'none';'none';'none'];
        edgecolors=thecolors(index,:);
    elseif fillstyle==2
        edgecolors=['none';'none';'none';'none';'none';'none';'none';'none';'none';'none'];
        facecolors=thecolors(index,:);
    else %fillstyle=3
        edgecolors=zeros(10,3);
        facecolors=thecolors(index,:);
    end
    
    %override for dots:
    if strcmp(udata.format.marker,'.') || strcmp(udata.format.marker,'+') || strcmp(udata.format.marker,'*')
        %  pvpairs={'markerfacecolor','none','markeredgecolor','flat'};
        facecolors=['none';'none';'none';'none';'none';'none';'none';'none';'none';'none'];
        edgecolors=thecolors(index,:);
    end
end

%initialize handles
h.main=zeros(rows,cols,10);

%set up grid of axes.
for r=1:1:numel(ycol),
    for c=1:1:numel(xcol),
        axPos = floor([pos(1)+(c-1)*(width+space) pos(2)+(rows-r)*(height+space) ...
            width height]);
        
        fakerax(r,c) = axes('units','pixels','Position',axPos,'parent',handles.wspanel,...
            'hittest','off','gridlinestyle',':','xcolor',[.5 .5 .5],'ycolor',[.5 .5 .5]);
        fakeax(r,c) = axes('units','pixels','Position',axPos,'parent',handles.wspanel,...
            'hittest','off','gridlinestyle','-','color','none','xcolor',[.5 .5 .5],'ycolor',[.5 .5 .5]);
        ax(r,c) = axes('units','pixels','Position',axPos,'parent',handles.wspanel,...
            'visible','on','nextplot','add','box','on','hittest','on','buttondownfcn',@buttondownfcnscattermatrix,...
            'userdata',handles.activeindex,'color','none'); %userdata contains the index of the bigax for this subgraph.
        set(ax(r,c),'visible','on','drawmode','normal');
        set([fakerax(r,c) fakeax(r,c)],'units','pixels','visible','on','XColor',[.5 .5 .5],'YColor',[.5 .5 .5]);
        subaxvars{r,c}=[xcol(c) ycol(r)];
        addlistener(ax(r,c),'XTick','PreSet',@subaxeschangelistener);
        addlistener(ax(r,c),'YTick','PreSet',@subaxeschangelistener);
        
        
        if ~udata.view.c %make as lines:
            
            rowcolors=handles.rowcolors{udata.view.dataset}{udata.view.analysis};
            %main data series:
            
            for s=1:10;
                if any(rowcolors==s & nothidden)
                    h.main(r,c,s)=line(xdata(rowcolors==s & nothidden,c),ydata(rowcolors==s & nothidden,r),'marker',format.marker,...
                        'markersize',format.markersize,'markerfacecolor',facecolors(s,:),'markeredgecolor',edgecolors(s,:),'hittest','off',...
                        'tag','main','userdata',s,'linestyle','none','buttondownfcn',@pointselectscattermatrix);
                elseif ~isempty(xdata)
                    h.main(r,c,s)=line(xdata(1,c),ydata(1,r),'marker',format.marker,'markersize',format.markersize,...
                        'markerfacecolor',facecolors(s,:),'markeredgecolor',edgecolors(s,:),'hittest','off',...
                        'tag','main','userdata',s,'visible','off','linestyle','none','buttondownfcn',@pointselectscattermatrix);
                else
                    h.main(r,c,s)=nan;
                end
            end
            
        else %make as scattergroup
            
            s=ones(size(xdata,1),1);
            
            %make cdata:
            cdata=handles.alldata{d}(nothidden,ccol);
            %normalize cdata to desired colormap:
            if min(cdata)~=max(cdata)
                cdata=(cdata-min(cdata))/(max(cdata)-min(cdata))*127+128*(udata.format.mcmap-1)+1;
            else
                cdata=(128*(udata.format.mcmap-1)+64)*ones(size(xdata,1),1);
            end
            
            main=specgraph.scattergroup('parent',ax(r,c),'xdata',xdata(nothidden,c),'ydata',ydata(nothidden,r),...
                'sizedata',s(nothidden),'cdata',cdata,'hittest','off','marker',format.marker,'clipping','on',...
                pvpairs{:});
            set(main,'refreshmode','auto'); 
            plotdoneevent(ax(r,c),main); 
            h.main(r,c)=double(main);
            set(h.main(r,c),'tag','main','buttondownfcn',@pointselectscattermatrix)
            
            set(get(h.main(r,c),'children'),'markersize',format.markersize) ;
            set(ax(r,c),'climmode','manual','clim',[1 size(get(handles.fig,'colormap'),1)]);
            
        end
        
        isselected=handles.isselected{d}{udata.view.analysis};
        selectedsize=format.markersize;
        
        %selected data series:
        if ~isempty(xdata) && any(nothidden & isselected)
            selected(r,c) = line(xdata(nothidden & isselected,c),ydata(nothidden & isselected,r),'marker',format.marker,...
                'markerfacecolor','none','markeredgecolor',[1 0 0],'linestyle','none',...
                'visible','on','markersize',selectedsize,'hittest','off',...
                'tag','selected','userdata',isselected,'linewidth',2);
        elseif ~isempty(xdata)
            selected(r,c) = line(xdata(1,c),ydata(1,r),'marker',format.marker,...
                'markerfacecolor','none','markeredgecolor',[1 0 0],'linestyle','none',...
                'visible','off','markersize',selectedsize,'hittest','off',...
                'tag','selected','userdata',false(length(xdata(:,c)),1),'linewidth',2);
        else
            selected(r,c)=nan;
            
        end
        
        set(ax(r,c),'xlimmode','auto','ylimmode','auto','xgrid','off','ygrid','off')
        axis(ax(r,c),'tight')
        
        xlim(r,c,:) = get(ax(r,c),'xlim');
        ylim(r,c,:) = get(ax(r,c),'ylim');
    end
end

xlimmin = min(xlim(:,:,1),[],1);
xlimmax = max(xlim(:,:,2),[],1);
ylimmin = min(ylim(:,:,1),[],2);
ylimmax = max(ylim(:,:,2),[],2);





%% fix axes limits:
inset = .05;
for r=1:rows,
    set(ax(r,1),'ylim',single([ylimmin(r,1) ylimmax(r,1)]))
    dy = diff(get(ax(r,1),'ylim'))*inset;
    set(ax(r,:),'ylim',single([ylimmin(r,1)-dy ylimmax(r,1)+dy]))
    
    set(fakeax(r,1),'ylim',single([ylimmin(r,1) ylimmax(r,1)]))
    dy = diff(get(fakeax(r,1),'ylim'))*inset;
    set(fakeax(r,:),'ylim',single([ylimmin(r,1)-dy ylimmax(r,1)+dy]))
    
    set(fakerax(r,1),'ylim',single([ylimmin(r,1) ylimmax(r,1)]))
    dy = diff(get(fakerax(r,1),'ylim'))*inset;
    set(fakerax(r,:),'ylim',single([ylimmin(r,1)-dy ylimmax(r,1)+dy]))
    
    %force non exponential labels:
    set(ax(r,1),'YTickLabel',get(ax(r,1),'YTick'))
    set(fakeax(r,1),'YTickLabel','')
    set(fakerax(r,1),'YTickLabel','')
    
end
dx = zeros(1,cols);

for c=1:cols,
    
    set(ax(1,c),'xlim',single([xlimmin(1,c) xlimmax(1,c)]))
    dx(c) = diff(get(ax(1,c),'xlim'))*inset;
    set(ax(:,c),'xlim',single([xlimmin(1,c)-dx(c) xlimmax(1,c)+dx(c)]))
    
    set(fakeax(1,c),'xlim',single([xlimmin(1,c) xlimmax(1,c)]))
    dx(c) = diff(get(fakeax(1,c),'xlim'))*inset;
    set(fakeax(:,c),'xlim',single([xlimmin(1,c)-dx(c) xlimmax(1,c)+dx(c)]))
    
    set(fakerax(1,c),'xlim',single([xlimmin(1,c) xlimmax(1,c)]))
    dx(c) = diff(get(fakerax(1,c),'xlim'))*inset;
    set(fakerax(:,c),'xlim',single([xlimmin(1,c)-dx(c) xlimmax(1,c)+dx(c)]))
    
    %force non exponential labels:
    set(ax(end,c),'XTickLabel',get(ax(end,c),'XTick'))
    set(fakeax(end,c),'XTickLabel','')
    set(fakerax(end,c),'XTickLabel','')
    
end

set(ax(1:rows-1,:),'xticklabel','')
set(ax(:,2:cols),'yticklabel','')

set(fakeax(1:rows-1,:),'xticklabel','')
set(fakeax(:,2:cols),'yticklabel','')

set(fakerax(1:rows-1,:),'xticklabel','')
set(fakerax(:,2:cols),'yticklabel','')

set(targetaxes,'XTick',get(ax(rows,1),'xtick'),'YTick',get(ax(rows,1),'ytick'), ...
    'userdata','scattermatrix','tag','');

set(ax,'xlimmode','manual','ylimmode','manual')

%% fix tick labels:

rotationz=90;
halign='right';
valign='default';


% X TICKS:
xlegendmultipliertext=cell(1,numel(xcol)); %first row is xcol, 2nd is ycol.

for q=1:numel(xcol)
    ticks=get(ax(end,q),'XTick');
    omag=round(log10(abs(ticks)));
    %set really small numbers=0
    ticks(omag<-15)=0;
    omag(omag<-15)=0;
    %only reformat if ANY omag is 10^3 or greater.
   % if sum(omag>2)>=(numel(omag)-1)    
     if any(omag>2)
        ticks=ticks/10^max(omag);
        if strcmp(handles.prefs.interpreter,'tex')
            xlegendmultipliertext{q} = [' x10^{',num2str(-max(omag)),'}'];
        else
            xlegendmultipliertext{q} = [' x10^',num2str(-max(omag))];
        end
    elseif  sum(omag<-2)>=(numel(omag)-1)
        ticks=ticks/10^min(omag);
        if strcmp(handles.prefs.interpreter,'tex')
            xlegendmultipliertext{q} = [' x10^{',num2str(-min(omag)),'}'];
        else
            xlegendmultipliertext{q} = [' x10^',num2str(-min(omag))];
        end
    else %no scaling necessary
        xlegendmultipliertext{q} ='';
    end
    
    set(ax(end,q),'XTickLabel',[]);
    set(fakeax(end,q),'XTickLabel',[]);
    set(fakerax(end,q),'XTickLabel',[]);
    
    switch udata.view.layout
        case {'square';'lower';'rectangle'}
            switch handles.datatype{d}{xcol(q)}            
                case {'continuous','constant','integer'}
                    putthemhere=get(ax(end,q),'xtick');
                    usethesevalues=cell(numel(ticks),1);
                    for w=1:numel(ticks)
                        usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                    end
                    
                case 'discrete'
                    discretez=handles.discretevalues{d}{a}{xcol(q)};
                    if numel(discretez)<=12;
                        putthemhere=discretez;
                        usethesevalues=cell(numel(discretez),1);
                        for w=1:numel(discretez)
                            usethesevalues{w}=strtrim(sprintf('%-6.4g',discretez(w)));
                        end
                        
                    else %too many, just treat as continuous
                        putthemhere=get(ax(end,q),'xtick');
                        usethesevalues=cell(numel(ticks),1);
                        for w=1:numel(ticks)
                            usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                        end
                    end
                case 'string'
                    usethesevalues=handles.discretevalues{d}{a}{xcol(q)};
                    putthemhere=unique(handles.alldata{d}(:,xcol(q)));
                    
                case 'logical'
                    usethesevalues={handles.prefs.logicalfalse;handles.prefs.logicaltrue};
                    putthemhere=[0 1];
            end
            

            ylimits=get(ax(end,1),'Ylim');
            ypos=ylimits(1)-.02*(ylimits(2)-ylimits(1));

            th=text(putthemhere,ypos*ones(1,length(putthemhere)),usethesevalues,...
                'HorizontalAlignment',halign,'rotation',rotationz,'parent',ax(end,q),'hittest','off',...
                'verticalalignment',valign,'handlevisibility','off','tag','xticklabel'); %#ok<NASGU>
            
        otherwise %upper
           switch handles.datatype{d}{xcol(q)}            
                case {'continuous','constant','integer'}
                    putthemhere=get(ax(1,q),'xtick');
                    usethesevalues=cell(numel(ticks),1);
                    for w=1:numel(ticks)
                        usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                    end
                    
                case 'discrete'
                    discretez=handles.discretevalues{d}{a}{xcol(q)};
                    if numel(discretez)<=12;
                        putthemhere=discretez;
                        usethesevalues=cell(numel(discretez),1);
                        for w=1:numel(discretez)
                            usethesevalues{w}=strtrim(sprintf('%-6.4g',discretez(w)));
                        end
                        
                    else %too many, just treat as continuous
                        putthemhere=get(ax(1,q),'xtick');
                        usethesevalues=cell(numel(ticks),1);
                        for w=1:numel(ticks)
                            usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                        end
                    end
                case 'string'
                    usethesevalues=handles.discretevalues{d}{a}{xcol(q)};
                    putthemhere=unique(handles.alldata{d}(:,xcol(q)));
                    
                case 'logical'
                    usethesevalues={handles.prefs.logicalfalse;handles.prefs.logicaltrue};
                    putthemhere=[0 1];
            end
            

                ylimits=get(ax(1,1),'Ylim');
                ypos=ylimits(2)+.02*(ylimits(2)-ylimits(1));

                    th=text(putthemhere,ypos*ones(1,length(putthemhere)),usethesevalues,...
                        'HorizontalAlignment','left','rotation',rotationz,'parent',ax(1,q),'hittest','off',...
                        'verticalalignment',valign,'handlevisibility','off','tag','xticklabel'); %#ok<NASGU>   
    end
end

% Y TICKS:
ylegendmultipliertext=cell(1,numel(ycol)); %first row is xcol, 2nd is ycol.

for q=1:numel(ycol)
    ticks=get(ax(q,1),'YTick');
    omag=round(log10(abs(ticks)));
    %set really small numbers=0
    ticks(omag<-15)=0;
    omag(omag<-15)=0;
    %only reformat if all omag is 10^3 or greater.
   % if sum(omag>2)>=(numel(omag)-1)%         all(omag>2)
    if any(omag>2)    
        
        ticks=ticks/10^max(omag);
        if strcmp(handles.prefs.interpreter,'tex')
            ylegendmultipliertext{q} = [' x10^{',num2str(-max(omag)),'}'];
        else
            ylegendmultipliertext{q} = [' x10^',num2str(-max(omag))];
        end
    elseif sum(omag<-2)>=(numel(omag)-1)%   all(omag<-2)
        ticks=ticks/10^min(omag);
        if strcmp(handles.prefs.interpreter,'tex')
            ylegendmultipliertext{q} = [' x10^{',num2str(-min(omag)),'}'];
        else
            ylegendmultipliertext{q} = [' x10^',num2str(-min(omag))];
        end
    else %no scaling necessary
        ylegendmultipliertext{q} ='';
    end
    set(ax(q,1),'YTickLabel',[]);
    set(fakeax(q,1),'YTickLabel',[]);
    set(fakerax(q,1),'YTickLabel',[]);
    
    switch udata.view.layout
        case {'square';'lower';'rectangle'}
            switch handles.datatype{d}{ycol(q)}
                case {'continuous','constant','integer'}
                    putthemhere=get(ax(q,1),'ytick');
                    usethesevalues=cell(numel(ticks),1);
                    for w=1:numel(ticks)
                        usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                    end
                case 'discrete'
                    discretez=handles.discretevalues{d}{a}{ycol(q)};
                    if numel(discretez)<=12;
                        putthemhere=discretez;
                        usethesevalues=cell(numel(discretez),1);
                        for w=1:numel(discretez)
                            usethesevalues{w}=strtrim(sprintf('%-6.4g',discretez(w)));
                        end
                    else %too many, just treat as continuous
                        putthemhere=get(ax(q,1),'ytick');
                        usethesevalues=cell(numel(ticks),1);
                        for w=1:numel(ticks)
                            usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                        end
                    end
                case 'string'
                    usethesevalues=handles.discretevalues{d}{a}{ycol(q)};
                    putthemhere=unique(handles.alldata{d}(:,ycol(q)));
                    
                case 'logical'
                    usethesevalues={handles.prefs.logicalfalse;handles.prefs.logicaltrue};
                    putthemhere=[0 1];
            end       

            xlimits=get(ax(end,1),'xlim');
            xpos=xlimits(1)-.02*(xlimits(2)-xlimits(1));
            
            th=text(xpos*ones(1,length(putthemhere)),putthemhere,usethesevalues,...
                'HorizontalAlignment','right','rotation',0,'parent',ax(q,1),'hittest','off',...
                'verticalalignment','middle','handlevisibility','off','tag','yticklabel'); %#ok<NASGU>

        otherwise %upper
             switch handles.datatype{d}{ycol(q)}
                case {'continuous','constant','integer'}
                    putthemhere=get(ax(q,end),'ytick');
                    usethesevalues=cell(numel(ticks),1);
                    for w=1:numel(ticks)
                        usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                    end
                case 'discrete'
                    discretez=handles.discretevalues{d}{a}{ycol(q)};
                    if numel(discretez)<=12;
                        putthemhere=discretez;
                        usethesevalues=cell(numel(discretez),1);
                        for w=1:numel(discretez)
                            usethesevalues{w}=strtrim(sprintf('%-6.4g',discretez(w)));
                        end
                    else %too many, just treat as continuous
                        putthemhere=get(ax(q,end),'ytick');
                        usethesevalues=cell(numel(ticks),1);
                        for w=1:numel(ticks)
                            usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                        end
                    end
                case 'string'
                    usethesevalues=handles.discretevalues{d}{a}{ycol(q)};
                    putthemhere=unique(handles.alldata{d}(:,ycol(q)));
                    
                case 'logical'
                    usethesevalues={handles.prefs.logicalfalse;handles.prefs.logicaltrue};
                    putthemhere=[0 1];
             end       

             xlimits=get(ax(end,end),'xlim');
             xpos=xlimits(2)+.02*(xlimits(2)-xlimits(1));
             
             th=text(xpos*ones(1,length(putthemhere)),putthemhere,usethesevalues,...
                 'HorizontalAlignment','left','rotation',0,'parent',ax(q,end),'hittest','off',...
                 'verticalalignment','middle','handlevisibility','off','tag','yticklabel'); %#ok<NASGU>

    end
end


%% make variable name labels:

switch udata.view.layout
    case {'square';'upper';'lower'}
        for k=1:numel(udata.view.x)
            try %gives error if data set is empty
                set(h.main(k,k,:),'visible','off','tag','nevershow');
                set(selected(k,k),'visible','off','tag','nevershow');
            end
            lims=get(ax(k,k),'xlim');
            tt(k)=text(mean(lims),mean(lims),{xnames{k};xlegendmultipliertext{k}},...
                'parent',ax(k,k),...
                'horizontalAlignment','center','verticalAlignment','middle','interpreter',handles.prefs.interpreter,...
                'backgroundcolor',get(ax(1),'color'),'tag','axeslabel');
        end
    otherwise %rectangle
        %make x/y labels on ax(r,c)
        for k=1:numel(udata.view.x)
            set(get(ax(end,k),'xlabel'),'string',{' ';xnames{k}},'interpreter',handles.prefs.interpreter,...
                'verticalalignment','top');
        end
        
        for k=1:numel(udata.view.y)
            set(get(ax(k,1),'ylabel'),'string',{ynames{k};' '},'interpreter',handles.prefs.interpreter,...
                'verticalalignment','bottom');
        end
        
        
end

%% turn off visibility of large axes
set(handles.fakerax(handles.activeindex),'visible','off');
set(handles.fakeax(handles.activeindex),'visible','off');
set(targetaxes,'nextplot','add')


%store handles to the subplot axes in "sub" handles structure:
handles.subax{handles.activeindex}=ax;
handles.subfakeax{handles.activeindex}=fakeax;
handles.subfakerax{handles.activeindex}=fakerax;
handles.statsax{handles.activeindex}=[];
handles.subaxvars{handles.activeindex}=[];

%select the subaxes in position (1,2) to start:
try
    handles.activesubgraph=handles.subax{handles.activeindex}(2);
catch
    handles.activesubgraph=handles.subax{handles.activeindex}(1);
end

%store handles to the data sets:
%handles.scatterplots.main{handles.activeindex}=h.main;
%handles.scatterplots.text{handles.activeindex}=tt; %the text labels for this graph.

%resize axes so axes labels/ticks fit inside the bounding box.
%targetindex=find(handles.allgraphs==targetaxes);

%adjust gridlines to match settings:
if format.grid==1 %no grid
    set(handles.subfakerax{handles.activeindex},'XminorGrid','off','YminorGrid','off')
    set(handles.subfakerax{handles.activeindex},'XGrid','off','YGrid','off')
    set(handles.subfakeax{handles.activeindex},'XGrid','off','YGrid','off')
elseif format.grid==2 %major grid
    set(handles.subfakerax{handles.activeindex},'XGrid','on','YGrid','on')
    set(handles.subfakerax{handles.activeindex},'XminorGrid','off','YminorGrid','off')
    set(handles.subfakeax{handles.activeindex},'XGrid','off','YGrid','off')
elseif format.grid==3; %minor grid
    set(handles.subfakerax{handles.activeindex},'XminorGrid','on','YminorGrid','on')
    set(handles.subfakerax{handles.activeindex},'XGrid','off','YGrid','off')
    set(handles.subfakeax{handles.activeindex},'XGrid','on','YGrid','on')
end

%adjust background and text labels to match settings:
%make background color data matrix:
bgcolordata={[1 1 1;.6 .6 .6;0 0 0]; %white
    [.753 .753 .753;1 1 1;0 0 0]; %light
    [.502 .502 .502;.6 .6 .6;1 1 1]; %dark
    [0 0 0;.3 .3 .3;1 1 1]}; %black

set(handles.subfakerax{handles.activeindex},'color',bgcolordata{format.bg}(1,:));
set(handles.subfakerax{handles.activeindex},'xcolor',bgcolordata{format.bg}(2,:),'ycolor',bgcolordata{format.bg}(2,:));
set(handles.subfakeax{handles.activeindex},'xcolor',bgcolordata{format.bg}(2,:),'ycolor',bgcolordata{format.bg}(2,:));
try %#ok<TRYNC>
    set(tt,'color',bgcolordata{format.bg}(3,:),'backgroundcolor',bgcolordata{format.bg}(1,:));
end
%for upper/lower triangles, hide appropriate axes/data
switch udata.view.layout
    case 'upper'
        hidefilter=logical(tril(ones(numel(xcol))));
        kidz=findall(ax(hidefilter),'-not','tag','axeslabel');
        set(kidz,'visible','off','tag','nevershow')
        set(ax(hidefilter),'visible','off')
        set(fakeax(hidefilter),'visible','off')
        set(fakerax(hidefilter),'visible','off')
    case 'lower'
        hidefilter=logical(triu(ones(numel(xcol))));
        kidz=findall(ax(hidefilter),'-not','tag','axeslabel');
        set(kidz,'visible','off','tag','nevershow')
        set(ax(hidefilter),'visible','off')
        set(fakeax(hidefilter),'visible','off')
        set(fakerax(hidefilter),'visible','off')
end

%update da handles
guidata(src,handles)
raveresizeaxes(handles.fig,[],handles.activeindex)
ravemakecolorbar(handles.fig,[]);

