function formattabhistogram(src,eventdata) %#ok<INUSD>
%rave format tab for line graphs.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata'); 
format=udata.format; %get format tab structure.

%move front tabs to front if necessary
if handles.fronttabsfront==0 %move swap fronttabs and backtabs
    backpos=get(handles.backtabaxes,'position');
    frontpos=get(handles.fronttabaxes,'position');
    set(handles.fronttabaxes,'position',backpos)
    set(handles.backtabaxes,'position',frontpos)
    [x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
    set(handles.backtablines,'xdata',x,'ydata',y);       
end

handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,3,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');

%% highlight the selected subplot... this only gets highlighted on the format tab, and goes away when you click another tab.
set(handles.activesubgraph,'xcolor',[1 .5 0],'ycolor',[1 .5 0]);


%% clear existing tab
delete(handles.tabobjects)

%% text headers 
graphtext=text(.15-.10317,.97*1.0317-.0317,'Graph','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
axestext=text(.15-.10317,.85*1.0317-.0317,'Axes','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
dataseriestext=text(.15-.10317,.515*1.0317-.0317,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
legendtext=text(.15-.10317,.09*1.0317-.0317,'Legends','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% GRAPH UICONTROLS

bgtext=uicontrol('position',[origin(1)+25 origin(2)+575 180 20],...
    'style','text','string','Background Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
gridtext=uicontrol('position',[origin(1)+25 origin(2)+550 180 20],...
    'style','text','string','Grid Lines:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

%bgcolor=get(handles.fakerax(handles.activeindex),'color');

bgindex=eye(4);
bgin=format.bg;

% color blobs: (userdata = gridcolor; textcolor; index)
bgwhite=uicontrol('position',[origin(1)+155 origin(2)+568 20 20],...
    'style','togglebutton','string','','backgroundcolor',[1 1 1],'value',bgindex(1,bgin),...
    'callback',@bgblob_callback,'userdata',[.6 .6 .6;0 0 0;1 0 0]);
bglight=uicontrol('position',[origin(1)+180 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',[.753 .753 .753],'value',bgindex(2,bgin),...
    'callback',@bgblob_callback,'userdata',[1 1 1;0 0 0;2 0 0]);
bgdark=uicontrol('position',[origin(1)+205 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',[.502 .502 .502],'value',bgindex(3,bgin),...
    'callback',@bgblob_callback,'userdata',[.6 .6 .6;1 1 1;3 0 0]);
bgblack=uicontrol('position',[origin(1)+230 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',[0 0 0],'value',bgindex(4,bgin),...
    'callback',@bgblob_callback,'userdata',[.3 .3 .3;1 1 1;4 0 0]);

handles.bgblobs=[bgwhite bglight bgdark bgblack];


gridindex=eye(3);
gridin=format.grid;


%grid radios
nogrid=uicontrol('position',[origin(1)+100 origin(2)+551 60 20],...
    'style','radiobutton','string','None','backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(1,gridin),'callback',@nogrid_callback);
magrid=uicontrol('position',[origin(1)+152 origin(2)+551 60 20],...
    'style','radiobutton','string','Major','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(2,gridin),'callback',@magrid_callback);
migrid=uicontrol('position',[origin(1)+204 origin(2)+551 55 20],...
    'style','radiobutton','string','Minor','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(3,gridin),'callback',@migrid_callback);

handles.gridchecks=[nogrid magrid migrid];

%% Axes UICONTROLS

xlim=get(handles.activesubgraph,'xlim');
ylim=get(handles.activesubgraph,'ylim');

xlabeltext=uicontrol('position',[origin(1)+25 origin(2)+498 180 20],...
    'style','text','string','X Label:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xlimtext=uicontrol('position',[origin(1)+35 origin(2)+474 180 20],...
    'style','text','string','Limits:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xto=uicontrol('position',[origin(1)+158 origin(2)+474 20 20],...
    'style','text','string','to','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xlabelbox=uicontrol('position',[origin(1)+85 origin(2)+500 160 20],...
    'style','edit','string',get(get(handles.activegraph,'xlabel'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@xlabel_callback);
xminbox=uicontrol('position',[origin(1)+85 origin(2)+476 70 20],...
    'style','edit','string',xlim(1),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@xmin_callback,...
    'tag','xminbox');
xmaxbox=uicontrol('position',[origin(1)+175 origin(2)+476 70 20],...
    'style','edit','string',xlim(2),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@xmax_callback,...
    'tag','xmaxbox');
xresetbutton=uicontrol('position',[origin(1)+85 origin(2)+452 160 20],...
    'style','pushbutton','string','Reset X Axis Limits','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','callback',@xresetbutton_callback);

ylabeltext=uicontrol('position',[origin(1)+25 origin(2)+410 180 20],...
    'style','text','string','Y Label:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
ylimtext=uicontrol('position',[origin(1)+35 origin(2)+386 180 20],...
    'style','text','string','Limits:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
yto=uicontrol('position',[origin(1)+158 origin(2)+386 20 20],...
    'style','text','string','to','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
ylabelbox=uicontrol('position',[origin(1)+85 origin(2)+412 160 20],...
    'style','edit','string',get(get(handles.activegraph,'ylabel'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@ylabel_callback);
yminbox=uicontrol('position',[origin(1)+85 origin(2)+388 70 20],...
    'style','edit','string',ylim(1),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@ymin_callback,...
    'tag','yminbox');
ymaxbox=uicontrol('position',[origin(1)+175 origin(2)+388 70 20],...
    'style','edit','string',ylim(2),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@ymax_callback,...
    'tag','ymaxbox');
yresetbutton=uicontrol('position',[origin(1)+85 origin(2)+364 160 20],...
    'style','pushbutton','string','Reset Y Axis Limits','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','callback',@yresetbutton_callback);


titletext=uicontrol('position',[origin(1)+25 origin(2)+332 180 20],...
    'style','text','string','Title:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
titlebox=uicontrol('position',[origin(1)+65 origin(2)+333 180 20],...
    'style','edit','string',get(get(handles.activegraph,'title'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@title_callback);

%% DATA SERIES UICONTROLS


if udata.view.nbars>20
    value=(udata.view.nbars-20)/2+20;
else
    value=udata.view.nbars;
end

 

nbarstext=uicontrol('position',[origin(1)+25 origin(2)+280 150 20],...
    'style','text','string','Number of Bars:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
nbars=uicontrol('position',[107+3.6*value origin(2)+235 50 20],...
    'style','text','string',num2str(udata.view.nbars),'backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',12,'horizontalalignment','left','fontweight','bold');

%scale slider:
[jslider1 hslider1]=javacomponent('javax.swing.JSlider',[origin(1)+40 origin(2)+255 190 30]);
jslider1.setPaintTicks(1)

jslider1.setSnapToTicks(0)
jslider1.setMinimum(2)
jslider1.setMaximum(50)
jslider1.setMajorTickSpacing(2);
%jslider.setMinorTickSpacing(1);
jslider1.setValue(value);
set(jslider1,'background',java.awt.Color.white)
set(jslider1,'StateChangedCallback',{@nbars_Callback,nbars});



value=udata.view.barwidth/5*100;


barwidthtext=uicontrol('position',[origin(1)+25 origin(2)+210 150 20],...
    'style','text','string','Bar Width:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
barwidth=uicontrol('position',[49+50+9.2*value origin(2)+165 50 20],...
    'style','text','string',[sprintf('%3.0f',value*5),'%'],'backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',12,'horizontalalignment','left','fontweight','bold');

%scale slider:
[jslider2 hslider2]=javacomponent('javax.swing.JSlider',[origin(1)+40 origin(2)+185 190 30]);
jslider2.setPaintTicks(1)

jslider2.setSnapToTicks(0)
jslider2.setMinimum(1)
jslider2.setMaximum(20)
jslider2.setMajorTickSpacing(1);
%jslider.setMinorTickSpacing(1);
jslider2.setValue(value);
set(jslider2,'background',java.awt.Color.white)
set(jslider2,'StateChangedCallback',{@barwidth_Callback,barwidth});

handles.javacomps={jslider1;jslider2};

facetext=uicontrol('position',[origin(1)+25 origin(2)+150-25 150 20],...
    'style','text','string','Face Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

facecolor=uicontrol('position',[origin(1)+115-6 origin(2)+152-25 25 25],...
    'style','pushbutton','string','','backgroundcolor',udata.format.facecolor,'value',bgindex(1,bgin),...
    'callback',@faceblob_callback);

edgecheck=uicontrol('position',[origin(1)+08 origin(2)+122-25 150 20],'style','checkbox',...
    'string','','value',udata.format.drawedge,'backgroundcolor',handles.colors.tabbg,...
    'callback',@edgecheck_callback);

edgetext=uicontrol('position',[origin(1)+25 origin(2)+120-25 150 20],...
    'style','text','string','Edge Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

edgecolor=uicontrol('position',[origin(1)+115-6 origin(2)+122-25 25 25],...
    'style','pushbutton','string','','backgroundcolor',udata.format.edgecolor,'value',bgindex(1,bgin),...
    'callback',@edgeblob_callback);


selecttext=uicontrol('position',[origin(1)+25 origin(2)+90-25 150 20],...
    'style','text','string','Select Data:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');


pointbutton=uicontrol('position',[origin(1)+115-6 origin(2)+66 25 25],...
    'style','togglebutton','string','','value',0,'backgroundcolor',handles.colors.togglebuttonbg,...
    'fontname','arial narrow','fontsize',6,'horizontalalignment','left','callback',@pointbutton_callback,...
    'cdata',imread('singlepoint.bmp'),'tag','pointbutton');


%%
orienttext=uicontrol('position',[origin(1)+155 origin(2)+152-25 100 20],...
    'style','text','string','Orientation:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');


if strcmp(udata.format.orient,'vertical')
    orien=[0 1];
else
    orien=[1 0];
end

horzcheck=uicontrol('position',[origin(1)+165 origin(2)+132-25 90 20],'style','radiobutton',...
    'string','Horizontal','value',orien(1),'backgroundcolor',handles.colors.tabbg,...
    'callback',@orientcheck_callback,'tag','horzcheck');
vertcheck=uicontrol('position',[origin(1)+165 origin(2)+112-25 90 20],'style','radiobutton',...
    'string','Vertical','value',orien(2),'backgroundcolor',handles.colors.tabbg,...
    'callback',@orientcheck_callback,'tag','vertcheck');


%% legend uicontrols
legendbutton=uicontrol('position',[origin(1)+85 origin(2)+06 160 20],...
    'style','pushbutton','string','Format Legend','fontname','arial','fontsize',10,...
    'backgroundcolor',handles.colors.buttonbg,'callback',@formatlegend_callback,...
    'fontweight','bold');



%% group items and finish up
handles.tabobjects=[graphtext axestext dataseriestext bgtext gridtext bgwhite...
    bglight bgdark bgblack nogrid magrid migrid xlabeltext ylabeltext...
    xlimtext ylimtext xlabelbox ylabelbox xminbox yminbox xto xmaxbox...
    xresetbutton yto ymaxbox yresetbutton legendtext legendbutton titletext titlebox...
    nbarstext nbars barwidth barwidthtext hslider1 hslider2 facetext edgetext selecttext facecolor...
    edgecolor edgecheck pointbutton orienttext horzcheck vertcheck];

handles.activetab='format';

set(handles.infobar,'string','');

guidata(src,handles);

%% HERE BE CALLBACKS
%
%
%
%
%
%
%
%
%% 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function bgblob_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);

uicontrol(handles.dummyobj)

colors=get(src,'userdata');
gridcolor=colors(1,:);
textcolor=colors(2,:);

plotcolor=get(src,'backgroundcolor');

%find text labels to recolor: (text type objects onthe diagonal.)

%update fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.bg=colors(3,1);
set(handles.fakeax(handles.activeindex),'userdata',udata);

%unpop blobs:
set(handles.bgblobs,'value',0);
set(src,'value',1)

%change colors
set(handles.subfakerax{handles.activeindex},'color',plotcolor,'xcolor',gridcolor,'ycolor',gridcolor)
set(handles.subfakeax{handles.activeindex},'xcolor',gridcolor,'ycolor',gridcolor);



%fix tick marks
if all(plotcolor==[0 0 0])
    set(handles.subax{handles.activeindex},'ticklength',[0 0])
else
    set(handles.subax{handles.activeindex},'tickdir','in','ticklength',[.01 .025])
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function magrid_callback(src, eventdata) %#ok<INUSD>
handles=guidata(src);
%inputs=get(handles.fakerax{handles.activeindex},'userdata');

set(handles.gridchecks,'value',0)
set(src,'value',1)

%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=2;
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')

function migrid_callback(src, eventdata) %#ok<INUSD>
handles=guidata(src);

set(handles.gridchecks,'value',0)
set(src,'value',1)


%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=3;
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on')
set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on')

function nogrid_callback(src, eventdata) %#ok<INUSD>
handles=guidata(src);

set(handles.gridchecks,'value',0)
set(src,'value',1)

%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=1;
set(handles.fakeax(handles.activeindex),'userdata',udata);


set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function xmin_callback(src, eventdata) %#ok<INUSD>

handles=guidata(src);
newval=str2double(get(src,'string'));
xlim=get(handles.activesubgraph,'xlim');
xlim(1)=newval;

%get coords of this subgraph:
[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph);

set(handles.subax{handles.activeindex}(r,c),'xlim',xlim)
set(handles.subfakeax{handles.activeindex}(r,c),'xlim',xlim)
set(handles.subfakerax{handles.activeindex}(r,c),'xlim',xlim)

function xmax_callback(src, eventdata) %#ok<INUSD>

handles=guidata(src);
newval=str2double(get(src,'string'));
xlim=get(handles.activesubgraph,'xlim');
xlim(2)=newval;
[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph);

set(handles.subax{handles.activeindex}(r,c),'xlim',xlim)
set(handles.subfakeax{handles.activeindex}(r,c),'xlim',xlim)
set(handles.subfakerax{handles.activeindex}(r,c),'xlim',xlim)

function ymin_callback(src, eventdata) %#ok<INUSD>

handles=guidata(src);
newval=str2double(get(src,'string'));
ylim=get(handles.activesubgraph,'ylim');
ylim(1)=newval;
[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph); 
set(handles.subax{handles.activeindex}(r,c),'ylim',ylim)
set(handles.subfakeax{handles.activeindex}(r,c),'ylim',ylim)
set(handles.subfakerax{handles.activeindex}(r,c),'ylim',ylim)


function ymax_callback(src, eventdata) %#ok<INUSD>

handles=guidata(src);
newval=str2double(get(src,'string'));
ylim=get(handles.activesubgraph,'ylim');
ylim(2)=newval;
[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph); 


set(handles.subax{handles.activeindex}(r,c),'ylim',ylim)
ytick=get(handles.activesubgraph,'ytick');
set(handles.subfakeax{handles.activeindex}(r,c),'ylim',ylim,'ytick',ytick)
set(handles.subfakerax{handles.activeindex}(r,c),'ylim',ylim,'ytick',ytick)


function xresetbutton_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
set(handles.activesubgraph,'xlimmode','auto')
xlim=get(handles.activesubgraph,'xlim');

[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph);
set(handles.subax{handles.activeindex}(r,c),'xlim',xlim)
set(handles.subfakeax{handles.activeindex}(r,c),'xlim',xlim)
set(handles.subfakerax{handles.activeindex}(r,c),'xlim',xlim)


xminbox=findobj(handles.tabobjects,'tag','xminbox');
xmaxbox=findobj(handles.tabobjects,'tag','xmaxbox');

set(xminbox,'string',num2str(xlim(1)))
set(xmaxbox,'string',num2str(xlim(2)))

function yresetbutton_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
set(handles.activesubgraph,'ylimmode','auto')
ylim=get(handles.activesubgraph,'ylim');


[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph); %#ok<NASGU>
set(handles.subax{handles.activeindex}(r,c),'ylim',ylim)
set(handles.subfakeax{handles.activeindex}(r,c),'ylim',ylim)
set(handles.subfakerax{handles.activeindex}(r,c),'ylim',ylim)

%set(handles.fakeax{handles.activeindex},'ylim',ylim)
%set(handles.fakerax{handles.activeindex},'ylim',ylim)

yminbox=findobj(handles.tabobjects,'tag','yminbox');
ymaxbox=findobj(handles.tabobjects,'tag','ymaxbox');

set(yminbox,'string',num2str(ylim(1)))
set(ymaxbox,'string',num2str(ylim(2)))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xlabel_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
set(get(handles.activesubgraph,'xlabel'),'string',get(src,'string'))
raveresizeaxes(src,eventdata,handles.activeindex);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ylabel_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
set(get(handles.activesubgraph,'ylabel'),'string',get(src,'string'))
raveresizeaxes(src,eventdata,handles.activeindex);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function title_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);
set(get(handles.activegraph,'title'),'string',get(src,'string'),'fontsize',14);
raveresizeaxes(src,[],handles.activeindex);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
function formatlegend_callback(src,eventdata) 


function nbars_Callback(src,ev,hObject) %#ok<INUSL>
%hObject = nbars 
numbars=src.getValue;
if numbars>20;numbars=20+(numbars-20)*2;end
pos=get(hObject,'position');
set(hObject,'string',sprintf('%3.0f',numbars),'position',[49+58+3.6*src.getValue pos(2:4)]);

if src.getValueIsAdjusting;return;end

handles=guidata(hObject);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.nbars=numbars;
set(handles.fakeax(handles.activeindex),'userdata',udata)
guidata(hObject,handles);
%update histogram:
ravecreatehistogram(handles.fig,[]);




function barwidth_Callback(src,ev,hObject) %#ok<INUSL>

barwidth=src.getValue*5;

pos=get(hObject,'position');
set(hObject,'string',[sprintf('%3.0f',barwidth),'%'],'position',[49+50+9.2*src.getValue pos(2:4)]);


if src.getValueIsAdjusting;return;end

handles=guidata(hObject);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.barwidth=barwidth/100;
set(handles.fakeax(handles.activeindex),'userdata',udata)
guidata(hObject,handles);
%update histogram:
ravecreatehistogram(handles.fig,[]);

function faceblob_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
newcolor=ravecolorchooser(handles.fig,udata.format.facecolor,'Choose a bar fill color');
set(src,'backgroundcolor',newcolor);
udata.format.facecolor=newcolor;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%update histogram:
ravecreatehistogram(handles.fig,[]);


function edgeblob_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
newcolor=ravecolorchooser(handles.fig,udata.format.edgecolor,'Choose a bar fill color');
set(src,'backgroundcolor',newcolor);
udata.format.edgecolor=newcolor;
set(handles.fakeax(handles.activeindex),'userdata',udata);
%update histogram:
ravecreatehistogram(handles.fig,[]);


function edgecheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.drawedge=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);
%update histogram:
ravecreatehistogram(handles.fig,[]);


function orientcheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

horzcheck=findobj(handles.tabobjects,'tag','horzcheck');
vertcheck=findobj(handles.tabobjects,'tag','vertcheck');

set([horzcheck vertcheck],'value',0);
set(src,'value',1);

if src==horzcheck
    udata.format.orient='horizontal';
    for t=1:numel(handles.subax{handles.activeindex})
        view(handles.subax{handles.activeindex}(t),90,90);
        view(handles.subfakeax{handles.activeindex}(t),90,90);
        view(handles.subfakerax{handles.activeindex}(t),90,90);
        
        set([handles.subfakeax{handles.activeindex}(t) handles.subfakerax{handles.activeindex}(t)],'xlim',get(handles.subax{handles.activeindex}(t),'xlim'),'ylim',get(handles.subax{handles.activeindex}(t),'ylim'),'zlim',get(handles.subax{handles.activeindex}(t),'zlim'));
        set([handles.subfakeax{handles.activeindex}(t) handles.subfakerax{handles.activeindex}(t)],'xtick',get(handles.subax{handles.activeindex}(t),'xtick'),'ytick',get(handles.subax{handles.activeindex}(t),'ytick'),'ztick',get(handles.subax{handles.activeindex}(t),'ztick'))
        
        
    end
else
    udata.format.orient='vertical';
    for t=1:numel(handles.subax{handles.activeindex})
        view(handles.subax{handles.activeindex}(t),0,90);
        view(handles.subfakeax{handles.activeindex}(t),0,90);
        view(handles.subfakerax{handles.activeindex}(t),0,90);
        
        set([handles.subfakeax{handles.activeindex}(t) handles.subfakerax{handles.activeindex}(t)],'xlim',get(handles.subax{handles.activeindex}(t),'xlim'),'ylim',get(handles.subax{handles.activeindex}(t),'ylim'),'zlim',get(handles.subax{handles.activeindex}(t),'zlim'));
        set([handles.subfakeax{handles.activeindex}(t) handles.subfakerax{handles.activeindex}(t)],'xtick',get(handles.subax{handles.activeindex}(t),'xtick'),'ytick',get(handles.subax{handles.activeindex}(t),'ytick'),'ztick',get(handles.subax{handles.activeindex}(t),'ztick'))
        
    end
end

set(handles.fakeax(handles.activeindex),'userdata',udata);


function pointbutton_callback(src,ev)
handles=guidata(src);

if get(src,'value')
    handles.selectionmode='point';
else
    handles.selectionmode='none';
end
guidata(src,handles);






