function ravecreatehistogram3D(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');
%format=udata.format;

%inputs=varargin{3};
targetaxes=handles.activegraph;
xcol=udata.view.x(1); %ones are used to take first var in case 2 vars have same name.
ycol=udata.view.y(1);

%pscales=udata.view.pscale;
%gscales=udata.view.gscale;
analysis=udata.view.analysis;
d=udata.view.dataset; %data set index

%clear existing graph
cla(targetaxes);
legend(targetaxes,'off');


%font formatting
fweight='normal';
labelsize=12;

%get data
x=handles.alldata{d}(:,xcol);
y=handles.alldata{d}(:,ycol);

%maintain hidden points:
nothidden=handles.isvisible{udata.view.dataset}{udata.view.analysis};


x=x(nothidden);
y=y(nothidden);


%nbars=min(udata.view.nbars-1,numel(unique(x))); %dont make more bars than there are unique values.
nbars=udata.view.nbars-1;

if min(x)==max(x) %x is constant
    xedges=[x(1)-1 x(1)+1];
elseif ~isempty(x)
    xedges=linspace(min(x),max(x)+eps(max(x)),nbars+1);
else
    xedges=[0 1];
end

if min(y)==max(y) %x is constant
    yedges=[y(1)-1 y(1)+1];
elseif ~isempty(x)
    yedges=linspace(min(y),max(y)+eps(max(y)),nbars+1);
else
    yedges=[0 1];
end



[xcount,xbins]=histc(x,xedges);

ycount=zeros(nbars+1);
if ~isempty(x)
    for j=1:nbars
        ycount(j,:)=histc(y(xbins==j),yedges)';
    end
end

if udata.view.normalize %normalize counts (make pdfs)
    ycount=ycount./length(x);
end


ycount=ycount(1:end-1,1:end-1); %remove last row/col, cause they are always empty.

%main data series:
if ~isempty(x)
    mainhandle=bar3(targetaxes,yedges(2:end),ycount,udata.view.barwidth);
else
    mainhandle=[];
end


if udata.format.drawedge
    edgecolor=udata.format.edgecolor;
else
    edgecolor=udata.format.facecolor;
end

set(mainhandle,'facecolor',udata.format.facecolor,'hittest','off','tag','main','edgecolor',edgecolor);


%fix xdata to match actual x coordinates:
%xdata of bars is normalized from 1 to nbars, with barwidth calculated
%according to udata.view.barwidth. need to stretch these values out to
%fill proper range of x data.

set(targetaxes,'xlimmode','auto','ylimmode','auto','zlimmode','auto');

xrange=max(xedges)-min(xedges);
xstep=xrange./(nbars);
xwidth=xstep*udata.view.barwidth/2;

if ~isempty(x)
    for g=1:nbars;
        xdata=get(mainhandle(g),'xdata');
        newxdata=xdata;
        newxdata(xdata==min(min(xdata)))=min(x)+xstep*g-2*xwidth;
        newxdata(xdata==max(max(xdata)))=min(x)+xstep*g;
        set(mainhandle(g),'xdata',newxdata);
        
        ydata=get(mainhandle(g),'ydata');
        ybarwidth=yedges(2)-yedges(1);
        ydata=ydata-ybarwidth;
        set(mainhandle(g),'ydata',ydata);
        
        set(mainhandle(g),'facecolor','flat')
        cdata=get(mainhandle(g),'cdata');
        
        c=ycount(:,g);
        maxc=max(max(ycount));
        minc=min(min(ycount));
        
        if udata.format.hideempty
            for h=1:nbars
                if ycount(h,g)==0
                    newxdata((h-1)*6+1:h*6,:)=nan;
                    ydata((h-1)*6+1:h*6,:)=nan;
                end
                
            end
            set(mainhandle(g),'xdata',newxdata,'ydata',ydata);
        end
        
        
        % c=(c-min(c))/(max(c)-min(c))*127+128*(udata.format.mcmap-1)+1;
        c=(c-minc)/(maxc-minc)*127+128*(udata.format.mcmap-1)+1;
        
        e=repmat(1:nbars,[6 1]);
        e=e(:);
        
        f=repmat(c(e),[1 4]);
        
        set(mainhandle(g),'cdata',f)
        
        
    end
    set(targetaxes,'xlim',[min(x)-xwidth min(x)+xrange+2*xwidth],'xtickmode','auto')
    set(handles.fakeax(handles.activeindex),'xlim',[min(x)-xwidth min(x)+xrange+2*xwidth],'xtickmode','auto')
    set(handles.fakerax(handles.activeindex),'xlim',[min(x)-xwidth min(x)+xrange+2*xwidth],'xtickmode','auto')
    
    
end

set(targetaxes,'climmode','manual','clim',[1 size(get(handles.fig,'colormap'),1)]);
set(mainhandle,'tag','main');
set(handles.activegraph,'xgrid','off','ygrid','off','zgrid','off','color','none')



%set axes labels
set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter','none','fontname','arial')
set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
    'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname','arial')
set(get(targetaxes,'ZLabel'),'String','Count',...
    'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname','arial')

%resize axes so axes labels/ticks fit inside the bounding box.
targetindex=find(handles.allgraphs==targetaxes);

%adjust gridlines to match settings:
if udata.format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off','zminorgrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
elseif udata.format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on','zgrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off','zminorgrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
elseif udata.format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on','zminorgrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on','zgrid','on')
end

%reposition camera for iso view:
view(targetaxes,3);

%make all axes have same limits
Q=[handles.fakerax(handles.activeindex) handles.fakeax(handles.activeindex)];


if udata.format.boxon
    set([targetaxes Q],'box','on');
else
    set([targetaxes Q],'box','off');
end


set(Q,'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'),'zlim',get(targetaxes,'zlim'),...
    'xtickmode','manual','xtick',get(targetaxes,'xtick'),...
    'ytickmode','manual','ytick',get(targetaxes,'ytick'),...
    'ztickmode','manual','ztick',get(targetaxes,'ztick'));

set(Q,...
    'xticklabel',get(targetaxes,'xticklabel'),...
    'yticklabel',get(targetaxes,'yticklabel'),...
    'zticklabel',get(targetaxes,'zticklabel'));

set(Q,'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'),'zlim',get(targetaxes,'zlim'))
set(targetaxes,'nextplot','add')

set(Q,'ticklength',[0 0]);



set(Q,'cameraviewangle',get(handles.activegraph,'cameraviewangle'),'dataaspectratio',get(targetaxes,'dataaspectratio'),...
    'plotboxas',get(handles.activegraph,'plotbox'),'view',get(targetaxes,'view'))



%update da handles
guidata(handles.fig,handles)


%if there is a colorbar, fix it:
% %NOTE: shrinking graph to make room for colorbar is handled by raveresizeaxes.
% if udata.format.colorbar && udata.view.c;
%         ravefixcolorbar(handles.fig,[]);
% end

set(Q,...
    'xticklabel','',...
    'yticklabel','',...
    'zticklabel','');

set(targetaxes,'userdata','histogram3D')
raveresizeaxes(src,[],targetindex) %call this before and after maketexticks to avoid a problem with zaxis not updating.
ravemaketextticks(targetaxes,[],xcol,ycol,[],[])
raveresizeaxes(src,[],targetindex)

xxx=get(handles.activegraph,'xlabel');
set(xxx,'buttondownfcn',@ravemove3dlabels,'hittest','on');
yyy=get(handles.activegraph,'ylabel');
set(yyy,'buttondownfcn',@ravemove3dlabels,'hittest','on');






