function formattabcdf(src,eventdata) %#ok<INUSD>
%rave format tab for line graphs.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata'); 
format=udata.format; %get format tab structure.

%move front tabs to front if necessary
if handles.fronttabsfront==0 %move swap fronttabs and backtabs
    backpos=get(handles.backtabaxes,'position');
    frontpos=get(handles.fronttabaxes,'position');
    set(handles.fronttabaxes,'position',backpos)
    set(handles.backtabaxes,'position',frontpos)
    [x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
    set(handles.backtablines,'xdata',x,'ydata',y);       
end
handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,3,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');

%% highlight the selected subplot... this only gets highlighted on the format tab, and goes away when you click another tab.
set(handles.activesubgraph,'xcolor',[1 .5 0],'ycolor',[1 .5 0]);


%% clear existing tab
delete(handles.tabobjects)

%% text headers 
graphtext=text(.15-.10317,.97*1.0317-.0317,'Graph','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
axestext=text(.15-.10317,.85*1.0317-.0317,'Axes','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
dataseriestext=text(.15-.10317,.515*1.0317-.0317,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
legendtext=text(.15-.10317,.09*1.0317-.0317,'Legends','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% GRAPH UICONTROLS

bgtext=uicontrol('position',[origin(1)+25 origin(2)+575 180 20],...
    'style','text','string','Background Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
gridtext=uicontrol('position',[origin(1)+25 origin(2)+550 180 20],...
    'style','text','string','Grid Lines:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

%bgcolor=get(handles.fakerax(handles.activeindex),'color');

bgindex=eye(4);
bgin=format.bg;

% color blobs: (userdata = gridcolor; textcolor; index)
bgwhite=uicontrol('position',[origin(1)+155 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',[1 1 1],'value',bgindex(1,bgin),...
    'callback',@bgblob_callback,'userdata',[.6 .6 .6;0 0 0;1 0 0]);
bglight=uicontrol('position',[origin(1)+180 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',[.753 .753 .753],'value',bgindex(2,bgin),...
    'callback',@bgblob_callback,'userdata',[1 1 1;0 0 0;2 0 0]);
bgdark=uicontrol('position',[origin(1)+205 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',[.502 .502 .502],'value',bgindex(3,bgin),...
    'callback',@bgblob_callback,'userdata',[.6 .6 .6;1 1 1;3 0 0]);
bgblack=uicontrol('position',[origin(1)+230 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',[0 0 0],'value',bgindex(4,bgin),...
    'callback',@bgblob_callback,'userdata',[.3 .3 .3;1 1 1;4 0 0]);

handles.bgblobs=[bgwhite bglight bgdark bgblack];


gridindex=eye(3);
gridin=format.grid;


%grid radios
nogrid=uicontrol('position',[origin(1)+100 origin(2)+551 60 20],...
    'style','radiobutton','string','None','backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(1,gridin),'callback',@nogrid_callback);
magrid=uicontrol('position',[origin(1)+152 origin(2)+551 60 20],...
    'style','radiobutton','string','Major','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(2,gridin),'callback',@magrid_callback);
migrid=uicontrol('position',[origin(1)+204 origin(2)+551 55 20],...
    'style','radiobutton','string','Minor','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(3,gridin),'callback',@migrid_callback);

handles.gridchecks=[nogrid magrid migrid];

%% Axes UICONTROLS

xlim=get(handles.activesubgraph,'xlim');
ylim=get(handles.activesubgraph,'ylim');

xlabeltext=uicontrol('position',[origin(1)+25 origin(2)+498 180 20],...
    'style','text','string','X Label:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xlimtext=uicontrol('position',[origin(1)+35 origin(2)+474 180 20],...
    'style','text','string','Limits:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xto=uicontrol('position',[origin(1)+158 origin(2)+474 20 20],...
    'style','text','string','to','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xlabelbox=uicontrol('position',[origin(1)+85 origin(2)+500 160 20],...
    'style','edit','string',get(get(handles.activegraph,'xlabel'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@xlabel_callback);
xminbox=uicontrol('position',[origin(1)+85 origin(2)+476 70 20],...
    'style','edit','string',xlim(1),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@xmin_callback,...
    'tag','xminbox');
xmaxbox=uicontrol('position',[origin(1)+175 origin(2)+476 70 20],...
    'style','edit','string',xlim(2),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@xmax_callback,...
    'tag','xmaxbox');
xresetbutton=uicontrol('position',[origin(1)+85 origin(2)+452 160 20],...
    'style','pushbutton','string','Reset X Axis Limits','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','callback',@xresetbutton_callback);

ylabeltext=uicontrol('position',[origin(1)+25 origin(2)+410 180 20],...
    'style','text','string','Y Label:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
ylimtext=uicontrol('position',[origin(1)+65 origin(2)+386 180 20],...
    'style','text','string','Limits:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
yto=uicontrol('position',[origin(1)+158 origin(2)+386 20 20],...
    'style','text','string','to','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
ylabelbox=uicontrol('position',[origin(1)+85 origin(2)+412 160 20],...
    'style','edit','string',get(get(handles.activegraph,'ylabel'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@ylabel_callback);
yminbox=uicontrol('position',[origin(1)+85 origin(2)+388 70 20],...
    'style','edit','string',ylim(1),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@ymin_callback,...
    'tag','yminbox');
ymaxbox=uicontrol('position',[origin(1)+175 origin(2)+388 70 20],...
    'style','edit','string',ylim(2),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@ymax_callback,...
    'tag','ymaxbox');
yresetbutton=uicontrol('position',[origin(1)+85 origin(2)+364 160 20],...
    'style','pushbutton','string','Reset Y Axis Limits','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','callback',@yresetbutton_callback);


titletext=uicontrol('position',[origin(1)+25 origin(2)+332 180 20],...
    'style','text','string','Title:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
titlebox=uicontrol('position',[origin(1)+65 origin(2)+333 180 20],...
    'style','edit','string',get(get(handles.activegraph,'title'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@title_callback);

%% DATA SERIES UICONTROLS

solidcheck=uicontrol('position',[origin(1)+30 origin(2)+280 120 20],'style','radiobutton',...
    'string','Single Color:','value',~udata.view.usevariablecolors,'backgroundcolor',handles.colors.tabbg,...
    'callback',@solidcheck_callback,'tag','solidcheck','fontsize',10,'fontweight','bold');

variablecheck=uicontrol('position',[origin(1)+30 origin(2)+255 200 20],'style','radiobutton',...
    'string','Use Variable Colors','value',udata.view.usevariablecolors,'backgroundcolor',handles.colors.tabbg,...
    'callback',@variablecheck_callback,'tag','variablecheck','fontsize',10,'fontweight','bold');

linecolor=uicontrol('position',[origin(1)+140 origin(2)+278 25 25],...
    'style','pushbutton','string','','backgroundcolor',udata.format.linecolor,...
    'callback',@colorblob_callback);

widthtext=uicontrol('position',[origin(1)+55 origin(2)+222 180 20],...
    'style','text','string','Line Width:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

%width slider:
[jslider hslider]=javacomponent('javax.swing.JSlider',[origin(1)+40+25 origin(2)+197 100 27]);
jslider.setPaintTicks(1)
jslider.setSnapToTicks(1)
jslider.setMinimum(1)
jslider.setMaximum(5)
jslider.setMajorTickSpacing(1);
jslider.setValue(format.linesize);
set(jslider,'background',java.awt.Color.white)
set(jslider,'StateChangedCallback',{@linewidth_callback,handles.fig});

handles.javacomps=jslider;

widthaxes=axes('units','pixels','position',[origin(1)+30+25 origin(2)+197-10 110 5],'visible','off');
gauge=imread('widthgauge.bmp');
image(gauge,'parent',widthaxes)
set(widthaxes,'visible','off');





%%
orienttext=uicontrol('position',[origin(1)+25 origin(2)+152 100 20],...
    'style','text','string','Orientation:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');




if strcmp(udata.format.orient,'vertical')
    orien=[0 1];
else
    orien=[1 0];
end

horzcheck=uicontrol('position',[origin(1)+40 origin(2)+162-25 90 20],'style','radiobutton',...
    'string','Horizontal','value',orien(1),'backgroundcolor',handles.colors.tabbg,...
    'callback',@orientcheck_callback,'tag','horzcheck');


vertcheck=uicontrol('position',[origin(1)+40 origin(2)+112 90 20],'style','radiobutton',...
    'string','Vertical','value',orien(2),'backgroundcolor',handles.colors.tabbg,...
    'callback',@orientcheck_callback,'tag','vertcheck');


%% legend uicontrols
legendbutton=uicontrol('position',[origin(1)+85 origin(2)+06 160 20],...
    'style','pushbutton','string','Format Legend','fontname','arial','fontsize',10,...
    'backgroundcolor',handles.colors.buttonbg,'callback',@formatlegend_callback,...
    'fontweight','bold');



%% group items and finish up
handles.tabobjects=[graphtext axestext dataseriestext bgtext gridtext bgwhite...
    bglight bgdark bgblack nogrid magrid migrid xlabeltext ylabeltext...
    xlimtext ylimtext xlabelbox ylabelbox xminbox yminbox xto xmaxbox...
    xresetbutton yto ymaxbox yresetbutton legendtext legendbutton titletext titlebox...
    orienttext horzcheck vertcheck solidcheck variablecheck linecolor hslider widthaxes widthtext];

handles.activetab='format';

set(handles.infobar,'string','');

guidata(src,handles);

%% HERE BE CALLBACKS
%
%
%
%
%
%
%
%
%% 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function bgblob_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);

uicontrol(handles.dummyobj)

colors=get(src,'userdata');
gridcolor=colors(1,:);
textcolor=colors(2,:);

plotcolor=get(src,'backgroundcolor');

%find text labels to recolor: (text type objects onthe diagonal.)

%update fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.bg=colors(3,1);
set(handles.fakeax(handles.activeindex),'userdata',udata);

%unpop blobs:
set(handles.bgblobs,'value',0);
set(src,'value',1)

%change colors
set(handles.subfakerax{handles.activeindex},'color',plotcolor,'xcolor',gridcolor,'ycolor',gridcolor)
set(handles.subfakeax{handles.activeindex},'xcolor',gridcolor,'ycolor',gridcolor);



%fix tick marks
if all(plotcolor==[0 0 0])
    set(handles.subax{handles.activeindex},'ticklength',[0 0])
else
    set(handles.subax{handles.activeindex},'tickdir','in','ticklength',[.01 .025])
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function magrid_callback(src, eventdata) %#ok<INUSD>
handles=guidata(src);
%inputs=get(handles.fakerax{handles.activeindex},'userdata');

set(handles.gridchecks,'value',0)
set(src,'value',1)

%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=2;
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')

function migrid_callback(src, eventdata) %#ok<INUSD>
handles=guidata(src);

set(handles.gridchecks,'value',0)
set(src,'value',1)


%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=3;
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on')
set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on')

function nogrid_callback(src, eventdata) %#ok<INUSD>
handles=guidata(src);

set(handles.gridchecks,'value',0)
set(src,'value',1)

%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=1;
set(handles.fakeax(handles.activeindex),'userdata',udata);


set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function xmin_callback(src, eventdata) %#ok<INUSD>

handles=guidata(src);
newval=str2double(get(src,'string'));
xlim=get(handles.activesubgraph,'xlim');
xlim(1)=newval;

%get coords of this subgraph:
[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph);

set(handles.subax{handles.activeindex}(r,c),'xlim',xlim)
set(handles.subfakeax{handles.activeindex}(r,c),'xlim',xlim)
set(handles.subfakerax{handles.activeindex}(r,c),'xlim',xlim)

function xmax_callback(src, eventdata) %#ok<INUSD>

handles=guidata(src);
newval=str2double(get(src,'string'));
xlim=get(handles.activesubgraph,'xlim');
xlim(2)=newval;
[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph);

set(handles.subax{handles.activeindex}(r,c),'xlim',xlim)
set(handles.subfakeax{handles.activeindex}(r,c),'xlim',xlim)
set(handles.subfakerax{handles.activeindex}(r,c),'xlim',xlim)

function ymin_callback(src, eventdata) %#ok<INUSD>

handles=guidata(src);
newval=str2double(get(src,'string'));
ylim=get(handles.activesubgraph,'ylim');
ylim(1)=newval;
[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph); 
set(handles.subax{handles.activeindex}(r,c),'ylim',ylim)
set(handles.subfakeax{handles.activeindex}(r,c),'ylim',ylim)
set(handles.subfakerax{handles.activeindex}(r,c),'ylim',ylim)


function ymax_callback(src, eventdata) %#ok<INUSD>

handles=guidata(src);
newval=str2double(get(src,'string'));
ylim=get(handles.activesubgraph,'ylim');
ylim(2)=newval;
[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph); 
set(handles.subax{handles.activeindex}(r,c),'ylim',ylim)
set(handles.subfakeax{handles.activeindex}(r,c),'ylim',ylim)
set(handles.subfakerax{handles.activeindex}(r,c),'ylim',ylim)


function xresetbutton_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
set(handles.activesubgraph,'xlimmode','auto')
xlim=get(handles.activesubgraph,'xlim');

[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph);
set(handles.subax{handles.activeindex}(r,c),'xlim',xlim)
set(handles.subfakeax{handles.activeindex}(r,c),'xlim',xlim)
set(handles.subfakerax{handles.activeindex}(r,c),'xlim',xlim)


xminbox=findobj(handles.tabobjects,'tag','xminbox');
xmaxbox=findobj(handles.tabobjects,'tag','xmaxbox');

set(xminbox,'string',num2str(xlim(1)))
set(xmaxbox,'string',num2str(xlim(2)))

function yresetbutton_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
set(handles.activesubgraph,'ylimmode','auto')
ylim=get(handles.activesubgraph,'ylim');


[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph); %#ok<NASGU>
set(handles.subax{handles.activeindex}(r,c),'ylim',ylim)
set(handles.subfakeax{handles.activeindex}(r,c),'ylim',ylim)
set(handles.subfakerax{handles.activeindex}(r,c),'ylim',ylim)

%set(handles.fakeax{handles.activeindex},'ylim',ylim)
%set(handles.fakerax{handles.activeindex},'ylim',ylim)

yminbox=findobj(handles.tabobjects,'tag','yminbox');
ymaxbox=findobj(handles.tabobjects,'tag','ymaxbox');

set(yminbox,'string',num2str(ylim(1)))
set(ymaxbox,'string',num2str(ylim(2)))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xlabel_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
set(get(handles.activesubgraph,'xlabel'),'string',get(src,'string'))
raveresizeaxes(src,eventdata,handles.activeindex);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ylabel_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
set(get(handles.activesubgraph,'ylabel'),'string',get(src,'string'))
raveresizeaxes(src,eventdata,handles.activeindex);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function title_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);
set(get(handles.activegraph,'title'),'string',get(src,'string'),'fontsize',14);
raveresizeaxes(src,[],handles.activeindex);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
function formatlegend_callback(src,eventdata) 


function nbars_Callback(src,ev,hObject) %#ok<INUSL>
%hObject = nbars 
numbars=src.getValue;
if numbars>20;numbars=20+(numbars-20)*2;end
pos=get(hObject,'position');
set(hObject,'string',sprintf('%3.0f',numbars),'position',[49+58+3.6*src.getValue pos(2:4)]);

if src.getValueIsAdjusting;return;end

handles=guidata(hObject);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.nbars=numbars;
set(handles.fakeax(handles.activeindex),'userdata',udata)
guidata(hObject,handles);
%update histogram:
ravecreatehistogram(handles.fig,[]);




function barwidth_Callback(src,ev,hObject) %#ok<INUSL>

barwidth=src.getValue*5;

pos=get(hObject,'position');
set(hObject,'string',[sprintf('%3.0f',barwidth),'%'],'position',[49+50+9.2*src.getValue pos(2:4)]);


if src.getValueIsAdjusting;return;end

handles=guidata(hObject);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.barwidth=barwidth/100;
set(handles.fakeax(handles.activeindex),'userdata',udata)
guidata(hObject,handles);
%update histogram:
ravecreatehistogram(handles.fig,[]);

function faceblob_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
newcolor=ravecolorchooser(handles.fig,udata.format.facecolor,'Choose a bar fill color');
set(src,'backgroundcolor',newcolor);
udata.format.facecolor=newcolor;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%update histogram:
ravecreatehistogram(handles.fig,[]);


function edgeblob_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
newcolor=ravecolorchooser(handles.fig,udata.format.edgecolor,'Choose a bar fill color');
set(src,'backgroundcolor',newcolor);
udata.format.edgecolor=newcolor;
set(handles.fakeax(handles.activeindex),'userdata',udata);
%update histogram:
ravecreatehistogram(handles.fig,[]);


function edgecheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.drawedge=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);
%update histogram:
ravecreatehistogram(handles.fig,[]);

function pointbutton_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

for c=1:numel(udata.view.x)
    maindata=findobj(handles.subax{handles.activeindex},'tag','main');

    if get(src,'value')
        set(maindata,'HitTest','on','ButtonDownFcn',@pointselect)
        set(handles.subax{handles.activeindex},'HitTest','on','ButtonDownFcn',@deselectall) %this allows deselecting all points by clicking anywhere.
    else
        set(maindata,'hittest','off','ButtonDownFcn','')
        set(handles.subax{handles.activeindex},'HitTest','on','ButtonDownFcn','')
    end
end


function pointselect(src,ev)

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
xdata=get(src,'xdata');

subax=get(src,'parent');
clicked=get(subax,'currentpoint');

xclick=clicked(1,1,1);

c=find(handles.subax{handles.activeindex}==subax);

x=handles.alldata{udata.view.dataset}(:,udata.view.x(c));
nbars=udata.view.nbars-1;
edges=linspace(min(x),max(x),nbars+1);
[count bins]=histc(x,edges);


%find corresponding data range:
thisbin=find(abs((edges-xclick))==min(abs(edges-xclick)),1); %index of next largest edge. 

indata=bins==thisbin;


% 
% xmin=edges(index-1);
% xmax=edges(index);


%indata=x>=xmin & x<=xmax

handles.isselected{udata.view.dataset}{udata.view.analysis}=indata;

for c=1:numel(handles.subax{handles.activeindex})
    selected=findobj(handles.subax{handles.activeindex}(c),'tag','selected');
    x=handles.alldata{udata.view.dataset}(:,udata.view.x(c));
    nbars=udata.view.nbars-1; %dont make more bars than there are unique values.
    edges=linspace(min(x),max(x),nbars+1);
    selectedcount=histc(x(indata),edges);
    set(selected,'ydata',selectedcount);
end
guidata(handles.fig,handles);

raveupdateanalysis(src,ev,'isselected')

function deselectall(src,ev)
handles=guidata(src);

function orientcheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

horzcheck=findobj(handles.tabobjects,'tag','horzcheck');
vertcheck=findobj(handles.tabobjects,'tag','vertcheck');

set([horzcheck vertcheck],'value',0);
set(src,'value',1);

if src==horzcheck
    udata.format.orient='horizontal';
    for t=1:numel(handles.subax{handles.activeindex})
        view(handles.subax{handles.activeindex}(t),90,90);
        view(handles.subfakeax{handles.activeindex}(t),90,90);
        view(handles.subfakerax{handles.activeindex}(t),90,90);
        
        set([handles.subfakeax{handles.activeindex}(t) handles.subfakerax{handles.activeindex}(t)],'xlim',get(handles.subax{handles.activeindex}(t),'xlim'),'ylim',get(handles.subax{handles.activeindex}(t),'ylim'),'zlim',get(handles.subax{handles.activeindex}(t),'zlim'));
        set([handles.subfakeax{handles.activeindex}(t) handles.subfakerax{handles.activeindex}(t)],'xtick',get(handles.subax{handles.activeindex}(t),'xtick'),'ytick',get(handles.subax{handles.activeindex}(t),'ytick'),'ztick',get(handles.subax{handles.activeindex}(t),'ztick'))
        
        
    end
else
    udata.format.orient='vertical';
    for t=1:numel(handles.subax{handles.activeindex})
        view(handles.subax{handles.activeindex}(t),0,90);
        view(handles.subfakeax{handles.activeindex}(t),0,90);
        view(handles.subfakerax{handles.activeindex}(t),0,90);
        
        set([handles.subfakeax{handles.activeindex}(t) handles.subfakerax{handles.activeindex}(t)],'xlim',get(handles.subax{handles.activeindex}(t),'xlim'),'ylim',get(handles.subax{handles.activeindex}(t),'ylim'),'zlim',get(handles.subax{handles.activeindex}(t),'zlim'));
        set([handles.subfakeax{handles.activeindex}(t) handles.subfakerax{handles.activeindex}(t)],'xtick',get(handles.subax{handles.activeindex}(t),'xtick'),'ytick',get(handles.subax{handles.activeindex}(t),'ytick'),'ztick',get(handles.subax{handles.activeindex}(t),'ztick'))
        
    end
end

set(handles.fakeax(handles.activeindex),'userdata',udata);





function solidcheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
variablecheck=findobj(handles.tabobjects,'tag','variablecheck');
set(variablecheck,'value',0);
set(src,'value',1)

udata.view.usevariablecolors=0;
set(handles.fakeax(handles.activeindex),'userdata',udata);
ravecreatecdf(handles.fig,[]);



function variablecheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
solidcheck=findobj(handles.tabobjects,'tag','solidcheck');
set(solidcheck,'value',0);
set(src,'value',1)

udata.view.usevariablecolors=1;
set(handles.fakeax(handles.activeindex),'userdata',udata);
ravecreatecdf(handles.fig,[]);



function colorblob_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
newcolor=ravecolorchooser(handles.fig,udata.format.linecolor,'Choose a line color');
set(src,'backgroundcolor',newcolor);
udata.format.linecolor=newcolor;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%update histogram:
ravecreatecdf(handles.fig,[]);




function linewidth_callback(jsrc,jev,src)

handles=guidata(src);

%update inputs

udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.linesize=jsrc.getValue;
set(handles.fakeax(handles.activeindex),'userdata',udata);



%get target data series
targets=get(handles.subax{handles.activeindex},'children');

if iscell(targets);targets=cell2mat(targets);end;


%filter out line series with no marker
nottargets=findobj(targets,'linestyle','none');

targets=setdiff(targets,nottargets);

%update graph
set(targets,'linewidth',udata.format.linesize);





