function formattabparallelcoords(src,eventdata) %#ok<INUSD>
%rave format tab for line graphs.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata'); %i think the (1) is for if multiple graphs are selected.
format=udata.format; %get format tab structure.

%move front tabs to front if necessary
if handles.fronttabsfront==0 %move swap fronttabs and backtabs
    backpos=get(handles.backtabaxes,'position');
    frontpos=get(handles.fronttabaxes,'position');
    set(handles.fronttabaxes,'position',backpos)
    set(handles.backtabaxes,'position',frontpos)
    [x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
    set(handles.backtablines,'xdata',x,'ydata',y);
end
handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,3,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');


%% clear existing tab
delete(handles.tabobjects)

%% text headers
graphtext=text(.15-.10317,.97*1.0317-.0317,'Graph','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
axestext=text(.15-.10317,.85*1.0317-.0317,'Axes','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
dataseriestext=text(.15-.10317,.58*1.0317-.0317,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
legendtext=text(.15-.10317,.12*1.0317-.0317,'Legends','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% GRAPH UICONTROLS

bgtext=uicontrol('position',[origin(1)+25 origin(2)+575 180 20],...
    'style','text','string','Background Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
gridtext=uicontrol('position',[origin(1)+25 origin(2)+550 180 20],...
    'style','text','string','Grid Lines:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

bgindex=eye(4);
bgin=format.bg;

% color blobs: (userdata = gridcolor; textcolor; index)
bgwhite=uicontrol('position',[origin(1)+155 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',[1 1 1],'value',bgindex(1,bgin),...
    'callback',@bgblob_callback,'userdata',[.6 .6 .6;0 0 0;1 0 0]);
bglight=uicontrol('position',[origin(1)+180 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',[.753 .753 .753],'value',bgindex(2,bgin),...
    'callback',@bgblob_callback,'userdata',[1 1 1;0 0 0;2 0 0]);
bgdark=uicontrol('position',[origin(1)+205 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',[.502 .502 .502],'value',bgindex(3,bgin),...
    'callback',@bgblob_callback,'userdata',[.6 .6 .6;1 1 1;3 0 0]);
bgblack=uicontrol('position',[origin(1)+230 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',[0 0 0],'value',bgindex(4,bgin),...
    'callback',@bgblob_callback,'userdata',[.3 .3 .3;1 1 1;4 0 0]);

handles.bgblobs=[bgwhite bglight bgdark bgblack];


gridindex=eye(3);
gridin=format.grid;


%grid radios
nogrid=uicontrol('position',[origin(1)+100 origin(2)+551 60 20],...
    'style','radiobutton','string','None','backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(1,gridin),'callback',@nogrid_callback,'enable','off');
magrid=uicontrol('position',[origin(1)+152 origin(2)+551 60 20],...
    'style','radiobutton','string','Major','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(2,gridin),'callback',@magrid_callback,'enable','off');
migrid=uicontrol('position',[origin(1)+204 origin(2)+551 55 20],...
    'style','radiobutton','string','Minor','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(3,gridin),'callback',@migrid_callback,'enable','off');

handles.gridchecks=[nogrid magrid migrid];

%% Axes UICONTROLS

xlim=get(handles.activegraph,'xlim');
texts=findobj(handles.activegraph,'tag','axeslabel');
oldstring=unlinewrap(get(texts(1),'string'));







xvartext=uicontrol('position',[origin(1)+20 origin(2)+498 180 20],...
    'style','text','string','Variable:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

xvarmenu=uicontrol('position',[origin(1)+85 origin(2)+500 160 20],...
    'style','popupmenu','string',handles.allnames{udata.view.dataset}(udata.view.x),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@xvarmenu_callback,'tag','xvarmenu');

xlabeltext=uicontrol('position',[origin(1)+25 origin(2)+498-25 180 20],...
    'style','text','string','X Label:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xlimtext=uicontrol('position',[origin(1)+36 origin(2)+474-25 180 20],...
    'style','text','string','Limits:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xto=uicontrol('position',[origin(1)+158 origin(2)+474-25 20 20],...
    'style','text','string','to','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xlabelbox=uicontrol('position',[origin(1)+85 origin(2)+500-25 160 20],...
    'style','edit','string',oldstring,'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@xlabel_callback,'tag','xlabelbox');
xminbox=uicontrol('position',[origin(1)+85 origin(2)+476-25 70 20],...
    'style','edit','string',xlim(1),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@xmin_callback,...
    'tag','xminbox','enable','off');
xmaxbox=uicontrol('position',[origin(1)+185 origin(2)+476-25 70 20],...
    'style','edit','string',xlim(2),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@xmax_callback,...
    'tag','xmaxbox','enable','off');
xresetbutton=uicontrol('position',[origin(1)+85 origin(2)+452-25 160 20],...
    'style','pushbutton','string','Reset X Axis Limits','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','callback',@xresetbutton_callback,'enable','off');



titletext=uicontrol('position',[origin(1)+25 origin(2)+332+50 180 20],...
    'style','text','string','Title:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
titlebox=uicontrol('position',[origin(1)+65 origin(2)+333+50 180 20],...
    'style','edit','string',get(get(handles.activegraph,'title'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@title_callback);

%%  marker DATA SERIES UICONTROLS

markersizetext=uicontrol('position',[origin(1)+30 origin(2)+275-25-50 80 20],...
    'style','text','string','Filter Size:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
markercolortext=uicontrol('position',[origin(1)+24 origin(2)+240-25-50 180 20],...
    'style','text','string','Filter Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
selecttext=uicontrol('position',[origin(1)+21 origin(2)+64+65 180 20],...
    'style','text','string','Select Data:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');


offcolor=[.9 .9 .9];
oncolor=handles.colors.text;

markersizeaxes=axes('units','pixels','position',[origin(1)+100 origin(2)+277-25-50 144 20],...
    'xlimmode','manual','ylimmode','manual','visible','off','nextplot','add','xcolor',[1 1 1],...
    'ycolor',[1 1 1],'zcolor',[1 1 1]);

%markersizeoptions:
line(.05,.5,'marker','s','parent',markersizeaxes,'markersize',2,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.12,.5,'marker','s','parent',markersizeaxes,'markersize',4,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.22,.5,'marker','s','parent',markersizeaxes,'markersize',5,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.33,.5,'marker','s','parent',markersizeaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.45,.5,'marker','s','parent',markersizeaxes,'markersize',10,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.59,.5,'marker','s','parent',markersizeaxes,'markersize',12,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.75,.5,'marker','s','parent',markersizeaxes,'markersize',14,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.95,.5,'marker','s','parent',markersizeaxes,'markersize',18,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')

%handles.sizeblobs=get(markersizeaxes,'children');

%make size blobs the right style
set(get(markersizeaxes,'children'),'marker',format.marker)

%color the selected blob
hhh=findobj(markersizeaxes,'tag','sizeblob','markersize',format.markersize);
set(hhh,'markerfacecolor',oncolor);


%cdata=ravemakecdata(handles.fig,[],'hsv');

%cdata=ravemakecdata(handles.fig,[],handles.cmapnames{udata.format.mcmap});



noisycolorscheck=uicontrol('position',[origin(1)+41 origin(2)+250 150 20],...
    'style','checkbox','string','Use Noisy Colors','backgroundcolor',handles.colors.tabbg,...
    'fontsize',10,'horizontalalignment','left','callback',@ncolor_callback,'fontweight','bold','fontname','arial',...
    'value',udata.format.ncolor);



markercolor=uicontrol('position',[origin(1)+115 origin(2)+242-25-50 20 20],...
    'style','pushbutton','string','','backgroundcolor',format.markercolor,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@mcolor_callback);

resetfilterbutton=uicontrol('position',[origin(1)+115+35 origin(2)+242-25-50 100 20],...
    'style','pushbutton','string','Reset ALL Filters','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',8,'horizontalalignment','left','callback',@resetfilter_callback);


%%  line DATA SERIES UICONTROLS

linesizetext=uicontrol('position',[origin(1)+26 origin(2)+163-30+180 80 20],...
    'style','text','string','Line Width:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
linecolortext=uicontrol('position',[origin(1)+60 origin(2)+130-30+180 180 20],...
    'style','text','string','Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

%linestyleoptions:
offcolor=[.5 .5 .5];
oncolor=handles.colors.text;

%width slider:
[jslider hslider]=javacomponent('javax.swing.JSlider',[origin(1)+130 origin(2)+167-30+180 100 27]);
jslider.setPaintTicks(1)
jslider.setSnapToTicks(1)
jslider.setMinimum(1)
jslider.setMaximum(5)
jslider.setMajorTickSpacing(1);
jslider.setValue(format.linesize);
set(jslider,'background',java.awt.Color.white)
set(jslider,'StateChangedCallback',{@linewidth_callback,handles.fig});

handles.javacomps=jslider;

widthaxes=axes('units','pixels','position',[origin(1)+125 origin(2)+167-35+180 110 5],'visible','off');
gauge=imread('widthgauge.bmp');
image(gauge,'parent',widthaxes)
set(widthaxes,'visible','off');

cdata=ravemakecdata(handles.fig,[],handles.cmapnames{udata.format.lcmap});

linecolor=uicontrol('position',[origin(1)+115 origin(2)+105+1-5+180 120 20],...
    'style','pushbutton','string','','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','cdata',cdata,'callback',@lcolor_callback);



%% legend uicontrols


selectinbutton=uicontrol('position',[origin(1)+50+60 origin(2)+64+65 60 22],...
    'style','pushbutton','string','Feasible','backgroundcolor',handles.colors.togglebuttonbg,...
    'fontname','arial narrow','fontsize',10,'horizontalalignment','left','callback',@selectinbutton_callback,'tag','hidefilteredbutton');


selectoutbutton=uicontrol('position',[origin(1)+115+60 origin(2)+64+65 60 22],...
    'style','pushbutton','string','Infeasible','backgroundcolor',handles.colors.togglebuttonbg,...
    'fontname','arial narrow','fontsize',10,'horizontalalignment','left','callback',@selectoutbutton_callback,'tag','hidefilteredbutton');


hidefilteredbutton=uicontrol('position',[origin(1)+110 origin(2)+64+65-26 125 22],...
    'style','pushbutton','string','Hide Selected Rows','value',0,'backgroundcolor',handles.colors.togglebuttonbg,...
    'fontname','arial narrow','fontsize',10,'horizontalalignment','left','callback',@hidebutton_callback,'tag','hidefilteredbutton');


colortestbutton=zeros(1,10);
index=handles.prefs.rowcolors;
thecolors=get(handles.fig,'colormap');

for c=1:10
    colortestbutton(c)=uicontrol('position',[origin(1)+10+22*c origin(2)+75 18 18],...
    'style','pushbutton','string','','value',0,'backgroundcolor',thecolors(index(c),:),...
    'fontname','arial narrow','fontsize',6,'horizontalalignment','left',...
    'callback',@rowcolorblob_callback,'tag','rowcolorblob','userdata',c);

end


legendcheck=uicontrol('position',[origin(1)+85 origin(2)+48-20 160 20],...
    'style','checkbox','string','Show Legend','fontname','arial narrow','fontsize',10,...
    'backgroundcolor',handles.colors.tabbg,'callback',@legendcheck_callback,...
    'fontweight','normal','horizontalalignment','left','value',0,'enable','off');

colorbarcheck=uicontrol('position',[origin(1)+85 origin(2)+26-20 160 20],...
    'style','checkbox','string','Show Colorbar','fontname','arial narrow','fontsize',10,...
    'backgroundcolor',handles.colors.tabbg,'callback',@colorbarcheck_callback,...
    'fontweight','normal','horizontalalignment','left','value',udata.format.colorbar);


%% group items and finish up
handles.tabobjects=[graphtext axestext dataseriestext bgtext gridtext bgwhite...
    bglight bgdark bgblack nogrid magrid migrid xlabeltext...
    xlimtext xlabelbox xminbox xto xmaxbox...
    xresetbutton markersizetext markercolortext...
    markersizeaxes markercolor linesizetext linecolortext...
    widthaxes linecolor legendtext  titletext titlebox...
    selecttext hslider noisycolorscheck selectinbutton selectoutbutton...
    xvartext xvarmenu legendcheck colorbarcheck hidefilteredbutton resetfilterbutton colortestbutton];

handles.activetab='format';

set(handles.infobar,'string','');

guidata(src,handles);

%% HERE BE CALLBACKS
%
%
%
%
%
%
%
%
%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function sizeblob_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);

offcolor=[.9 .9 .9];

sizeblobs=findobj(handles.tabobjects,'tag','sizeblob');

set(sizeblobs,'markerfacecolor',offcolor)
set(src,'markerfacecolor',handles.colors.text)
%update fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.markersize=get(src,'markersize');
set(handles.fakeax(handles.activeindex),'userdata',udata);

%get target data series
targets=findobj(handles.activegraph,'tag','mindragger','-or','tag','maxdragger');

%update graph
set(targets,'markersize',get(src,'markersize'));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function bgblob_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);

uicontrol(handles.dummyobj)

colors=get(src,'userdata');
gridcolor=colors(1,:);
%textcolor=colors(2,:);

plotcolor=get(src,'backgroundcolor');

%update fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.bg=colors(3,1);


%unpop blobs:
set(handles.bgblobs,'value',0);
set(src,'value',1)

%change colors

bg=findobj(handles.fakerax(handles.activeindex),'tag','background');
set(bg,'facecolor',plotcolor);

texters=findobj(handles.activegraph,'tag','mintexter','-or','tag','maxtexter');
set(texters,'backgroundcolor',plotcolor);

if udata.format.bg==4 %black
    linez=findobj(handles.activegraph,'tag','parallelaxes');
    set(linez,'color',[.8 .8 .8]);
    set(texters,'color',[.8 .8 .8]);
    udata.view.offcolor=[.2 .2 .2];
elseif udata.format.bg==3 %dark gray
    linez=findobj(handles.activegraph,'tag','parallelaxes');
    set(linez,'color',[0 0 0]);
    set(texters,'color',[0 0 0]);
    udata.view.offcolor=[.6 .6 .6];
elseif udata.format.bg==2 %light gray
    linez=findobj(handles.activegraph,'tag','parallelaxes');
    set(linez,'color',[0 0 0]);
    set(texters,'color',[0 0 0]);
    udata.view.offcolor=[.65 .65 .65];
else
    linez=findobj(handles.activegraph,'tag','parallelaxes');
    set(linez,'color',[0 0 0]);
    set(texters,'color',[0 0 0]);
    udata.view.offcolor=[.85 .85 .85];
end



allkidz=sort(findobj(handles.activegraph,'tag','main'));
selected=sort(findobj(handles.activegraph,'tag','selected'));


set(allkidz(udata.view.filtered),'color',udata.view.offcolor)
set(selected(udata.view.filtered),'color',udata.view.offcolor)





set(handles.fakeax(handles.activeindex),'userdata',udata);



%ravecreateparallelcoords(handles.fig,[]);




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function magrid_callback(src, eventdata) %#ok<INUSD>
handles=guidata(src);
%inputs=get(handles.fakerax{handles.activeindex},'userdata');

set(handles.gridchecks,'value',0)
set(src,'value',1)

%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=2;
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')

function migrid_callback(src, eventdata) %#ok<INUSD>
handles=guidata(src);

set(handles.gridchecks,'value',0)
set(src,'value',1)


%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=3;
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on')
set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on')

function nogrid_callback(src, eventdata) %#ok<INUSD>
handles=guidata(src);

set(handles.gridchecks,'value',0)
set(src,'value',1)

%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=1;
set(handles.fakeax(handles.activeindex),'userdata',udata);


set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function xmin_callback(src, eventdata) %#ok<INUSD>

handles=guidata(src);
newval=str2double(get(src,'string'));
xlim=get(handles.activegraph,'xlim');
xlim(1)=newval;
set(handles.activegraph,'xlim',xlim)
set(handles.fakeax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'))
set(handles.fakerax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'))

function xmax_callback(src, eventdata) %#ok<INUSD>

handles=guidata(src);
newval=str2double(get(src,'string'));
xlim=get(handles.activegraph,'xlim');
xlim(2)=newval;
set(handles.activegraph,'xlim',xlim)
set(handles.fakeax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'))
set(handles.fakerax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'))


function xresetbutton_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
set(handles.activegraph,'xlimmode','auto')
xlim=get(handles.activegraph,'xlim');

set(handles.fakeax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'))
set(handles.fakerax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'))

xminbox=findobj(handles.tabobjects,'tag','xminbox');
xmaxbox=findobj(handles.tabobjects,'tag','xmaxbox');

set(xminbox,'string',num2str(xlim(1)))
set(xmaxbox,'string',num2str(xlim(2)))


function linewidth_callback(src,eventdata,fighandle) %#ok<INUSL>

handles=guidata(fighandle);

%update inputs

udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.linesize=src.getValue;
set(handles.fakeax(handles.activeindex),'userdata',udata);



%get target data series
targets=findobj(handles.activegraph,'tag','main');
set(targets,'linewidth',udata.format.linesize);

selected=findobj(handles.activegraph,'tag','selected');
set(selected,'linewidth',udata.format.linesize+1);


%update graph


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xlabel_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);

axpos=get(handles.activegraph,'position');
texts=findobj(handles.activegraph,'tag','axeslabel');

nvars=numel(texts);
width=axpos(3)/(nvars-1);
maxchars=round(width/8);


xvarmenu=findobj(handles.tabobjects,'tag','xvarmenu');

set(texts(get(xvarmenu,'value')),'string',linewrap(get(src,'string'),maxchars))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function mcolor_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
uicontrol(handles.dummyobj)

udata.format.markercolor=ravecolorchooser(handles.fig,udata.format.markercolor,'Pick a Color');


set(src,'backgroundcolor',udata.format.markercolor)

targets=findobj(handles.activegraph,'tag','mindragger','-or','tag','maxdragger');
set(targets,'markerfacecolor',udata.format.markercolor)

set(handles.fakeax(handles.activeindex),'userdata',udata);




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function lcolor_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
uicontrol(handles.dummyobj)


newcmap=ravechoosecolormap(src,[],udata.format.lcmap);

if newcmap==udata.format.lcmap;return;end
udata.format.lcmap=newcmap;

colorz=handles.cmapnames;
newcolormap=colorz{newcmap};

cdata=ravemakecdata(handles.fig,[],newcolormap);
set(src,'cdata',cdata);


set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreateparallelcoords(handles.fig,[]);


if udata.format.colorbar;
    if udata.view.c     
        ravefixcolorbar(handles.fig,[]);
    end
end




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function title_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);
set(get(handles.activegraph,'title'),'string',get(src,'string'),'fontsize',14,'visible','on','clipping','off');
raveresizeaxes(src,[],handles.activeindex);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5


function xvarmenu_callback(src,ev)
handles=guidata(src);


texts=findobj(handles.activegraph,'tag','axeslabel');
xlabelbox=findobj(handles.tabobjects,'tag','xlabelbox');

oldstring=unlinewrap(get(texts(get(src,'value')),'string'));

set(xlabelbox,'string',oldstring);


function legendcheck_callback(src,ev)





function resetfilter_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.maxfilter=ones(1,numel(udata.view.maxfilter));
udata.view.minfilter=zeros(1,numel(udata.view.minfilter));

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreateparallelcoords(handles.fig,[]);




function selectinbutton_callback(src,ev)
%need to base selection on ALL rows, including hidden ones. otherwise
%indexing gets screwed up. 
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

%filter on ALL data:
allydata=handles.alldata{udata.view.dataset}(:,udata.view.x);
%normalize:
data1=bsxfun(@minus,allydata,min(allydata));
data2=bsxfun(@rdivide,data1,max(data1));
data2(isnan(data2))=.5;
allydata=data2;


maxoutlier=bsxfun(@gt,allydata,udata.view.maxfilter(udata.view.x));
minoutlier=bsxfun(@lt,allydata,udata.view.minfilter(udata.view.x));

outliers=minoutlier | maxoutlier;
outliers=any(outliers,2);

selected=sort(findobj(handles.activegraph,'tag','selected'));

set(selected,'visible','off');
set(selected(~udata.view.filtered),'visible','on');

handles.isselected{udata.view.dataset}{udata.view.analysis}=~outliers;

set(handles.fakeax(handles.activeindex),'userdata',udata);
guidata(handles.fig,handles);

raveupdateanalysis(handles.fig,[],'isselected');


function selectoutbutton_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

%filter on ALL data:
allydata=handles.alldata{udata.view.dataset}(:,udata.view.x);
%normalize:
data1=bsxfun(@minus,allydata,min(allydata));
data2=bsxfun(@rdivide,data1,max(data1));
data2(isnan(data2))=.5;
allydata=data2;

%allydata=(allydata-udata.view.varmins(udata.view.x))./udata.view.varranges(udata.view.x);

maxoutlier=bsxfun(@gt,allydata,udata.view.maxfilter(udata.view.x));
minoutlier=bsxfun(@lt,allydata,udata.view.minfilter(udata.view.x));

outliers=minoutlier | maxoutlier;
outliers=any(outliers,2);


selected=sort(findobj(handles.activegraph,'tag','selected'));

set(selected,'visible','off');
set(selected(udata.view.filtered),'visible','on');

handles.isselected{udata.view.dataset}{udata.view.analysis}=outliers;

set(handles.fakeax(handles.activeindex),'userdata',udata);
guidata(handles.fig,handles);

raveupdateanalysis(handles.fig,[],'isselected');

function hidebutton_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

nothidden=handles.isvisible{udata.view.dataset}{udata.view.analysis};

newhidden=false(size(nothidden));
newhidden(handles.isselected{udata.view.dataset}{udata.view.analysis})=true;

allhidden= ~nothidden | newhidden;

handles.isvisible{udata.view.dataset}{udata.view.analysis}=~allhidden;

guidata(handles.fig,handles);


raveupdateanalysis(handles.fig,[],'visiblerows',1);









function ncolor_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.format.ncolor=get(src,'value');

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreateparallelcoords(handles.fig,[]);


function rowcolorblob_callback(src,ev)

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

oldindex=get(src,'userdata');

if strcmp(get(handles.fig,'currentmodifier'),'control')
    newindex=ravechoosecolormapcolor(handles.fig,[],oldindex);
    handles.prefs.rowcolors(get(src,'userdata'))=newindex;  
    
    thecolors=get(handles.fig,'colormap');
    set(src,'backgroundcolor',thecolors(newindex,:));
    
else
    newindex=oldindex;
end

handles.rowcolors{udata.view.dataset}{udata.view.analysis}(handles.isselected{udata.view.dataset}{udata.view.analysis})=get(src,'userdata');

guidata(handles.fig,handles);

ravecreateparallelcoords(src,[]);

raveupdateanalysis(src,[],'rowcolors');




function colorbarcheck_callback(src,ev)

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.format.colorbar=get(src,'value');

set(handles.fakeax(handles.activeindex),'userdata',udata);


ravecreateparallelcoords(src,[]);

