function parallelcoordsfilter(src,ev)
%filter a parallelcoords graph by dragging the top or bottomarrows.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');


clicked=get(handles.activegraph,'currentpoint');

xclick=clicked(1,1,1);

xdata=get(gco,'xdata');

uistack(gco,'top') %this makes sure the dragger doesnt get stuck underneath the opposite dragger.


index=find(abs(xdata-xclick)==min(abs(xdata-xclick)));

series=get(gco,'tag');
switch series
    case 'maxdragger'       
        texter=findobj(handles.activegraph,'tag','maxtexter');
        texter=texter(index);
        set(texter,'visible','on');
        set(handles.fig,'windowbuttonmotionfcn',{@maxfilter,index,src,texter})
    case 'mindragger'       
        texter=findobj(handles.activegraph,'tag','mintexter');
        texter=texter(index);
        set(texter,'visible','on');
        set(handles.fig,'windowbuttonmotionfcn',{@minfilter,index,src,texter})
end

set(handles.fig,'windowbuttonupfcn',@filterdone)


function maxfilter(src,ev,index,handy,texter)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

thisax=get(handy,'parent');

newvalue=get(thisax,'currentpoint');

yval=newvalue(1,2,1);

if yval<udata.view.minfilter(udata.view.x(index));yval=udata.view.minfilter(udata.view.x(index));end
if yval>1;yval=1;end

ydata=get(handy,'ydata');
ydata(index)=yval;
set(handy,'ydata',ydata);
textpos=get(texter,'position');

set(texter,'pos',[textpos(1) ydata(index)],...
    'string',['    ',strtrim(sprintf('%-12.5g',ydata(index)*udata.view.varranges(udata.view.x(index))+udata.view.varmins(udata.view.x(index))))]);

udata.view.maxfilter(udata.view.x(index))=ydata(index);
set(handles.fakeax(handles.activeindex),'userdata',udata);

%% do filtering

allkidz=findobj(handles.activegraph,'tag','main');
%selected=findobj(handles.activegraph,'tag','selected');
allydata=cell2mat(get(allkidz,'ydata'));

%note, with this setup, filtering is only with respect to currently visible vars, even though other
%vars' filters are saved. 
maxoutlier=bsxfun(@gt,allydata,udata.view.maxfilter(udata.view.x));
minoutlier=bsxfun(@lt,allydata,udata.view.minfilter(udata.view.x));

outliers=minoutlier | maxoutlier;
outliers=any(outliers,2);

%dim infeasible rows:

oncolor=get(allkidz,'userdata'); %this contains their oncolor cdata.
set(allkidz,{'color'},oncolor)
%set(selected,{'color'},oncolor)

set(allkidz(outliers),'color',udata.view.offcolor,'visible','on')
%set(selected(outliers),'color',udata.view.offcolor)


%uistack(selected(outliers),'bottom');
uistack(allkidz(outliers),'bottom');




function minfilter(src,ev,index,handy,texter)

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');


thisax=get(handy,'parent');

newvalue=get(thisax,'currentpoint');

yval=newvalue(1,2,1);

if yval<0;yval=0;end
if yval>udata.view.maxfilter(udata.view.x(index));yval=udata.view.maxfilter(udata.view.x(index));end

ydata=get(handy,'ydata');
ydata(index)=yval;
set(handy,'ydata',ydata);
textpos=get(texter,'position');

set(texter,'pos',[textpos(1) ydata(index)],...
    'string',['    ',strtrim(sprintf('%-12.5g',ydata(index)*udata.view.varranges(udata.view.x(index))+udata.view.varmins(udata.view.x(index))))]);


udata.view.minfilter(udata.view.x(index))=ydata(index);
set(handles.fakeax(handles.activeindex),'userdata',udata);

allkidz=findobj(handles.activegraph,'tag','main');
%selected=findobj(handles.activegraph,'tag','selected');
allydata=cell2mat(get(allkidz,'ydata'));

maxoutlier=bsxfun(@gt,allydata,udata.view.maxfilter(udata.view.x));
minoutlier=bsxfun(@lt,allydata,udata.view.minfilter(udata.view.x));

outliers=minoutlier | maxoutlier;
outliers=any(outliers,2);

%dim infeasible rows:


oncolor=get(allkidz,'userdata'); %this contains their oncolor cdata.
set(allkidz,{'color'},oncolor)
%set(selected,{'color'},oncolor)

set(allkidz(outliers),'color',udata.view.offcolor,'visible','on')
%set(selected(outliers),'color',udata.view.offcolor)


%uistack(selected(outliers),'bottom');
uistack(allkidz(outliers),'bottom');



function filterdone(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

set(handles.fig,'windowbuttonmotionfcn','')
set(handles.fig,'windowbuttonup','')

if ~udata.format.showlimits
    texter=findobj(handles.activegraph,'tag','maxtexter','-or','tag','mintexter');
    set(texter,'visible','off');
end


%one last filter....
allkidz=sort(findobj(handles.activegraph,'tag','main'));
selected=sort(findobj(handles.activegraph,'tag','selected'));
allydata=cell2mat(get(allkidz,'ydata'));

maxoutlier=bsxfun(@gt,allydata,udata.view.maxfilter(udata.view.x));
minoutlier=bsxfun(@lt,allydata,udata.view.minfilter(udata.view.x));

outliers=minoutlier | maxoutlier;
outliers=any(outliers,2);


oncolor=get(allkidz,'userdata'); %this contains their oncolor cdata.
set(selected,{'color'},oncolor)
set(selected(outliers),'color',udata.view.offcolor)

%do stacking:
uistack(selected(~outliers),'bottom');
uistack(allkidz(~outliers),'bottom');
uistack(selected(outliers),'bottom');
uistack(allkidz(outliers),'bottom');

% %one last filter because uistack changes handle order:
% allkidz=findobj(handles.activegraph,'tag','main');
% selected=findobj(handles.activegraph,'tag','selected');
% allydata=cell2mat(get(allkidz,'ydata'));
% 
% maxoutlier=bsxfun(@gt,allydata,udata.view.maxfilter(udata.view.x));
% minoutlier=bsxfun(@lt,allydata,udata.view.minfilter(udata.view.x));
% 
% outliers=minoutlier | maxoutlier;
% outliers=any(outliers,2);



udata.view.filtered=outliers;
set(handles.fakeax(handles.activeindex),'userdata',udata);




