function viewtabparallelcoords(src,eventdata) %#ok<INUSD>
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

%RAVE: creates uicontrols on the view tab for line graphs.

handles=guidata(src);

%move front tabs to front if necessary
if handles.fronttabsfront==0 %swap fronttabs and backtabs
    backpos=get(handles.backtabaxes,'position');
    frontpos=get(handles.fronttabaxes,'position');
    set(handles.fronttabaxes,'position',backpos)
    set(handles.backtabaxes,'position',frontpos)
    [x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
    set(handles.backtablines,'xdata',x,'ydata',y);
    set(handles.fronttabaxes,'xcolor',handles.colors.tabbg,'ycolor',handles.colors.tabbg);
    set(handles.backtabaxes,'xcolor',handles.colors.frame,'ycolor',handles.colors.frame);
end
handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,2,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

set(handles.infobar,'string','');

origin=get(handles.tabaxes,'position');

%% clear existing tab
%delete(handles.tabobjects)

%% get input info from graph:
udata=get(handles.fakeax(handles.activeindex),'userdata');
view=udata.view; %get view tab structure.
d=udata.view.dataset;

%% text headers
projecttext=text(.15-.10317,.95*1.0317-.0317,'Project','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
dataseriestext=text(.15-.10317,.75*1.0317-.0317,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
optionstext=text(.15-.10317,.23,'Options','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

%% text labels
analysistext=text(.25-.10317,.920*1.0317-.0317,'Choose Parent Analysis:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
datasettext=text(.25-.10317,.85*1.0317-.0317,'Choose a Data Set:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');
xaxistext=text(.25-.10317,.72*1.0317-.0317,'Display Variables:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

sorttext=text(.25-.10317,.50*1.0317-.0317,{'Use arrow buttons to change display order'},'horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',8,'fontname','arial','color',[0 0 0],'fontweight','normal');


colortext=text(.25-.10317,.46*1.0317-.0317,'Color Points by:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

% selecttext=text(.25,.43,'Constraint Slider Behavior:','horizontalalignment','left',...
%     'backgroundcolor',handles.colors.tabbg,...
%     'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');



%% variable lists
analysismenu=uicontrol('position',[origin(1)+55 origin(2)+535 180 30],...
    'style','popupmenu','string',handles.analysisnames,'callback',@analysismenu_callback,...
    'backgroundcolor',[1 1 1],'tag','analysismenu','value',view.analysis);
datasetmenu=uicontrol('position',[origin(1)+55 origin(2)+490 180 30],...
    'style','popupmenu','string',handles.datasetnames,'callback',@datasetmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','datasetmenu','value',view.dataset);


%check varorder... if new variables have been added since graph was
%created, varorder must be updated:

numvars=numel(handles.allnames{d});
if numvars>numel(udata.view.varorder)
   lastvar=max(udata.view.varorder);
   udata.view.varorder=[udata.view.varorder lastvar+1:numvars];
   udata.view.minfilter=[udata.view.minfilter zeros(1,numvars-lastvar)];
   udata.view.maxfilter=[udata.view.maxfilter ones(1,numvars-lastvar)]; 
   udata.view.varmins=[udata.view.varmins min(handles.alldata{d}(:,lastvar+1:numvars))];
   udata.view.varranges=[udata.view.varranges max(handles.alldata{d}(:,lastvar+1:numvars))-min(handles.alldata{d}(:,lastvar+1:numvars))];
   set(handles.fakeax(handles.activeindex),'userdata',udata);     
end



xaxismenu=uicontrol('position',[origin(1)+55 origin(2)+315 160 120],...
    'style','listbox','string',handles.allnames{d}(udata.view.varorder),'callback',@xaxismenu_callback,...
    'backgroundcolor',[1 1 1],'tag','xaxismenu','max',100);

colormenu=uicontrol('position',[origin(1)+55 origin(2)+235 180 30],...
    'style','popupmenu','string',[handles.allnames{d}(1:end),cellstr('<Use Row Colors>')],'callback',@colormenu_callback,...
    'backgroundcolor',[1 1 1],'tag','colormenu','value',length(handles.allnames{1}));


% selectmenu=uicontrol('position',[origin(1)+85 origin(2)+240 180 30],...
%     'style','popupmenu','string',{'Dim infeasible rows';'Select infeasible rows';'Select feasible rows'},'callback',@selectmenu_callback,...
%     'backgroundcolor',[1 1 1],'tag','selectmenu','value',view.filtermode,'enable','off');




upbutton=uicontrol('position',[origin(1)+218 origin(2)+377 35 59],...
    'style','pushbutton','string','^','backgroundcolor',handles.colors.buttonbg,...
    'callback',@upbutton_callback);
downbutton=uicontrol('position',[origin(1)+218 origin(2)+316 35 59],...
    'style','pushbutton','string','v','backgroundcolor',handles.colors.buttonbg,...
    'callback',@downbutton_callback);


showvaluescheck=uicontrol('position',[origin(1)+41 origin(2)+215 150 20],...
    'style','checkbox','string','Show Filter Values','backgroundcolor',handles.colors.tabbg,...
    'fontsize',10,'horizontalalignment','left','callback',@showvalues_callback,'fontweight','bold','fontname','arial',...
    'value',udata.format.showlimits);




usemaxlinescheck=uicontrol('position',[origin(1)+41 origin(2)+185 200 30],...
    'style','checkbox','string','Do not draw more than                lines',...
    'backgroundcolor',handles.colors.tabbg,'value',view.usemaxlines,'callback',@usemaxlinescheck_callback,'tag','usemaxlinescheck');

maxlinesedit=uicontrol('position',[origin(1)+173 origin(2)+191 40 18],...
    'style','edit','string',num2str(view.maxlines),...
    'backgroundcolor',handles.colors.tabbg,'value',view.exclude,'callback',@maxlinesedit_callback,'tag','maxlinesedit');

maxlinesmenu=uicontrol('position',[origin(1)+55 origin(2)+156 180 30],...
    'style','popupmenu','string',{'From the top of data table';'Selected randomly'},'callback',@maxlinesmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','maxlinesmenu','value',1);

if strcmp(view.maxlinesmode,'random');set(maxlinesmenu,'value',2);end


%% buttons

scalebutton=uicontrol('position',[origin(1)+55 origin(2)+95 180 30],...
    'style','pushbutton','string','Scale/Transform Variables','backgroundcolor',handles.colors.buttonbg);
changeunitsbutton=uicontrol('position',[origin(1)+55 origin(2)+60 180 30],...
    'style','pushbutton','string','Change Units','backgroundcolor',handles.colors.buttonbg);
useresponsedata=uicontrol('position',[origin(1)+55 origin(2)+28 200 30],...
    'style','checkbox','string','Limit Menus to Response Variables',...
    'backgroundcolor',handles.colors.tabbg,'value',view.response,'callback',@useresponsedata_callback,'tag','useresponsedata');

excludecheck=uicontrol('position',[origin(1)+5 origin(2)+06 200 30],...
    'style','checkbox','string','Hide constant columns in menus',...
    'backgroundcolor',handles.colors.tabbg,'value',view.exclude,'callback',@excludecheck_callback,'tag','excludecheck');


%% group items and finish up
handles.tabobjects=[datasettext xaxistext colortext datasetmenu xaxismenu colormenu,...
    scalebutton analysistext projecttext analysismenu dataseriestext optionstext useresponsedata,...
    changeunitsbutton excludecheck sorttext upbutton downbutton usemaxlinescheck,...
    maxlinesedit maxlinesmenu showvaluescheck];

handles.activetab='view';

guidata(src,handles);
% update menus to hide excluded vars
viewtabmenufilter(src,[])


%% Exclude hidden variables from menus:


function viewtabmenufilter(src,eventdata)

handles=guidata(src);

%get input info from graph:
udata=get(handles.fakeax(handles.activeindex),'userdata');
view=udata.view; %get view tab structure.

% datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');

xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
constmenu=findobj(handles.tabobjects,'tag','colormenu');
% useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
% excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

%get old strings/values
oldxvalue=view.x;
oldcvalue=view.c;

d=view.dataset;

if view.response && view.exclude && ~all(handles.isresponse{d}==0)
    %filter menus by responses and hide constant cols

    newxstring=handles.allnames{d}(~handles.isconstant{d} & handles.isresponse{d});
    
    newxvalue=find(ismember(newxstring,handles.allnames{d}(oldxvalue)));
    if isempty(newxvalue)
        newxvalue=1;
    end

    newcstring=[newystring,cellstr('<Use Row Colors>')]; 
    if oldcvalue~=0 %something selected
        newcvalue=find(strcmp(handles.allnames{d}(oldcvalue),newcstring));
    else %'none selected'
        newcvalue=numel(newcstring); %last entry in c string (='')
    end
    
    if isempty(newcvalue)
        newcvalue=1;
    end
    
    %store column numbers of available cols. pulldown menus index into these:
    handles.visiblexcols=find((~handles.isconstant{d} & ~handles.isresponse{d}));
    handles.visibleccols=[find(ismember(handles.allnames{d},newcstring)) 0];

    %reset menu strings/values to match displayed data.
    set(xaxismenu,'string',newxstring,'value',newxvalue);
    set(constmenu,'string',newcstring,'value',newcvalue);

%elseif get(useresponsedata,'value') && ~all(handles.isresponse{d}==0)
elseif view.response && ~all(handles.isresponse{d}==0)    
    %filter responses only

    newxstring=handles.allnames{d}(handles.isresponse{d});
    newxvalue=find(ismember(newxstring,handles.allnames{d}(oldxvalue)));

    newcstring=[newxstring,cellstr('<Use Row Colors>')];
    if oldcvalue~=0
        newcvalue=find(strcmp(handles.allnames{d}(oldcvalue),newcstring));
    else %'none selected'
        newcvalue=numel(newcstring);
    end

    %store column numbers of available cols. pulldown menus index into these:
    handles.visiblexcols=find((handles.isresponse{d}));
    handles.visibleccols=[find(ismember(handles.allnames{d},newcstring)) 0];
    
    %reset menu strings/values to match displayed data.
    set(xaxismenu,'string',newxstring,'value',newxvalue);
    set(constmenu,'string',newcstring,'value',newcvalue);

%elseif get(excludecheck,'value') %hide constant cols only
elseif view.exclude %hide constant cols only


    newxstring=handles.allnames{d}(~handles.isconstant{d});
    newxvalue=find(ismember(newxstring,handles.allnames{d}(oldxvalue)));
    newcstring=[newxstring,cellstr('<Use Row Colors>')];
    if oldcvalue~=0
        newcvalue=find(strcmp(handles.allnames{d}(oldcvalue),newcstring));
    else %'none selected'
        newcvalue=numel(newcstring);
    end
    
    %store column numbers of available cols. pulldown menus index into these:
    handles.visiblexcols=find((~handles.isconstant{d}));
    handles.visibleccols=[find(ismember(handles.allnames{d},newcstring)) 0];
    
    %reset menu strings/values to match displayed data.
    set(xaxismenu,'string',newxstring,'value',newxvalue);

    set(constmenu,'string',newcstring,'value',newcvalue);

else %show all variables

    newxstring=handles.allnames{d}(udata.view.varorder);
    newxvalue=find(ismember(newxstring,handles.allnames{d}(oldxvalue)));
    newcstring=[handles.allnames{d},cellstr('<Use Row Colors>')];
    if oldcvalue~=0
        newcvalue=find(strcmp(handles.allnames{d}(oldcvalue),newcstring));
    else %'none selected'
        newcvalue=numel(newcstring);
    end   
    
    %store column numbers of available cols. pulldown menus index into these:
    handles.visiblexcols=1:1:length(handles.allnames{d});
    handles.visibleccols=[find(ismember(handles.allnames{d},newcstring)) 0];

    %reset menu strings/values to match displayed data.
    set(xaxismenu,'string',newxstring,'value',newxvalue,'listboxtop',newxvalue(1));
    
    set(constmenu,'string',newcstring,'value',newcvalue);

end

drawnow %cant update listboxtop til it renders :(
set(xaxismenu,'listboxtop',newxvalue(1));
guidata(src,handles)



%% Here be callbacks:%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function datasetmenu_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
%update other menus to show correct variables

% xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
% yaxismenu=findobj(handles.tabobjects,'tag','yaxismenu');
% colormenu=findobj(handles.tabobjects,'tag','colormenu');
% sizemenu=findobj(handles.tabobjects,'tag','sizemenu');
analysismenu=findobj(handles.tabobjects,'tag','analysismenu');
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

d=get(datasetmenu,'value');

%get initial columns for this data set:
if ~all(handles.isresponse{d}==0)
    x=find((~handles.isconstant{d} & ~handles.isresponse{d}),1);
    y=find((~handles.isconstant{d} & handles.isresponse{d}),1);
    c=0;
    s=0;
else
    x=find((~handles.isconstant{d}),1);
    y=find((~handles.isconstant{d}),1);
    c=0;
    s=0;
end

%set fakeax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.analysis=get(analysismenu,'value');
udata.view.dataset=get(datasetmenu,'value');
udata.view.x=x;
udata.view.y=y;
udata.view.c=c;
udata.view.s=s;
udata.view.response=get(useresponsedata,'value'); %response check box value
udata.view.exclude=get(excludecheck,'value'); %exclude constant check box value

set(handles.fakeax(handles.activeindex),'userdata',udata);
handles.graphdataset(handles.activeindex)=get(src,'value');
guidata(src,handles);

viewtabmenufilter(src,[])

ravecreateparallelcoords(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function analysismenu_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);
handles.graphanalysis(handles.activeindex)=get(src,'value');
guidata(src,handles);

udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.analysis=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreateparallelcoords(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xaxismenu_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);

xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
% yaxismenu=findobj(handles.tabobjects,'tag','yaxismenu');
% colormenu=findobj(handles.tabobjects,'tag','colormenu');
% sizemenu=findobj(handles.tabobjects,'tag','sizemenu');
% analysismenu=findobj(handles.tabobjects,'tag','analysismenu');
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

%if selected ~response, turn off filter response button
if ~handles.isresponse{get(datasetmenu,'value')}(handles.visiblexcols(get(src,'value')))
    set(useresponsedata,'enable','off')  
else
    set(useresponsedata,'enable','on')    
end

%if selected isconstant, turn off filter constant button
if handles.isconstant{get(datasetmenu,'value')}(handles.visiblexcols(get(src,'value')))
    set(excludecheck,'enable','off')  
else
    set(excludecheck,'enable','on')    
end

guidata(src,handles);

%set fakeax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');

% udata.view.analysis=get(analysismenu,'value');
% udata.view.dataset=get(datasetmenu,'value');


udata.view.x=handles.visiblexcols(udata.view.varorder(get(xaxismenu,'value')));



% udata.view.y=handles.visiblexcols(get(yaxismenu,'value'));
% udata.view.c=handles.visibleccols(get(colormenu,'value'));
% udata.view.s=handles.visibleccols(get(sizemenu,'value'));

% udata.view.response=get(useresponsedata,'value'); %response check box value
% udata.view.exclude=get(excludecheck,'value'); %exclude constant check box value


% %reset filter
% udata.view.minfilter=zeros(1,numel(get(xaxismenu,'value')));
% udata.view.maxfilter=ones(1,numel(get(xaxismenu,'value')));
% 

set(handles.fakeax(handles.activeindex),'userdata',udata);

drawnow %unnecessary?
%
ravecreateparallelcoords(src,[]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function colormenu_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
yaxismenu=findobj(handles.tabobjects,'tag','yaxismenu');
colormenu=findobj(handles.tabobjects,'tag','colormenu');
sizemenu=findobj(handles.tabobjects,'tag','sizemenu');
analysismenu=findobj(handles.tabobjects,'tag','analysismenu');
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

%if selected isconstant, turn off filter constant button
if get(src,'value')~=numel(get(src,'string'))
    if any(handles.isconstant{get(datasetmenu,'value')}(handles.visibleccols(get(src,'value'))))
        set(excludecheck,'enable','off')
    else
        set(excludecheck,'enable','on')
    end
end

guidata(src,handles);




%set fakeax userdata
%udata=get(handles.fakeax(handles.activeindex),'userdata');

% udata.view.analysis=get(analysismenu,'value');
% udata.view.dataset=get(datasetmenu,'value');
% udata.view.x=handles.visiblexcols(get(xaxismenu,'value'));
% udata.view.y=handles.visiblexcols(get(yaxismenu,'value'));
udata.view.c=handles.visibleccols(get(colormenu,'value'));
% udata.view.s=handles.visibleccols(get(sizemenu,'value'));
% udata.view.response=get(useresponsedata,'value'); %response check box value
% udata.view.exclude=get(excludecheck,'value'); %exclude constant check box value

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreateparallelcoords(src,[]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function useresponsedata_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);

xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
yaxismenu=findobj(handles.tabobjects,'tag','yaxismenu');
colormenu=findobj(handles.tabobjects,'tag','colormenu');
sizemenu=findobj(handles.tabobjects,'tag','sizemenu');
analysismenu=findobj(handles.tabobjects,'tag','analysismenu');
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

%set fakeax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.analysis=get(analysismenu,'value');
udata.view.dataset=get(datasetmenu,'value');
udata.view.x=handles.visiblexcols(get(xaxismenu,'value'));
udata.view.y=handles.visiblexcols(get(yaxismenu,'value'));
udata.view.c=handles.visibleccols(get(colormenu,'value'));
udata.view.s=handles.visibleccols(get(sizemenu,'value'));
udata.view.response=get(useresponsedata,'value'); %response check box value
udata.view.exclude=get(excludecheck,'value'); %exclude constant check box value

set(handles.fakeax(handles.activeindex),'userdata',udata);

viewtabmenufilter(src,[])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function excludecheck_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);

xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
yaxismenu=findobj(handles.tabobjects,'tag','yaxismenu');
colormenu=findobj(handles.tabobjects,'tag','colormenu');
sizemenu=findobj(handles.tabobjects,'tag','sizemenu');
analysismenu=findobj(handles.tabobjects,'tag','analysismenu');
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

%set fakeax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.analysis=get(analysismenu,'value');
udata.view.dataset=get(datasetmenu,'value');
udata.view.x=handles.visiblexcols(get(xaxismenu,'value'));
udata.view.y=handles.visiblexcols(get(yaxismenu,'value'));
udata.view.c=handles.visibleccols(get(colormenu,'value'));
udata.view.s=handles.visibleccols(get(sizemenu,'value'));
udata.view.response=get(useresponsedata,'value'); %response check box value
udata.view.exclude=get(excludecheck,'value'); %exclude constant check box value

set(handles.fakeax(handles.activeindex),'userdata',udata);

viewtabmenufilter(src,[])




function upbutton_callback(src,ev)
%move all selected variables up one spot. 

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

varorder=udata.view.varorder;
topvar=find(ismember(varorder,udata.view.x),1,'first');

if topvar>2
    othervars=varorder(topvar-1:end);
    newvarorder=[varorder(1:topvar-2) udata.view.x othervars(~ismember(othervars,udata.view.x))];    
else
    othervars=varorder;
    newvarorder=[udata.view.x othervars(~ismember(othervars,udata.view.x))];
end

udata.view.varorder=newvarorder;
set(handles.fakeax(handles.activeindex),'userdata',udata);
viewtabmenufilter(handles.fig,[]);


function downbutton_callback(src,ev)
%move all selected variables down one spot. 

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

varorder=udata.view.varorder;
botvar=find(ismember(varorder,udata.view.x),1,'last');

if botvar<(numel(varorder)-1)
    othervars=varorder(1:botvar+1);
   % newvarorder=[varorder(1:topvar-2) udata.view.x othervars(~ismember(othervars,udata.view.x))];
    newvarorder=[othervars(~ismember(othervars,udata.view.x)) udata.view.x varorder(botvar+2:end)];
    
else
    othervars=varorder;
    newvarorder=[othervars(~ismember(othervars,udata.view.x)) udata.view.x];
end

udata.view.varorder=newvarorder;
set(handles.fakeax(handles.activeindex),'userdata',udata);
viewtabmenufilter(handles.fig,[]);









function selectmenu_callback(src,ev)


%%
function usemaxlinescheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.usemaxlines=get(src,'value');

set(handles.fakeax(handles.activeindex),'userdata',udata)

ravecreateparallelcoords(handles.fig,[]);

%%
function maxlinesedit_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.maxlines=str2double(get(src,'string'));

set(handles.fakeax(handles.activeindex),'userdata',udata)

ravecreateparallelcoords(handles.fig,[]);

%% 
function maxlinesmenu_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

if get(src,'value')==1
    udata.view.maxlinesmode='top';
else
    udata.view.maxlinesmode='random';
end

set(handles.fakeax(handles.activeindex),'userdata',udata)

ravecreateparallelcoords(handles.fig,[]);

%% 
function showvalues_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

texters=findobj(handles.activegraph,'tag','maxtexter','-or','tag','mintexter');
if get(src,'value')
    set(texters,'vis','on')
else
    set(texters,'vis','off')
end

udata.format.showlimits=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);


