function ravecreategoodness(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

raveresizeaxes(src,[],handles.activeindex)


%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;

targetaxes=handles.activegraph;
xcol=udata.view.x;
ycol=udata.view.y;
% pscales=udata.view.pscale;
% gscales=udata.view.gscale;
analysis=udata.view.analysis; %#ok<NASGU>
d=udata.view.dataset; %data set index
graphtype=udata.view.graphs;

%determine fit rows:
if udata.view.usefitcol
    fitrows=logical(handles.alldata{d}(:,udata.view.fitcol));
    if udata.view.invertfitcol
        fitrows=~fitrows;
    end
else %convert range to numbers:
    fitrows=false(size(handles.alldata{d}(:,udata.view.fitcol)));
    strg=udata.view.fitrows;
    strg=strg(strg~=32);
    strg(strg==45)=char(58);
    trues=eval(['[',strg,']']);
    fitrows(trues)=true;
end


if udata.view.useunfitcol
    unfitrows=logical(handles.alldata{d}(:,udata.view.unfitcol));
    if udata.view.invertunfitcol
        unfitrows=~unfitrows;
    end
else %convert range to numbers:
    unfitrows=false(size(handles.alldata{d}(:,udata.view.unfitcol)));
    strg=udata.view.unfitrows;
    strg=strg(strg~=32);
    strg(strg==45)=char(58);
    trues=eval(['[',strg,']']);
    unfitrows(trues)=true;
end


actual= handles.alldata{d}(:,xcol);
predicted= handles.alldata{d}(:,ycol);



%determine precision of predicted
teststring=num2str(actual(1),16);

%precise is the value that if you multiple predicted values by 10^precise
%and then round them, then divide by 10^precise, they'll be at same
%precision as actual value.

if numel(actual)<10;maxg=numel(actual);else maxg=10;end


if udata.view.round
    for g=1:maxg
        teststring=num2str(actual(g),12);
        
        
        if any(teststring==46)
            
            
            dec=find(teststring==46);
            precise(g)=(numel(teststring)-dec);
        else %is a whole number
            zerz=find(teststring~=48,1,'last');
            if isempty(zerz);
                precise(g)=1;
            else
                precise(g) = -numel(teststring)+zerz;
            end
            
        end
        
    end
    precise=max(precise);
    predicted=round(predicted.*(10.^precise))./(10.^precise);
    
end

resid=predicted-actual;



TSS = sum((actual(fitrows)-mean(actual(fitrows))).^2);
RSS = sum((actual(fitrows)-predicted(fitrows)).^2);
R2fit = 1 - RSS/TSS;


rmsefit=sqrt(sum(resid(fitrows).^2)/numel(resid(fitrows)));



TSS = sum((actual(unfitrows)-mean(actual(unfitrows))).^2);
RSS = sum((actual(unfitrows)-predicted(unfitrows)).^2);
R2unfit = 1 - RSS/TSS;

rmseunfit=sqrt(sum(resid(unfitrows).^2)/numel(resid(unfitrows)));



t=1;






%% make the graph.

%how many graphs to draw?

if graphtype<5
    numax=[1 1];
    ymargin=20;
    xmargin=0;
else
    numax=[2 2];
    ymargin=80;
    xmargin=80;
end

%clear existing graph
cla(targetaxes);

delete(handles.subax{handles.activeindex})
delete(handles.subfakeax{handles.activeindex})
delete(handles.subfakerax{handles.activeindex})

%maintain hidden points:
nothidden=logical(handles.isvisible{udata.view.dataset}{udata.view.analysis});

%get data
xdata=handles.alldata{d}(:,xcol);
xnames=handles.allnames{d}(xcol);

ydata=handles.alldata{d}(:,ycol);
ynames=handles.allnames{d}(ycol);

%default color:
% defaultcolor=[0 .5 1];

% Create/find BigAx and make it invisible
% BigAx = newplot(targetaxes);

set(targetaxes,'Visible','off','color','none')


%make overall title:

text(.5,1,['Goodness of fit for ',handles.allnames{d}{xcol}],'parent',targetaxes,'horizontalalignment','center','verticalalignment','bottom',...
    'interpreter',handles.prefs.interpreter,'fontsize',14,'tag','title')





% Create and plot into small axes

ax = zeros(numax);
fakeax = zeros(numax);
fakerax = zeros(numax);
selected= zeros(numax);
%tt=zeros(rows,cols,1);

rows=numax(1);
cols=numax(2);

pos = get(targetaxes,'Position');%+[30 30 -45 -30]; %shrink a little to fit tick labels. (now handled by raveresizeaxes)

width = (pos(3)-xmargin)/cols;
height = (pos(4)-ymargin-30)/rows;

% xlim = zeros([rows cols 2]);
% ylim = zeros([rows cols 2]);
%

%set up grid of axes.
for r=rows:-1:1,
    for c=cols:-1:1,
        
        %         axPos = [pos(1)+(c-1)*(width) pos(2)+(rows-r)*(height) ...
        %             width*(1-space) height*(1-space)];
        axPos = [pos(1)+(c-1)*(width+xmargin) pos(2)+(rows-r)*(height+ymargin) ...
            width height];
        
        fakerax(r,c) = axes('units','pixels','Position',axPos,'parent',handles.wspanel,...
            'hittest','off','gridlinestyle',':','xcolor',[.5 .5 .5],'ycolor',[.5 .5 .5]);
        fakeax(r,c) = axes('units','pixels','Position',axPos,'parent',handles.wspanel,...
            'hittest','off','gridlinestyle','-','color','none','xcolor',[.5 .5 .5],'ycolor',[.5 .5 .5]);
        ax(r,c) = axes('units','pixels','Position',axPos,'parent',handles.wspanel,...
            'visible','on','nextplot','add','box','on','hittest','on','buttondownfcn',@raveselectsubgraph,...
            'userdata',handles.activeindex,'color','none'); %userdata contains the index of the bigax for this subgraph.
        set(ax(r,c),'visible','on','drawmode','normal');
        set([fakerax(r,c) fakeax(r,c)],'units','pixels','visible','on','XColor',[.5 .5 .5],'YColor',[.5 .5 .5]);
        
        %set(ax(r,c),'xlimmode','auto','ylimmode','auto','xgrid','off','ygrid','off')
        %axis(ax(r,c),'tight')
        
        %         xlim(r,c,:) = get(ax(r,c),'xlim');
        %         ylim(r,c,:) = get(ax(r,c),'ylim');
    end
end

%% format graphs
%adjust gridlines to match settings:
if format.grid==1 %no grid
    set(fakerax,'XminorGrid','off','YminorGrid','off')
    set(fakerax,'XGrid','off','YGrid','off')
    set(fakeax,'XGrid','off','YGrid','off')
elseif format.grid==2 %major grid
    set(fakerax,'XGrid','on','YGrid','on')
    set(fakerax,'XminorGrid','off','YminorGrid','off')
    set(fakeax,'XGrid','off','YGrid','off')
elseif format.grid==3; %minor grid
    set(fakerax,'XminorGrid','on','YminorGrid','on')
    set(fakerax,'XGrid','off','YGrid','off')
    set(fakeax,'XGrid','on','YGrid','on')
end

%adjust background and text labels to match settings:
%make background color data matrix:
bgcolordata={[1 1 1;.6 .6 .6;0 0 0]; %white
    [.753 .753 .753;1 1 1;0 0 0]; %light
    [.502 .502 .502;.6 .6 .6;1 1 1]; %dark
    [0 0 0;.3 .3 .3;1 1 1]}; %black

set(fakerax,'color',bgcolordata{format.bg}(1,:));
set(fakerax,'xcolor',bgcolordata{format.bg}(2,:),'ycolor',bgcolordata{format.bg}(2,:));
set(fakeax,'xcolor',bgcolordata{format.bg}(2,:),'ycolor',bgcolordata{format.bg}(2,:));


%% make the actual graphs

if graphtype==5
    graphs=[1 2 3 4];
else
    graphs=graphtype;
end


%format for fillstyle:
fillstyle=format.markerfill;
if fillstyle==1
    fitpvpairs={'markerfacecolor','none','markeredgecolor',udata.format.markerfitcolor};
    unfitpvpairs={'markerfacecolor','none','markeredgecolor',udata.format.markerunfitcolor};
    
elseif fillstyle==2
    fitpvpairs={'markerfacecolor',udata.format.markerfitcolor,'markeredgecolor','none'};
    unfitpvpairs={'markerfacecolor',udata.format.markerunfitcolor,'markeredgecolor','none'};
else %fillstyle=3
    fitpvpairs={'markerfacecolor',udata.format.markerfitcolor,'markeredgecolor',[0 0 0]};
    unfitpvpairs={'markerfacecolor',udata.format.markerunfitcolor,'markeredgecolor',[0 0 0]};
end


flipax=ax'; %need to loop columnwise instead of rowwise

leg1=[];
leg2=[];

if width<400 || height<300
    fontsize=8;
else
    fontsize=10;
end

% NOTE: need to make ravemovegrpah update fontsize.


for g=1:prod(numax);
    thisax=flipax(g);
    switch graphs(g)
        case 1 %actual vs predicted
            %line(0,0,'marker','p','markerfacecolor',[1 0 0],'parent',thisax,'tag','main','markersize',20);
            fit=line(predicted(fitrows),actual(fitrows),'parent',thisax,'linestyle','none','marker',udata.format.fitmarker,...
                'markersize',udata.format.fitmarkersize,fitpvpairs{:});
            unfit=line(predicted(unfitrows),actual(unfitrows),'parent',thisax,'linestyle','none','marker',udata.format.unfitmarker,...
                'markersize',udata.format.unfitmarkersize,unfitpvpairs{:});
            
            set(get(thisax,'xlabel'),'string','Predicted Value')
            set(get(thisax,'ylabel'),'string','Actual Value')
            set(get(thisax,'title'),'string','Actual vs. Predicted Values')
            
            warning('off','MATLAB:legend:IgnoringExtraEntries')
            leg1=legend([fit unfit],{'Fit Data';'Test Data'},'location','northwest');
            warning('on','MATLAB:legend:IgnoringExtraEntries')
            
            xlim=get(thisax,'xlim');
            ylim=get(thisax,'ylim');
            
            set(thisax,'xlim',[min([xlim ylim]) max([xlim ylim])],'ylim',[min([xlim ylim]) max([xlim ylim])])
            
            xlim=get(thisax,'xlim');
            ylim=get(thisax,'ylim');
            
            if any(fitrows) && any(unfitrows)
                Rstring={['R^2_{fit} = ',num2str(R2fit)];['R^2_{test} = ',num2str(R2unfit)]};
            elseif any(fitrows)
                Rstring=['R^2_{fit} = ',num2str(R2fit)];
            elseif any(unfitrows)
                Rstring=['R^2_{test} = ',num2str(R2unfit)];
            else
                Rstring='';
            end
            
            
            
            
            text(xlim(2)-0.05*(xlim(2)-xlim(1)),ylim(1)+0.05*(ylim(2)-ylim(1)),Rstring,'parent',thisax,'interpreter','tex','horizontalalignment','right',...
                'verticalalignment','bottom','backgroundcolor',[1 1 1],'tag','textongraph','edgecolor',[0 0 0],'fontsize',fontsize)
            
            L=line(xlim,ylim,'linestyle','-','color',[0 .5 1],'tag','perfectfitline','buttondownfcn',@lineobject_callback,'parent',thisax);
            linedata.handles=L;
            linedata.delete=0;
            set(L,'userdata',linedata);
            uistack(L,'bottom');
            
            
        case 2 %residual vs predicted
            %line(0,0,'marker','p','markerfacecolor',[1 0 0],'parent',thisax,'tag','main','markersize',20);
            fit=line(predicted(fitrows),resid(fitrows),'parent',thisax,'linestyle','none','marker',udata.format.fitmarker,...
                'markersize',udata.format.fitmarkersize,fitpvpairs{:});
            unfit=line(predicted(unfitrows),resid(unfitrows),'parent',thisax,'linestyle','none','marker',udata.format.unfitmarker,...
                'markersize',udata.format.unfitmarkersize,unfitpvpairs{:});
            
            set(get(thisax,'xlabel'),'string','Predicted Value')
            set(get(thisax,'ylabel'),'string','Residual (Error)')
            set(get(thisax,'title'),'string','Residual vs. Predicted Values')
            
            warning('off','MATLAB:legend:IgnoringExtraEntries')
            leg2=legend([fit unfit],{'Fit Data';'Test Data'},'location','northwest');
            warning('on','MATLAB:legend:IgnoringExtraEntries')
            
            xlim=get(thisax,'xlim');
            ylim=get(thisax,'ylim');
            
            set(thisax,'xlimmode','manual','ylimmode','manual')
            
            if any(fitrows) && any(unfitrows)
                Rstring={['RMSE_{fit} = ',num2str(rmsefit)];['RMSE_{test} = ',num2str(rmseunfit)]};
            elseif any(fitrows)
                Rstring=['RMSE_{fit} = ',num2str(rmsefit)];
            elseif any(unfitrows)
                Rstring=['RMSE_{test} = ',num2str(rmseunfit)];
            else
                Rstring='';
            end
                  
           % text(xlim(2)-0.05*(xlim(2)-xlim(1)),ylim(1)+0.05*(ylim(2)-ylim(1)),Rstring,'parent',thisax,'interpreter','tex','horizontalalignment','right',...
           %     'verticalalignment','bottom','backgroundcolor',[1 1 1],'tag','textongraph','edgecolor',[0 0 0],'fontsize',fontsize)

             text(xlim(2)-0.4*(xlim(2)-xlim(1)),ylim(2)-0.22*(ylim(2)-ylim(1)),Rstring,'parent',thisax,'interpreter','tex','horizontalalignment','right',...
                'verticalalignment','bottom','backgroundcolor',[1 1 1],'tag','textongraph','edgecolor',[0 0 0],'fontsize',fontsize)

            
            
            L=line(xlim,[0 0],'linestyle','-','color',[0 .5 1],'tag','perfectfitline','buttondownfcn',@lineobject_callback,'parent',thisax);
            linedata.handles=L;
            linedata.delete=0;
            set(L,'userdata',linedata);
            uistack(L,'bottom');
            
        case 3 %MFE histogram
            
            
            x=resid(fitrows);
            
            if udata.format.percenterror %express errors as percent instead of absolute
                x=x./actual(fitrows)*100;
                set(get(thisax,'xlabel'),'string','Percent Error')
                pct='%';
            else
                set(get(thisax,'xlabel'),'string','Error')
                pct='';
            end
            
            %nbars=min(udata.view.nbars-1,numel(unique(x))); %dont make more bars than there are unique values.
            nbars=udata.format.nbars-1;
            
            if min(x)==max(x) %x is constant
                edges=[x(1)-1 x(1)+1];
            else
                edges=linspace(min(x),max(x),nbars+1);
            end
            
            count=histc(x,edges);
            
            mainhandle=bar(thisax,edges,count,udata.format.barwidth);
            if udata.format.drawedge
                edgecolor=udata.format.edgecolor;
            else
                edgecolor=udata.format.facecolor;
            end
            
            set(mainhandle,'facecolor',udata.format.facecolor,'hittest','off','tag','main','edgecolor',edgecolor);
            
            
            set(get(thisax,'ylabel'),'string','Count')
            set(get(thisax,'title'),'string','Fit Data Error')
            
            xlim=get(thisax,'xlim');
            ylim=get(thisax,'ylim');
            
            %text(xlim(1)+0.05*(xlim(2)-xlim(1)),ylim(2)-0.05*(ylim(2)-ylim(1)),{['\mu_{MFE}= ',strtrim(sprintf('%-10.5g',mean(x))),pct];'\fontsize{8} ';['\fontsize{',num2str(fontsize),'}\sigma^2_{MFE}= ',strtrim(sprintf('%-10.5g',std(x))),pct]},'parent',thisax,'interpreter','tex','horizontalalignment','left',...
            %    'verticalalignment','top','backgroundcolor',[1 1 1],'tag','textongraph','edgecolor',[0 0 0],'fontsize',fontsize)
            
              text(xlim(2)-0.3*(xlim(2)-xlim(1)),ylim(2)-0.05*(ylim(2)-ylim(1)),{['\mu_{MFE}= ',strtrim(sprintf('%-10.5g',mean(x))),pct];'\fontsize{8} ';['\fontsize{',num2str(fontsize),'}\sigma^2_{MFE}= ',strtrim(sprintf('%-10.5g',std(x))),pct]},'parent',thisax,'interpreter','tex','horizontalalignment','left',...
                'verticalalignment','top','backgroundcolor',[1 1 1],'tag','textongraph','edgecolor',[0 0 0],'fontsize',fontsize)
            
            
            
        case 4 %MRE histogram
            
            x=resid(unfitrows);
            
            if ~isempty(x)
                
                if udata.format.percenterror %express errors as percent instead of absolute
                    x=x./actual(unfitrows)*100;
                    set(get(thisax,'xlabel'),'string','Percent Error')
                    pct=' %';
                else
                    set(get(thisax,'xlabel'),'string','Error')
                    pct='';
                end
                
                %nbars=min(udata.view.nbars-1,numel(unique(x))); %dont make more bars than there are unique values.
                nbars=udata.format.nbars-1;
                
                if min(x)==max(x) %x is constant
                    edges=[x(1)-1 x(1)+1];
                else
                    edges=linspace(min(x),max(x),nbars+1);
                end
                
                count=histc(x,edges);
                
                mainhandle=bar(thisax,edges,count,udata.format.barwidth);
                if udata.format.drawedge
                    edgecolor=udata.format.edgecolor;
                else
                    edgecolor=udata.format.facecolor;
                end
                
                set(mainhandle,'facecolor',udata.format.facecolor,'hittest','off','tag','main','edgecolor',edgecolor);
                
                
                set(get(thisax,'ylabel'),'string','Count')
                set(get(thisax,'title'),'string','Validation Data Error')
                
                xlim=get(thisax,'xlim');
                ylim=get(thisax,'ylim');
                
               % text(xlim(1)+0.05*(xlim(2)-xlim(1)),ylim(2)-0.05*(ylim(2)-ylim(1)),{['\mu_{MRE}= ',strtrim(sprintf('%-10.5g',mean(x))),pct];'\fontsize{8} ';['\fontsize{',num2str(fontsize),'}\sigma^2_{MRE}= ',strtrim(sprintf('%-10.5g',std(x))),pct]},'parent',thisax,'interpreter','tex','horizontalalignment','left',...
               %     'verticalalignment','top','backgroundcolor',[1 1 1],'tag','textongraph','edgecolor',[0 0 0],'fontsize',fontsize)
                
                text(xlim(2)-0.3*(xlim(2)-xlim(1)),ylim(2)-0.05*(ylim(2)-ylim(1)),{['\mu_{MRE}= ',strtrim(sprintf('%-10.5g',mean(x))),pct];'\fontsize{8} ';['\fontsize{',num2str(fontsize),'}\sigma^2_{MRE}= ',strtrim(sprintf('%-10.5g',std(x))),pct]},'parent',thisax,'interpreter','tex','horizontalalignment','left',...
                    'verticalalignment','top','backgroundcolor',[1 1 1],'tag','textongraph','edgecolor',[0 0 0],'fontsize',fontsize)
                
                
            end
            
            
            
    end
    
end

%store legend handles:

LLL=[leg1 leg2];
set(handles.fakerax(handles.activeindex),'userdata',LLL);


%make all axes have same limits
for f=1:prod(numax)
    set(fakerax(f),'xlim',get(ax(f),'xlim'),'ylim',get(ax(f),'ylim'))
    set(fakeax(f),'xlim',get(ax(f),'xlim'),'ylim',get(ax(f),'ylim'))
    %set(targetaxes,'nextplot','add')
end


%% turn off visibility of large axes
set(handles.fakerax(handles.activeindex),'visible','off');
set(handles.fakeax(handles.activeindex),'visible','off');
set(targetaxes,'nextplot','add')


%store handles to the subplot axes in "sub" handles structure:
handles.subax{handles.activeindex}=ax;
handles.subfakeax{handles.activeindex}=fakeax;
handles.subfakerax{handles.activeindex}=fakerax;
handles.statsax{handles.activeindex}=[];

%select the subaxes in position (1,2) to start:
handles.activesubgraph=handles.subax{handles.activeindex}(1,1);


%store handles to the data sets:
%handles.scatterplots.main{handles.activeindex}=h.main;
%handles.scatterplots.text{handles.activeindex}=tt; %the text labels for this graph.

%resize axes so axes labels/ticks fit inside the bounding box.
%targetindex=find(handles.allgraphs==targetaxes);


%texts=findobj(handles.fig,'tag','textongraph');
%set(texts,'backgroundcolor',bgcolordata{format.bg}(1,:));



try %#ok<TRYNC>
    set(tt,'color',bgcolordata{format.bg}(3,:),'backgroundcolor',bgcolordata{format.bg}(1,:));
end

%{

%for upper/lower triangles, hide appropriate axes/data
switch udata.view.layout
    case 'upper'
        hidefilter=logical(tril(ones(numel(xcol))));
        kidz=findall(ax(hidefilter),'-not','tag','axeslabel');
        set(kidz,'visible','off','tag','nevershow')
        set(ax(hidefilter),'visible','off')
        set(fakeax(hidefilter),'visible','off')
        set(fakerax(hidefilter),'visible','off')
    case 'lower'
        hidefilter=logical(triu(ones(numel(xcol))));
        kidz=findall(ax(hidefilter),'-not','tag','axeslabel');
        set(kidz,'visible','off','tag','nevershow')
        set(ax(hidefilter),'visible','off')
        set(fakeax(hidefilter),'visible','off')
        set(fakerax(hidefilter),'visible','off')
end


%}

%update da handles
guidata(src,handles)
%
% set(handles.subax{handles.activeindex},'ticklength',[.015 .025]);
% set(handles.subfakeax{handles.activeindex},'ticklength',[.015 .025]);
% set(handles.subfakerax{handles.activeindex},'ticklength',[.015 .025]);

% %rasterize if colored by points:
%
% if udata.view.c && handles.prefs.alwaysrasterize
%    raverasterfarian(handles.fig,[],handles.activeindex);
% end

%
% for j=1:numel(xcol);
%     ravemaketextticks(ax(end,j),[],xcol(j),[],[],[])
% end
%
%
% for j=1:numel(ycol);
%     ravemaketextticks(ax(j,1),[],[],ycol(j),[],[])
% end


set(fakerax,'xlimmode','manual','ylimmode','manual')
set(fakeax,'xlimmode','manual','ylimmode','manual');
set(ax,'xlimmode','manual','ylimmode','manual');



raveresizeaxes(handles.fig,[],handles.activeindex)















