function ravecreatewate(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;

targetaxes=handles.activegraph;
targetindex=find(handles.allgraphs==targetaxes);

filename=udata.view.filename;
path=udata.view.path;
analysis=udata.view.analysis;
d=udata.view.dataset; %data set index

%quit if no file selected:
if udata.view.invalid || isempty(filename)
    text(.5,.5,'No File Selected','parent',targetaxes,'fontname','arial','fontsize',18,...
        'horizontalalignment','center','verticalalignment','middle','color',[0 0 0]);
    guidata(src,handles)
    raveresizeaxes(src,[],targetindex)
    return
end

numpaths=numel(filename);

%clear existing graph
cla(targetaxes);
cla(handles.fakeax(handles.activeindex));
cla(handles.fakerax(handles.activeindex));
legend(targetaxes,'off');

set(targetaxes,'xlimmode','auto','ylimmode','auto');


%% make the graph. %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%combine path and filenames to get full paths:

if path(end)~='\'
    path=[path,'\'];
end

fullpath=cell(numel(udata.view.filename),1);
for j=1:numpaths
    fullpath{j}=[path,udata.view.filename{j}];
end


%get platform from function call. assume unix if not specified.
[plat loc]=ismember({'unix' 'dos' 'mac'},udata.view.os); %#ok<NASGU>
if plat~=0
    platform=udata.view.os;
else
    platform='unix';
end

%store platform as a number
switch lower(platform)
    case {'unix','linux'}
        pl=1;
    case {'dos','win','windows'}
        pl=2;
    case {'mac','macintosh'}
        pl=3;
end

%open file for to be read

for pathindex=1:numpaths


    %get plot formatting
    linefrmt=udata.format.linecolor(pathindex,:);
    linewidth=udata.format.bold(pathindex)+.5;

    fid=fopen(fullpath{pathindex});
    %--------------------------------------------------------------------------

    %FLOWPATH COMPLEXITY DIALS:
    fdata=zeros(23500,28); %< -- Adjust # of rows
    x=zeros(1,7000); %<-- Adjust # of columns
    y=zeros(1,7000); %<-- Adjust # of columns
    rowstart=zeros(1,700); %<-- Adjust # of columns

    %--------------------------------------------------------------------------

    %figure out how many bytes are in file:
    fseek(fid,0,1);
    nobytes=ftell(fid);

    %rewind to beginning of file
    fseek(fid,0,-1);

    %convert data to column vector:
    bigpimpin=fread(fid,nobytes);

    %convert data to ascii text
    %bigreadable=char(bigpimpin); %<-- comment this out in final version

    %initialize position counters
    r=1;
    c=1;

    %Get rid of header data.
    vectorstart=8837; %<-- # of bytes in header of ps file. Doesnt need to be exact. Just be close.
    mediumpimpin=bigpimpin(vectorstart:end); %trim off header

    %Determine where vector data ends. This cuts off file where the Text
    %(weights and stuff) begins. This is at the fifth "/" in the body of the ps
    %file. Also cuts off everything before the fourth "/", which is the WATE
    %axes.
    slashes=find(mediumpimpin==47,5,'first'); %returns indices of slashes
    lilpimpin=mediumpimpin(slashes(4)+101:slashes(5)); %trim off footer.

    %convert from vector to array, with each row containing one "word"
    for j=1:numel(lilpimpin);
        %if current character is a space or new line (new line in dos=13&10, in unix=10)
        if lilpimpin(j)==32 || lilpimpin(j)==10
            r=r+1; %go to next row
            c=1; %go back to first column
            if pl==2 && lilpimpin(j)==10 %If DOS/Win
                j=j+1; %#ok<FXSET> %skip additional row
            end
        else
            fdata(r,c)=lilpimpin(j);
            c=c+1;
        end
    end

    r=1;
    q=2;
    mlast=0;

    %Check if row contains a number.
    for i=1:size(fdata,1)-2 %need to cut off last 2 words for this to work

        %LINETOS
        %---------------------------------------------------------------%
        if fdata(i+2,1)==76 %if theres an L ahead
            if mlast==1
                pathstart(1)=x(r);
                x(r+1)=eval(char(fdata(i,:)));
            else
                x(r)=eval(char(fdata(i,:)));
            end

        end

        if fdata(i+1,1)==76 %if theres an L ahead
            if mlast==1
                pathstart(2)=y(r);
                y(r+1)=eval(char(fdata(i,:)));
                rowstart(q)=r;
                r=r+2;
                mlast=0;
            else
                y(r)=eval(char(fdata(i,:)));
                r=r+1;
                mlast=0;
            end


        end
        %---------------------------------------------------------------%

        %MOVETOS
        %---------------------------------------------------------------%
        if fdata(i,1)==77 && mlast~=1 %if this is an M and its not more than one M in a row
            %rowstop(q)=r; %mark previous row as where to stop a plot
            x(r)=eval(char(fdata(i-2,:)));
            y(r)=eval(char(fdata(i-1,:)));
            mlast=1;
            q=q+1;
        end
        %---------------------------------------------------------------%

        %CLOSEPATHS
        %---------------------------------------------------------------%
        if fdata(i,1)==67 %if this is an C as in CP
            x(r)=pathstart(1);
            y(r)=pathstart(2);
            r=r+1;
        end
        %---------------------------------------------------------------%

    end

    rowstart(q+1)=r;

    %transpose for easier viewing. These may be commented out to save time.
    %     x=x';
    %     y=y';

    %wait=1;
    %SCALE ENGINE TO FULL SIZE --> APPROXIMATE

    %move origin to tip of spinner
    x=x-x(rowstart(11));
    %%move y origin to centerline
    y=y-y(rowstart(4)-1);

    %scale to full size
    scale=(140--60)/(9.388-0.5);
    x=x.*scale;
    y=y.*scale;


    if udata.view.newnose %replace round spinner with pointy spinner.
        %find spinner segments: minimum y coordinate with negative x coordinate.

        %find start point of first spinner:
        spinstartx=x(rowstart(3));
        spinstarty=y(rowstart(3));
        spinslope=(y(rowstart(3)+1)-y(rowstart(3)))/(x(rowstart(3)+1)-x(rowstart(3)));
        spinendx=-spinstarty/spinslope;
        spinendy=0;

        line([spinstartx spinendx],[spinstarty spinendy],'color',linefrmt,'parent',targetaxes,...
            'userdata',pathindex,'linewidth',linewidth);
        line([spinstartx spinendx],[-spinstarty -spinendy],'color',linefrmt,'parent',targetaxes,...
            'userdata',pathindex,'linewidth',linewidth);


    else
        %draw actual spinner:
        line(x(rowstart(3):rowstart(3+1)-1),y(rowstart(3):rowstart(3+1)-1),'color',linefrmt,'parent',targetaxes,...
            'userdata',pathindex,'linewidth',linewidth);
        line(x(rowstart(4):rowstart(4+1)-1),y(rowstart(4):rowstart(4+1)-1),'color',linefrmt,'parent',targetaxes,...
            'userdata',pathindex,'linewidth',linewidth);

    end


    %PLOT RESULTS




    for k=5:q %for each line segment
        line(x(rowstart(k):rowstart(k+1)-1),y(rowstart(k):rowstart(k+1)-1),'Color',linefrmt,'parent',targetaxes,...
            'userdata',pathindex,'linewidth',linewidth);
    end

    fclose(fid);

end

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%copy to fakeaxes for proper axis scaling. lame, i know:
copyobj(get(targetaxes,'children'),handles.fakeax(handles.activeindex))
copyobj(get(targetaxes,'children'),handles.fakerax(handles.activeindex))
axis(targetaxes,'equal');
axis(handles.fakeax(handles.activeindex),'equal');
axis(handles.fakerax(handles.activeindex),'equal');
xl=get(targetaxes,'xlim');
yl=get(targetaxes,'ylim');
set(targetaxes,'xlim',[-40 1.05*xl(2)],'ylim',[-1.05*yl(2) 1.05*yl(2)]);
set(handles.fakeax(handles.activeindex),'xlim',[-40 1.05*xl(2)],'ylim',[-1.05*yl(2) 1.05*yl(2)]);
set(handles.fakerax(handles.activeindex),'xlim',[-40 1.05*xl(2)],'ylim',[-1.05*yl(2) 1.05*yl(2)]);


set(get(handles.fakeax(handles.activeindex),'children'),'linewidth',1)
set(get(handles.fakerax(handles.activeindex),'children'),'linewidth',1)


%adjust gridlines to match settings:
if format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on')
end

%update da handles
guidata(src,handles)

raveresizeaxes(src,[],targetindex)
















