function ravecreatetableallpoints(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;
activetype=get(handles.activegraph,'userdata');
targetaxes=handles.activegraph;

maxpos=udata.view.maxpos;


rownames=udata.view.rownames;
colnames=udata.view.colnames;
colwidths=udata.view.colwidths;
rowheight=udata.view.rowheight;
data=udata.view.data;
headerheight=udata.view.headerheight;
headerwidth=udata.view.headerwidth;

p=handles.fakerax(handles.activeindex); %the panel
pos=get(p,'position');

numrows=size(data,1);
numcols=size(data,2);
%note, numrows/numcols refer to the main data table only, headers may have different number of
%elements.

nudge=3; %number of pixels occupied by border of uipanel.. avoid drawing into these.

%clear existing table
delete(get(handles.activegraph,'children'));
delete(get(handles.fakeax(handles.activeindex),'children'));
delete(handles.subax{handles.activeindex}); %column/row headers
delete(findobj(handles.fakerax(handles.activeindex),'tag','hscroll','-or','tag','vscroll'));

%convert data into a cell array of strings:
data=mat2cell(data,ones(size(data,1),1),ones(1,size(data,2)));
for r=1:numrows
    for c=1:numcols
        data{r,c}=sprintf('%-12.5g',data{r,c});
    end
end

%make column header image:
if ~isempty(colnames)
    ch=ones(1,sum(colwidths),3);
    ch(:,:,1)=format.colheadercolor(1);
    ch(:,:,2)=format.colheadercolor(2);
    ch(:,:,3)=format.colheadercolor(3);

    %color column gridlines:
    collines=[1 cumsum(colwidths)];
    ch(:,collines,1)=format.gridlinecolor(1);
    ch(:,collines,2)=format.gridlinecolor(2);
    ch(:,collines,3)=format.gridlinecolor(3);

    chax=axes('parent',p,'units','pixels','position',[headerwidth pos(4)-headerheight sum(colwidths) headerheight],'tag','colheader');
    image(ch,'parent',chax)
    for c=1:numel(colnames)
        text(collines(c)+3,1,colnames{c},'parent',chax);
    end

    set(chax,'visible','off')
    
    line([0.5 sum(colwidths)+.5],[1.5 1.5],'color',udata.format.gridlinecolor,'parent',chax);

else
    chax=[];
end

%make row header image:
if ~isempty(rownames)
    rh=ones(rowheight*numrows,1,3);
    rh(:,:,1)=format.rowheadercolor(1);
    rh(:,:,2)=format.rowheadercolor(2);
    rh(:,:,3)=format.rowheadercolor(3);

    %color row gridlines:
    rowlines=[1 rowheight:rowheight:numrows*rowheight];
    rh(rowlines,:,1)=format.gridlinecolor(1);
    rh(rowlines,:,2)=format.gridlinecolor(2);
    rh(rowlines,:,3)=format.gridlinecolor(3);

    rhax=axes('parent',p,'units','pixels','position',[0 pos(4)-size(rh,1)-headerheight headerwidth numrows*rowheight],...
        'tag','rowheader','xlim',[.5 headerwidth+.5]);
    image(rh,'parent',rhax)
    %set(rhax,'xlim',[.5 headerwidth+.5]) %this must come after image call.
    for r=1:numel(rownames)
        text(.5+3/headerwidth,r*rowheight,rownames{r},'parent',rhax,'verticalalignment','bottom')
    end
    set(rhax,'visible','off')
    
    line([1.5 1.5],[.5 rowheight*numrows+.5],'parent',rhax,'color',udata.format.gridlinecolor);

    %rowoffset=colwidths(1);
    %colwidths=colwidths(2:end); %first col is the row headers, don't duplicate it in actual table.
else
    rhax=[];
    rowoffset=0;
end


handles.subax{handles.activeindex}=[chax rhax];

if ~isempty(colnames) &&  ~isempty(rownames)
    %make a little block in upper corner to scroll both at once:
    crapax=axes('parent',p,'units','pixels','position',[0 pos(4)-headerheight headerwidth headerheight],'color',[1 1 1]);
    image(ones(1,1,3)*.95,'parent',crapax);
    set(crapax,'visible','off');
    line([.5 1.5],[1.5 1.5],'parent',crapax,'color',udata.format.gridlinecolor);
    line([.5 .5],[.5 1.5],'parent',crapax,'color',udata.format.gridlinecolor);
end


%% make main table image:

%t=ones(numrows*rowheight,sum(colwidths),3);
t=ones(numrows*rowheight,1,3);

% %color vertical grid lines:
% collines=[1 cumsum(colwidths)];
% t(:,collines,1)=0;
% t(:,collines,2)=0;
% t(:,collines,3)=0;

%color horizontal grid lines:
rowlines=[1 rowheight:rowheight:(rowheight*numrows)];
t(rowlines,:,1)=0;
t(rowlines,:,2)=0;
t(rowlines,:,3)=0;

%display:
set(handles.fakeax(handles.activeindex),'position',[headerwidth pos(4)-size(t,1)-headerheight sum(colwidths) size(t,1)],'color',[1 0 1]);


tax=handles.fakeax(handles.activeindex);

image(t,'parent',handles.fakeax(handles.activeindex));
set(handles.fakeax(handles.activeindex),'visible','off');
tpos=get(handles.fakeax(handles.activeindex),'position');

%tt=1;

%% make text for table:
texthandles=zeros(size(data));

collines=[1 cumsum(colwidths)];
rowlines=rowheight:rowheight:(rowheight*numrows);
% 
rowcoords=rowlines-rowheight/2+1;
for r=1:numrows
    for c=1:numcols
        texthandles(r,c)=text(collines(c)+3,-rowcoords(r),data{r,c},'parent',handles.activegraph,'erasemode','normal');
    end
end

set(handles.activegraph,'position',get(handles.fakeax(handles.activeindex),'position'),'color','none');
ylim=get(handles.fakeax(handles.activeindex),'ylim');
set(handles.activegraph,'ylim',[-ylim(2) -ylim(1)],'visible','off','xlim',[.5 sum(colwidths)+.5]);

%make vertical gridlines:
for cc=1:numel(collines)
    line([collines(cc) collines(cc)],[-ylim(2) -ylim(1)],'color',udata.format.gridlinecolor,'tag','gridlines','parent',handles.activegraph);
end


dax=handles.activegraph;


%hide invisible text:
firstvisiblerow=0;
lastvisiblerow=ceil((pos(4)-headerheight)/rowheight);
firstvisiblecol=0;
lastvisiblecol=find([0 cumsum(colwidths)]>pos(3)-headerwidth,1);

set(texthandles(lastvisiblerow+3:end,:),'visible','off');
set(texthandles(:,lastvisiblecol+1:end),'visible','off');

if udata.view.tight && tpos(1)+tpos(3)<maxpos(3) && tpos(4)+headerheight<maxpos(4)
    %table is smaller than the uipanel, so shrink uipanel to just hold table.
    set(p,'position',[pos(1) pos(2) tpos(1)+tpos(3) tpos(4)]);
    pos=get(p,'position');

    set(chax,'position',[0 pos(4)-headerheight sum(colwidths) headerheight]);
    set(rhax,'position',[0 pos(4)-size(rh,1)-nudge colwidths(1) numrows*rowheight]);
    set(tax,'position',[headerwidth pos(4)-size(t,1)-nudge tpos(3) tpos(4)]);
    set(dax,'position',[headerwidth pos(4)-size(t,1)-nudge tpos(3) tpos(4)]);

end




%% make scrollbars:

%image command resets userdata:
udata.view.texthandles=texthandles;
set(handles.fakeax(handles.activeindex),'userdata',udata);
set(handles.activegraph,'userdata',activetype);
set(rhax,'tag','rowheader');
set(chax,'tag','colheader');


% vstep=max(pos(4)/size(t,1)-pos(4),0.0001);
% hstep=max(pos(3)/size(t,2)-pos(3),0.0001);
hstep=(pos(3)-headerwidth);
vstep=(pos(4)-headerheight);

% hscroll=uicontrol('parent',p,'units','pixels','style','slider','position',[-1 -1 pos(3)-20+4 16],...
%     'callback',{@horztablescroll,handles.fig},'min',0,'max',max(size(t,2)-pos(3)+16,1),...
%     'sliderstep',[.05 hstep],'value',0);


%NOTE: SINCE I WROTE THIS FUNCTION I CHANGED SYNTAX SO CALLBACKS OF THESE
%SCROLLBARS NOW GET TARGETAXES AS FIRST INPUT INSTEAD OF HANDLES.FIG. i
%HAVE NOT YET TESTED THESE FUNCTIONS WITH THIS CHANGE. 


hscroll=drawhscroll(handles.wspanel,targetaxes,[pos(1) pos(2)-16 pos(3)-1 17],0,tpos(3),hstep,0,@horztablescroll,1);


% if size(t,2)<=pos(3)
%     set(hscroll,'visible','off')
% end

vscroll=drawvscroll(handles.wspanel,targetaxes,[pos(1)+pos(3)-1 pos(2)+1 17 pos(4)-1],0,tpos(4),vstep,tpos(4),@verttablescroll,1);

%vscroll=uicontrol('parent',p,'units','pixels','style','slider','position',[pos(3)-17 15 16 pos(4)-22+5],...
%    'callback',{@verttablescroll,handles.fig},'min',-15,'max',max(size(t,1)-pos(4),0),...
%    'sliderstep',[.05 vstep],'value',size(t,1)-pos(4));


% if size(t,1)<=pos(4)
%     set(vscroll,'visible','off')
% end

%crap=uicontrol('parent',p,'style','pushbutton','units','pixels','position',[pos(3)-17 -1 16 16],...
%    'backgroundcolor',[.92 .92 .92],'enable','inactive','tag','bottomblocker');



udata.view.vscroll=vscroll;
udata.view.hscroll=hscroll;
udata.view.rowheader=rhax;
udata.view.colheader=chax;


set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.allgraphs(handles.activeindex),'userdata','table');

guidata(handles.fig,handles);





