function ravemaintablechangedcallback(jsrc,jev,src)
%tracks changes to the main table so they can be undone. Also handles
%re-evaluating functions when you change an independent variable.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);


if ~handles.maintable.ignoreupdates 
        
    %make rave table change event:
    evt.d = handles.maintable.d;
    evt.a = handles.maintable.a;
    try %certain updates cause an error here, but we don't care about undoing those kinds of updates anyway
        evt.row=jev.getFirstRow:jev.getLastRow; %these are MODEL indices
    catch %#ok<CTCH>
        return
    end
    
    evt.col=jev.getColumn; %model index
    
    %only track changes to independent variables since changes to
    %depenedent variables must have been caused as a direct result of a
    %change to an independent variable, tracking both would be redundant.
    if handles.isfunction{evt.d}(evt.col+1)
        return
    end
       
    evt.oldvalue=handles.alldata{evt.d}(evt.row+1,evt.col+1);
    evt.oldvalue=num2cell(evt.oldvalue);
    evt.oldvalue=cellfun(@num2str,evt.oldvalue,'UniformOutput',false);
    
    
    evt.newvalue=cell(numel(evt.row),1);
    evt.updatetype=jev.getType;
    for r=1:numel(evt.row)
        evt.newvalue{r}=handles.maintable.jtable.getValueAt(evt.row(r),evt.col);
    end
    
    %check that this is a legal change: (i.e. new data matches column type)
    
    
    
    %store event:
    handles.maintable.undos(2:50)=handles.maintable.undos(1:49);
    handles.maintable.undos{1} = evt;
    handles.maintable.redos=cell(50,1); %clear all redos
    
    %turn undo button on:
    undobutton=findobj(handles.fig,'tag','undobutton');
    set(undobutton,'enable','on');
    
    %update data set:
    handles.alldata{evt.d}(evt.row+1,evt.col+1)=str2double(evt.newvalue);
    handles.datasetupdatespending{evt.d}=union(handles.datasetupdatespending{evt.d},'allrows');
    guidata(handles.fig,handles);
    
    %% recalculate funtions:
    
    %deteremine what columns need to be recalculated:
    recalc=evt.col+1;
    for c=1:size(handles.alldata{evt.d},2)
        if any(ismember(recalc,handles.funvars{evt.d}{c}))
            recalc(end+1)=c; %#ok<AGROW>
        end
    end
    
    recalc=setdiff(recalc,evt.col+1);
    
    if isempty(recalc)
        return
    end
    
    
    newvalues=raveevaluateregression(src,evt.d,recalc,handles.alldata{evt.d}(evt.row+1,:));
    oldvalues=handles.alldata{evt.d}(evt.row+1,recalc);
    handles.alldata{evt.d}(evt.row+1,recalc)=newvalues;
    
    %update tablemodel:
    handles.maintable.ignoreupdates=1;
    guidata(handles.fig,handles);
    
    for r=1:numel(evt.row)
        for c=1:numel(recalc)
            handles.maintable.models{evt.d}.setValueAt(java.lang.String(num2str(newvalues(r,c))),evt.row(r),recalc(c)-1);
        end
    end
    
    handles.maintable.ignoreupdates=0;
    
    %convert newvalues to cell strings:
    newvalues=num2cell(newvalues);
    newvalues=cellfun(@num2str,newvalues,'UniformOutput',false);
    
    oldvalues=num2cell(oldvalues);
    oldvalues=cellfun(@num2str,oldvalues,'UniformOutput',false);
    
    %update undo eventdata to account for all columns that changed:
    evt.col=[jev.getColumn recalc-1]; %model index
    evt.oldvalue=[evt.oldvalue oldvalues];
    evt.newvalue=[evt.newvalue newvalues];
    
    handles.maintable.undos{1} = evt;
    
    
    %save updates:
    guidata(handles.fig,handles);
    
end


