function ravemultivariatemontecarlo(src,ev,settings)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

%settings is a structure with the following fields:
% .method ='multivariatemontecarlo';
% .d = dataset to use
% .a = analysis to use
% .x = list of variables to use as objectives
% .activeconstraints = index of constraints to use
% .startbutton = handle of startbutton
% .pop = total size of monte carlo sample
% .batchsize = max number of points to run at once

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

d=settings.d;
a=settings.a;
x=settings.x;
pop=settings.pop;
batch=settings.batchsize;

set(settings.startbutton,'enable','off')
set(handles.infobar,'string','Finding Pareto efficient points. Please wait...')
drawnow;

%determine scaling factors
ranges=[handles.minlimit{d}{a}; handles.maxlimit{d}{a}];
adderz=handles.minlimit{d}{a};
multiplierz=handles.maxlimit{d}{a}-handles.minlimit{d}{a};

paretotarget=settings.targets;%[str2double(get(targ1edit,'string')) str2double(get(targ2edit,'string'))];
paretotarget=-paretotarget;

%constraintlist=findobj(handles.tabobjects,'tag','constraintlist');
handles.temp.activeconstraints=settings.activeconstraints; %get(constraintlist,'value')-1;
handles.temp.activeconstraints=handles.temp.activeconstraints(logical(handles.temp.activeconstraints)); %remove 0 from list.


M=numel(settings.x); %number of objectives
%get variables for these functions:

indexx=handles.funvars{d}{x(1)};
for k=2:M
    indexx=union(indexx,handles.funvars{d}{x(k)});
end

nvars=numel(indexx);%(number of input vars) not the same as size(d,2)

adderz=adderz(indexx);
multiplierz=multiplierz(indexx);

%determine total number of batches:
nbatches=ceil(pop./batch);

batches=repmat(batch,nbatches,1);
if rem(pop,batch)~=0
    batches(end)=rem(pop,batch);
end

paretodesigns=[];

colorz=hsv(10);

for g=1:nbatches
    
    
    %make random samples:
    data=rand(batches(g),nvars);
    data=bsxfun(@times,data,multiplierz);
    data=bsxfun(@plus,data,adderz);
    %evaluate objectives:
    objective=zeros(batches(g),M);
    
    prefs=handles.preferences{d};
    targets=handles.targetvalues{d};
    
    
   
    objective=raveevaluateregression(src,settings.d,settings.x,data);
    
    
    
    %% Reformat data such that a larger number is always best.
    for j=1:M
        if paretotarget(j)==inf
            objective(:,j)=objective(:,j); %max is best
        elseif paretotarget(j)==-inf
            objective(:,j)=-objective(:,j); %min is best
        elseif ~isnan(paretotarget(j))
            objective(:,j)=-abs(objective(:,j)-paretotarget(j)); %nominal is best
        end
    end
    
    
    %% evaluate constraints:
    
    
    cviolation=zeros(length(objective(:,1)),numel(handles.temp.activeconstraints));
    if isempty(handles.temp.activeconstraints)
        cviolation=zeros(length(objective(:,1)),1);
        
    else
        
        
        for c=1:numel(handles.temp.activeconstraints)
            thisconstraint=handles.temp.activeconstraints(c);
            ignore=0;
            cviolation(:,c)=raveevaluateconstraint(handles.fig,thisconstraint,Preal,ignore);
            
        end
        
        cviolation(cviolation<0)=0; %set feasible points to 0 violation
        scalers=handles.constraintscalers{settings.d}{settings.a}(handles.temp.activeconstraints);
        cviolation=bsxfun(@rdivide,cviolation,scalers);
        cviolation=sum(cviolation,2);
        
    end
    isfeasible=~cviolation;
    
    %do pareto:
    if any(isfeasible)
        feasibleobj=objective(isfeasible,:);
        isnondominated=paretofront(feasibleobj);
    else
        isnondominated=false(size(objective,1),1);
    end
    
   % line(-objective(:,1),-objective(:,2),'linestyle','none','marker','+','color',colorz(g,:),'parent',handles.activegraph);
    paretodesigns=[paretodesigns;data(isnondominated,:)];
   % line(-objective(isnondominated,1),-objective(isnondominated,2),'linestyle','none','marker','+','color',[0 0 0],'parent',handles.activegraph);
    
end







%do final filter:
data=paretodesigns;
  %evaluate objectives:
  
    objective=raveevaluateregression(src,settings.d,settings.x,data);

    
    
    %% Reformat data such that a larger number is always best.
    for j=1:M
        if paretotarget(j)==inf
            objective(:,j)=objective(:,j); %max is best
        elseif paretotarget(j)==-inf
            objective(:,j)=-objective(:,j); %min is best
        elseif ~isnan(paretotarget(j))
            objective(:,j)=-abs(objective(:,j)-paretotarget(j)); %nominal is best
        end
    end
    
    
    %% evaluate constraints:
    
    
    cviolation=zeros(length(objective(:,1)),numel(handles.temp.activeconstraints));
    if isempty(handles.temp.activeconstraints)
        cviolation=zeros(length(objective(:,1)),1);
        
    else
        
        
        for c=1:numel(handles.temp.activeconstraints)
            thisconstraint=handles.temp.activeconstraints(c);
            ignore=0;
            cviolation(:,c)=raveevaluateconstraint(handles.fig,thisconstraint,Preal,ignore);
            
        end
        
        cviolation(cviolation<0)=0; %set feasible points to 0 violation
        scalers=handles.constraintscalers{settings.d}{settings.a}(handles.temp.activeconstraints);
        cviolation=bsxfun(@rdivide,cviolation,scalers);
        cviolation=sum(cviolation,2);
        
    end
    isfeasible=~cviolation;
    
    %do pareto:
    if any(isfeasible)
        feasibleobj=objective(isfeasible,:);
        isnondominated=paretofront(feasibleobj);
    else
        isnondominated=false(size(objective,1),1);
    end
    
    
paretodesigns=data(isnondominated,:);

data=paretodesigns;
%evaluate objectives:

    objective=raveevaluateregression(src,settings.d,settings.x,data);



[sortedpop,index]=sortrows(objective,1:size(objective,2));

line(objective(:,1),objective(:,2),'parent',handles.activegraph,'linestyle','none','color',[1 0 0],'marker','+')

%store results:
handles.temp.finalpop=paretodesigns(index,:);

set(settings.startbutton,'enable','on')

set(handles.infobar,'string','Your optimizer has finished.');

%ravefinish;

guidata(handles.fig,handles);





