function state = raveanimategads(options, state, flag)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
settings=get(0,'userdata');
src=settings.src;
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

%delete retarded pop up window that appears:
handles.temp.crapwindow=gcf;
set(gcf,'visible','off')
guidata(src,handles);

switch settings.method
    case {'ga','gamultiobj'}
        pop=state.Population;
    case {'simulatedannealing','simulannealbnd'}
        pop=[state.x;state.bestx];
    case 'patternsearch'
        state=options;
        pop=state.x;
        
end
plotvars=settings.plotvars;
N=size(pop,1); %population size

%distribute variables to correct columns:
indexx=settings.funvars;
%indata=zeros(N,max(indexx));

indata=repmat(handles.xdials{udata.view.dataset}{udata.view.analysis},N,1); %need this in case objective fun and plot fun's dont have same input variable set.
indata(:,indexx)=pop;

indata(:,indexx)=pop;


objective=raveevaluateregression(src,settings.d,plotvars,indata);


cviolation=zeros(N,1);

if strcmp(settings.method,'gamultiobj')
    rank=nonDominatedRank(state.Score);
end

switch settings.method
    case 'ga' %MULTIOBJECTIVE METHODS, USE RANK
        colorz=[1 0 0];
        delete(findobj(handles.activegraph,'tag','pareto'))
        
        timestr=sprintf('%6.0f',cputime-state.StartTime);
        lastgoodgen=sprintf('%4.0f',state.Generation-state.LastImprovement);
        
        set(handles.infobar,'string',['Gens: \fontname{monospace} ',sprintf('%4.0f',state.Generation),'\fontname{',handles.prefs.fonts.tab,'}   Elapsed Time: ' timestr 's     Gens since improvement:' lastgoodgen])
        
        switch handles.activetype
            case 'scatter'
                %NOTE: SHOULD REALLY BE SORTING ACCORDING TO OPTIMIZED
                %VARIABLES, NOT PLOTTED VARS. this method can cause noise
                %when plot vars ~= objectives.
                plotdata=sortrows(objective(cviolation==0,:));
                L=line(plotdata(:,1),plotdata(:,2),'marker','none','marker','o','parent',handles.activegraph,...
                    'tag','pareto','linewidth',1,'markeredgecolor','none','markerfacecolor',[1 0 0],'markersize',5,'buttondownfcn',@lineobject_callback,'linestyle','none');
                userdata.handles=L;
                userdata.delete=1;
                set(L,'userdata',userdata); %for use in lineobjectcallback
                
                try %#ok<TRYNC>
                    L=line(plotdata(state.Score==state.Best(end),1),plotdata(state.Score==state.Best(end),2),'marker','none','marker','o','parent',handles.activegraph,...
                        'tag','pareto','linewidth',1,'markeredgecolor',[0 0 0],'markerfacecolor',[0 1 0],'markersize',5,'buttondownfcn',@lineobject_callback,'linestyle','none');
                    userdata.handles=L;
                    userdata.delete=1;
                    set(L,'userdata',userdata);
                end
                
                %%plot infeasible points:
                %plotdata=objective(cviolation~=0,:);
                %L=line(plotdata(:,1),plotdata(:,2),'marker','+','linestyle','none','parent',handles.activegraph,...
                %    'tag','pareto','linewidth',1,'color',[0 0 0],'markersize',2,'buttondownfcn',@lineobject_callback);
                %userdata.handles=L;
                %userdata.delete=1;
                %set(L,'userdata',userdata); %for use in lineobjectcallback
                
            case 'scatter3D'
                
                for q=max(rank):-1:1
                    %plotdata=objective(rank==q & cviolation==0,:);
                    
                    if numel(settings.x)==2 %draw line instead of points;
                        linestyle={'linestyle','-','marker','none'};
                        plotdata=sortrows(objective(rank==q & cviolation==0,:));
                    else
                        linestyle={'linestyle','none','marker','+'};
                        plotdata=objective(rank==q & cviolation==0,:);
                    end
                    
                    L=line(plotdata(:,1),plotdata(:,2),plotdata(:,3),'parent',handles.activegraph,...
                        'tag','pareto','linewidth',1,'color',colorz(q,:),'markersize',2,'buttondownfcn',@lineobject_callback,linestyle{:});
                    userdata.handles=L;
                    userdata.delete=1;
                    set(L,'userdata',userdata); %for use in lineobjectcallback
                    
                end
                
                %plot infeasible points:
                plotdata=objective(cviolation~=0,:);
                L=line(plotdata(:,1),plotdata(:,2),plotdata(:,3),'marker','+','linestyle','none','parent',handles.activegraph,...
                    'tag','pareto','linewidth',1,'color',[0 0 0],'markersize',2,'buttondownfcn',@lineobject_callback);
                userdata.handles=L;
                userdata.delete=1;
                set(L,'userdata',userdata); %for use in lineobjectcallback
        end
        
    case 'gamultiobj' %MULTIOBJECTIVE METHODS, USE RANK
        colorz=hsv(8);
        
        if max(rank)>8
            colorz=[colorz;repmat(colorz(end,:),(max(rank)-8),1)];
        end
        delete(findobj(handles.activegraph,'tag','pareto'))
        
        timestr=sprintf('%6.0f',cputime-state.StartTime);
        set(handles.infobar,'string',['Gens: \fontname{monospace} ',sprintf('%4.0f',state.Generation),'\fontname{',handles.prefs.fonts.tab,'} Generations Completed.    Elapsed Time: ' timestr 's'])
        
        switch handles.activetype
            case 'scatter'
                %NOTE: SHOULD REALLY BE SORTING ACCORDING TO OPTIMIZED
                %VARIABLES, NOT PLOTTED VARS. this method can cause noise
                %when plot vars ~= objectives.
                for q=max(rank):-1:2
                    plotdata=sortrows(objective(rank==q & cviolation==0,:));
                    L=line(plotdata(:,1),plotdata(:,2),'marker','none','marker','+','parent',handles.activegraph,...
                        'tag','pareto','linewidth',1,'color',colorz(q,:),'markersize',2,'buttondownfcn',@lineobject_callback,'linestyle','none');
                    userdata.handles=L;
                    userdata.delete=1;
                    set(L,'userdata',userdata); %for use in lineobjectcallback
                end
                plotdata=sortrows(objective(rank==1 & cviolation==0,:));
                L=line(plotdata(:,1),plotdata(:,2),'marker','none','marker','+','parent',handles.activegraph,...
                    'tag','pareto','linewidth',1,'color',colorz(1,:),'markersize',2,'buttondownfcn',@lineobject_callback);
                userdata.handles=L;
                userdata.delete=1;
                set(L,'userdata',userdata); %for use in lineobjectcallback
                
                %%plot infeasible points:
                % plotdata=objective(cviolation~=0,:);
                %L=line(plotdata(:,1),plotdata(:,2),'marker','+','linestyle','none','parent',handles.activegraph,...
                %    'tag','pareto','linewidth',1,'color',[0 0 0],'markersize',2,'buttondownfcn',@lineobject_callback);
                %userdata.handles=L;
                %userdata.delete=1;
                %set(L,'userdata',userdata); %for use in lineobjectcallback
                
            case 'scatter3D'
                for q=max(rank):-1:1
                    %plotdata=objective(rank==q & cviolation==0,:);
                    if numel(settings.x)==2 %draw line instead of points;
                        linestyle={'linestyle','-','marker','none'};
                        plotdata=sortrows(objective(rank==q & cviolation==0,:));
                    else
                        linestyle={'linestyle','none','marker','+'};
                        plotdata=objective(rank==q & cviolation==0,:);
                    end
                    L=line(plotdata(:,1),plotdata(:,2),plotdata(:,3),'parent',handles.activegraph,...
                        'tag','pareto','linewidth',1,'color',colorz(q,:),'markersize',2,'buttondownfcn',@lineobject_callback,linestyle{:});
                    userdata.handles=L;
                    userdata.delete=1;
                    set(L,'userdata',userdata); %for use in lineobjectcallback
                end
                %plot infeasible points:
                plotdata=objective(cviolation~=0,:);
                L=line(plotdata(:,1),plotdata(:,2),plotdata(:,3),'marker','+','linestyle','none','parent',handles.activegraph,...
                    'tag','pareto','linewidth',1,'color',[0 0 0],'markersize',2,'buttondownfcn',@lineobject_callback);
                userdata.handles=L;
                userdata.delete=1;
                set(L,'userdata',userdata); %for use in lineobjectcallback
        end
        
        
        
    case {'simulatedannealing','simulannealbnd'}
        delete(findobj(handles.activegraph,'tag','pareto'))
        set(handles.infobar,'string',['Iteration: ',num2str(state.iteration),'  Function Evaluations: ',num2str(state.funccount),'  Avg Temperature: ',num2str(mean(state.temperature))])
        
        
        
        switch handles.activetype
            case 'scatter'
                %NOTE: SHOULD REALLY BE SORTING ACCORDING TO OPTIMIZED
                %VARIABLES, NOT PLOTTED VARS. this method can cause noise
                %when plot vars ~= objectives.
                L=line(objective(1,1),objective(1,2),'marker','none','marker','o','parent',handles.activegraph,...
                    'tag','pareto','linewidth',1,'markerfacecolor',[1 0 0],'markeredgecolor',[0 0 0],'markersize',5,'buttondownfcn',@lineobject_callback,'linestyle','none');
                userdata.handles=L;
                userdata.delete=1;
                set(L,'userdata',userdata); %for use in lineobjectcallback
                
                L=line(objective(2,1),objective(2,2),'marker','none','marker','o','parent',handles.activegraph,...
                    'tag','pareto','linewidth',1,'markerfacecolor',[0 1 0],'markeredgecolor',[0 0 0],'markersize',5,'buttondownfcn',@lineobject_callback,'linestyle','none');
                userdata.handles=L;
                userdata.delete=1;
                set(L,'userdata',userdata); %for use in lineobjectcallback
                
                
            case 'scatter3D'
                
        end
        
    case 'patternsearch'
        delete(findobj(handles.activegraph,'tag','pareto'))
        set(handles.infobar,'string',['Iteration: ',num2str(state.iteration),'  Function Evaluations: ',num2str(state.funccount)])

        switch handles.activetype
            case 'scatter'
                %NOTE: SHOULD REALLY BE SORTING ACCORDING TO OPTIMIZED
                %VARIABLES, NOT PLOTTED VARS. this method can cause noise
                %when plot vars ~= objectives.
                L=line(objective(1,1),objective(1,2),'marker','none','marker','o','parent',handles.activegraph,...
                    'tag','pareto','linewidth',1,'markerfacecolor',[0 1 0],'markeredgecolor',[0 0 0],'markersize',5,'buttondownfcn',@lineobject_callback,'linestyle','none');
                userdata.handles=L;
                userdata.delete=1;
                set(L,'userdata',userdata); %for use in lineobjectcallback
                
            case 'scatter3D'     
        end
        
end

%fix axes limits:
set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],...
    'xlim',get(handles.activegraph,'xlim'),'ylim',get(handles.activegraph,'ylim'),'zlim',get(handles.activegraph,'zlim'))
drawnow;

%export animation if necessary:
if settings.animation && strcmp(handles.prefs.animation.format,'gif')
    handles.temp.Framez(state.Generation+1) = getscreen(handles.fig,handles.temp.framebounds);
    guidata(src,handles);
elseif settings.animation
    F = getscreen(handles.fig,handles.temp.framebounds);
    handles.temp.mov = addframe(mov,F);
    guidata(src,handles);
end

%state.temperature


if strcmp(settings.method,'simulatedannealing') || strcmp(settings.method,'simulannealbnd') || strcmp(settings.method,'patternsearch')
    state=false; %set this to true to stop at the current iteration.
    if get(handles.temp.stopbutton,'value');state=true;end
end




