function rave
% Welcome to the rave. 
% For help with Rave you should read the
% Rave User Manual pdf file in your rave directory. If you don't
% know where that is, use the MATLAB command "which rave".
%
% Bare minimum help: type "rave" to run the program. Then click Manage Data Sets
% to load a data set (must be "flat file" format). Then click "Create New"
% to make a graph. 
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% If you would like to obtain RAVE under a different license, contact
% Georgia Tech Reserach Corporation http://otl.gtrc.gatech.edu/
%
% Rave was created by Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu
% Please contact me with bug reports, feature requests, collaboration
% ideas, or feedback on how you are using RAVE. 

warning('off','MATLAB:HandleGraphics:ObsoletedProperty:JavaFrame');

if ~usejava('swing') || ~usejava('awt')
    errordlg('RAVE requires Java to run.');
    return
end

if isempty(which('RaveTableSorter'))
    javaaddpath(which('RaveTable.jar'));
end

set(0,'showhiddenhandles','off')

preffile=which('raveprefs.mat');
if isempty(preffile);
    raveinstall;
    return;
end

load raveprefs
if raveprefs.isportable %#ok<NODEF>
    raveroot=which('rave');
    raveroot=raveroot(1:end-6);
    addpath(genpath(raveroot))
    raveprefs.defaultdir=fullfile(raveroot,'ravefiles\');
end

%check if monitor position has changed:
mpos=get(0,'MonitorPositions');
if numel(mpos(:))~=numel(raveprefs.monitorpositions(:)) || ~all(mpos(:)==raveprefs.monitorpositions(:))
    answer=questdlg('Your screen resolution has changed since you last used Rave. Do you want to resize the Rave window to better fit your new resolution?','Resize Rave Window');
    if strcmpi(answer,'yes')
       hhh=raveresizewindow([],[]);   
       uiwait(hhh);
       load raveprefs
       raveprefs.monitorpositions=get(0,'MonitorPositions');
    end
end
raveprefs.monitorpositions=get(0,'MonitorPositions');
prefspath=which('raveprefs.mat');
save(prefspath,'raveprefs');



handles.prefs=raveprefs;

%make tray icon:
sysTray = java.awt.SystemTray.getSystemTray;
if (sysTray.isSupported)
    raveroot=which('rave');
    raveroot=raveroot(1:end-6);    
    myIcon = fullfile(raveroot,'images\tinyrave.png');
    iconImage = java.awt.Toolkit.getDefaultToolkit.createImage(myIcon);
    handles.trayIcon = java.awt.TrayIcon(iconImage, 'initial tooltip');
    handles.trayIcon.setToolTip('RAVE is running.');
    sysTray.add(handles.trayIcon)
else
    handles.trayIcon=[];
end

%do madness:
handles.fig=figure('visible','on','toolbar','none','name','RAVE','menubar','none','numbertitle','off',...
    'resize','off','color',raveprefs.colors.frame,'units','pixels','renderermode','manual','renderer','painters',...
    'handlevisibility','callback','integerhandle','off','doublebuffer','on');

outerwidth=raveprefs.preferredposition(3); %new
outerheight=raveprefs.preferredposition(4); %new

%set up colors
handles.colors=raveprefs.colors;

%set work dir to default dir to start.
handles.defaultdir=raveprefs.defaultdir;
if ~strcmp(handles.defaultdir(end),char(92)) %if it doesnt end in a \, put one there.
    handles.defaultdir(end+1)=char(92);
end


%default colormap:
guidata(handles.fig,handles);
[cmap cmapnames]=ravemakecolormap(handles.fig);

set(handles.fig,'colormap',cmap);
handles.cmapnames=cmapnames;

set(handles.fig,'position',raveprefs.preferredposition)

%set up data structure for passing info between windows:
handles.alldata=cell(0);
handles.datasetnames=cell(0);
handles.analysisnames={'Default Analysis'};
handles.allnames=cell(0);
handles.isresponse=cell(0);
handles.nargout=cell(0);
handles.allunits=cell(0);
handles.isincluded=cell(0);
handles.alltext=cell(0);
handles.istext=cell(0);
handles.isdiscrete=cell(0);
handles.discretevalues=cell(0);
%analysis stuff:
handles.targetvalues=cell(0);
handles.xdials=cell(0);
handles.minlimit=cell(0);
handles.maxlimit=cell(0);
handles.rowcolors=cell(0);
handles.ineqconstraints=cell(0);
handles.isselected=cell(0);
handles.isvisible=cell(0);
handles.variablecolors=cell(0);
handles.preferences=cell(0);
handles.normalizepreferences=cell(0);
handles.isobjective=cell(0); % a special type of function that uses preferences/targets
handles.projectname='';
handles.rankingcolumn=cell(0);
handles.roi=[]; %print region of interest;
handles.selectionmode='none'; %current selection mode: none, box, lasso, or point. 

raveroot = which('rave');
handles.raveroot=raveroot(1:end-6);

%initialize variables
handles.newgraphtype='blank';
handles.currentgraph=[];
handles.allgraphs=[];
handles.graphanalysis=[]; %same length as allgraphs, contains number of analysis for indicated graph.
handles.graphdataset=[]; %ditto for data sets. 
handles.fakeax=[];
handles.fakerax=[]; %allgraphs, fakeax, and fakerax contain the handles to the outermost (biggest) axes for a graph. They are always dim=(1,1), so they dont need to be cells.
handles.allblockers=[];
handles.expblockers=[];
handles.expAblockers=[];
handles.activeindex=[];
handles.activegraph=[];
handles.activetype='';
handles.activetab='';
handles.activedir=[]; %the directory this project resides in.
handles.activesubgraph=[]; %for use with plot matrices. contains the index of a SINGLE selected sub-axes. To get all subaxes for a graph, use handles.subax{activeindex}(:,:);
handles.subax=cell(0);
handles.subfakeax=cell(0);
handles.subfakerax=cell(0);
handles.statsax=cell(0);
handles.tempplots=[];
handles.limlinks=[];
handles.tabflexgrid=[];
handles.rotater=[];
handles.rasters=cell(0);
handles.grabbers=cell(0);
handles.javacomps=[];
handles.textboxj=cell(0);
handles.textboxh=cell(0);
handles.Agrabbers=cell(0);
handles.putaway=[]; %index of graphs that are currently put away.
handles.graphnames=cell(0);


%intitialize "udata" for draw tab:
handles.draw.textbg=[1 1 .8]; %post-it yellow
handles.draw.edittextmode=0;%if 1, process drawtab callbacks on current text box.
handles.draw.fontname=handles.prefs.fonts.textbox;
handles.draw.fontstyle=4; %plain by default
handles.draw.fontsize=14;
handles.draw.showborder=1;

%save last export config for easy exporting:
handles.lastexport.scale=10;
handles.lastexport.antialias='none';
handles.lastexport.filetype='.jpg';


%maintable stuff:
handles.tablemode=0; %true when table is visible.
handles.maintable.undos=cell(50,1); %history of table changes, used to undo.
handles.maintable.redos=cell(50,1); %for updates that have been undone.
handles.maintable.x=cell(0); %visible columns for each data set.
handles.maintable.wasdragged=0; %draggings sets this to 1 so mousereleasecallback knows now to select column.
handles.maintable.colorrows=1; %use row colors?
handles.maintable.shadehidden=1; %shade hidden rows?
handles.maintable.allowedits=1;
handles.maintable.allowselection=1;
handles.datasetupdatespending=cell(0);
handles.analysisupdatespending=cell(0);
handles.maintable.d=0;
handles.maintable.a=0;
handles.maintable.ignoreupdates=false;


%% load data for all graphtypes:
raveroot=which('rave');
raveroot=raveroot(1:end-6);
handles.graphinfo=struct;
graphclasses={'discrete';'continuous';'special';'table';'control';'annotation'};
for G=1:numel(graphclasses)

    maindir=fullfile(raveroot,'graphics\',[graphclasses{G} '\']);
    graphtypes=dir(maindir);

    for g=3:numel(graphtypes)
       % h=g-2;
        if graphtypes(g).isdir && graphtypes(g).name(1)~='X'
            thisname=graphtypes(g).name;
            %remove leading numbers:
            firstletter=find(isstrprop(thisname,'alpha'),1);
            keyword=thisname(firstletter:end);
            %get graphinfo:
            fid=fopen(fullfile(maindir,thisname,'graphinfo.txt'),'r');
              
            %get properties for this graph type:
            properties=struct;
            try
            eval(['graphproperties',keyword]);
            catch %#ok<CTCH>
               addpath([maindir,thisname],'-begin');
               rehash toolboxcache
               try
                   eval(['graphproperties',keyword]);
               catch %#ok<CTCH>
                   error(['rave:filenotfound','The graphproperties.txt file could not be found for graph type ' ...
                       keyword '. Either create this file or remove the ' keyword ' folder from Rave''s graphics directory']);
               end
                
            end
            
            handles.graphinfo.(keyword)=properties;
            handles.graphinfo.(keyword).longname=fgetl(fid);
            handles.graphinfo.(keyword).class=graphclasses{G};
            
            fclose(fid);
        end
    end
end


%% Load data for optimizers:

raveroot=which('rave');
raveroot=raveroot(1:end-6);
handles.optimizerinfo=struct;
optimizerclasses={'single';'multi';'ranking'};
for G=1:numel(optimizerclasses)

    maindir=fullfile(raveroot,'optimizers\',[optimizerclasses{G} '\']);
    optimizertypes=dir(maindir);

    for g=3:numel(optimizertypes)
       % h=g-2;
        if optimizertypes(g).isdir
            thisname=optimizertypes(g).name;
            %remove leading numbers:
            firstletter=find(isstrprop(thisname,'alpha'),1);
            keyword=thisname(firstletter:end);
            %get graphinfo:
            fid=fopen(fullfile(maindir,thisname,'optimizerinfo.txt'),'r');
              
            settings=struct;
            eval(['optimizersettings',keyword]);

            handles.optimizerinfo.(keyword)=settings;
            handles.optimizerinfo.(keyword).longname=fgetl(fid);
            handles.optimizerinfo.(keyword).class=optimizerclasses{G};
            
            fclose(fid);
        end
    end
end


%%
handles.tabsvisible=1;

innerposition=get(handles.fig,'position'); 
handles.width=innerposition(3);
handles.height=innerposition(4);


%% make main data table
guidata(handles.fig,handles);
ravecreatemaintable(handles.fig,[]);
handles=guidata(handles.fig);

%depending on which table method i'm currently using, invisible is either "off" or "0"
set(handles.maintable.handle,'visible','off');

%% set up main (workspace) axes

handles.wspanel=uipanel('position',[0 0 .3 .3],'backgroundcolor',[1 1 1],'units','pixels',...
    'foregroundcolor',handles.colors.frame,'parent',handles.fig,'highlightcolor',handles.colors.frame,'shadowcolor',handles.colors.frame);
%uistackold(handles.wspanel,'bottom');

handles.wscols=handles.prefs.wswidth/handles.prefs.gridsize;
handles.wsrows=handles.prefs.wsheight/handles.prefs.gridsize;

%% set up frame axes

handles.leftmargin=axes('units','pixels','position',[0 -5 30+315*handles.prefs.tabscale handles.height+11],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xcolor',handles.colors.frame,'ycolor',handles.colors.frame,'zcolor',handles.colors.frame,'color',handles.colors.frame);

handles.rightmargin=axes('units','pixels','position',[handles.width-13 -5 40 handles.height+20],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xcolor',handles.colors.frame,'ycolor',handles.colors.frame,'zcolor',handles.colors.frame,'color',handles.colors.frame);

handles.topmargin=axes('units','pixels','position',[-5 handles.height-35 handles.width+5 60],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xcolor',handles.colors.frame,'ycolor',handles.colors.frame,'zcolor',handles.colors.frame,'color',handles.colors.frame,'xlimmode','manual','ylimmode','manual',...
    'xlim',[0 1],'ylim',[0 1]);
handles.bottommargin=axes('units','pixels','position',[-5 1 handles.width+10 17],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xcolor',handles.colors.frame,'ycolor',handles.colors.frame,'zcolor',handles.colors.frame,'color',handles.colors.frame);

leftpos=get(handles.leftmargin,'position');
rightpos=get(handles.rightmargin,'position');
toppos=get(handles.topmargin,'position');
botpos=get(handles.bottommargin,'position');

handles.workspacedims=[rightpos(1)-leftpos(1)-leftpos(3),toppos(2)-botpos(2)-botpos(4)];
handles.workspacepos=[leftpos(1)+leftpos(3) botpos(2)+botpos(4) handles.workspacedims(1) handles.workspacedims(2)];

%make infobar:
infopos=get(handles.topmargin,'position');
infoaxes=axes('units','pixels','position',infopos+[360 0 -360 0],'color','none','visible','off','parent',handles.fig);
handles.infobar=text(.0,.05,'','horizontalalignment','left',...
    'fontsize',18*raveprefs.fonts.tabscale,'fontname',raveprefs.fonts.tab,'color',[.3 .6 .8],'parent',infoaxes,'verticalalignment','baseline','interpreter','tex');

handles.mainaxes=axes('units','pixels','position',[350 toppos(2)-handles.wsrows*handles.prefs.gridsize-1 handles.wscols*handles.prefs.gridsize handles.wsrows*handles.prefs.gridsize],...
    'parent',handles.fig,'ytickmode','manual','yticklabel','','xtickmode',...
    'manual','xticklabel','','box','off','clipping','off',...
    'xlimmode','manual','ylimmode','manual','color',handles.colors.wsbg,'buttondownfcn',@ravedragview);
%uistackold(handles.mainaxes,'bottom');


axessize=get(handles.mainaxes,'position');
axessize(3:4)=handles.prefs.gridsize.*floor(axessize(3:4)./handles.prefs.gridsize);
set(handles.mainaxes,'position',axessize);

%draw grids
set(handles.mainaxes,'xlim',[0 axessize(3)],'ylim',[0 axessize(4)])

for r=0:(handles.prefs.gridsize/4):axessize(4)
    line([0 axessize(3)],[r r],'color',(handles.colors.wsgrid+handles.colors.wsbg)/2,'parent',handles.mainaxes,'hittest','off','tag','sgrid');
end
for c=0:(handles.prefs.gridsize/4):axessize(3)
    line([c c],[0 axessize(4)],'color',(handles.colors.wsgrid+handles.colors.wsbg)/2,'parent',handles.mainaxes,'hittest','off','tag','sgrid');
end

for r=0:(handles.prefs.gridsize/2):axessize(4)
    line([0 axessize(3)],[r r],'color',(handles.colors.wsgrid+handles.colors.wsbg)/2,'parent',handles.mainaxes,'hittest','off','tag','mgrid');
end
for c=0:(handles.prefs.gridsize/2):axessize(3)
    line([c c],[0 axessize(4)],'color',(handles.colors.wsgrid+handles.colors.wsbg)/2,'parent',handles.mainaxes,'hittest','off','tag','mgrid');
end

for r=0:handles.prefs.gridsize:axessize(4)
    line([0 axessize(3)],[r r],'color',handles.colors.wsgrid,'parent',handles.mainaxes,'hittest','off','tag','lgrid');
end
for c=0:handles.prefs.gridsize:axessize(3)
    line([c c],[0 axessize(4)],'color',handles.colors.wsgrid,'parent',handles.mainaxes,'hittest','off','tag','lgrid');
end

handles.lgrid=findobj(handles.mainaxes,'tag','lgrid');
handles.mgrid=findobj(handles.mainaxes,'tag','mgrid');
handles.sgrid=findobj(handles.mainaxes,'tag','sgrid');

%fix grid visibility
if handles.prefs.gridmode==1
    set(handles.lgrid,'visible','on')
    set(handles.mgrid,'visible','off')
    set(handles.sgrid,'visible','off')
    handles.snapsize=handles.prefs.gridsize;
elseif handles.prefs.gridmode==2
    set(handles.lgrid,'visible','on')
    set(handles.mgrid,'visible','on')
    set(handles.sgrid,'visible','off')
    handles.snapsize=handles.prefs.gridsize/2;
else
    set(handles.lgrid,'visible','on')
    set(handles.mgrid,'visible','on')
    set(handles.sgrid,'visible','on')
    handles.snapsize=handles.prefs.gridsize/4;
end

if ~handles.prefs.snaptogrid
    handles.snapsize=1;
end

handles.wsrows=axessize(4)/handles.prefs.gridsize;
handles.wscols =axessize(3)/handles.prefs.gridsize;
handles.wsdims=[axessize(3) axessize(4)];

%draw additional box around axes that is always visible
for r=0:axessize(4):axessize(4)
    line([0 axessize(3)],[r r],'color',handles.colors.wsgrid,'parent',handles.mainaxes,'hittest','off','tag','wspanelborder');
end
for c=0:axessize(3):axessize(3)
    line([c c],[0 axessize(4)],'color',handles.colors.wsgrid,'parent',handles.mainaxes,'hittest','off','tag','wspanelborder');
end

%move uipanel to encompass mainaxes, and make uipanel mainaxes's parent
putpanelhere=get(handles.mainaxes,'position');
set(handles.wspanel,'position',[putpanelhere(1)-5 putpanelhere(2)-3 putpanelhere(3)+5 putpanelhere(4)+5]);
set(handles.mainaxes,'parent',handles.wspanel);
set(handles.mainaxes,'position',[1 2 putpanelhere(3) putpanelhere(4)])


%% set up workspace explorer axes
if handles.height>=900
    pushdown=30; %move down a little so its not so cramped
else
    pushdown=0;
end

if handles.wscols>=240/150*handles.wsrows %width constrained

    handles.explorer=axes('units','pixels','position',[100 handles.height-155-pushdown 240 handles.wsrows/handles.wscols*240],...
        'parent',handles.fig,...
        'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off','clipping','off',...
        'xlimmode','manual','ylimmode','manual','xlim',get(handles.mainaxes,'xlim'),...
        'ylim',get(handles.mainaxes,'ylim'),'ycolor',[1 1 1],'xcolor',[1 1 1],...
        'buttondownfcn',@ravemoveview,'hittest','on','color',handles.colors.wsbg);

else%height constrained
    handles.explorer=axes('units','pixels','position',[340-handles.wscols/handles.wsrows*150 handles.height-155-pushdown handles.wscols/handles.wsrows*150 150],...
        'parent',handles.fig,...
        'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off','clipping','off',...
        'xlimmode','manual','ylimmode','manual','xlim',get(handles.mainaxes,'xlim'),'ylim',get(handles.mainaxes,'ylim'),...
        'ycolor',[1 1 1],'xcolor',[1 1 1],...
        'buttondownfcn',@ravemoveview,'hittest','on','color',handles.colors.wsbg);
end

%draw explorer grid
handles.explorergrid=copyobj(handles.lgrid,handles.explorer);
set(handles.explorergrid,'color',(handles.colors.wsgrid+handles.colors.wsbg)/2)

%% draw current-view frame

%calculate where to put it.
mainaxespos=get(handles.wspanel,'position');

makethistall=handles.workspacepos(4);
makethiswide=handles.workspacepos(3);
putthisfarup=handles.workspacepos(2)-mainaxespos(2);
putthisfarleft=handles.workspacepos(1)-mainaxespos(1);

%draw it
handles.currentview=rectangle('position',[putthisfarleft putthisfarup makethiswide makethistall],'parent',handles.explorer,...
    'linewidth',2,'clipping','off','edgecolor',handles.colors.text,'hittest','off','tag','currentview');

%get x y coords of rectangle in explorer
cvpos=get(handles.currentview,'position');

%% make explorer title if on large screen
if handles.height>900

    handles.explorertitle=axes('units','pixels','position',[340-handles.wscols/handles.wsrows*150 handles.height-1-pushdown 200 50],'parent',handles.fig,...
        'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off','clipping','off',...
        'xlimmode','manual','ylimmode','manual','xlim',get(handles.mainaxes,'xlim'),'ylim',get(handles.mainaxes,'ylim'),'ycolor',handles.colors.frame,'xcolor',handles.colors.frame,...
        'buttondownfcn','','hittest','off','visible','off','color','none');
    text(0,.2,'Navigator','fontname',handles.prefs.fonts.title,'fontsize',20*handles.prefs.fonts.titlescale,'color',handles.colors.title,...
        'parent',handles.explorertitle,'verticalalignment','baseline')
else
    handles.explorertitle=[];

end


%% make scrollbars
%originalLnF = javax.swing.UIManager.getLookAndFeel;  %class
% try %#ok<TRYNC>
%     newLnF = 'com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel';
%     javax.swing.UIManager.setLookAndFeel(newLnF);
% end

handles.vscroll=uicontrol('position',[handles.width-17 17 16 handles.height-53],'style','slider',...
    'callback',{@vertscroll,handles.fig},'min',cvpos(4)/2,'max',handles.prefs.gridsize*handles.wsrows-cvpos(4)/2,...
    'sliderstep',[.05 cvpos(4)/((handles.prefs.gridsize*handles.wsrows)-cvpos(4))],...
    'value',handles.prefs.gridsize*handles.wsrows-cvpos(4)/2,'parent',handles.fig);

%horizontalslider
handles.hscroll=uicontrol('position',[345+1 1 handles.width-345-17 16],'style','slider',...
    'callback',{@horizscroll,handles.fig},'min',cvpos(3)/2,'max',handles.prefs.gridsize*handles.wscols-cvpos(3)/2,...
    'sliderstep',[.02 cvpos(3)/((handles.prefs.gridsize*handles.wscols)-cvpos(3))],...
    'value',cvpos(3)/2,'parent',handles.fig);
drawnow;
%javax.swing.UIManager.setLookAndFeel(originalLnF);

set([handles.vscroll handles.hscroll],'handlevisibility','off');

%% set up title axes if on a large monitor
if handles.height>=900
    handles.titleaxes=axes('units','pixels','position',[20 20 320 40],'parent',handles.fig,...
       'visible','off','xlim',[0 1],'ylim',[0 1]);
    text(.5,0,'rave','fontname','magneto','parent',handles.titleaxes,'fontsize',80,'color',handles.colors.title,...
        'verticalalignment','baseline','backgroundcolor','none','hor','center','hittest','on','buttondownfcn',@ravefixtabs)
    axesoffset=85;
else
    handles.titleaxes=[];
    axesoffset=25;
end

%% set up tab axes

tabwidth=1; %pixel width of tab borders

handles.bgtabaxes=axes('units','pixels','position',[20+30*(handles.prefs.tabscale) 15+axesoffset-20-650*(handles.prefs.tabscale-1) 261*handles.prefs.tabscale 631*handles.prefs.tabscale],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xlimmode','manual','ylimmode','manual','xlim',[0 1],'ylim',[0 1],'clipping','off',...
    'ycolor',handles.colors.frame,'xcolor',handles.colors.frame,'zcolor',handles.colors.tabfg,'color','none','visible','on');
handles.tabaxes=axes('units','pixels','position',[20+60*(handles.prefs.tabscale) 20+15-20+axesoffset-650*(handles.prefs.tabscale-1) 261*handles.prefs.tabscale 631*handles.prefs.tabscale],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xlimmode','manual','ylimmode','manual','xlim',[0 1],'ylim',[0 1],'clipping','off',...
    'ycolor',handles.colors.frame,'xcolor',handles.colors.frame,'zcolor',handles.colors.tabfg,'color','none','visible','on');
handles.fronttabaxes=axes('units','pixels','position',[20+30*(handles.prefs.tabscale) 35-20+axesoffset-650*(handles.prefs.tabscale-1) 30*handles.prefs.tabscale 631*handles.prefs.tabscale],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xlimmode','manual','ylimmode','manual','xlim',[0 1],'ylim',[0 1],'color','none',...
    'ycolor',[0 0 0],'xcolor',[0 0 0],'zcolor',handles.colors.tabbg,...
    'visible','off');
handles.backtabaxes=axes('units','pixels','position',[20 15-20+axesoffset-650*(handles.prefs.tabscale-1) 30*handles.prefs.tabscale 630*handles.prefs.tabscale],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xlimmode','manual','ylimmode','manual','xlim',[0 1],'ylim',[0 1],'color','none',...
    'ycolor',[0 0 0],'xcolor',[0 0 0],'zcolor',handles.colors.frame,...
    'visible','off');

%set up non-tab tab borders
%background
patch([0 1 1 0],[1 1 0 0],handles.colors.tabbg,'parent',handles.tabaxes,'edgecolor','none','tag','tabbg')
patch([0 1 1 0],[1 1 0 0],handles.colors.tabbg,'parent',handles.bgtabaxes,'edgecolor','none','tag','tabbg')

line([0 1],[0 0],'linewidth',tabwidth,'color',handles.colors.tabfg,'parent',handles.tabaxes);%bottom
line([0 1],[1 1],'linewidth',tabwidth,'color',handles.colors.tabfg,'parent',handles.tabaxes);%top
line([1 1],[0 1],'linewidth',tabwidth,'color',handles.colors.tabfg,'parent',handles.tabaxes);%right
line([0 1],[0 0],'linewidth',tabwidth,'color',handles.colors.tabfg,'parent',handles.bgtabaxes);%subbottom
line([1 1],[0 1],'linewidth',tabwidth,'color',handles.colors.tabfg,'parent',handles.bgtabaxes);%subright

if handles.height>=900
    %show title on controlpanel:
    handles.controltitle=text(.15,1,'Control Panel','backgroundcolor','none','color',handles.colors.title,'fontsize',20*handles.prefs.fonts.titlescale,...
        'horizontalalignment','left','verticalalignment','baseline','parent',handles.tabaxes,...
        'fontname',handles.prefs.fonts.title,'margin',1,'handlevisibility','off','tag','controltitle');
else
    handles.controltitle=[];
end

%draw tabs:
handles.fronttabprofile=[1 .8 1 1 1];
handles.backtabprofile=[1 1.1 1.2 .9 1.2];
%front row:
[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,1,1);

handles.fronttabbg=patch(x,y,handles.colors.tabbg,'parent',handles.fronttabaxes,'edgecolor','none','tag','tabbg');
handles.fronttablines=line(x,y,'parent',handles.fronttabaxes,'linewidth',tabwidth,'color',handles.colors.tabfg);

tabfs=18*((handles.prefs.tabscale-1)*2+1);

%put text in tabs:
%front row
text(.15,v(1),'  Manage','fontname',raveprefs.fonts.tab,'color',handles.colors.text,'fontsize',tabfs*raveprefs.fonts.tabscale,...
    'rotation',90,'parent',handles.fronttabaxes,'verticalalignment','cap',...
    'buttondownfcn',@managetab,'interruptible','off','tag','managetab')
text(.17,v(2),'  View','fontname',raveprefs.fonts.tab,'color',handles.colors.text,'fontsize',tabfs*raveprefs.fonts.tabscale,...
    'rotation',90,'parent',handles.fronttabaxes,'verticalalignment','cap',...
    'buttondownfcn',@viewtab,'hittest','off','interruptible','off','tag','viewtab')
text(.17,v(3),'  Format','fontname',raveprefs.fonts.tab,'color',handles.colors.text,'fontsize',tabfs*raveprefs.fonts.tabscale,...
    'rotation',90,'parent',handles.fronttabaxes,'verticalalignment','cap',...
    'buttondownfcn',@formattab,'hittest','off','interruptible','off','tag','formattab')
text(.17,v(4),'  Explore','fontname',raveprefs.fonts.tab,'color',handles.colors.text,'fontsize',tabfs*raveprefs.fonts.tabscale,...
    'rotation',90,'parent',handles.fronttabaxes,'verticalalignment','cap',...
    'buttondownfcn',@exploretab,'hittest','off','interruptible','off','tag','exploretab')
text(.17,v(5),'   Export','fontname',raveprefs.fonts.tab,'color',handles.colors.text,'fontsize',tabfs*raveprefs.fonts.tabscale,...
    'rotation',90,'parent',handles.fronttabaxes,'verticalalignment','cap',...
    'buttondownfcn',@exporttab,'interruptible','off','tag','exporttab')
%back row:

[x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0);

handles.backtabbg=patch(x,y,handles.colors.tabbg,'parent',handles.backtabaxes,'edgecolor','none','tag','tabbg');
handles.backtablines=line(x,y,'parent',handles.backtabaxes,'linewidth',tabwidth,'color',handles.colors.tabfg);

%put text in tabs:
text(.17,v(3),' Optimize','fontname',raveprefs.fonts.tab,'color',handles.colors.texttint,'fontsize',tabfs*raveprefs.fonts.tabscale,...
    'rotation',90,'parent',handles.backtabaxes,'verticalalignment','cap',...
    'buttondownfcn',@optimizetab,'interruptible','off','tag','optimizetab')
text(.17,v(4),'   Draw  ','fontname',raveprefs.fonts.tab,'color',handles.colors.texttint,'fontsize',tabfs*raveprefs.fonts.tabscale,...
    'rotation',90,'parent',handles.backtabaxes,'verticalalignment','cap',...
    'buttondownfcn',@drawtab,'interruptible','off','tag','drawtab')
text(.17,v(1),'  Model','fontname',raveprefs.fonts.tab,'color',handles.colors.texttint,'fontsize',tabfs*raveprefs.fonts.tabscale,...
    'rotation',90,'parent',handles.backtabaxes,'verticalalignment','cap',...
    'buttondownfcn',@modeltab,'interruptible','off','tag','modeltab')
text(.17,v(2),'Constrain','fontname',raveprefs.fonts.tab,'color',handles.colors.texttint,'fontsize',tabfs*raveprefs.fonts.tabscale,...
    'rotation',90,'parent',handles.backtabaxes,'verticalalignment','cap',...
    'buttondownfcn',@constraintab,'interruptible','off','tag','constraintab')
text(.17,v(5),' Configure','fontname',raveprefs.fonts.tab,'color',handles.colors.texttint,'fontsize',tabfs*raveprefs.fonts.tabscale,...
    'rotation',90,'parent',handles.backtabaxes,'verticalalignment','cap',...
    'buttondownfcn',@configuretab,'interruptible','off','tag','configuretab')

handles.alltabs=findobj([handles.fronttabaxes,handles.backtabaxes],'type','text');

%make tabs unclickable until the user loads a data set (managedatagui turns
%them on)
set(handles.alltabs,'hittest','off');

handles.fronttabsfront=1; %1 if fronttabs in front, 0 if backtabs in front


%% set up non-tabbed ui controls:
handles.stopbutton=uicontrol('style','pushbutton','position',[handles.width-105 handles.height-32 100 32],...
    'string','debug','fontname','arial narrow','fontsize',20,'foregroundcolor',handles.colors.title,...
    'callback',@stopbutton,'backgroundcolor',handles.colors.buttonbg,'parent',handles.fig);

handles.dummyobj=uicontrol('style','pushbutton','position',[-100 -100 50 50],'parent',handles.fig);



handles.tablemodebutton=uicontrol('style','pushbutton','position',[18 handles.height-35-pushdown 73 32],...
    'string','Table','callback',@showhideworkspace,'cdata',[],'tooltipstring','View Data Table','parent',handles.fig,...
    'fontsize',12,'tag','tablemodebutton');
handles.newbutton=uicontrol('style','pushbutton','position',[18 handles.height-75-pushdown 32 32],...
    'string','','callback',@newbutton,'cdata',imread('newgraph.bmp'),'tooltipstring','Create New Plot','parent',handles.fig);
handles.zoombutton=uicontrol('style','togglebutton','position',[59 handles.height-75-pushdown 32 32],...
    'string','','callback','','cdata',imread('zoom.bmp'),'tooltipstring',' Zoom ','parent',handles.fig);
handles.showgridbutton=uicontrol('style','togglebutton','position',[18 handles.height-155-pushdown 32 32],...
    'string','','callback',@showgridbutton,'value',1,...
    'tooltipstring','Show/Hide Gridlines','cdata',imread('showgrid.bmp'),'parent',handles.fig,...
    'backgroundcolor',handles.colors.togglebuttonbg);
handles.openglbutton=uicontrol('style','togglebutton','position',[59 handles.height-115-pushdown 32 32],...
    'string','','callback',@changerenderer_callback,'tooltipstring','Accelerate','parent',handles.fig,...
    'cdata',imread('raveclock.bmp'),'backgroundcolor',handles.colors.togglebuttonbg);
handles.hideshowbutton=uicontrol('style','pushbutton','position',[18 handles.height-115-pushdown 32 32],...
    'string','','callback',@hidetabs,'cdata',imread('hideshow.bmp'),...
    'tooltipstring','Hide/Show Control Panel','parent',handles.fig);




%% initialize handles.tabobjects so tab changefcn has something to delete
dummyobj=uicontrol('position',[-200 -200 10 10],'style','pushbutton','parent',handles.fig);
handles.tabobjects=dummyobj;
handles.tabobjects=[];

%% load error sound:
%[handles.ravewave,handles.ravewaveF]=wavread('raveding.wav');
handles.sounds.finish=wavread('ravefinish.wav');

%% set up close request function:
set(handles.fig,'closerequestfcn',@ravequitfunction)

%% show splash screen
drawnow;
pause(0.2); %need to pause because on vista figure window "fades in" and screws up getscreen.

splash=imread('ravesplashlite.bmp','bmp');

splashposition=floor([(outerwidth-size(splash,2))/2+28 (outerheight-size(splash,1))/2 size(splash,2) size(splash,1)]);
splashax=axes('units','pixels','position',splashposition,'parent',handles.fig,'color','none','visible','off');
%get background pixels:
splashback=getscreen(splashax);

splashalpha=zeros(size(splash));
splashalpha(:,:,1)=splash(:,:,1)==splash(1,1,1);
splashalpha(:,:,2)=splash(:,:,2)==splash(1,1,2);
splashalpha(:,:,3)=splash(:,:,3)==splash(1,1,3);
splashalpha=all(splashalpha,3);
splashbeta=~splashalpha;

splashalpha=uint8(splashalpha);
splashbeta=uint8(splashbeta);

splashback=splashback.cdata;

splashback=bsxfun(@times,splashback,splashalpha);
splash=bsxfun(@times,splash,splashbeta);

SPAH=splash+splashback;

handles.splash=image(SPAH,'parent',splashax);
set(splashax,'visible','off')

text(635,190,['Version ',ravever],...
    'parent',splashax,'fontname','times','fontsize',12,'hor','right','fontangle','italic','fontweight','bold')
%Notice to users -- do not modify/remove this line:
text(635,330,{'Created by Matthew Daskilewicz';'Copyright 2011';'Georgia Tech Research Corporation';'All Rights Reserved'},...
    'parent',splashax,'fontname','times','fontsize',12,'hor','right','fontangle','italic')
%Notice to users -- do not modify/remove this line:
text(635,390,{'Distributed under GNU General Public License Ver. 2'},...
    'parent',splashax,'fontname','times','fontsize',11,'hor','right','fontangle','normal')

%% wrap it up...
%Choose default command line output for rave
handles.output = handles.fig;
% Update handles structure
guidata(handles.fig, handles);

%% create manage tab

managetab(handles.fig,[]);
handles=guidata(handles.fig);

%% start gpu:


%% TURN IT ON
set(handles.fig,'visible','on','windowscrollwheelfcn',@ravewheelscroll)

% Update handles structure
guidata(handles.fig, handles);

%load context menus:
ravemakecontextmenus(handles.fig,[]);
handles=guidata(handles.fig);

%% %do evil java bidding:
%add rave icon:
pause(2); %wait to make sure java handles are valid.

handles.jvscroll=findjobj(handles.vscroll);
handles.jhscroll=findjobj(handles.hscroll);
try %#ok<TRYNC>
    handles.jvscroll.MouseDraggedCallback = {@vertscroll,handles.fig};
    handles.jhscroll.MouseDraggedCallback = {@horizscroll,handles.fig};
end

guidata(handles.fig,handles);
delete(splashax)
set(handles.infobar,'string',['Welcome to the \fontsize{24}\fontname{magneto}\color[rgb]{',num2str(handles.colors.text),'}rave\fontsize{18}\fontname{' handles.prefs.fonts.tab '}\color[rgb]{',num2str(handles.colors.text),'}, click "Manage Data Sets" to begin.']);

drawnow;

%%
function stopbutton(hObject,eventdata)  %#ok<INUSD>
handles=guidata(hObject); 
udata=get(handles.fakeax(handles.activeindex),'userdata'); %#ok<NASGU>
keyboard


function newbutton(hObject,eventdata) 
handles=guidata(hObject);
uicontrol(handles.dummyobj);

ravenewgraph(handles.fig,[])


function showgridbutton(hObject,eventdata) 
handles=guidata(hObject);
uicontrol(handles.dummyobj);


%do not alter the currently selected graph, or any annotation class graph. 
changethese=false(size(handles.allgraphs));

for c=1:numel(changethese)
   if handles.allgraphs(c)~=handles.activegraph && ~strcmp(handles.graphinfo.(get(handles.allgraphs(c),'userdata')).class,'annotation')
       changethese(c)=true;
   end
end

if get(hObject,'value')
    if handles.prefs.gridmode==1
        set(handles.lgrid,'visible','on')
    elseif handles.prefs.gridmode==2
        set(handles.lgrid,'visible','on')
        set(handles.mgrid,'visible','on')
    else
        set(handles.lgrid,'visible','on')
        set(handles.mgrid,'visible','on')
        set(handles.sgrid,'visible','on')
    end

    set(handles.allblockers(changethese),'edgecolor',handles.colors.wsgrid)

else
    set(handles.sgrid,'visible','off')
    set(handles.mgrid,'visible','off')
    set(handles.lgrid,'visible','off')
    set(handles.allblockers(changethese),'edgecolor','none')

end

function highlightbutton(hObject,eventdata) 
handles=guidata(hObject);
uicontrol(handles.dummyobj);

if numel(handles.activeindex)==0;return;end; %quit if no graphs

if get(hObject,'value')
    raveselectgraph(handles.activegraph,1);
else
    if get(handles.showgridbutton,'value')
        set(handles.allblockers,'linewidth',1,'edgecolor',handles.colors.wsgrid);
    else
        set(handles.allblockers(handles.activeindex),'linewidth',1,'edgecolor','none');
    end

    if ~handles.prefs.automovemode
        set(handles.grabbers{handles.activeindex},'visible','off')
    end

end

function hidetabs(hObject,eventdata) %#ok<INUSD> % %hide / show tabs.
handles=guidata(hObject);

if isempty(handles.activeindex);return;end
%uicontrol(handles.wspanel);

%if get(handles.hideshowbutton,'value') %hide em

set(handles.fig,'name','RAVE <<HIT ESC TO EXIT FULL-SCREEN MODE>>');
set(handles.stopbutton,'visible','off')

%make manage tab active since it is known to have no issues
managetab(hObject,[])

%turn off graph hittests so no new tabs appear:
set(handles.allblockers,'hittest','off','visible','off');

%update handles
handles=guidata(hObject);

hideobjs=findobj([handles.explorer,handles.tabaxes,handles.fronttabaxes,...
    handles.explorertitle,handles.titleaxes,handles.backtabaxes,handles.bgtabaxes handles.controltitle]);

set(hideobjs,'visible','off')
set(handles.tabobjects,'visible','off')

%reduce leftmarginposition to 1 pixel
leftpos=get(handles.leftmargin,'position');
leftpos(3)=1;
set(handles.leftmargin,'position',leftpos);

toppos=get(handles.topmargin,'position');
set(handles.topmargin,'position',toppos+[0 35 0 0])

%update workspaceposition
rightpos=get(handles.rightmargin,'position');
toppos=get(handles.topmargin,'position');
botpos=get(handles.bottommargin,'position');

handles.workspacedims=[rightpos(1)-leftpos(1)-leftpos(3),toppos(2)-botpos(2)-botpos(4)];
handles.workspacepos=[leftpos(1)+leftpos(3) botpos(2)+botpos(4) handles.workspacedims(1) handles.workspacedims(2)];

%turn off grid:
set(handles.lgrid,'visible','off')
set(handles.mgrid,'visible','off')
set(handles.sgrid,'visible','off')

set(handles.grabbers{handles.activeindex},'visible','off')

%stretch out the scroll bar:
cvpos=get(handles.currentview,'position');

cvpos(3)=handles.workspacepos(3);
cvpos(4)=handles.workspacepos(4);
set(handles.currentview,'position',cvpos);

set(handles.hscroll,'position',[1 1 handles.width-13 16],...
    'sliderstep',[.02 cvpos(3)/((handles.prefs.gridsize*handles.wscols)-cvpos(3))],...
    'min',cvpos(3)/2,'max',handles.prefs.gridsize*handles.wscols-cvpos(3)/2,'value',cvpos(3)/2);
set(handles.vscroll,'position',[handles.width-16 18 16 handles.height-54+35],...
    'min',cvpos(4)/2,'max',handles.prefs.gridsize*handles.wsrows-cvpos(4)/2,...
     'sliderstep',[.05 cvpos(4)/((handles.prefs.gridsize*handles.wsrows)-cvpos(4))],...
     'value',handles.prefs.gridsize*handles.wsrows-cvpos(4)/2)

%update handles
guidata(hObject,handles);

%move view to new origin
ravemoveview(handles.fig,[],[0 handles.prefs.wsheight-1+35]);

set(handles.tablemodebutton,'visible','off');
set(handles.showgridbutton,'visible','off');
%set(handles.highlightbutton,'visible','off');
%set(handles.unhidebutton,'visible','off');
set(handles.openglbutton,'visible','off');
set(handles.zoombutton,'visible','off');
set(handles.hideshowbutton,'visible','off');
set(handles.newbutton,'visible','off');
%set(handles.putawaybutton,'visible','off');
set(handles.infobar,'visible','off');

set(handles.fig,'keypressfcn',@showtabs);

function showtabs(src,ev) %show em
handles=guidata(src);

if ~strcmp(ev.Key,'escape');return;end;

set(handles.fig,'name','RAVE');
set(handles.stopbutton,'visible','on')

hideobjs=findobj([handles.explorer,handles.tabaxes,handles.fronttabaxes,...
    handles.explorertitle,handles.titleaxes,handles.backtabaxes,handles.bgtabaxes handles.controltitle]);
set(hideobjs,'visible','on')
set([handles.fronttabaxes handles.backtabaxes,handles.bgtabaxes,handles.titleaxes],'visible','off')

set(handles.tabobjects,'visible','on')

%turn on graph hittests:
set(handles.allblockers,'hittest','on');
set(handles.allblockers(ishandle(handles.allgraphs)),'visible','on')

cvpos=get(handles.currentview,'position');

%turn on grid:
if get(handles.showgridbutton,'value')
    if handles.prefs.gridmode==1
        set(handles.lgrid,'visible','on')
    elseif handles.prefs.gridmode==2
        set(handles.lgrid,'visible','on')
        set(handles.mgrid,'visible','on')
    else
        set(handles.lgrid,'visible','on')
        set(handles.mgrid,'visible','on')
        set(handles.sgrid,'visible','on')
    end

     set(handles.grabbers{handles.activeindex},'visible','on')
end

%stretch out the scroll bar:
set(handles.hscroll,'position',[357 1 handles.width-373 16],...
    'sliderstep',[.02 cvpos(3)/((handles.prefs.gridsize*handles.wscols)-cvpos(3))],...
    'min',cvpos(3)/2,'max',handles.prefs.gridsize*handles.wscols-cvpos(3)/2,'value',cvpos(3)/2);

%restore leftmarginposition to 355 pixel
leftpos=get(handles.leftmargin,'position');
leftpos(3)=355;
set(handles.leftmargin,'position',leftpos);

toppos=get(handles.topmargin,'position');
set(handles.topmargin,'position',toppos+[0 -35 0 0])

%update workspaceposition
rightpos=get(handles.rightmargin,'position');
toppos=get(handles.topmargin,'position');
botpos=get(handles.bottommargin,'position');

handles.workspacedims=[rightpos(1)-leftpos(1)-leftpos(3),toppos(2)-botpos(2)-botpos(4)];
handles.workspacepos=[leftpos(1)+leftpos(3) botpos(2)+botpos(4) handles.workspacedims(1) handles.workspacedims(2)];

%stretch out the scroll bar:
cvpos=get(handles.currentview,'position');

cvpos(3)=handles.workspacepos(3);
cvpos(4)=handles.workspacepos(4);
set(handles.currentview,'position',cvpos);

set(handles.hscroll,'position',[357 1 handles.width-373 16],...
    'sliderstep',[.02 cvpos(3)/((handles.prefs.gridsize*handles.wscols)-cvpos(3))],...
    'min',cvpos(3)/2,'max',handles.prefs.gridsize*handles.wscols-cvpos(3)/2,'value',cvpos(3)/2);
set(handles.vscroll,'position',[handles.width-16 18 16 handles.height-54],...
    'min',cvpos(4)/2,'max',handles.prefs.gridsize*handles.wsrows-cvpos(4)/2,...
     'sliderstep',[.05 cvpos(4)/((handles.prefs.gridsize*handles.wsrows)-cvpos(4))],...
     'value',handles.prefs.gridsize*handles.wsrows-cvpos(4)/2)

%update handles
guidata(handles.fig,handles);

%move view to new origin
ravemoveview(handles.fig,[],[0 handles.prefs.gridsize*handles.wsrows]);

set(handles.tablemodebutton,'visible','on');
set(handles.showgridbutton,'visible','on');
%set(handles.highlightbutton,'visible','on');
%set(handles.unhidebutton,'visible','on');
set(handles.openglbutton,'visible','on');
set(handles.zoombutton,'visible','on');
set(handles.hideshowbutton,'visible','on');
set(handles.newbutton,'visible','on');
%set(handles.putawaybutton,'visible','on');
set(handles.infobar,'visible','on');

set(handles.fig,'keypressfcn',[]);

function changerenderer_callback(src,ev) %#ok<INUSD>
handles=guidata(src);

if get(src,'value')
    set(handles.fig,'renderer','opengl')
    set(handles.infobar,'string','Renderer set to Open GL')
else
    set(handles.fig,'renderer','painters');
    set(handles.infobar,'string','Renderer set to Painters')
end





