function [xsteps totalrows]=ravecalculatexsteps(src,d,a,x,numpts,includecurrent)
%calculates a sweep of values of variable x that can be used to evaluate a
%function. If x is continuous, outputs "numpts" values. Otherwise outputs
%all discrete values of x.
%
%This helper function is used by almost all continous graphs.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

if nargin<6;
    includecurrent=true;
end

handles=guidata(src);

datatypes=handles.datatype{d}(x);
totalrows=zeros(size(datatypes));
xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};
step=(xmax-xmin)/numpts;
xsteps=cell(1,numel(datatypes));
xdials=handles.xdials{d}{a};

for j=1:numel(datatypes);
    if numpts==1
        xsteps{j}=xdials(x(j));
        totalrows(j)=1;
    else
        
        switch datatypes{j}
            case 'constant'
                xsteps{j}=xdials(x(j));
                totalrows(j)=1;
            case 'continuous'
                if ~includecurrent
                    xsteps{j}=sort([xmin(x(j)):step(x(j)):xmax(x(j))]);
                    totalrows(j)=numel(xsteps{j});
                else
                    xsteps{j}=sort([xmin(x(j)):step(x(j)):xmax(x(j)) xdials(x(j))]);
                    totalrows(j)=numel(xsteps{j});
                end
            case 'integer'
                if numel(handles.discretevalues{d}{a}{x(j)})<20
                    xsteps{j}=handles.discretevalues{d}{a}{x(j)};
                    totalrows(j)=numel(xsteps{j});
                else
                    if ~includecurrent
                        xsteps{j}=sort([xmin(gridcolz(j)):step(x(j)):xmax(x(j))]);
                        totalrows(j)=numel(xsteps{j});
                    else
                        xsteps{j}=sort([xmin(gridcolz(j)):step(x(j)):xmax(x(j)) xdials(x(j))]);
                        totalrows(j)=numel(xsteps{j});
                    end
                end
            case 'discrete'
                xsteps{j}=handles.discretevalues{d}{a}{x(j)};
                totalrows(j)=numel(xsteps{j});
            case 'string'
                xsteps{j}=handles.discretevalues{d}{a}{x(j)};
                totalrows(j)=numel(xsteps{j});
            case 'logical'
                xsteps{j}=[0 1];
                totalrows(j)=numel(xsteps{j});
        end
        
    end
end
