function f=ravechangerangesgui2(d,a,oldhandles)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


%make figure:
f=figure('units','pixels','position',[0 0 651 700],'name',...
    'Define Roles and Data Types','numbertitle','off','menubar','none','resize','off');
handles.gui=f;
origin=get(0,'userdata');
thispos=get(f,'position');
thispos(1)=origin(1)+origin(3)/2-thispos(3)/2;
thispos(2)=thispos(2)+origin(2)+origin(4)/2-thispos(4)/2;
set(f,'position',thispos);

handles.d=d;
handles.a=a;
handles.oldhandles=oldhandles;

%get theme
load raveprefs
handles.prefs=raveprefs;
set(f,'color',raveprefs.colors.frame)


%make initial uicontrols:
handles.background=axes('units','pixels','position',[88 55 506 570],'visible','off','xlimmode','manual','ylimmode','manual');

handles.tabaxes=axes('units','pixels','position',[58 75 30 550],'parent',handles.gui,'visible','off','tag','tabaxes');
handles.titleaxes=axes('units','pixels','position',[11 660 485 41],'parent',handles.gui,'visible','off','tag','titleaxes');

[x,y,v]=calculatetabprofile([1 1.2 .9 1 .8 .9],.1,1,1);
handles.tabbg=patch(x,y,raveprefs.colors.tabbg,'parent',handles.tabaxes,'edgecolor','none');
handles.bg=patch([0 1 1 0],[0 0 1 1],raveprefs.colors.tabbg,'parent',handles.background,'edgecolor','none');

handles.tablines=line(x,y,'parent',handles.tabaxes,'color',[.5 .5 .5]);

handles.constant=text(.2,v(1),' Constant','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',14*raveprefs.fonts.tabscale,...
    'color',raveprefs.colors.text,'vert','cap','buttondownfcn',@constant_tab);
handles.continuous=text(.2,v(2),' Continuous','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',14*raveprefs.fonts.tabscale,...
    'color',raveprefs.colors.tabfg,'vert','cap','buttondownfcn',@continuous_tab);
handles.integer=text(.2,v(3),' Integer','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',14*raveprefs.fonts.tabscale,...
    'color',raveprefs.colors.tabfg,'vert','cap','buttondownfcn',@integer_tab);
handles.discrete=text(.2,v(4),' Discrete','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',14*raveprefs.fonts.tabscale,...
    'color',raveprefs.colors.tabfg,'vert','cap','buttondownfcn',@discrete_tab);
handles.string=text(.2,v(5),' String','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',14*raveprefs.fonts.tabscale,...
    'color',raveprefs.colors.tabfg,'vert','cap','buttondownfcn',@string_tab);
handles.logical=text(.2,v(6),' Logical','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',14*raveprefs.fonts.tabscale,...
    'color',raveprefs.colors.tabfg,'vert','cap','buttondownfcn',@logical_tab);

handles.tabs=[handles.constant handles.continuous handles.integer handles.discrete handles.string handles.logical];

line([0 0 1 1 0],[.355 0 0 1 1],'color',raveprefs.colors.tabfg,'parent',handles.background);


handles.donebutton=uicontrol('style','pushbutton','units','pixels','position',[265 11 156 35],'tag','donebutton',...
    'fontname',raveprefs.fonts.tab,'fontsize',14*raveprefs.fonts.tabscale,'string','Ok','callback',@donebutton_Callback);
handles.cancelbutton=uicontrol('style','pushbutton','units','pixels','position',[438 11 156 35],'tag','donebutton',...
    'fontname',raveprefs.fonts.tab,'fontsize',14*raveprefs.fonts.tabscale,'string','Cancel','callback',@cancelbutton_Callback);


%make titles
set(handles.titleaxes,'color','none','xcolor',raveprefs.colors.frame,...
    'ycolor',raveprefs.colors.frame,'zcolor',raveprefs.colors.frame);


text(0,.2,'Change feasible values of input variables','parent',handles.titleaxes,'fontname',handles.prefs.fonts.title,'fontsize',20*handles.prefs.fonts.titlescale,...
    'color',raveprefs.colors.title,'horizontalalignment','left','verticalalignment','baseline');


%get data from managedatagui
handles.indata=guidata(gcbo); %gcbo = "define roles" button in managedatagui



handles.savebutton=uicontrol('style','pushbutton','units','pix','parent',handles.gui,'position',[320 65 120 26],...
    'string','Save Settings','callback',@savesettings_callback,'enable','off');
handles.loadbutton=uicontrol('style','pushbutton','units','pix','parent',handles.gui,'position',[460 65 120 26],...
    'string','Load Settings','callback',@loadsettings_callback,'enable','off');



% Choose default command line output for ravedefinerolesgui
handles.output = f;

handles.tabobjects=[];

% Update handles structure
guidata(f, handles);


%show first tab:
constant_tab(handles.constant,[]);


function constant_tab(src,ev)
%display uicontrols for defining column roles (constant, continuous, discrete)
handles=guidata(src);
delete(handles.tabobjects);

d=handles.d;
a=handles.a;
a=a(1); %in case multiple are selected

[x,y,v]=calculatetabprofile([1 1.2 .9 1 .8 .9],.1,1,1);
set(handles.tablines,'xdata',x,'ydata',y);
set(handles.tabs,'color',handles.prefs.colors.tabfg);
set(src,'color',handles.prefs.colors.text);



%colformat={'char',{'constant','continuous','integer','discrete','string','logical'}};
varnamez=handles.oldhandles.allnames{d};

thistype=strcmp('constant',handles.oldhandles.datatype{d});

varnamez=varnamez(thistype);
data=handles.oldhandles.xdials{d}{a}(thistype);

data=num2cell(data);

warning('off','MATLAB:uitable:OldTableUsage');

[table, buttons] = createTable (handles.gui,{'Variable Name','     Value      '}, [varnamez' data'], false,...
    'Units','pixels','Position',[120 100 440 460]);
set(table,'datachangedcallback',{@constantchanged_callback,handles.gui})

table=get(table,'UIContainer'); %convert to double handle;

warning('on','MATLAB:uitable:OldTableUsage');


handles.tabobjects=table;
guidata(src,handles);

function continuous_tab(src,ev)
%display uicontrols for defining column roles (constant, continuous, discrete)
handles=guidata(src);
delete(handles.tabobjects);

d=handles.d;
a=handles.a;
a=a(1); %in case multiple are selected

[x,y,v]=calculatetabprofile([1 1.2 .9 1 .8 .9],.1,1,1);
set(handles.tablines,'xdata',x,'ydata',y);
set(handles.tabs,'color',handles.prefs.colors.tabfg);
set(src,'color',handles.prefs.colors.text);


varnamez=handles.oldhandles.allnames{d};

thistype=strcmp('continuous',handles.oldhandles.datatype{d});

varnamez=varnamez(thistype);
data=[handles.oldhandles.minlimit{d}{a}(thistype);handles.oldhandles.maxlimit{d}{a}(thistype)];

data=num2cell(data);

warning('off','MATLAB:uitable:OldTableUsage');

[table, buttons] = createTable (handles.gui,{'Variable Name            ','   Min Value    ','    Max Value    '}, [varnamez' data'], false,...
    'Units','pixels','Position',[120 100 440 460]);
set(table,'datachangedcallback',{@continuouschanged_callback,handles.gui})

table=get(table,'UIContainer'); %convert to double handle;

warning('on','MATLAB:uitable:OldTableUsage');


handles.tabobjects=table;
guidata(src,handles);


function integer_tab(src,ev)
%display uicontrols for defining column roles (constant, continuous, discrete)
handles=guidata(src);
delete(handles.tabobjects);


d=handles.d;
a=handles.a;
a=a(1); %in case multiple are selected

[x,y,v]=calculatetabprofile([1 1.2 .9 1 .8 .9],.1,1,1);
set(handles.tablines,'xdata',x,'ydata',y);
set(handles.tabs,'color',handles.prefs.colors.tabfg);
set(src,'color',handles.prefs.colors.text);



%colformat={'char',{'constant','continuous','integer','discrete','string','logical'}};
varnamez=handles.oldhandles.allnames{d};

thistype=strcmp('integer',handles.oldhandles.datatype{d});

varnamez=varnamez(thistype);
data=[handles.oldhandles.minlimit{d}{a}(thistype);handles.oldhandles.maxlimit{d}{a}(thistype)];


data=num2cell(data);

warning('off','MATLAB:uitable:OldTableUsage');

[table, buttons] = createTable (handles.gui,{'Variable Name            ','   Min Value    ','    Max Value    '}, [varnamez' data'], false,...
    'Units','pixels','Position',[120 100 440 460]);
set(table,'datachangedcallback',{@integerchanged_callback,handles.gui})

table=get(table,'UIContainer'); %convert to double handle;

warning('on','MATLAB:uitable:OldTableUsage');


handles.tabobjects=table;
guidata(src,handles);


function discrete_tab(src,ev)
%display uicontrols for defining column roles (constant, continuous, discrete)
handles=guidata(src);
delete(handles.tabobjects);


d=handles.d;
a=handles.a;
a=a(1); %in case multiple are selected

[x,y,v]=calculatetabprofile([1 1.2 .9 1 .8 .9],.1,1,1);
set(handles.tablines,'xdata',x,'ydata',y);
set(handles.tabs,'color',handles.prefs.colors.tabfg);
set(src,'color',handles.prefs.colors.text);



%colformat={'char',{'constant','continuous','integer','discrete','string','logical'}};
varnamez=handles.oldhandles.allnames{d};

thistype=strcmp('discrete',handles.oldhandles.datatype{d});


varnamez=varnamez(thistype);
tempdata=handles.oldhandles.discretevalues{d}{a}(thistype);

datasizes=cellfun(@size,tempdata','UniformOutput',0);

datasizes=cell2mat(datasizes);
if any(thistype)
    data=cell(max(datasizes(:,1)),numel(varnamez));
    for v=1:numel(varnamez)
        data(1:datasizes(v,1),v)=num2cell(tempdata{v});
    end
else
    data=cell(0);
end


warning('off','MATLAB:uitable:OldTableUsage');

[table, buttons] = createTable (handles.gui,varnamez,data, false,...
    'Units','pixels','Position',[120 100 440 460]);

if any(thistype)
    set(table,'NumRows',max(datasizes(:,1))+10)
    set(table,'datachangedcallback',{@discretechanged_callback,handles.gui})
end

table=get(table,'UIContainer'); %convert to double handle;

warning('on','MATLAB:uitable:OldTableUsage');


handles.tabobjects=table;


guidata(src,handles);


function string_tab(src,ev)
%display uicontrols for defining column roles (constant, continuous, discrete)
handles=guidata(src);
delete(handles.tabobjects);


d=handles.d;
a=handles.a;
a=a(1); %in case multiple are selected

[x,y,v]=calculatetabprofile([1 1.2 .9 1 .8 .9],.1,1,1);
set(handles.tablines,'xdata',x,'ydata',y);
set(handles.tabs,'color',handles.prefs.colors.tabfg);
set(src,'color',handles.prefs.colors.text);



%colformat={'char',{'constant','continuous','integer','discrete','string','logical'}};
varnamez=handles.oldhandles.allnames{d};

thistype=strcmp('string',handles.oldhandles.datatype{d});

varnamez=varnamez(thistype);
tempdata=handles.oldhandles.discretevalues{d}{a}(thistype);

datasizes=cellfun(@size,tempdata','UniformOutput',0);

datasizes=cell2mat(datasizes);

if any(thistype)
    data=cell(max(datasizes(:,1)),numel(varnamez));

    for v=1:numel(varnamez)
        data(1:datasizes(v,1),v)=tempdata{v};
    end
else
    data=cell(0);
end


warning('off','MATLAB:uitable:OldTableUsage');

[table, buttons] = createTable (handles.gui,varnamez,data, false,...
    'Units','pixels','Position',[120 100 440 460]);

if any(thistype)
    set(table,'NumRows',max(datasizes(:,1))+10)
    set(table,'datachangedcallback',{@stringchanged_callback,handles.gui})
end

table=get(table,'UIContainer'); %convert to double handle;

warning('on','MATLAB:uitable:OldTableUsage');

handles.tabobjects=table;
guidata(src,handles);


function logical_tab(src,ev)
%display uicontrols for defining column roles (constant, continuous, discrete)
handles=guidata(src);
delete(handles.tabobjects);


d=handles.d;
a=handles.a;
a=a(1); %in case multiple are selected

[x,y,v]=calculatetabprofile([1 1.2 .9 1 .8 .9],.1,1,1);
set(handles.tablines,'xdata',x,'ydata',y);
set(handles.tabs,'color',handles.prefs.colors.tabfg);
set(src,'color',handles.prefs.colors.text);



%colformat={'char',{'constant','continuous','integer','discrete','string','logical'}};
varnamez=handles.oldhandles.allnames{d};

thistype=strcmp('logical',handles.oldhandles.datatype{d});

varnamez=varnamez(thistype);
falsedata=handles.oldhandles.minlimit{d}{a}(thistype);
falsedata=~falsedata;
truedata=logical(handles.oldhandles.maxlimit{d}{a}(thistype));


data=num2cell([falsedata;truedata]);

warning('off','MATLAB:uitable:OldTableUsage');

[table, buttons] = createTable (handles.gui,{'Variable Name            ','   False    ','    True    '}, [varnamez' data'], false,...
    'Units','pixels','Position',[120 100 440 460]);
set(table,'datachangedcallback',{@logicalchanged_callback,handles.gui})
table.setCheckBoxEditor(2);
table.setCheckBoxEditor(3);


table=get(table,'UIContainer'); %convert to double handle;

warning('on','MATLAB:uitable:OldTableUsage');


handles.tabobjects=table;
guidata(src,handles);









function donebutton_Callback(src, eventdata)
handles=guidata(src);

set(0,'userdata',handles.oldhandles);
close(handles.gui);

function cancelbutton_Callback(src,ev)
close(gcbf);



function constantchanged_callback(jsrc,jev,src)

handles=guidata(src);

%determine which cell is selected:
eventDetails = jev.getEvent;

c=eventDetails.getColumn+1;
r=eventDetails.getFirstRow+1;

data = jsrc.getData;
newvalue = data(r,c);

if ischar(newvalue);newvalue=str2double(newvalue);end

d=handles.d;
a=handles.a;
datatypes=handles.oldhandles.datatype{d};
thistype=find(strcmp('constant',datatypes));

index=thistype(r);

[handles.oldhandles.xdials{d}{a}(index)]=deal(newvalue);
[handles.oldhandles.minlimit{d}{a}(index)]=deal(newvalue);
[handles.oldhandles.maxlimit{d}{a}(index)]=deal(newvalue);

guidata(src,handles);



function continuouschanged_callback(jsrc,jev,src)

handles=guidata(src);

%determine which cell is selected:
eventDetails = jev.getEvent;

c=eventDetails.getColumn+1;
r=eventDetails.getFirstRow+1;

data = jsrc.getData;
newmin = data(r,2);
newmax = data(r,3);

if ischar(newmin);newvalue=str2double(newmin);end
if ischar(newmax);newvalue=str2double(newmax);end

if newmin>newmax
    errordlg('Minimum value must be less than or equal to maximum value','You can''t do that');
    return
end

d=handles.d;
a=handles.a;
datatypes=handles.oldhandles.datatype{d};
thistype=find(strcmp('continuous',datatypes));

index=thistype(r);

[handles.oldhandles.minlimit{d}{a}(index)]=deal(newvalue);
[handles.oldhandles.maxlimit{d}{a}(index)]=deal(newvalue);

guidata(src,handles);


function integerchanged_callback(jsrc,jev,src)

handles=guidata(src);

%determine which cell is selected:
eventDetails = jev.getEvent;

c=eventDetails.getColumn+1;
r=eventDetails.getFirstRow+1;

data = jsrc.getData;
newmin = data(r,2);
newmax = data(r,3);

if ischar(newmin);newvalue=str2double(newmin);end
if ischar(newmax);newvalue=str2double(newmax);end

if newmin>newmax
    errordlg('Minimum value must be less than or equal to maximum value','You can''t do that');
    return
end

d=handles.d;
a=handles.a;
datatypes=handles.oldhandles.datatype{d};
thistype=find(strcmp('integer',datatypes));

index=thistype(r);

[handles.oldhandles.minlimit{d}{a}(index)]=deal(newvalue);
[handles.oldhandles.maxlimit{d}{a}(index)]=deal(newvalue);
[handles.oldhandles.discretevalues{d}{a}{index}]=deal(newmin:newmax);


guidata(src,handles);


function discretechanged_callback(jsrc,jev,src)


handles=guidata(src);

%determine which cell is selected:
eventDetails = jev.getEvent;

c=eventDetails.getColumn+1;
r=eventDetails.getFirstRow+1;

if c==0;return;end %for some reason this runs when table is created with c=0

data = jsrc.getData;


newdata=cell(data(:,c)); %cell converts to matlab object

for j=1:numel(newdata)
    if ischar(newdata{j})
        try
            newdata{j}=str2double(newdata{j});
        catch %#ok<CTCH>
            newdata{j}=[]; %if they didnt enter a number
        end
    end
end

newdata=cell2mat(newdata);
newdata=unique(newdata); %in case of repetitions

d=handles.d;
a=handles.a;
datatypes=handles.oldhandles.datatype{d};
thistype=find(strcmp('discrete',datatypes));

index=thistype(c);

[handles.oldhandles.minlimit{d}{a}(index)]=deal(min(newdata));
[handles.oldhandles.maxlimit{d}{a}(index)]=deal(max(newdata));
[handles.oldhandles.discretevalues{d}{a}{index}]=deal(newdata);

guidata(src,handles);


function stringchanged_callback(jsrc,jev,src)


handles=guidata(src);

%determine which cell is selected:
eventDetails = jev.getEvent;

c=eventDetails.getColumn+1;
r=eventDetails.getFirstRow+1;

if c==0;return;end %for some reason this runs when table is created with c=0

data = jsrc.getData;


newdata=cell(data(:,c)); %cell converts to matlab object
charz=cellfun(@ischar,newdata);

%make charz true for all values that appear in alldata, even if their
%string is now empty:


d=handles.d;
a=handles.a;
datatypes=handles.oldhandles.datatype{d};
thistype=find(strcmp('string',datatypes));
index=thistype(c);

truevalues=unique(handles.oldhandles.alldata{d}(:,index));

charz(truevalues)=true;

newdata=newdata(charz);

%updata alltext to reflect new names:
handles.oldhandles.alltext{d}(:,index)=newdata(handles.oldhandles.alldata{d}(:,index));
%[handles.oldhandles.minlimit{d}{a}(index)]=deal(min(newdata)); %minlimit is always 1 for strings
[handles.oldhandles.maxlimit{d}{a}(index)]=deal(numel(newdata));
[handles.oldhandles.discretevalues{d}{a}{index}]=deal(newdata);

guidata(src,handles);


function logicalchanged_callback(jsrc,jev,src)

handles=guidata(src);

%determine which cell is selected:
eventDetails = jev.getEvent;

c=eventDetails.getColumn+1;
r=eventDetails.getFirstRow+1;

data = jsrc.getData;


newmin = data(r,2);
newmax = data(r,3);


if ~newmin && ~newmax
    errordlg('You must enable at least one option for each variable','You can''t do that');
    return
end

d=handles.d;
a=handles.a;
datatypes=handles.oldhandles.datatype{d};
thistype=find(strcmp('logical',datatypes));

index=thistype(r);

if newmin
    [handles.oldhandles.minlimit{d}{a}(index)]=deal(0);
else
    [handles.oldhandles.minlimit{d}{a}(index)]=deal(1);
end

if ~newmax
    [handles.oldhandles.maxlimit{d}{a}(index)]=deal(0);
else
    [handles.oldhandles.maxlimit{d}{a}(index)]=deal(1);
end

guidata(src,handles);




