function ravecreatemodelgui(src,ev)
%gui to create a new metamodel.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
load raveprefs

origin=get(handles.fig,'position');

hhh=figure('menubar','none','units','pixels','position',[origin(1)+origin(3)/2-600 origin(2)+origin(4)/2-400 1200 800],'color',raveprefs.colors.frame,...
    'name','Create New Model','numbertitle','off','resize','off','windowstyle','normal');

titleaxes=axes('units','pixels','position',[20 750 400 40],'parent',hhh,'visible','off');
text(0,.4,'Fit a model to your data','parent',titleaxes,'fontname',handles.prefs.fonts.title,'fontsize',20*handles.prefs.fonts.titlescale,...
    'color',raveprefs.colors.title,'horizontalalignment','left','verticalalignment','baseline');


u=uipanel('units','pixels','position',[50 300+40 1100 410],'title','','bordertype','etchedin','backgroundcolor',[1 1 1]);

uicontrol('style','text','parent',u,'units','pixels','position',[10 420-40 200 22],'string','Select a data set:','backgroundcolor',get(u,'backgroundcolor'),...
    'fontsize',10,'fontname','arial','fontweight','bold','horizontalalign','left')

uicontrol('style','popup','parent',u,'units','pixels','position',[20 400-40 200 20],'string',handles.datasetnames,...
    'backgroundcolor',get(u,'backgroundcolor'),'tag','datasetlist')


h1axes=axes('parent',u,'units','pixels','position',[10 360-40 200 35],'visible','off');
text(0,0,'Variables to Model','parent',h1axes,'fontname',raveprefs.fonts.tab,'fontsize',18*raveprefs.fonts.tabscale,'color',handles.colors.text,...
    'hor','left','ver','baseline')
uicontrol('style','listbox','parent',u,'units','pixels','position',[20 50-40 200 300],'string',handles.allnames{1},...
    'backgroundcolor',get(u,'backgroundcolor'),'max',100,'tag','responselist')

h2axes=axes('parent',u,'units','pixels','position',[260 360-40 200 35],'visible','off');
text(0,0,'Input Variables','parent',h2axes,'fontname',raveprefs.fonts.tab,'fontsize',18*raveprefs.fonts.tabscale,'color',handles.colors.text,...
    'hor','left','ver','baseline')
uicontrol('style','listbox','parent',u,'units','pixels','position',[270 50-40 200 300],'string',handles.allnames{1},...
    'backgroundcolor',get(u,'backgroundcolor'),'max',100,'tag','inputlist')

h3axes=axes('parent',u,'units','pixels','position',[260+250 360-40 200 35],'visible','off');
text(0,0,'Model Type & Settings','parent',h3axes,'fontname',raveprefs.fonts.tab,'fontsize',18*raveprefs.fonts.tabscale,'color',handles.colors.text,...
    'hor','left','ver','baseline')
uicontrol('style','popup','parent',u,'units','pixels','position',[270+250 350-22-40 200 22],'string',{'Polynomial (RSE)';'Feed-Forward Neural Net (MLP)';'Radial Basis Function';'Gaussian Process';'Self-Organizing Map';'Support Vector Machine'},...
    'backgroundcolor',get(u,'backgroundcolor'),'callback',@makemethodcontrols,'value',1)

h4axes=axes('parent',u,'units','pixels','position',[835-10 320 200 35],'visible','off');
text(0,0,'Fit & Validation Data','parent',h4axes,'fontname',raveprefs.fonts.tab,'fontsize',18*raveprefs.fonts.tabscale,'color',handles.colors.text,...
    'hor','left','ver','baseline')

uicontrol('style','text','Position',[840 294 200 20],'string','Choose Fit-Data Column:','fontsize',10,'fontweight','bold',...
    'hor','left','backgroundcolor',[1 1 1],'parent',u);
uicontrol('style','popup','parent',u,'units','pixels','position',[865-10 276 200 20],'string',handles.allnames{1},...
    'backgroundcolor',get(u,'backgroundcolor'))
uicontrol('style','text','Position',[855 252 200 20],'string','Fit model to rows whose value is:','fontsize',10,'fontweight','normal',...
    'hor','left','backgroundcolor',[1 1 1],'parent',u);
uicontrol('style','edit','Position',[870 234 150 20],'string','0',...
    'hor','center','backgroundcolor',[1 1 1],'tag','','parent',u);
uicontrol('style','text','Position',[855 208 240 20],'string','Validate model with rows whose value is:','fontsize',10,'fontweight','normal',...
    'hor','left','backgroundcolor',[1 1 1],'parent',u);
uicontrol('style','edit','Position',[870 190 150 20],'string','1',...
    'hor','center','backgroundcolor',[1 1 1],'tag','','parent',u);
uicontrol('style','text','Position',[855 208-44 240 20],'string','Test model with rows whose value is:','fontsize',10,'fontweight','normal',...
    'hor','left','backgroundcolor',[1 1 1],'parent',u);
uicontrol('style','edit','Position',[870 190-44 150 20],'string','2',...
    'hor','center','backgroundcolor',[1 1 1],'tag','','parent',u);

uicontrol('style','text','Position',[840 150-30 200 20],'string','Or...','fontsize',10,'fontweight','bold',...
    'hor','left','backgroundcolor',[1 1 1],'parent',u);
uicontrol('style','pushbutton','parent',u,'units','pixels','position',[880-10 110-30 210 40],'string','Make New Fit-Data Column',...
    'fontsize',12,'enable','on','callback',@ravemakenewfitcolumn)

uicontrol('style','text','Position',[840 50 200 20],'string','All set?','fontsize',12,'fontweight','bold',...
    'hor','left','backgroundcolor',[1 1 1],'parent',u);
uicontrol('style','pushbutton','parent',u,'units','pixels','position',[880-10 10 210 40],'string','Add Model(s) to Queue',...
    'fontsize',12,'enable','on','foregroundcolor',[0 .8 0],'callback',@addmodeltoqueue)

makemethodcontrols(src,[]);

a=axes('units','pixels','parent',hhh,'position',[15 320 1170 10],'visible','off');
line([0 1],[.5 .5],'parent',a,'color',[0 0 0],'linewidth',1);

tabz=uitable(hhh,'position',[25 15 1050 300],'columnname',{'Response','Model Type','Status','Effects'},'ColumnWidth',{200 200 100 'auto'});


gobutton=uicontrol('style','pushbutton','parent',hhh,'position',[1090 130 90 90],'string',{'Create';'Models'},'fontsize',12*raveprefs.fonts.tabscale,'enable','on','foregroundcolor',[0 .8 0]);

guidata(hhh,handles);


function makemethodcontrols(src,ev)

contrlz=findobj(ancestor(src,'figure'),'userdata','methodcontrol');
delete(contrlz);

method=get(src,'value');

if method==1 %Polynomail RSE
    ui(1)=uicontrol('style','text','Position',[570 600 300 20],'string','Effects:','fontsize',10,'fontweight','bold',...
        'hor','left','backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(2)=uicontrol('style','text','Position',[590 580 300 20],'string','Polynomial degree:','fontsize',10,'fontweight','normal',...
        'hor','left','backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(3)=uicontrol('style','edit','Position',[710 580 20 20],'string','2',...
        'hor','center','backgroundcolor',[1 1 1],'tag','degreeedit','userdata','methodcontrol');
    
    ui(4)=uicontrol('style','checkbox','Position',[590 560 300 20],'string','Include cross terms','fontsize',10,'fontweight','normal',...
        'hor','left','backgroundcolor',[1 1 1],'value',1,'tag','includecrosscheck','userdata','methodcontrol');
    
    ui(5)=uicontrol('style','text','Position',[570 530 300 20],'string','Responses:','fontsize',10,'fontweight','bold',...
        'hor','left','backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(6)=uicontrol('style','text','Position',[590 510 300 20],'string','Before fitting, transform by: ','fontsize',10,'fontweight','normal',...
        'hor','left','backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(7)=uicontrol('style','popup','position',[750 512 100 20],'string',{'none';'exp(y)';'log(y)';'1/y'},...
        'backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(8)=uicontrol('style','text','Position',[570 480 300 20],'string','Post Processing:','fontsize',10,'fontweight','bold',...
        'hor','left','backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(9)=uicontrol('style','text','Position',[590 460 300 20],'string','Save only largest         effects','fontsize',10,'fontweight','normal',...
        'hor','left','backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(10)=uicontrol('style','edit','Position',[693 462 30 20],'string','inf','fontsize',10,'fontweight','normal',...
        'hor','center','backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(11)=uicontrol('style','text','Position',[590 438 300 20],'string','Exclude effects with t-ratio larger than','fontsize',10,'fontweight','normal',...
        'hor','left','backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(12)=uicontrol('style','edit','Position',[813 440 30 20],'string','.999','fontsize',10,'fontweight','normal',...
        'hor','center','backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(13)=uicontrol('style','checkbox','Position',[590 418 300 20],'string','Optimize equations for MATLAB','fontsize',10,'fontweight','normal',...
        'hor','left','backgroundcolor',[1 1 1],'value',1,'tag','includecrosscheck','userdata','methodcontrol');
    
elseif method==2 %feed forward nn
    
    ui(1)=uicontrol('style','text','Position',[570 600 300 20],'string','Topology:','fontsize',10,'fontweight','bold',...
        'hor','left','backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(2)=uicontrol('style','text','Position',[590 580 300 20],'string','Hidden nodes:','fontsize',10,'fontweight','normal',...
        'hor','left','backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(3)=uicontrol('style','edit','Position',[680 580 30 20],'string','5',...
        'hor','center','backgroundcolor',[1 1 1],'tag','degreeedit','userdata','methodcontrol');
    
    ui(4)=uicontrol('style','text','Position',[590 554 300 20],'string','Transfer fcn:','fontsize',10,'fontweight','normal',...
        'hor','left','backgroundcolor',[1 1 1],'value',1,'tag','includecrosscheck','userdata','methodcontrol');
    
    ui(5)=uicontrol('style','popup','position',[670 554 80 20],'string',{'logsig';'tansig'},...
        'backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(6)=uicontrol('style','text','Position',[570 530 300 20],'string','Training algorithm:','fontsize',10,'fontweight','bold',...
        'hor','left','backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(7)=uicontrol('style','popup','position',[590 512 200 20],'string',{'Bayesian Regularization';...
        'Levenburg Marquardt';'One-Step Secant (Quasi-Newton)';'BFGS (Quasi-Newton)';'Backtracking';'Charalambous Search';'Hybrid Bisection-Cubic Search';...
        'Brent''s Search';'Golden Section';'Scaled Conjugate Gradient';'Powell-Beale Restarts';'Polak-Ribiere Update';'Fletcher-Reeves Update';...
        'Resilient Backpropogation';'Adaptive Rate Gradient Descent w/ Momentum';'Adaptive Rate Gradient Descent';'Gradient Descent w/ Momentum';'Gradient Descent'},...
        'backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(8)=uicontrol('style','text','Position',[590 483 300 20],'string','Max epochs:','fontsize',10,'fontweight','normal',...
        'hor','left','backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(9)=uicontrol('style','edit','Position',[673 485 50 20],'string','5000','fontsize',10,'fontweight','normal',...
        'hor','center','backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(10)=uicontrol('style','text','Position',[590 460 300 20],'string','Max training time (s):','fontsize',10,'fontweight','normal',...
        'hor','left','backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(11)=uicontrol('style','edit','Position',[723 462 40 20],'string','inf','fontsize',10,'fontweight','normal',...
        'hor','center','backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(12)=uicontrol('style','text','Position',[590 460-23 300 20],'string','Iterations:','fontsize',10,'fontweight','normal',...
        'hor','left','backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(13)=uicontrol('style','edit','Position',[656 460-23 40 20],'string','1','fontsize',10,'fontweight','normal',...
        'hor','center','backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(14)=uicontrol('style','text','Position',[570 440-23 300 20],'string','Improved Generalization:','fontsize',10,'fontweight','bold',...
        'hor','left','backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
    ui(15)=uicontrol('style','checkbox','Position',[590 418-23 300 20],'string','Regularization','fontsize',10,'fontweight','normal',...
        'hor','left','backgroundcolor',[1 1 1],'value',0,'tag','includecrosscheck','userdata','methodcontrol');
    
    ui(16)=uicontrol('style','checkbox','Position',[590 418-20-23 300 20],'string','Early Stopping','fontsize',10,'fontweight','normal',...
        'hor','left','backgroundcolor',[1 1 1],'value',0,'tag','includecrosscheck','userdata','methodcontrol');
    
    ui(17)=uicontrol('style','edit','Position',[700 418-23 30 20],'string','0.5','fontsize',10,'fontweight','normal',...
        'hor','center','backgroundcolor',[1 1 1],'userdata','methodcontrol');
    
end

%
%
function addmodeltoqueue(src,ev)
handles=guidata(src);
tfig=ancestor(src,'figure');

datasetlist=findobj(tfig,'tag','datasetlist');
responselist=findobj(tfig,'tag','responselist');
inputlist=findobj(tfig,'tag','inputlist');






