function f=ravedefinerolesgui
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

%make figure:
f=figure('units','pixels','position',[0 0 651 534],'name',...
    'Define Roles and Data Types','numbertitle','off','menubar','none','resize','off');
handles.gui=f;
origin=get(0,'userdata');
thispos=get(f,'position');
thispos(1)=origin(1)+origin(3)/2-thispos(3)/2;
thispos(2)=thispos(2)+origin(2)+origin(4)/2-thispos(4)/2;
set(f,'position',thispos);

%make tabs:

%get theme
load raveprefs
handles.prefs=raveprefs;
set(f,'color',raveprefs.colors.frame)

%make initial uicontrols:
handles.background=axes('units','pixels','position',[88 55 506 427],'visible','off','xlimmode','manual','ylimmode','manual');
   
handles.tabaxes=axes('units','pixels','position',[58 207 30 275],'parent',handles.gui,'visible','off','tag','tabaxes');  
handles.titleaxes=axes('units','pixels','position',[11 489 485 41],'parent',handles.gui,'visible','off','tag','titleaxes');




[x,y,v]=calculatetabprofile([1.4 .8],.1,2,1); 
handles.tabbg=patch(x,y,raveprefs.colors.tabbg,'parent',handles.tabaxes,'edgecolor','none');
handles.bg=patch([0 1 1 0],[0 0 1 1],raveprefs.colors.tabbg,'parent',handles.background,'edgecolor','none');

handles.tablines=line(x,y,'parent',handles.tabaxes,'color',[.5 .5 .5]);

handles.types=text(.2,v(1),'  Data Types','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',18*raveprefs.fonts.tabscale,...
    'color',raveprefs.colors.text,'vert','cap','buttondownfcn',@types_callback);
handles.roles=text(.2,v(2),'  Roles','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',18*raveprefs.fonts.tabscale,...
    'color',raveprefs.colors.tabfg,'vert','cap','buttondownfcn','');

line([0 0 1 1 0],[.355 0 0 1 1],'color',raveprefs.colors.tabfg,'parent',handles.background);


handles.donebutton=uicontrol('style','pushbutton','units','pixels','position',[265 11 156 35],'tag','donebutton',...
    'fontname',raveprefs.fonts.tab,'fontsize',14*raveprefs.fonts.tabscale,'string','Ok','callback',@donebutton_Callback);
handles.cancelbutton=uicontrol('style','pushbutton','units','pixels','position',[438 11 156 35],'tag','donebutton',...
        'fontname',raveprefs.fonts.tab,'fontsize',14*raveprefs.fonts.tabscale,'string','Cancel','callback',@cancelbutton_Callback);
    

%make titles
set(handles.titleaxes,'color','none','xcolor',raveprefs.colors.frame,...
    'ycolor',raveprefs.colors.frame,'zcolor',raveprefs.colors.frame);


text(0,.2,'Define the role of each column','parent',handles.titleaxes,'fontname',handles.prefs.fonts.title,'fontsize',20*handles.prefs.fonts.titlescale,...
   'color',raveprefs.colors.title,'horizontalalignment','left','verticalalignment','baseline');


%get data from managedatagui
handles.indata=guidata(gcbo); %gcbo = "define roles" button in managedatagui


% Choose default command line output for ravedefinerolesgui
handles.output = f;

handles.tabobjects=[];

% Update handles structure
guidata(f, handles);


%show data types tab:

types_callback(handles.types,[]);





function roles_callback(src,ev)
%display uicontrols for defining column roles (input or response)
handles=guidata(src);
delete(handles.tabobjects);


[x,y,v]=calculatetabprofile([1.4 .8],.1,2,1);
set(handles.tablines,'xdata',x,'ydata',y);

set(src,'color',handles.prefs.colors.text);
set(handles.types,'color',handles.prefs.colors.tabfg);

%make uicontrols:
makex=uicontrol('style','pushbutton','units','pixels','position',[280+88 15+55 200 37],'parent',handles.gui,'callback',@makex_callback,...
    'string','<<< Make These Independent');
makey=uicontrol('style','pushbutton','units','pixels','position',[28+88 15+55 200 37],'parent',handles.gui,'callback',@makey_callback,...
    'string','Make These Responses >>>');

xbox=uicontrol('style','listbox','units','pixels','position',[28+88 56+55 200 323],'parent',handles.gui,'max',100,'backgroundcolor',[1 1 1],'tag','xbox');
ybox=uicontrol('style','listbox','units','pixels','position',[280+88 56+55 200 323],'parent',handles.gui,'max',100,'backgroundcolor',[1 1 1],'tag','ybox');

leftaxes=axes('units','pixels','position',[29+88 381+55 200 33],'parent',handles.gui,'visible','off');
rightaxes=axes('units','pixels','position',[281+88 382+55 200 33],'parent',handles.gui,'visible','off');

handles.tabobjects=[makex makey xbox ybox leftaxes rightaxes];


load raveprefs;

text(0,.2,'Independent','parent',leftaxes,'fontname',raveprefs.fonts.tab,'fontsize',18*raveprefs.fonts.tabscale,...
   'color',handles.prefs.colors.text,'horizontalalignment','left','verticalalignment','baseline');
text(0,.2,'Response','parent',rightaxes,'fontname',raveprefs.fonts.tab,'fontsize',18*raveprefs.fonts.tabscale,...
   'color',handles.prefs.colors.text,'horizontalalignment','left','verticalalignment','baseline');



%find current data set:
d=get(handles.indata.listbox1,'value');


names=handles.indata.indata.allnames{d};
handles.names=names; %make accessible to the done button

set(xbox,'string',names) %load all data into x box first
set(ybox,'string',[]) %make ybox empty

set(xbox,'max',length(names))
set(ybox,'max',length(names))

handles.exes=1:1:length(names);
handles.whys=[];

set(xbox,'value',[]);
set(ybox,'value',[]);



guidata(src,handles);



function types_callback(src,ev)
%display uicontrols for defining column roles (constant, continuous, discrete)
handles=guidata(src);
delete(handles.tabobjects);

[x,y,v]=calculatetabprofile([1.4 .8],.1,1,1);
set(handles.tablines,'xdata',x,'ydata',y);

set(src,'color',handles.prefs.colors.text);
set(handles.roles,'color',handles.prefs.colors.tabfg);

d=get(handles.indata.listbox1,'value');


data=handles.indata.indata.datatype{d};





%the data types are:


%convert data to indices:

colformat={'char',{'constant','continuous','integer','discrete','string','logical'}};
varnamez=handles.indata.indata.allnames{d};

% try
% table=uitable('Parent',handles.gui,'Units','pixels','Position',[150 100 300 300],...
%     'Data',[varnamez' data'],'ColumnName',{'Variable Name','     Data Type      '},'columnformat',colformat,...
%     'columneditable',true,'celleditcallback',@tablevaluechanged_callback);
% catch

warning('off','MATLAB:uitable:OldTableUsage');


   [table, buttons] = createTable (handles.gui,{'Variable Name','     Data Type      '}, [varnamez' data'], false,...
       'Units','pixels','Position',[150 100 300 300]);
   set(table,'datachangedcallback',{@oldtablevaluechanged_callback,handles.gui})
   
   jtable=table.getTable;
   cb = javax.swing.JComboBox({'constant','continuous','integer','discrete','string','logical'});
   cb.setEditable(false);  % prepare an editable drop-down CellEditor
   editor = javax.swing.DefaultCellEditor(cb);
   jtable.getColumnModel.getColumn(1).setCellEditor(editor);  % assign this editor to second column (see Note 2)
   
   table=get(table,'UIContainer'); %convert to double handle;
   
   
   warning('on','MATLAB:uitable:OldTableUsage');
    
%end

%{
maxname=size(char(varnamez),2);
newheaderwidth=maxname*7;

h=findjobj('class','com.mathworks.hg.peer.ui.table.RowHeader'); %find the stupid header object;
k=h.getParent; %get its parent viewport
 
drawnow;
 
%determine how tall it should be (don't change this value)
dimz=h.getSize;
height=dimz.height;

k.setPreferredSize(java.awt.Dimension(newheaderwidth,height));
k.setSize(java.awt.Dimension(newheaderwidth,height));
h.setSize(java.awt.Dimension(newheaderwidth,height));

%}

strnng={'Use popup menus to select the data type for each variable.';...
    'Note: Allowable discrete values are defined by the Analysis, and must be set in the Manage Analyses window.';...
    'The save button here will save data types AND allowable values for use in future sessions.'};
    
    
    infotext=uicontrol('style','text','units','pix','parent',handles.gui,'position',[110 410 450 60],...
        'backgroundcolor',[1 1 1],'string',strnng,'hor','left');
    
    
    savebutton=uicontrol('style','pushbutton','units','pix','parent',handles.gui,'position',[460 150 120 26],...
        'string','Save Settings','callback',@savesettings_callback,'enable','off');
    loadbutton=uicontrol('style','pushbutton','units','pix','parent',handles.gui,'position',[460 100 120 26],...
        'string','Load Settings','callback',@loadsettings_callback,'enable','off');
    
handles.tabobjects=[table,infotext,savebutton,loadbutton];



guidata(src,handles);


function oldtablevaluechanged_callback(jsrc,jev,src)
handles=guidata(src);

%determine which cell is selected:
eventDetails = jev.getEvent;

c=eventDetails.getColumn+1;
r=eventDetails.getFirstRow+1;

data = jsrc.getData;
newvalue = data(r,c);

d=get(handles.indata.listbox1,'value');
handles.indata.indata.datatype{d}{r}=newvalue;

switch newvalue
    case 'constant'
        minval=min(handles.indata.indata.alldata{d}(:,r));
        
        [handles.indata.indata.isconstant{d}(r)]=deal(true);
        [handles.indata.indata.minlimit{d}{:}(r)]=deal(minval);
        [handles.indata.indata.maxlimit{d}{:}(r)]=deal(minval);
        [handles.indata.indata.discretevalues{d}{:}(r)]=deal({[]});
        [handles.indata.indata.xdials{d}{:}(r)]=deal(minval);
        
    case 'continuous'
        minval=min(handles.indata.indata.alldata{d}(:,r));
        maxval=max(handles.indata.indata.alldata{d}(:,r));
        
        handles.indata.indata.isconstant{d}(r)=deal(false);
        [handles.indata.indata.minlimit{d}{:}(r)]=deal(minval);
        [handles.indata.indata.maxlimit{d}{:}(r)]=deal(maxval);
        [handles.indata.indata.discretevalues{d}{:}(r)]=deal({[]});
        
    case 'integer'
        minval=floor(min(handles.indata.indata.alldata{d}(:,r)));
        maxval=ceil(max(handles.indata.indata.alldata{d}(:,r)));
        
        handles.indata.indata.isconstant{d}(r)=deal(false);
        [handles.indata.indata.minlimit{d}{:}(r)]=deal(minval);
        [handles.indata.indata.maxlimit{d}{:}(r)]=deal(maxval);
        [handles.indata.indata.discretevalues{d}{:}(r)]=deal({minval:maxval});
        [handles.indata.indata.xdials{d}{:}(r)]=deal(minval);
        
        
    case 'discrete'
        %minval=floor(min(handles.indata.indata.alldata{d}(:,r)));
        %maxval=ceil(max(handles.indata.indata.alldata{d}(:,r)));
        uniquez=unique(handles.indata.indata.alldata{d}(:,r));        
        minval=min(uniquez);
        maxval=max(uniquez);
        
        handles.indata.indata.isconstant{d}(r)=deal(false);
        [handles.indata.indata.minlimit{d}{:}(r)]=deal(minval);
        [handles.indata.indata.maxlimit{d}{:}(r)]=deal(maxval);
        [handles.indata.indata.discretevalues{d}{:}(r)]=deal({uniquez});
        [handles.indata.indata.xdials{d}{:}(r)]=deal(minval);
        
    case 'string'
        minval=floor(min(handles.indata.indata.alldata{d}(:,r)));
        maxval=ceil(max(handles.indata.indata.alldata{d}(:,r)));
        uniquez=unique(handles.indata.indata.alltext{d}(:,r));
        
        handles.indata.indata.isconstant{d}(r)=deal(false);
        [handles.indata.indata.minlimit{d}{:}(r)]=deal(minval);
        [handles.indata.indata.maxlimit{d}{:}(r)]=deal(maxval);
        [handles.indata.indata.discretevalues{d}{:}(r)]=deal({uniquez});
        [handles.indata.indata.xdials{d}{:}(r)]=deal(minval);
        
    case 'logical'
        minval=0;
        maxval=1;
        handles.indata.indata.isconstant{d}(r)=deal(false);
        [handles.indata.indata.minlimit{d}{:}(r)]=deal(minval);
        [handles.indata.indata.maxlimit{d}{:}(r)]=deal(maxval);
        [handles.indata.indata.discretevalues{d}{:}(r)]=deal({[0 1]});
        [handles.indata.indata.xdials{d}{:}(r)]=deal(minval);
end

guidata(src,handles);



function makey_callback(src, eventdata)
handles=guidata(src);

xbox=findobj(handles.tabobjects,'tag','xbox');
ybox=findobj(handles.tabobjects,'tag','ybox');

selected=get(xbox,'value');
xnames=get(xbox,'string');
ynames=get(ybox,'string');

%this much i know is correct:
set(xbox,'value',[]);
set(ybox,'value',[]);

movethesepts=find(ismember(handles.names,xnames(selected)));

if get(ybox,'userdata')==1
    set(xbox,'string',setdiff(xnames,handles.names(movethesepts)));
    set(ybox,'string',union(ynames,handles.names(movethesepts)));
else
    set(xbox,'string',setdiff(xnames,handles.names(movethesepts)));
    set(ybox,'string',handles.names(movethesepts),'userdata',1);
end

d=get(handles.indata.listbox1,'value');
isx=find(ismember(handles.names,get(xbox,'string')));
isy=find(ismember(handles.names,get(ybox,'string')));

%check if any isx is actually a function, and if so, move it back to the isy column.  
%isfun=?

%update handles to reflect changes: 

handles.indata.indata.isresponse{d}=false(1,numel(isx)+numel(isy));
handles.indata.indata.isresponse{d}(isy)=true;

guidata(src,handles);



% --- Executes on button press in makex.
function makex_callback(src, eventdata)
handles=guidata(src);

xbox=findobj(handles.tabobjects,'tag','xbox');
ybox=findobj(handles.tabobjects,'tag','ybox');

selected=get(ybox,'value');
xnames=get(xbox,'string');
ynames=get(ybox,'string');

%this much i know is correct:
set(xbox,'value',[]);
set(ybox,'value',[]);


movethesepts=find(ismember(handles.names,ynames(selected)));

set(xbox,'string',union(xnames,handles.names(movethesepts)));
set(ybox,'string',setdiff(ynames,handles.names(movethesepts)));


d=get(handles.indata.listbox1,'value');
isx=find(ismember(handles.names,get(xbox,'string')));
isy=find(ismember(handles.names,get(ybox,'string')));

%check if any isx is actually a function, and if so, move it back to the isy column.  
%isfun=?

%update handles to reflect changes: 

handles.indata.indata.isresponse{d}=false(1,numel(isx)+numel(isy));
handles.indata.indata.isresponse{d}(isy)=true;

guidata(src,handles);




% --- Executes on button press in donebutton.
function donebutton_Callback(src, eventdata)
handles=guidata(src);

%isresponse=ismember(handles.names,get(handles.ybox,'string'));

set(0,'userdata',handles.indata.indata);
close(handles.gui);

function cancelbutton_Callback(src,ev)
close(gcbf);



% --- Executes on button press in hideconstantcheck.
function hideconstantcheck_Callback(hObject)
handles=guidata(src);
isconstant=handles.isconstant;
hideem=get(hObject,'value');

if hideem
    handles.names=handles.allnames(~isconstant);
    set(handles.xbox,'string',setdiff(get(handles.xbox,'string'),handles.allnames(isconstant)));
    set(handles.ybox,'string',setdiff(get(handles.ybox,'string'),handles.allnames(isconstant)));
else
    handles.names=handles.allnames;
    set(handles.xbox,'string',union(get(handles.xbox,'string'),handles.allnames(isconstant)));
end

guidata(hObject,handles);



