function askbox=raveeditthemecolorsgui(src,ev)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


if ~isempty(src);
    handles=guidata(src);
    screensize=get(handles.fig,'position');
    clear handles
    load raveprefs
else
    screensize=get(0,'monitorposition');
    screensize=screensize(1,:);
    load raveprefstemp
end



askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-350 screensize(2)+screensize(4)/2-225 700 450],...
    'numbertitle','off','menubar','none','name','Change RAVE colors','color',raveprefs.colors.frame,...
    'resize','off','windowstyle','modal');

uicontrol('style','text','string','Use the dropdown menu to select a component to change its color.',...
    'position',[30 400 640 40],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left');


a1=axes('units','pixels','parent',askbox,'position',[86 160 528 216],'ylim',[0 300],'xlim',[0 200],'visible','off','color','none');
idata=imread('setcolor.bmp');



colornames={'tabbg';'tabfg';'wsbg';'wsgrid';'frame';'title';'text';'texttint'};
colorz=[0 255 255;255 0 0;255 255 255;0 0 255;0 255 0;255 255 0;128 128 0;0 128 128];

%original image:
R=idata(:,:,1);
G=idata(:,:,2);
B=idata(:,:,3);

%updated image:
r=R;
g=G;
b=B;

for f=1:8
    
    thiscolor=raveprefs.colors.(colornames{f});
    matchcolor=colorz(f,:);
    
    idx = R==matchcolor(1) & G==matchcolor(2) & B==matchcolor(3);
    r(idx)=round(thiscolor(1)*255);
    g(idx)=round(thiscolor(2)*255);
    b(idx)=round(thiscolor(3)*255);
    
    if f==4
        matchcolor=[255 0 255];
        idx = R==matchcolor(1) & G==matchcolor(2) & B==matchcolor(3);
        r(idx)=round((raveprefs.colors.wsgrid(1)+raveprefs.colors.wsbg(1))./2*255);
        g(idx)=round((raveprefs.colors.wsgrid(2)+raveprefs.colors.wsbg(2))./2*255);
        b(idx)=round((raveprefs.colors.wsgrid(3)+raveprefs.colors.wsbg(3))./2*255);
    end
    
end

%draw it:

idata(:,:,1)=r;
idata(:,:,2)=g;
idata(:,:,3)=b;

i=image(idata,'parent',a1);
set(i,'hittest','off');

set(a1,'xtick',[],'ytick',[],'hittest','off','buttondownfcn',@imageclick);

uicontrol('style','popup','string',{'Tab Background';'Tab Border';'Workspace Background';'Grid';'Frame';'Title';'Text';'Text2'},...
    'position',[250 100 200 18],'backgroundcolor',[1 1 1],'hor','left','callback',@colormenu_Callback);

if ~isempty(src)
    cancelbutton=uicontrol('style','pushbutton','position',[420 10 120 26],'string','Cancel','fontsize',12,'callback',@cancelbutton_Callback);
    donebutton=uicontrol('style','pushbutton','position',[560 10 120 26],'string','Done','fontsize',12,'callback',@donebutton_Callback);
else
    cancelbutton=uicontrol('style','pushbutton','position',[420 10 120 26],'string','Cancel','fontsize',12,'callback',@cancelbutton2_Callback);
    donebutton=uicontrol('style','pushbutton','position',[560 10 120 26],'string','Next','fontsize',12,'callback',@donebutton2_Callback);
end



handles.colors=raveprefs.colors;
guidata(askbox,handles);


function cancelbutton_Callback(src,ev)
close(gcf);

function donebutton_Callback(src,ev)

handles=guidata(src);
prefpath=which('raveprefs.mat');
load raveprefs
raveprefs.colors=handles.colors;
save(prefpath,'raveprefs');
close(gcf);

function cancelbutton2_Callback(src,ev)
close(gcf);

function donebutton2_Callback(src,ev)
handles=guidata(src);
prefpath=which('raveprefstemp.mat');
load raveprefstemp
raveprefs.colors=handles.colors;
save(prefpath,'raveprefs');
close(gcf);

function imageclick(src,ev)

clicked=get(src,'currentpoint');
xclick=clicked(1,1,1);
yclick=clicked(1,2,1);

idata=imread('setcolor.bmp');

r=idata(xclick,yclick,1);
g=idata(xclick,yclick,2);
b=idata(xclick,yclick,3);


colornames={'tabbg';'tabfg';'wsbg';'wsgrid';'frame';'title';'text';'texttint'};
colorz=[0 255 255;255 0 0;255 255 255;0 0 255;0 255 0;255 255 0;128 128 0;0 128 128];


function colormenu_Callback(src,ev)
handles=guidata(src);

value=get(src,'value');


idata=imread('setcolor.bmp');
R=idata(:,:,1);
G=idata(:,:,2);
B=idata(:,:,3);

setcolorimage=findobj(gcf,'type','image');
newdata=get(setcolorimage,'cdata');

r=newdata(:,:,1);
g=newdata(:,:,2);
b=newdata(:,:,3);


displaynames={'Tab Background';'Tab Border';'Workspace Background';'Grid';'Frame';'Title';'Text';'Text2'};
colornames={'tabbg';'tabfg';'wsbg';'wsgrid';'frame';'title';'text';'texttint'};
colorz=[0 255 255;255 0 0;255 255 255;0 0 255;0 255 0;255 255 0;128 128 0;0 128 128];

idx= R==colorz(value,1) & G==colorz(value,2) & B==colorz(value,3);
newcolor=ravecolorchooser(gcf,handles.colors.(colornames{value}),['Choose ',displaynames{value},' Color']);

r(idx)=round(newcolor(1).*255);
g(idx)=round(newcolor(2).*255);
b(idx)=round(newcolor(3).*255);


newdata(:,:,1)=r;
newdata(:,:,2)=g;
newdata(:,:,3)=b;

set(setcolorimage,'cdata',newdata);

handles.colors.(colornames{value})=newcolor;
guidata(src,handles);

