function askbox=raveeditthemefontsgui(src,ev)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


if ~isempty(src);
    handles=guidata(src);
    screensize=get(handles.fig,'position');
    load raveprefs

else
    screensize=get(0,'monitorposition');
    screensize=screensize(1,:);
    load raveprefstemp
end


askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-350 screensize(2)+screensize(4)/2-225 700 450],...
    'numbertitle','off','menubar','none','name','Change RAVE fonts','color',raveprefs.colors.frame,...
    'resize','off','windowstyle','normal');

if ~isempty(src);
    uicontrol('style','text','string','For title and tabs, select a font then use slider to scale font so that the text fits (mostly) within the bounds of the rectangles.',...
        'position',[30 400 640 40],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left');
else
    uicontrol('style','text','string','Select fonts used by RAVE interface (you can change these later from within RAVE). Select a font then use slider to scale font so that the text fits (mostly) within the bounds of the rectangles. Click "Next" when you are finished.',...
        'position',[20 400 660 40],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left');
end


%guidata(askbox,handles);

R=raveprefs.colors.frame(1);
G=raveprefs.colors.frame(2);
B=raveprefs.colors.frame(3);

fontz=listfonts;

[junk idx]=ismember(lower(raveprefs.fonts.title),lower(fontz));
if idx==0;idx=1;end

a1=axes('units','pixels','parent',askbox,'position',[40 120 200 300],'ylim',[0 300],'xlim',[0 200],'visible','off','color','none');
l1=uicontrol('style','popup','pos',[50 90 180 20],'string',fontz,'back',[1 1 1],'callback',@titlefont_Callback,'value',idx,'tag','titlemenu');
[jslider h1]=javacomponent('javax.swing.JSlider',[50 50 180 30]);
jslider.setPaintTicks(1)
jslider.setSnapToTicks(0)
jslider.setMinimum(12)
jslider.setMaximum(40)
jslider.setMajorTickSpacing(2);
jslider.setValue(raveprefs.fonts.titlescale*20);
set(jslider,'background',java.awt.Color(R,G,B))
set(jslider,'foreground',java.awt.Color.black)
set(jslider,'StateChangedCallback',{@titlescale_Callback,askbox});
set(h1,'tag','titlescale');

r1(1)=rectangle('position',[0 18 117 18],'facecolor',[1 1 1],'edgecolor',[0 0 1]);
r1(2)=rectangle('position',[0 68 160 18],'facecolor',[1 1 1],'edgecolor',[0 0 1]);
r1(3)=rectangle('position',[0 118 81 18],'facecolor',[1 1 1],'edgecolor',[0 0 1]);
r1(4)=rectangle('position',[0 168 178 18],'facecolor',[1 1 1],'edgecolor',[0 0 1]);
r1(5)=rectangle('position',[0 218 128 18],'facecolor',[1 1 1],'edgecolor',[0 0 1]);

t1(1)=text(0,25,'Legends','parent',a1,'fontname',raveprefs.fonts.title,'fontsize',18*raveprefs.fonts.titlescale,'hor','left','tag','t1');
t1(2)=text(0,75,'Data Series','parent',a1,'fontname',raveprefs.fonts.title,'fontsize',18*raveprefs.fonts.titlescale,'hor','left','tag','t1');
t1(3)=text(0,125,'Graph','parent',a1,'fontname',raveprefs.fonts.title,'fontsize',18*raveprefs.fonts.titlescale,'hor','left','tag','t1');
t1(4)=text(0,175,'Control Panel','parent',a1,'fontname',raveprefs.fonts.title,'fontsize',18*raveprefs.fonts.titlescale,'hor','left','tag','t1');
t1(5)=text(0,225,'Navigator','parent',a1,'fontname',raveprefs.fonts.title,'fontsize',18*raveprefs.fonts.titlescale,'hor','left','tag','t1');

text(0,265,'Title Font:','parent',a1,'fontsize',12,'hor','left','fontweight','bold');


%tab font:

[junk idx]=ismember(lower(raveprefs.fonts.tab),lower(fontz));

if idx==0;idx=1;end

a2=axes('units','pixels','parent',askbox,'position',[220+40 120 200 300],'ylim',[0 300],'xlim',[0 200],'visible','off','color','none');
l1=uicontrol('style','popup','pos',[220+50 90 180 20],'string',fontz,'back',[1 1 1],'callback',@tabfont_Callback,'value',idx,'tag','tabmenu');
[jslider h2]=javacomponent('javax.swing.JSlider',[220+50 50 180 30]);
jslider.setPaintTicks(1)
jslider.setSnapToTicks(0)
jslider.setMinimum(12)
jslider.setMaximum(40)
jslider.setMajorTickSpacing(2);
jslider.setValue(raveprefs.fonts.tabscale*20);
set(jslider,'background',java.awt.Color(R,G,B))
set(jslider,'foreground',java.awt.Color.black)
set(jslider,'StateChangedCallback',{@tabscale_Callback,askbox});
set(h2,'tag','tabscale');

r2(1)=rectangle('position',[0 18 85 18],'facecolor',[1 1 1],'edgecolor',[0 0 1]);
r2(2)=rectangle('position',[0 68 50 18],'facecolor',[1 1 1],'edgecolor',[0 0 1]);
r2(3)=rectangle('position',[0 118 110 18],'facecolor',[1 1 1],'edgecolor',[0 0 1]);
r2(4)=rectangle('position',[0 168 80 18],'facecolor',[1 1 1],'edgecolor',[0 0 1]);
r2(5)=rectangle('position',[0 218 98 18],'facecolor',[1 1 1],'edgecolor',[0 0 1]);

t2(1)=text(0,27,'Manage','parent',a2,'fontname',raveprefs.fonts.tab,'fontsize',18*raveprefs.fonts.tabscale,'hor','left','tag','t2');
t2(2)=text(0,77,'View','parent',a2,'fontname',raveprefs.fonts.tab,'fontsize',18*raveprefs.fonts.tabscale,'hor','left','tag','t2');
t2(3)=text(0,127,'Constrain','parent',a2,'fontname',raveprefs.fonts.tab,'fontsize',18*raveprefs.fonts.tabscale,'hor','left','tag','t2');
t2(4)=text(0,177,'Explore','parent',a2,'fontname',raveprefs.fonts.tab,'fontsize',18*raveprefs.fonts.tabscale,'hor','left','tag','t2');
t2(5)=text(0,227,'Optimize','parent',a2,'fontname',raveprefs.fonts.tab,'fontsize',18*raveprefs.fonts.tabscale,'hor','left','tag','t2');

text(0,265,'Tab Font:','parent',a2,'fontsize',12,'hor','left','fontweight','bold');

if ~isempty(src)
    cancelbutton=uicontrol('style','pushbutton','position',[420 10 120 26],'string','Cancel','fontsize',12,'callback',@cancelbutton_Callback);
    donebutton=uicontrol('style','pushbutton','position',[560 10 120 26],'string','Done','fontsize',12,'callback',@donebutton_Callback);
else
    cancelbutton=uicontrol('style','pushbutton','position',[420 10 120 26],'string','Cancel','fontsize',12,'callback',@cancelbutton2_Callback);
    donebutton=uicontrol('style','pushbutton','position',[560 10 120 26],'string','Next','fontsize',12,'callback',@donebutton2_Callback);
end



function titlescale_Callback(jsrc,jev,src)
moving = get(jsrc,'ValueIsAdjusting');
if strcmp(moving,'off')
    return
end
newscale=jsrc.getValue./20;
t1=findobj(gcf,'tag','t1');
set(t1,'fontsize',18*newscale);


function titlefont_Callback(src,ev)
fontz=get(src,'string');
newfont=fontz{get(src,'value')};
t1=findobj(gcf,'tag','t1');
set(t1,'fontname',newfont);


function tabscale_Callback(jsrc,jev,src)

moving = get(jsrc,'ValueIsAdjusting');
if strcmp(moving,'off')
    return
end
newscale=jsrc.getValue./20;
t2=findobj(gcf,'tag','t2');
set(t2,'fontsize',18*newscale);


function tabfont_Callback(src,ev)

fontz=get(src,'string');
newfont=fontz{get(src,'value')};
t2=findobj(gcf,'tag','t2');
set(t2,'fontname',newfont);


function cancelbutton_Callback(src,ev)
close(gcf);

function donebutton_Callback(src,ev)

tabscale=findobj(gcf,'tag','tabscale');
titlescale=findobj(gcf,'tag','titlescale');

tabscale=get(tabscale,'javapeer');
titlescale=get(titlescale,'javapeer');

tabmenu=findobj(gcf,'tag','tabmenu');
titlemenu=findobj(gcf,'tag','titlemenu');




prefspath=which('raveprefs.mat');
load raveprefs
fontz=listfonts;
raveprefs.fonts.tab=fontz{get(tabmenu,'value')};
raveprefs.fonts.tabscale=tabscale.getValue./20;
raveprefs.fonts.title=fontz{get(titlemenu,'value')};
raveprefs.fonts.titlescale=titlescale.getValue./20;

save(prefspath,'raveprefs');

close(gcf);

function cancelbutton2_Callback(src,ev)
close(gcf);

function donebutton2_Callback(src,ev)

tabscale=findobj(gcf,'tag','tabscale');
titlescale=findobj(gcf,'tag','titlescale');

tabscale=get(tabscale,'javapeer');
titlescale=get(titlescale,'javapeer');

tabmenu=findobj(gcf,'tag','tabmenu');
titlemenu=findobj(gcf,'tag','titlemenu');

prefspath=which('raveprefstemp.mat');
load raveprefstemp
fontz=listfonts;
raveprefs.fonts.tab=fontz{get(tabmenu,'value')};
raveprefs.fonts.tabscale=tabscale.getValue./20;
raveprefs.fonts.title=fontz{get(titlemenu,'value')};
raveprefs.fonts.titlescale=titlescale.getValue./20;

save(prefspath,'raveprefs');

close(gcf);




