function varargout = raveexcludecolumnsgui(varargin)
% gui to define variables as being indepent or responses.
% part of the RAVE toolbox
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @raveexcludecolumnsgui_OpeningFcn, ...
                   'gui_OutputFcn',  @raveexcludecolumnsgui_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before raveexcludecolumnsgui is made visible.
function raveexcludecolumnsgui_OpeningFcn(hObject, eventdata, handles, varargin)

%get theme
load raveprefs

set(hObject,'color',raveprefs.colors.frame)
set(handles.uipanel1,'backgroundcolor',raveprefs.colors.tabbg)

%make titles
set(handles.titleaxes,'color','none','xcolor',raveprefs.colors.frame,...
    'ycolor',raveprefs.colors.frame,'zcolor',raveprefs.colors.frame);
set(handles.leftaxes,'color','none','xcolor',raveprefs.colors.tabbg,...
    'ycolor',raveprefs.colors.tabbg,'zcolor',raveprefs.colors.tabbg);
set(handles.rightaxes,'color','none','xcolor',raveprefs.colors.tabbg,...
    'ycolor',raveprefs.colors.tabbg,'zcolor',raveprefs.colors.tabbg);

text(0,.5,'Choose which variables to include/exclude','parent',handles.titleaxes,'fontname',handles.prefs.fonts.title,'fontsize',20*handles.prefs.fonts.titlescale,...
   'color',raveprefs.colors.title,'horizontalalignment','left','verticalalignment','baseline');
text(.05,.05,'Exclude variables that are not of interest and they will not appear in variable menus and lists.','parent',handles.titleaxes,'fontname','arial','fontsize',8,...
   'color',[0 0 0],'horizontalalignment','left','verticalalignment','baseline');
text(0,.2,'Included','parent',handles.leftaxes,'fontname',raveprefs.fonts.tab,'fontsize',18*raveprefs.fonts.tabscale,...
   'color',raveprefs.colors.text,'horizontalalignment','left','verticalalignment','baseline');
text(0,.2,'Excluded','parent',handles.rightaxes,'fontname',raveprefs.fonts.tab,'fontsize',18*raveprefs.fonts.tabscale,...
   'color',raveprefs.colors.text,'horizontalalignment','left','verticalalignment','baseline');

%get data from managedatagui
handles.indata=guidata(gcbo); %gcbo = "define roles" button in managedatagui

%find current data set:
d=get(handles.indata.listbox1,'value');


names=handles.indata.indata.allnames{d};
handles.names=names; %make accessible to the done button

set(handles.xbox,'string',names) %load all data into x box first
set(handles.ybox,'string',[]) %make ybox empty

set(handles.xbox,'max',length(names))
set(handles.ybox,'max',length(names))

handles.exes=1:1:length(names);
handles.whys=[];

set(handles.xbox,'value',[]);
set(handles.ybox,'value',[]);


% Choose default command line output for raveexcludecolumnsgui
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes raveexcludecolumnsgui wait for user response (see UIRESUME)
% uiwait(handles.figure1);

% --- Outputs from this function are returned to the command line.
function varargout = raveexcludecolumnsgui_OutputFcn(hObject, eventdata, handles) 
varargout{1} = handles.output;

%% stuff that matters
% --- Executes on button press in makey.
function makey_Callback(hObject, eventdata, handles)

selected=get(handles.xbox,'value');
xnames=get(handles.xbox,'string');
ynames=get(handles.ybox,'string');

%this much i know is correct:
set(handles.xbox,'value',[]);
set(handles.ybox,'value',[]);



movethesepts=find(ismember(handles.names,xnames(selected)));

if get(handles.ybox,'userdata')==1
    set(handles.xbox,'string',setdiff(xnames,handles.names(movethesepts)));
    set(handles.ybox,'string',union(ynames,handles.names(movethesepts)));
else
    set(handles.xbox,'string',setdiff(xnames,handles.names(movethesepts)));
    set(handles.ybox,'string',handles.names(movethesepts),'userdata',1);
end



% --- Executes on button press in makex.
function makex_Callback(hObject, eventdata, handles)

selected=get(handles.ybox,'value');
xnames=get(handles.xbox,'string');
ynames=get(handles.ybox,'string');

%this much i know is correct:
set(handles.xbox,'value',[]);
set(handles.ybox,'value',[]);



movethesepts=find(ismember(handles.names,ynames(selected)));

set(handles.xbox,'string',union(xnames,handles.names(movethesepts)));
set(handles.ybox,'string',setdiff(ynames,handles.names(movethesepts)));

% --- Executes on button press in donebutton.
function donebutton_Callback(hObject, eventdata, handles)

isincluded=ismember(handles.names,get(handles.xbox,'string'));
set(0,'userdata',isincluded);
close(gcf);




% --- Executes on button press in hideconstantcheck.
function hideconstantcheck_Callback(hObject, eventdata, handles)
isconstant=handles.isconstant;
hideem=get(hObject,'value');

if hideem
    handles.names=handles.allnames(~isconstant);
    set(handles.xbox,'string',setdiff(get(handles.xbox,'string'),handles.allnames(isconstant)));
    set(handles.ybox,'string',setdiff(get(handles.ybox,'string'),handles.allnames(isconstant)));
else
    handles.names=handles.allnames;
    set(handles.xbox,'string',union(get(handles.xbox,'string'),handles.allnames(isconstant)));
end

guidata(hObject,handles);


%% stuff that doesnt matter

% --- Executes on selection change in xbox.
function xbox_Callback(hObject, eventdata, handles)
function xbox_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on selection change in ybox.
function ybox_Callback(hObject, eventdata, handles)
function ybox_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end





