function [handles,success]=raveloadmetadata(handles,d,path)
%NOTE: THIS funciton takes in handles directly instead of retrieving it
%from guidata (to avoid problems with managedatagui.
%path is path of file to be read.
%results is a structure whose fields depend on the filetype of the file in "path".
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

success=true;

if isempty(path);return;end

thistype=path(end-2:end);

switch thistype
    
    case 'fun'
        
        evalthese=[];
        %nvars=size(handles.alldata{d}); %initial number of variables;
        
        contents=importdata(path);
        for c=1:numel(contents)
            tabz=find(contents{c}==char(9));
            lastindex=find(isstrprop(contents{c},'alphanum'),1,'last');
            tabz=[tabz(tabz<lastindex) lastindex+1];
            thisvarname=contents{c}(1:tabz(1)-1);
            
            if thisvarname(1)=='*';
                thisvarname=thisvarname(2:end);
                isobjective=1;
            else
                isobjective=0;
            end
            
            thisfunpath=contents{c}(tabz(1)+1:tabz(2)-1);
            if ~exist(thisfunpath,'file')
                errordlg(['Function file ' thisfunpath ' does not exist. Fix it, or reload data set without metadata.'])
                return
            end
            
            if ~strcmp(thisfunpath(end-1:end),'.m');errordlg('.rvefun Files can only reference .m functions');end
            [pathstr, name, ext, versn] = fileparts(thisfunpath);
            if isempty(which(name))
                try %gives error if a file with same name/different capitalization exists on path.
                    copyfile(thisfunpath,fullfile(handles.prefs.defaultdir,[name ext]),'f')
                end
                rehash
            end
            
            thisnargout=contents{c}(tabz(2)+1:tabz(3)-1);
            
            for t=3:numel(tabz)-1
                invars(t-2)=find(strcmp(handles.allnames{d},contents{c}(tabz(t)+1:tabz(t+1)-1))); %#ok<AGROW>
            end
            thisinvars=sort(invars);
            %load all this info, but done evaluate functions yet. first
            %check for rveopt file in case any of these are objectives.
            
            handles.allnames{d}{end+1}=thisvarname;
            handles.isfunction{d}(end+1)=1;
            handles.isresponse{d}(end+1)=1;
            handles.allunits{d}{end+1}=''; %havent implemented this yet
            handles.isincluded{d}(end+1)=1;
            handles.isconstant{d}(end+1)=0; %note, should really use an if statement here to check if it's constant.
            handles.funnames{d}{end+1}=name;
            handles.funvars{d}{end+1}=sort(thisinvars);
            handles.nargout{d}(end+1)=str2num(thisnargout);
            handles.datatype{d}{end+1}='continuous';
            handles.preferences{d}(end+1)=1;
            handles.targetvalues{d}(end+1)=-inf; %load prefs/targs here in case rveopt file is not found.
            handles.isobjective{d}(end+1)=isobjective;
            handles.variablecolors{d}(end+1,:)=rand(1,3);
            
            for a=1:numel(handles.analysisnames)
                handles.xdials{d}{a}(end+1)=0;
                handles.minlimit{d}{a}(end+1)=0;
                handles.maxlimit{d}{a}(end+1)=0;
                handles.discretevalues{d}{a}{end+1}=[];
            end
            
            evalthese(end+1)=numel(handles.allnames{d}); %#ok<AGROW>
            
        end
        
        optpath=[path(1:end-3) 'opt'];
        if exist(optpath,'file')
            handles=raveloadmetadata(handles,d,optpath);
        end
        
        %evaluate functions and fill in alldata:
        
        %this code copied from raveevaluateregression since we can't use
        %guidata here:
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        outnum=evalthese;
        
        if isempty(outnum);return;end
        
        data=handles.alldata{d};
        prefs=handles.preferences{d};
        targets=handles.targetvalues{d};
        
        %deterine dependency:
        num=sort(outnum,'descend');
        f=numel(num);
        while f<=numel(num)
            thesefunvars=handles.funvars{d}{num(f)};
            num=union(num,thesefunvars(handles.isfunction{d}(thesefunvars)));
            f=f+1;
        end
        name=handles.funnames{d}(num);
        narg=handles.nargout{d}(num);
        if ~iscell(name) %one response only
            if handles.isfunction{d}(num)
                xfun=str2func(name);
                alloutputs=cell(1,nargout(xfun));
                [alloutputs{:}]=xfun(data,prefs,targets);
                y=alloutputs{narg};
            elseif num<=size(data,2)
                y=data(:,num);
            end
        else %multiple responses:
            
            %enlarge (if necessary) data to have as many columns as largest index:
            if size(data,2)<max(num)
                data=[data zeros(size(data,1),max(num)-size(data,2))];
            end
            
            isfuns=find(handles.isfunction{d}(num));
            if ~isempty(isfuns)
                
                [junk,index]=unique(name(isfuns)); %get index of first occurence of unique funs
                uniquefuns = name(sort(index)); %extract unique funs without changing their evaluation order.
                
                for g=1:numel(uniquefuns);
                    thisfun=uniquefuns{g};
                    isthisfun=find(strcmp(name,thisfun)); %logical, responses matching current function
                    xfun=str2func(thisfun);
                    try
                    alloutputs=cell(1,nargout(xfun));
                    catch
                        errordlg(['Could not find the function ''',thisfun,''' on your path. If this is a new install, make sure your ''ravefiles'' folder is on your path. Otherwise, make sure this function exists or remove it from the .rvefun metadata file.'],'File not found');
                        success=false;
                        return
                    end
                        
                    [alloutputs{:}]=xfun(data,prefs,targets);
                    alloutputs=cell2mat(alloutputs);
                    data(:,num(isthisfun))=alloutputs(:,narg(isthisfun));
                end
                
            end
            
            y=data(:,outnum);
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        end
        
        handles.alldata{d}(:,evalthese)=y;
        handles.alltext{d}=[handles.alltext{d},cell(size(y))];
        
        
        for a=1:numel(handles.analysisnames)
            handles.minlimit{d}{a}(evalthese)=min(handles.alldata{d}(:,evalthese));
            handles.maxlimit{d}{a}(evalthese)=max(handles.alldata{d}(:,evalthese));
        end
        
    case 'val'
        fid = fopen(path);
        tline = fgetl(fid);
        while ischar(tline)
            tabz=find(tline==char(9));
            lastindex=find(isstrprop(tline,'alphanum'),1,'last');
            tabz=[tabz(tabz<lastindex) lastindex+1];
            thisvarname=tline(1:tabz(1)-1);
            thistype=lower(tline(tabz(1)+1:tabz(2)-1));
            thisvar=find(strcmp(handles.allnames{d},thisvarname));
            if isempty(thisvar)
                clear values
                tline = fgetl(fid);
                continue
            end
            if ~ismember(thistype,{'constant','continuous','discrete','integer','string'})
                thistype='continuous';
            end
            
            %store type:
            handles.datatype{d}{thisvar}=thistype;
            
            %store values:
            if ~strcmp(thistype,'string')
                for t=2:numel(tabz)-1
                    values(t-1)=str2double(tline(tabz(t)+1:tabz(t+1)-1)); %#ok<AGROW>
                end
                values=unique(values);
            else
                for t=2:numel(tabz)-1
                    values{t-1}=tline(tabz(t)+1:tabz(t+1)-1);
                end
                values=unique(values);
            end
            
            switch lower(thistype)
                case 'constant'
                    for a=1:numel(handles.analysisnames)
                        handles.minlimit{d}{a}(thisvar)=values(1);
                        handles.maxlimit{d}{a}(thisvar)=values(1);
                    end
                case {'contiuous','integer','logical'}
                    for a=1:numel(handles.analysisnames)
                        handles.minlimit{d}{a}(thisvar)=min(values);
                        handles.maxlimit{d}{a}(thisvar)=max(values);
                    end
                case 'discrete'
                    for a=1:numel(handles.analysisnames)
                        handles.minlimit{d}{a}(thisvar)=min(values);
                        handles.maxlimit{d}{a}(thisvar)=max(values);
                        handles.discretevalues{d}{a}{thisvar}=values';
                    end
                case 'string'
                    for a=1:numel(handles.analysisnames)
                        handles.minlimit{d}{a}(thisvar)=1;
                        handles.maxlimit{d}{a}(thisvar)=numel(values);
                        handles.discretevalues{d}{a}{thisvar}=values';
                    end
            end
            clear values
            tline = fgetl(fid);
        end
        fclose(fid);
        
        
    case 'con'
        
        
        evalthese=[];
        nvars=size(handles.alldata{d}); %initial number of variables;
        
        contents=importdata(path);
        for c=1:numel(contents)
            thisstr=contents{c};
            thisstr=thisstr(thisstr~=char(9));
            %determine color:
            brack=find(thisstr=='[' | thisstr==']');
            if numel(brack==2)
                color=eval(thisstr(brack(1):brack(2)));
                thisstr=thisstr(1:(brack(1)-1));
            else
                color=handles.colors.constraint;
            end
            
            
            %determine type of constraint:
            ttt=1;
            
            if ~isempty(strfind(thisstr,'<='))
                thisequality='<=';
                k=strfind(thisstr,'<=');
                constraintstring=thisstr(1:k-1);
                constraintvalue=eval(thisstr(k+2:end));
            elseif ~isempty(strfind(thisstr,'>='))
                thisequality='>=';
                k=strfind(thisstr,'>=');
                constraintstring=thisstr(1:k-1);
                constraintvalue=eval(thisstr(k+2:end));
                
            elseif ~isempty(strfind(thisstr,'='))
                thisequality='=';
                k=strfind(thisstr,'=');
                constraintstring=thisstr(1:k-1);
                constraintvalue=eval(thisstr(k+1:end));
                
            elseif ~isempty(strfind(thisstr,'<'))
                thisequality='<';
                k=strfind(thisstr,'<');
                constraintstring=thisstr(1:k-1);
                constraintvalue=eval(thisstr(k+1:end));
                
            elseif ~isempty(strfind(thisstr,'>'))
                thisequality='>';
                k=strfind(thisstr,'>');
                constraintstring=thisstr(1:k-1);
                constraintvalue=eval(thisstr(k+1:end));
                
            else
                continue %invalid constraint
            end
            
            [constraintfunctionstring,constrainttype,constraintvars]=raveprocessconstraint(handles,d,constraintstring,thisequality,constraintvalue);
            
            %store this function annonymously in handles.
            
            try
                eval(['newhandle=@(data) ',constraintfunctionstring,';']);
            catch %#ok<CTCH>
                errordlg('Something went wrong while creating a constraint. Check to see if you spelled everything right and used correct MATLAB syntax.')
            end
            
            for a=1:numel(handles.analysisnames)
                handles.constrainthandles{d}{a}{end+1}=newhandle;
                handles.constrainttext{d}{a}{end+1}=strtrim([constraintstring,thisequality,' ',num2str(constraintvalue)]);
                handles.constrainttype{d}{a}{end+1}=constrainttype;
                handles.constraintvars{d}{a}{end+1}=constraintvars;
                handles.constraintcolors{d}{a}{end+1}=color;
            end
            
            %evaluate this constraint on current data set, and store results in handles.constraintvalues:
            
            %    thishandle=handles.constrainthandles{udata.view.dataset}{udata.view.analysis}{end};
            
            thisindex=numel(handles.constrainthandles{d}{1});
            
            guidata(handles.fig,handles);
            
            %evaluate constraint:
            
            for a=1:numel(handles.analysisnames)
                
                
                handles.constraintvalues{d}{a}{end+1}=evaluateconstraint(handles,d,thisindex,handles.alldata{d});
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%
                
                %evaluate constraint at end points to get normalization parameters:
                mins=handles.minlimit{d}{a};
                maxs=handles.maxlimit{d}{a};
                
                minvalue=evaluateconstraint(handles,d,thisindex,mins,0);
                maxvalue=evaluateconstraint(handles,d,thisindex,maxs,0);
                meanvalue=evaluateconstraint(handles,d,thisindex,(mins+maxs)./2,0);
                
                sample=[minvalue maxvalue meanvalue];
                
                sample(isnan(sample))=1; %not really sure what to do if nan pops up (most likely from 0/0 expression)
                
                offset=mean(sample);
                range=max(sample)-min(sample);
                
                if range==0
                    range=1;
                end
                
                %handles.constraintadders{udata.view.dataset}{udata.view.analysis}{end+1}=offset;
                handles.constraintscalers{d}{a}(end+1)=range;
                
                guidata(handles.fig,handles);
                
            end
            
        end
        
        
    case 'cur'
        contents=importdata(path,'\t');
        
        varnames=contents.colheaders;
        values=contents.data(1,:);
        
        for c=1:numel(varnames);
            thisindex=find(strcmp(handles.allnames{d},varnames{c}));
            for a=1:numel(handles.analysisnames)
                handles.xdials{d}{a}(thisindex)=values(c); %#ok<FNDSB>
            end
        end
        
    case 'opt'
        
        contents=importdata(path,'\t');
        
        varnames=contents.colheaders;
        targs=contents.data(1,:);
        prefs=contents.data(2,:);
        
        for c=1:numel(varnames);
            thisindex=find(strcmp(handles.allnames{d},varnames{c}));
            handles.preferences{d}(thisindex)=prefs(c);
            handles.targetvalues{d}(thisindex)=targs(c);
        end
        
        
    case 'col'
        contents=importdata(path,'\t');
        
        varnames=contents{1};
        colors=contents{2};
        
        nametabz=find(varnames==char(9));
        lastindex=find(isstrprop(varnames,'alphanum'),1,'last');
        nametabz=[0 nametabz(nametabz<lastindex) lastindex+1];
        colortabz=find(colors==char(9));
        lastindex=find(colors==']',1,'last');
        colortabz=[0 colortabz(colortabz<lastindex) lastindex+1];
        
        for c=1:min(numel(nametabz,colortabz))-1
            thisindex=find(strcmp(handles.allnames{d},varnames(nametabz(c)+1:nametabz(c+1)-1)));
            handles.variablecolors{d}(thisindex,:)=eval(colors(colortabz(c)+1:colortabz(c+1)-1)); %#ok<FNDSB>
        end
        
    case 'row'
        contents=importdata(path,'\t');
        
        for a=1:numel(handles.analysisnames)
            handles.rowcolors{d}{a}(1:size(contents,1))=contents(:,1);
            handles.isvisible{d}{a}(1:size(contents,1))=contents(:,2);
            handles.isselected{d}{a}(1:size(contents,1))=contents(:,3);
        end
        
    case 'poi'
        %not yet implemented
        
end


function g=evaluateconstraint(handles,dataset,thisconstraint,indata,ignore)
%evaluates the regression with index stored in thisconstraint using the data stored in data.
%thisconstraint is an index into:
%handles.constrainthandles{udata.view.dataset}{udata.view.analysis}{thisconstraint}
%ignore is a logical vector of length=size(indata,2) (numvars) where 1 means do not evaluate this
%regression (if that varaible isfunction). use this to do things like sweeps through regressed
%variables. or just make ignore==0, to  evaluate all constraints.


%{

%% determine which variables appear in this constraint:
thisstring=char(handles.constrainthandles{dataset}{a}{thisconstraint});
thisstring=thisstring(8:end);

varnumbers=handles.constraintvars{dataset}{a}{thisconstraint};

varnumbers=setdiff(varnumbers,find(ignore)); %dont evaluate regressions of ignored vars

if ~isempty(varnumbers)
    %evaluate all regressions necessary for this constraint:
    respdata=raveevaluateregression(handles.fig,udata.view.dataset,varnumbers,indata);
    indata(:,varnumbers)=respdata;
end
%}
%solve evaluate this constraint for y variable:

thishandle=handles.constrainthandles{dataset}{1}{thisconstraint};
g=thishandle(indata);


