function ravemakecolorbar(src,ev)
%fix the colorbar associated with the current graph to display the correct range of colors and
%ylabelticks. this function doesnt check if the figure actually HAS a colorbar, so make sure to do
%that before calling this.
%
%also, this uses the mcmap property. doesn't work with lcmap. 
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
legdata=get(handles.fakerax(handles.activeindex),'userdata');




%remove old cbar from legdata (for rasterizing/exporting);
oldcbar=findobj(handles.fig,'tag','Colorbar','userdata',handles.activeindex);

if ~isempty(oldcbar)
    legdata=setdiff(legdata,oldcbar);
    delete(oldcbar)
    set(handles.fakerax(handles.activeindex),'userdata',legdata);
end

% %remove existing colorbar:
% colorbar('off','peer',handles.activegraph); %if is real colorbar
% %try
% if ~isempty(oldcbar)
%     
% end
% %end

%make a new colorbar if needed
if ~udata.format.colorbar;return;end
if ~udata.view.c;return;end
if strcmp(handles.datatype{udata.view.dataset}{udata.view.c},'constant');return;end %trying to make constant colorbar = errors. 
    

%{
%make/updata cbar:
cbar=colorbar('peer',handles.activegraph,'location','eastoutside','units','pixels');

%fix the image to display the real colormap:
theimage=get(cbar,'children');
realcmap=get(handles.fig,'colormap');
newcdata=permute(realcmap,[1 3 2]);
% 
% if udata.format.reversecolors
%     newcdata=newcdata(end:-1:1,:,:);
% end


set(theimage,'cdata',newcdata);

%fix ylims and yticks to reflect non-scaled data:
cbardims=get(cbar,'position');

colorbar('off','peer',handles.activegraph);
%}
axpos=get(handles.activegraph,'position');

aaa=axes('parent',handles.wspanel,'visible','on','units','pixels','position',[axpos(1)+axpos(3)+30 axpos(2) 27 axpos(4)]);
% theimage=copyobj(theimage,aaa);

cdata=get(handles.fig,'colormap');
cdata=permute(cdata,[1,3,2]);
image(cdata,'parent',aaa);
%set(aaa,'xlim',[-1 2],'xdir','reverse')


switch handles.activetype
    case {'contour','hyperslice'} %scale to current profiler range
        set(aaa,'xtick',[],'xticklabel','','yaxislocation','right','ylim',udata.view.clim)
    otherwise %scale to data table
        ccoldata=handles.alldata{udata.view.dataset}(:,udata.view.c);
        try
            set(aaa,'xtick',[],'xticklabel','','yaxislocation','right','ylim',[min(ccoldata) max(ccoldata)]);
        catch %#ok<CTCH> %is ccol is a constant
            set(aaa,'xtick',[],'xticklabel','','yaxislocation','right','ylimmode','auto');
        end
end


newyticks=get(aaa,'ytick');
newyticklabels=get(aaa,'yticklabel');


if strcmp(handles.activetype,'parallelcoords')
    properYrange=[128*(udata.format.lcmap-1)+1 128*udata.format.lcmap];
else
    properYrange=[128*(udata.format.mcmap-1)+1 128*udata.format.mcmap];
end

set(aaa,'ylim',properYrange,'ylimmode','manual');

if udata.format.reversecolors
    set(aaa,'ydir','reverse')
    newyticklabels=flipud(newyticklabels);
else
    set(aaa,'ydir','normal')
end


% try %#ok<TRYNC>

switch handles.activetype
    case {'contour','hyperslice'} %scale to current profiler range
       newscaledticks=interp1(udata.view.clim,properYrange,newyticks);
    otherwise %scale to data table
       newscaledticks=interp1([min(ccoldata) max(ccoldata)],properYrange,newyticks);
end

set(aaa,'ytick',newscaledticks,'yticklabel',newyticklabels);

set(aaa,'userdata',handles.activeindex,'tag','Colorbar');
udata.format.colorbar=1;

legdata=union(legdata,aaa);

set(handles.fakerax(handles.activeindex),'userdata',legdata);

ylab=get(aaa,'ylabel');
set(ylab,'string',handles.allnames{udata.view.dataset}{udata.view.c},'interpreter','none');



% for profiler type graphs, make markers showing current value:

switch handles.activetype
    case {'contour','hyperslice'}
        xlim=get(aaa,'xlim');
        ylim=get(aaa,'ylim');
        pos=get(aaa,'pos');
        clim=udata.view.clim;
        set(aaa,'xlimmode','manual')
        
        
        
        %determine current c value:
        thisvalue=raveevaluateregression(src,udata.view.dataset,udata.view.c,handles.xdials{udata.view.dataset}{udata.view.analysis});
        
        thiscoord=interp1(clim,ylim,thisvalue);
        
        pixels=pos(3)./diff(xlim); %pixels per unit
        
        %make triangle patch at this value:
        
        patch([xlim(1)-7/pixels xlim(1)-7/pixels xlim(1)-1/pixels],[thiscoord-1.5 thiscoord+1.5 thiscoord],udata.format.tracercolor,'parent',aaa,'tag','currentvalue')
        text(xlim(2)+3/pixels,thiscoord,sprintf('%5.5g',thisvalue),'color',udata.format.tracercolor,'parent',aaa,'tag','currentvaluetxt')
       
        
end







