function ravenewgraph(src,eventdata,newgraphtype)  %#ok<INUSL>
%optional 3rd arguement is string of graph type to create (overrides gui)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


handles=guidata(src);

if handles.tablemode;return;end %cant make graphs while in table mode.

if isempty(handles.alldata);
    set(handles.infobar,'string','You must load a data set before making graphs. Click the "Manage Data Sets" button.')
    return
end


if nargin==2
    set(0,'userdata',get(handles.fig,'position'));
    handy=ravenewgraphgui(handles.fig);
    uiwait(handy);
    
    %get plot type:
    handles.newgraphtype=get(0,'userdata');
    set(0,'userdata',[]);
    
else
    handles.newgraphtype=newgraphtype;
end


%prompt for placement:
if strcmp(handles.newgraphtype,'cancel');
    %do nothing
else
    set(handles.infobar,'string','Click and drag to place this graph.')
    set(handles.mainaxes,'buttondownfcn',{@raveplacegraph,handles})
    set(handles.explorer,'buttondownfcn',{@raveplacegraph,handles})
    buttons=findobj(handles.tabobjects,'type','uicontrol');
    set(buttons,'enable','inactive');
end

guidata(src,handles)

function raveplacegraph(src,eventdata,handles) %#ok<INUSL>
set(handles.fig,'windowbuttondownfcn',[]);
buttons=findobj(handles.tabobjects,'type','uicontrol');
set(buttons,'enable','on');

%check that they clicked on exploreraxes or mainaxes:
if ~any([gco==handles.explorer,gco==handles.mainaxes])
    return
end

%clicked=get(handles.mainaxes,'currentpoint');
clicked=get(gco,'currentpoint');

xclick=clicked(1,1,1);
yclick=clicked(1,2,1);
handles.temp.startx=xclick;
handles.temp.starty=yclick;

xclick=handles.snapsize*floor(xclick/handles.snapsize);
yclick=handles.snapsize*floor(yclick/handles.snapsize);


if strcmp(handles.graphinfo.(handles.newgraphtype).class,'control');
    edgecolor=[0 0.5 1];
elseif strcmp(handles.graphinfo.(handles.newgraphtype).class,'annotation');
    edgecolor=[0.7 0 1];
else
    edgecolor=[1 0.5 0];
end

handles.temp.putgraphhere(1)=rectangle('position',[xclick yclick handles.prefs.gridsize handles.prefs.gridsize],'linewidth',2,'edgecolor',edgecolor,...
    'parent',handles.mainaxes,'hittest','on','buttondownfcn','','facecolor',handles.colors.wsbg);
handles.temp.putgraphhere(2)=rectangle('position',[xclick yclick handles.prefs.gridsize handles.prefs.gridsize],'linewidth',2,'edgecolor',edgecolor,...
    'parent',handles.explorer,'hittest','on','buttondownfcn','');

%allow dragging
set(handles.fig,'WindowButtonMotionFcn',@ravesizegraph);
set(handles.fig,'WindowButtonUpFcn',@ravesizegraphdone);
set(handles.fig,'windowbuttondownfcn',[]);

guidata(src,handles);

function ravesizegraph(src,eventdata) %#ok<INUSD>

handles=guidata(src);

finish=get(gco,'currentpoint');
starty=handles.temp.starty;
startx=handles.temp.startx;
endx=finish(1,1,1);
endy=finish(1,2,1);

pos1=handles.snapsize*floor(min(startx,endx)/handles.snapsize);
pos3=handles.snapsize*ceil(max(startx,endx)/handles.snapsize)-pos1;
pos2=handles.snapsize*floor(min(starty,endy)/handles.snapsize);
pos4=handles.snapsize*ceil(max(starty,endy)/handles.snapsize)-pos2;

widthinches=sprintf('%6.2f',round(pos3/96*4)/4);
heightinches=sprintf('%6.2f',round(pos4/96*4)/4);

set(handles.temp.putgraphhere,'position',[pos1 pos2 pos3 pos4]);

set(handles.infobar,'string',['Width=',widthinches,'" Height=',heightinches,'"  at 96 dpi'])

function ravesizegraphdone(src,eventdata) %#ok<INUSD>
handles=guidata(src);
%stop dragging
set(handles.fig,'WindowButtonMotionFcn',[]);
set(handles.fig,'WindowButtonUpFcn',[]);

ttt=get(handles.temp.putgraphhere,'position');

%make graph in this position

fakeraxes=axes('parent',handles.wspanel,'units','pixels','box','on','position',ttt{1},...
    'userdata',handles.newgraphtype,'hittest','off','xcolor',[.5 .5 .5],'ycolor',[.5 .5 .5],...
    'zcolor',[.5 .5 .5],'xticklabel','','yticklabel','',...
    'gridlinestyle',':','userdata','');

fakeaxes=axes('parent',handles.wspanel,'units','pixels','box','on','position',ttt{1},...
    'userdata',handles.newgraphtype,'hittest','off','xcolor',[.5 .5 .5],'ycolor',[.5 .5 .5],...
    'zcolor',[.5 .5 .5],'xticklabel','','yticklabel','',...
    'gridlinestyle','-','color','none','userdata','');

newaxes=axes('parent',handles.wspanel,'units','pixels','box','on','position',ttt{1},...
    'userdata',handles.newgraphtype,'hittest','off','color','none','buttondownfcn',@raveselectgraph);

if strcmp(handles.graphinfo.(handles.newgraphtype).class,'control');
    facecolor=[0 0.5 1];
elseif strcmp(handles.graphinfo.(handles.newgraphtype).class,'annotation');
    facecolor=[0.7 0 1];
else
    facecolor=[1 0.5 0];
end
%make movegraph grabbers:
grab1=line(ttt{1}(1)+2,ttt{1}(2)+3,'marker','s','markersize',5,'markerfacecolor',facecolor,'markeredgecolor',[0 0 0],...
    'tag','SW','parent',handles.mainaxes,'buttondownfcn',@ravemovegraph);
grab2=line(ttt{1}(1)+2,ttt{1}(2)+ttt{1}(4)-2,'marker','s','markersize',5,'markerfacecolor',facecolor,'markeredgecolor',[0 0 0],...
    'tag','NW','parent',handles.mainaxes,'buttondownfcn',@ravemovegraph);
grab3=line(ttt{1}(1)+ttt{1}(3)-3,ttt{1}(2)+ttt{1}(4)-2,'marker','s','markersize',5,'markerfacecolor',facecolor,'markeredgecolor',[0 0 0],...
    'tag','NE','parent',handles.mainaxes,'buttondownfcn',@ravemovegraph);
grab4=line(ttt{1}(1)+ttt{1}(3)-3,ttt{1}(2)+3,'marker','s','markersize',5,'markerfacecolor',facecolor,'markeredgecolor',[0 0 0],...
    'tag','SE','parent',handles.mainaxes,'buttondownfcn',@ravemovegraph);

%adjust margins to encompass tick mark labels
inset=get(newaxes,'tightinset');
%handles.inset=inset;

ttt{1}=ttt{1}+[inset(1)+4 inset(2)+2 -inset(1)-inset(3)-4 -inset(2)-inset(4)];
try
    set(newaxes,'position',ttt{1})
    set(fakeraxes,'position',get(newaxes,'position'))
    set(fakeaxes,'position',get(newaxes,'position'))
catch %#ok<CTCH> %enforce minimum graph size:
    ttt=get(handles.temp.putgraphhere,'position');
    ttt{1}(3:4)=[198 198];
    set(handles.temp.putgraphhere,'position',ttt{1})
    
    set(grab1,'xdata',ttt{1}(1)+2,'ydata',ttt{1}(2)+3)
    set(grab2,'xdata',ttt{1}(1)+2,'ydata',ttt{1}(2)+ttt{1}(4)-2)
    set(grab3,'xdata',ttt{1}(1)+ttt{1}(3)-3,'ydata',ttt{1}(2)+ttt{1}(4)-2)
    set(grab4,'xdata',ttt{1}(1)+ttt{1}(3)-3,'ydata',ttt{1}(2)+3)
    ttt{1}=ttt{1}+[inset(1)+4 inset(2)+2 -inset(1)-inset(3)-4 -inset(2)-inset(4)];
    set(newaxes,'position',ttt{1})
    set(fakeraxes,'position',get(newaxes,'position'))
    set(fakeaxes,'position',get(newaxes,'position'))
    
end

%reorder to bring axes to top:
uistack(fakeraxes,'top')
uistack(fakeaxes,'top')
uistack(newaxes,'top')

%recolor position rectangles
set(handles.temp.putgraphhere(1),'facecolor',handles.colors.wsbg,'edgecolor',handles.colors.wsgrid,'linewidth',1,'hittest','on',...
    'buttondownfcn',@raveselectgraph);
set(handles.temp.putgraphhere(2),'facecolor',[.7 .7 .7],'edgecolor',[.5 .5 .5],'linewidth',1,'hittest','off');
uistack(handles.currentview,'top');

%make axes limit listeners:
addlistener(newaxes,'XTick','PreSet',@axeschangelistener);
addlistener(newaxes,'YTick','PreSet',@axeschangelistener);
addlistener(newaxes,'ZTick','PreSet',@axeschangelistener);

%store handles of axes
handles.allgraphs(end+1)=newaxes;
handles.fakeax(end+1)=fakeaxes;
handles.fakerax(end+1)=fakeraxes;
handles.grabbers{end+1}=[grab1 grab2 grab3 grab4];
handles.allblockers(end+1)=handles.temp.putgraphhere(1);
handles.expblockers(end+1)=handles.temp.putgraphhere(2);
handles.subax{end+1}=[];
handles.subfakeax{end+1}=[];
handles.subfakerax{end+1}=[];
handles.statsax{end+1}=[];
handles.rasters{end+1}=[];
handles.graphanalysis(end+1)=1;
handles.graphdataset(end+1)=1;

%ditch the temp crap
handles.temp=[];

%clear the infobar
set(handles.infobar,'string','');

%undo creation function
set(handles.mainaxes,'buttondownfcn',@ravedragview);
set(handles.explorer,'buttondownfcn',@ravemoveview);

%update guidata
guidata(handles.fig,handles);

%make this graph active:
raveselectgraph(newaxes,[])

%refresh guidata
handles=guidata(handles.fig);

%create the initial graph:

if exist(which(['raveinitialize',handles.newgraphtype]),'file')
    eval(['raveinitialize',handles.newgraphtype,'(handles.fig,[]);']);
else
    errordlg(['Function ''raveinitialize',handles.newgraphtype,'.m not found on path. Ensure that file exists and is on path.'],'File Not Found');   
    return;
end


%refresh handles
handles=guidata(src);
%turn on the tabs that need a graph to exist:
%set(handles.alltabs,'hittest','on')

%make this graph active:
raveselectgraph(newaxes,[])

%display view tab:
if ~isempty(handles.activegraph) %avoid error if graph didn't create successfully
    viewtab(handles.fig,[]);
end


