function [constraintfunctionstring,constrainttype,constraintvars]=raveprocessconstraint(handles,d,constraintstring,thisequality,constraintvalue) %#ok<INUSL>
%process the string of rave variable names in constraintstring and make it into something suitable
%for a matlab annonymous function. output constrainttype is either 'equality' or 'inequality'. does
%not handle the actual creation/storage of the anonymous function. 
%
%handles=guidata(src);
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


indata=constraintstring;

if any(indata==char(58)) %if any colons
    errordlg('Vectorized constraints are not allowed (get rid of the colons).','You can''t do that')
end


%extract variable names from indata:
% varnames=symvar(indata);
% nvars=length(varnames);
% 

%check that all function variables match up with variable names in data
% %set.
% importnames=varnames;
% existingnames=strrep(handles.allnames{udata.view.dataset},char(32),'');
% if sum(ismember(importnames,existingnames))<length(importnames)
%     %some names arent in data set
%     numbermissing=length(importnames)-sum(ismember(importnames,existingnames));
%     warnd=warndlg(['I can''t find matches for all of this function''s independent variables in the selected graph''s data set. ',...
%         num2str(numbermissing),' of the ',num2str(nvars),' variables in this equation cannot be found.'],'Variable Not Found');
%     uiwait(warnd);
% end


%% replace actual variable names with indices into alldata matrix:

%this is going to read through ALL variable names whether they appear in the string or not, cause i
%think this will give better resistance to symbols/spaces in variable names. (this way dont need to
%rely on symvars function.)

varnames=handles.allnames{d};
%replace longest names first in case a shorter name is a subset of a longer name.
varlengths=cellfun(@length,varnames);
[sortednames varlengthorder]=sort(varlengths,'descend');
isinput=false(1,numel(varnames));
for g=1:numel(varnames)
    thisvar=varnames{varlengthorder(g)};
    thisindex=num2str(find(ismember(varnames,thisvar)));
    isinput(g)=any(strfind(indata,thisvar));
    indata=strrep(indata,thisvar,['data(:,',thisindex,')']);
end
constraintvars=sort(varlengthorder(isinput));

%note, need to replace variable names before replacing any symbols/functions in case names contain
%these symbols:

%remove empty spaces and colons before variable names.
%indata=indata(indata~=char(32) & indata~=char(9));

%remove newlines:
indata=indata(indata~=char(15));

%replace retarded "+-" business with just "-"
indata=strrep(indata,'+-','-');

%remove tabs:
%indata=strrep(indata,char(9),'');

%replace vector multiply/divide with scalar: (NOTE, KEEP ALL 6 OF THESE STATMENTS)
indata=strrep(indata,'.*','*');
indata=strrep(indata,'./','/');
indata=strrep(indata,'.^','^');
indata=strrep(indata,'*','.*');
indata=strrep(indata,'/','./');
indata=strrep(indata,'^','.^');

%convert funtion names to matlabese:
indata=strrep(indata,'Exp','exp');
indata=strrep(indata,'Sine','sin');
indata=strrep(indata,'Squish','logsig');
indata=strrep(indata,'Cosine','cos');
indata=strrep(indata,'Tangent','tan');
indata=strrep(indata,'Log','log');
indata=strrep(indata,'Log','log');
indata=strrep(indata,'Abs','abs');
indata=strrep(indata,'Round','round');
indata=strrep(indata,'Floor','floor');
indata=strrep(indata,'Ceiling','ceil');




%% process equality and target value:
%convert all inequality constraints to less than or equal to 0.
%convert equality constraitns to equal to 0.
switch thisequality
    case '='
        indata=[indata,' -',num2str(constraintvalue)];
        constrainttype='equality';
    case {'<','<='}
        indata=[indata,' -',num2str(constraintvalue)];
        constrainttype='inequality';
    case {'>=','>'}
        indata=[indata,' -',num2str(constraintvalue)];
        indata=['-(',indata,')'];
        constrainttype='inequality';   
end

constraintfunctionstring=indata;











