function raveputaway(src,ev)
%put away the current graph.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


handles=guidata(src);

set(0,'userdata',[]);

screensize=get(handles.fig,'position');

askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-250 screensize(2)+screensize(4)/2-75 500 150],...
    'numbertitle','off','menubar','none','name','Put Away Selected Graph','color',handles.colors.frame,...
    'resize','off','windowstyle','modal');
uicontrol('style','text','backgroundcolor',handles.colors.frame,'units','pixels',...
    'position',[20 80 225 40],...
    'string','Enter a name for the selected graph: ','horizontalalignment','left')
uicontrol('style','edit','string',ravenamegraph(handles.fig,[]),'backgroundcolor',[1 1 1],'units','pixels',...
    'position',[30 70 440 22],'tag','namebox','horizontalalignment','left')
uicontrol('style','pushbutton','string','Put Away','parent',askbox,'units','pixels',...
    'position',[270 10 100 30],'callback',@putawaypopup_callback,'fontsize',10);
uicontrol('style','pushbutton','string','Cancel','parent',askbox,'units','pixels',...
    'position',[380 10 100 30],'callback',@putawaypopup_callback,'fontsize',10);


uiwait(askbox)



name=get(0,'userdata');


if isempty(name)
    return %user hit cancel
end



%else put away this graph:

if ~any(strcmp(handles.activetype,{'scatter';'scattermatrix';'line';'scatter3D';'derivativeprofiler';'wate';'histogram';'parallelcoords';'surfaceprofiler'}))

    set(handles.infobar,'string','This is lame, but I don''t know how to put away this type of graph.')
    return
    

else %put away this graph
    guidata(src,handles);
    %   raverasterize(src,[]);
    raverasterfarian(src,[]);
    handles=guidata(src);
    
    
    raster=handles.rasters{handles.activeindex};
    set(get(raster,'children'),'visible','off');

    handles.graphnames{handles.activeindex}=name;
    handles.putaway=union(handles.putaway,handles.activeindex);

    set(handles.allgraphs(handles.activeindex),'visible','off');
    set(handles.fakeax(handles.activeindex),'visible','off');
    set(handles.fakerax(handles.activeindex),'visible','off');
    % delete(handles.allblockers(handles.activeindex));
    set(handles.expblockers(handles.activeindex),'visible','off');
    set(handles.grabbers{handles.activeindex},'visible','off');

    set(handles.allblockers(handles.activeindex),'visible','off')

    switch handles.activetype
        case {'scattermatrix','derivativeprofiler'}
            set(handles.subax{handles.activeindex},'visible','off');
            set(handles.subfakeax{handles.activeindex},'visible','off');
            set(handles.subfakerax{handles.activeindex},'visible','off');
            set(get(handles.allgraphs(handles.activeindex),'children'),'visible','off');
        case 'histogram'
            set(handles.subax{handles.activeindex},'visible','off');
            set(handles.subfakeax{handles.activeindex},'visible','off');
            set(handles.subfakerax{handles.activeindex},'visible','off');
            set(handles.statsax{handles.activeindex},'visible','off');
            statz=get(handles.statsax{handles.activeindex},'children');
            if iscell(statz);statz=cell2mat(statz);end
            set(statz,'visible','off')
        case 'tableallpoints'
            set(udata.view.hscroll,'visible','off');
            set(udata.view.vscroll,'visible','off');
        case 'tablesinglepoint'
            set(udata.view.hscroll,'visible','off');
            set(udata.view.vscroll,'visible','off');
    end





    %make another graph the active graph:
    newgraph=[];
    for s=1:length(handles.allgraphs)
        if ishandle(handles.allgraphs(s)) && ~ismember(s,handles.putaway) %this graph hasnt been deleted and isnt put away
            newgraph=handles.allgraphs(s);
            break
        end
    end


    handles.activeindex=[];
    handles.activegraph=[];
    guidata(src,handles);

    if ~isempty(newgraph) %graphs exist
        raveselectgraph(newgraph,[]) %make this graph current.
    else %graphs dont exist
        %make tabs unclickable:
        set(handles.alltabs,'hittest','off')

        %go back to managetab:

        managetab(handles.fig,[]);
        set(handles.infobar,'string','All graphs have been deleted. Make a new one to continue.');

    end





end

function putawaypopup_callback(src,ev)

popup=get(src,'parent');
if strcmp(get(src,'string'),'Cancel');
    delete(popup);
    return
end




namebox=findobj(popup,'tag','namebox');
graphname=get(namebox,'string');

set(0,'userdata',graphname);

delete(popup);






