function raveresizeaxes(src,eventdata,targetindex) %#ok<INUSL>

%RAVE: resize the current target axes to fit inside their bounding box,
%including axes titles/labels.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

targetaxes=handles.allgraphs(targetindex);
targetpos=get(handles.allblockers(targetindex),'position');

%if any(strcmp(handles.activetype,{'scatter','scatter3D','scattermatrix','parallelcoords','surfaceprofiler','hyperslice','contour'}))
 if handles.graphinfo.(handles.activetype).hascolorbar;   
    if udata.format.colorbar && udata.view.c;
        targetpos(3)=targetpos(3)-100;
    end

end

%adjust margins to encompass tick mark labels

if ~ischar(handles.graphinfo.(handles.activetype).inset)
    inset=get(targetaxes,'tightinset')+handles.graphinfo.(handles.activetype).inset;
elseif strcmp(handles.graphinfo.(handles.activetype).inset,'image')
     if udata.view.invalid
            inset=[10 10 10 10];
        else
            imdata=findobj(handles.activegraph,'type','image');
            imdata=get(imdata,'cdata');

            %determine how to resize axes:
            maxsize=get(handles.allblockers(handles.activeindex),'position');
            maxsize=maxsize(3:4)-[24 20];
            maxratio=maxsize(1)/maxsize(2);

            ratio=size(imdata,2)/size(imdata,1);

            if ratio>maxratio %width constrained

                newheight=maxsize(1)/ratio;

                if newheight>size(imdata,1)
                    newheight=size(imdata,1);
                end

                heightdiff=maxsize(2)-newheight;
                oldpos=get(handles.allblockers(handles.activeindex),'position')+[10 10 -24 -20];
                
                if oldpos(3)>size(imdata,2)
                    widthdiff=oldpos(3)-size(imdata,2);
                    oldpos(3)=size(imdata,2);
                else
                    widthdiff=0;
                end

                set(handles.activegraph,'position',[oldpos(1)+widthdiff/2 oldpos(2)+heightdiff/2 oldpos(3) newheight])
            else %height constrained

                newwidth=maxsize(2)*ratio;

                if newwidth>size(imdata,2)
                    newwidth=size(imdata,2);
                end

                widthdiff=maxsize(1)-newwidth;
                %oldpos=get(handles.activegraph,'position');
                oldpos=get(handles.allblockers(handles.activeindex),'position')+[10 10 -24 -20];
                if oldpos(4)>size(imdata,1)
                    heightdiff=oldpos(4)-size(imdata,1);
                    oldpos(4)=size(imdata,1);
                else
                    heightdiff=0;
                end

                set(handles.activegraph,'position',[oldpos(1)+widthdiff/2 oldpos(2)+heightdiff/2 newwidth oldpos(4)])

            end
            return
     end
else

    if isempty(which(['raveresizeaxes',handles.activetype]))
        errordlg(['Function raveresizeaxes',handles.activetype,'.m not found. Add this function to your path or remove ''custom'' option from graphproperties',handles.activetype,'.m file.','File Not Found']);
        return
    end

    eval(['raveresizeaxes',handles.activetype,'(handles.fig,[],targetindex)']);
    return

end

newpos=floor([targetpos(1)+inset(1)+4 targetpos(2)+inset(2)+2 targetpos(3)-inset(1)-inset(3)-4 targetpos(4)-inset(2)-inset(4)]);

set(targetaxes,'position',newpos)
set(handles.fakeax(targetindex),'position',newpos)
set(handles.fakerax(targetindex),'position',newpos)

guidata(src,handles);