function ravesavemetadata(src,ev)
%these functions are described in the rave readme file. 
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);


set(0,'userdata',handles.fig)

hhh=savemetadatagui(src,ev);
uiwait(hhh);

results=get(0,'userdata');


if ~isstruct(results) %user hit cancel
    return
end

d=results.d;
a=results.a;
dest=results.path;
makethese=results.makethese;

name=handles.datasetnames{d};

if makethese(1);save_rvefun(src,d,a,fullfile(dest,[name '.rvefun']));end
if makethese(2);save_rveval(src,d,a,fullfile(dest,[name '.rveval']));end
if makethese(3);save_rvecon(src,d,a,fullfile(dest,[name '.rvecon']));end
if makethese(4);save_rvecur(src,d,a,fullfile(dest,[name '.rvecur']));end
if makethese(5);save_rveopt(src,d,a,fullfile(dest,[name '.rveopt']));end
if makethese(6);save_rvecol(src,d,a,fullfile(dest,[name '.rvecol']));end
if makethese(7);save_rverow(src,d,a,fullfile(dest,[name '.rverow']));end
if makethese(8);save_rvepoi(src,d,a,fullfile(dest,[name '.rvepoi']));end

ravefinish;
set(handles.infobar,'string','Metadata exported successfully.')

function save_rvefun(src,d,a,dest)
handles=guidata(src);

thesevars=find(handles.isfunction{d});

if isempty(thesevars);return;end %no functions in this data set

fid=fopen(dest,'wt');

for g=1:numel(thesevars)
    thisname=handles.allnames{d}{thesevars(g)};
    
    if handles.isobjective{d}(thesevars(g))
        thisname=['*' thisname]; %#ok<AGROW>
    end
    
    
    thisfile=which(handles.funnames{d}{thesevars(g)});
    thisout=handles.nargout{d}(thesevars(g));
    thisins=handles.funvars{d}{thesevars(g)};

    fprintf(fid,'%s\t%s\t%s\t',thisname,thisfile,num2str(thisout));

    for f=1:numel(thisins)
       fprintf(fid,[handles.allnames{d}{thisins(f)} '\t']);
    end
    
    fprintf(fid,'\n');
    
    
end
fclose(fid);


%function save_rvetyp(src,d,a,dest)

function save_rveval(src,d,a,dest)
handles=guidata(src);

thesevars=1:numel(handles.allnames{d});

fid=fopen(dest,'wt');

for g=1:numel(thesevars)
    thisname=handles.allnames{d}{thesevars(g)};
    fprintf(fid,'%s\t',thisname);   
    thistype=handles.datatype{d}{thesevars(g)};
    fprintf(fid,'%s\t',thistype);  
    switch thistype
        case 'constant'
            fprintf(fid,'%16.15g',handles.minlimit{d}{a(1)}(g)); 
        case {'continuous','integer','logical'}
            
            fprintf(fid,'%16.15g\t',handles.minlimit{d}{a(1)}(g)); 
            fprintf(fid,'%16.15g',handles.maxlimit{d}{a(1)}(g)); 
        case 'discrete'
            N=numel(handles.discretevalues{d}{a(1)}{g});
            for n=1:N
                fprintf(fid,'%16.15g\t',handles.discretevalues{d}{a(1)}{g}(n));
            end  
        case 'string'
            N=numel(handles.discretevalues{d}{a(1)}{g});
            for n=1:N
                fprintf(fid,'%s\t',handles.discretevalues{d}{a(1)}{g}{n});
            end   
            
    end
    fprintf(fid,'\n');
end
fclose(fid);


function save_rvecon(src,d,a,dest)
handles=guidata(src);
N=numel(handles.constrainttext{d}{a(1)});
if N==0;return;end

fid=fopen(dest,'wt');




for n=1:N
   fprintf(fid,'%s\t',handles.constrainttext{d}{a(1)}{n});
   fprintf(fid,'%s\n',mat2str(handles.constraintcolors{d}{a(1)}{n})); 

end

fclose(fid);

function save_rvecur(src,d,a,dest)
handles=guidata(src);

fid=fopen(dest,'wt');
for n=1:numel(handles.allnames{d})
fprintf(fid,'%s\t',handles.allnames{d}{n});
end
fprintf(fid,'\n');
for n=1:numel(handles.allnames{d})
fprintf(fid,'%s\t',num2str(handles.xdials{d}{a(1)}(n)));
end
fclose(fid);




function save_rveopt(src,d,a,dest)
handles=guidata(src);

fid=fopen(dest,'wt');
for n=1:numel(handles.allnames{d})
fprintf(fid,'%s\t',handles.allnames{d}{n});
end
fprintf(fid,'\n');
for n=1:numel(handles.allnames{d})
fprintf(fid,'%s\t',num2str(handles.targetvalues{d}(n)));
end
fprintf(fid,'\n');
for n=1:numel(handles.allnames{d})
fprintf(fid,'%s\t',num2str(handles.preferences{d}(n)));
end
fclose(fid);


function save_rvecol(src,d,a,dest)
handles=guidata(src);

fid=fopen(dest,'wt');
for n=1:numel(handles.allnames{d})
fprintf(fid,'%s\t',handles.allnames{d}{n});
end
fprintf(fid,'\n');
for n=1:numel(handles.allnames{d})
fprintf(fid,'%s\t',mat2str(handles.variablecolors{d}(n,:),4));
end

fclose(fid);

function save_rverow(src,d,a,dest)
handles=guidata(src);

fid=fopen(dest,'wt');
for n=1:size(handles.alldata{d},1)
fprintf(fid,'%s\t',num2str(handles.rowcolors{d}{a(1)}(n)));
fprintf(fid,'%s\t',num2str(handles.isvisible{d}{a(1)}(n)));
fprintf(fid,'%s\n',num2str(handles.isselected{d}{a(1)}(n)));
end
fclose(fid);

function save_rvepoi(src,d,a,dest)

