function raveupdatedataset(varargin)
%raveupdatedataset(src,ev,updatetype,dataset)
%applies the update in updatetype to all graphs that share the same
%dataset as the active graph or the dataset arguement.
%options for updatetype:
%'preferences','targets','replaceall','addrows','addcolumns','removerows',
%'removecolumns','datachanged'
%
%Note: addrows does not reset selection/coloration etc. Replaceall resets those if number of rows changes. (really should prompt and ask what to do). 
% columns just updates the maintable RaveTableModel
%
%NEED to make this not rasterize/vectorize when updating the currently selected graph.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

src=varargin{1};
ev=varargin{2};
updatetype=varargin{3};

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

if numel(varargin)==4
    d=varargin{4};
else
    d=udata.view.dataset;
end

%update table model:
a=1;
handles.maintable.models{d}=ravecreatemaintablemodel(handles.fig,[],d,a);
guidata(handles.fig,handles);
ravedisplaymaintable(handles.fig,[]);



othergraphs=find(handles.graphdataset==d);
needsupdating=false(size(othergraphs));
%apply changes to data set:
switch updatetype
    case {'preferences','targets'}
        %re-evaluate objective functions that use these variables:
        
        objectives=find(handles.isobjective{d});
        
        %NOTE: this loop won't work if one objective fcn is a function of another objective fcn.
        %should really check for dependency and evaluate in correct order.
        if ~isempty(objectives)
            
            newvalues=raveevaluateregression(src,d,objectives,handles.alldata{d});
            handles.alldata{d}(:,objectives)=newvalues;
        end
        
        %determine which graphs need updating:
        if ~isempty(objectives)
            for o=1:numel(othergraphs);
                udat=get(handles.fakeax(othergraphs(o)),'userdata');
                allvars=unique([udat.view.x(:);udat.view.y(:);udat.view.z(:);udat.view.c(:);udat.view.s(:)]);
                if any(ismember(allvars,objectives))
                    needsupdating(o)=true;
                end
            end
        end
            
    case {'replaceall','addrows','deleterows','datachanged'}
        needsupdating=true(size(othergraphs));

        
end

guidata(src,handles);

%find indices of other graphs with this dataset and analysis:
othergraphs=othergraphs(needsupdating);

if isempty(othergraphs);return;end

originalindex=handles.activeindex;


for ai=othergraphs
    handles.activeindex=ai;
    
    othergraph=handles.allgraphs(ai);
    if ~ishandle(othergraph);continue;end %skip if this graph has been deleted.
    thistype=get(othergraph,'userdata'); %type of other graph in this analysis
    
    handles.activetype=thistype;
    handles.activegraph=othergraph;
    guidata(handles.fig,handles);
       
    eval(['ravecreate',lower(thistype),'(handles.fig,[]);']);
        
    handles=guidata(handles.fig); %refresh handles;
    
end

othergraph=handles.allgraphs(originalindex);
thistype=get(othergraph,'userdata'); %type of other graph in this analysis

handles.activeindex=originalindex;
handles.activetype=thistype;
handles.activegraph=othergraph;
guidata(handles.fig,handles);



