function exporttab(src,eventdata)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

ravechangetab(src,[]);

handles=guidata(src);

%move front tabs to front if necessary
if handles.fronttabsfront==0 %move swap fronttabs and backtabs
    backpos=get(handles.backtabaxes,'position');
    frontpos=get(handles.fronttabaxes,'position');
    set(handles.fronttabaxes,'position',backpos)
    set(handles.backtabaxes,'position',frontpos)
    [x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
    set(handles.backtablines,'xdata',x,'ydata',y);
end
handles.fronttabsfront=1;

%raise this tab to top
[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,5,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');

handles.exportdestination='file';
handles.exportlayout=1;
handles.exportformat='xls';

set(handles.infobar,'string','');

set(handles.roi,'visible','on')


%% clear existing tab
delete(handles.tabobjects)

%% GRAPH STUFF

graphtext=text(.15-.10317,.95*1.0317-.0317,'Export Graphs','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

destinationtext=uicontrol('position',[origin(1)+25 origin(2)+565 180 20],...
    'style','text','string','Destination:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

filebutton=uicontrol('position',[origin(1)+50 origin(2)+528 35 35],...
    'style','togglebutton','string','','cdata',imread('image.bmp','bmp'),'value',1,'userdata','file',...
    'callback',@graphblob_callback,'backgroundcolor',handles.colors.togglebuttonbg);
printbutton=uicontrol('position',[origin(1)+95 origin(2)+528 35 35],...
    'style','togglebutton','string','','cdata',imread('printer.bmp','bmp'),'userdata','print',...
    'callback',@graphblob_callback,'backgroundcolor',handles.colors.togglebuttonbg);
pptbutton=uicontrol('position',[origin(1)+140 origin(2)+528 35 35],...
    'style','togglebutton','string','','cdata',imread('ppt.bmp','bmp'),'userdata','ppt',...
    'callback',@graphblob_callback,'backgroundcolor',handles.colors.togglebuttonbg,'visible','off');
wordbutton=uicontrol('position',[origin(1)+185 origin(2)+528 35 35],...
    'style','togglebutton','string','','cdata',imread('word.bmp','bmp'),'userdata','word',...
    'callback',@graphblob_callback,'backgroundcolor',handles.colors.togglebuttonbg,'visible','off');

handles.graphexportblobs=[filebutton printbutton pptbutton wordbutton];

destinationinfo=uicontrol('position',[origin(1)+25 origin(2)+504 220 16],...
    'style','text','string','Save as Image','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',8,'horizontalalignment','center','tag','destinationinfo');

layouttext=uicontrol('position',[origin(1)+25 origin(2)+490 50 20],...
    'style','text','string','Layout:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

oneinone=uicontrol('position',[origin(1)+35 origin(2)+470 220 20],...
    'style','radiobutton','string','Selected graph and its legends','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',1,'callback',@layout_callback,...
    'tag','oneinone');

allinone=uicontrol('position',[origin(1)+35 origin(2)+450 220 20],...
    'style','radiobutton','string','All graphs within a region of interest','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','callback',@layout_callback,'tag','allinone');
allinonetext=uicontrol('position',[origin(1)+62 origin(2)+430 180 20],...
    'style','text','string','(maintains relative positions)','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left');

aatext=uicontrol('position',[origin(1)+25 origin(2)+400 150 20],...
    'style','text','string','Anti-Aliasing:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

nosmooth=uicontrol('position',[origin(1)+35 origin(2)+380 220 20],...
    'style','radiobutton','string','None','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','tag','nosmooth','value',1,'callback',@aa_callback);
linesmooth=uicontrol('position',[origin(1)+35 origin(2)+360 220 20],...
    'style','radiobutton','string','LineSmoothing','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','tag','linesmooth','value',0,'callback',@aa_callback);
mayasmooth=uicontrol('position',[origin(1)+35 origin(2)+340 220 20],...
    'style','radiobutton','string','MYAA','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','tag','mayasmooth','value',0,'callback',@aa_callback);

%note, use handles.lastexport.antialias to maintain this setting between exports.

%use handles.lastexport.filetype to maintain filetype between exports.


exportgraphbutton=uicontrol('position',[origin(1)+30 origin(2)+210 220 30],...
    'style','pushbutton','string','Export Selected Graphs','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',12,'horizontalalignment','left','fontweight','bold','callback',@raveexportgraph);


if isempty(handles.activetype);set(exportgraphbutton,'enable','off');end


%% DATA STUFF

datatext=text(.15-.10317,.285*1.0317-.0317,'Export Data','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

formattext=uicontrol('position',[origin(1)+25 origin(2)+205-75 180 20],...
    'style','text','string','File Format:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

xlsbutton=uicontrol('position',[origin(1)+140 origin(2)+168-75 35 35],...
    'style','togglebutton','string','','cdata',imread('excel.bmp','bmp'),'value',0,'userdata','xls',...
    'callback',@datablob_callback,'backgroundcolor',handles.colors.togglebuttonbg,'tag','datablob');
matbutton=uicontrol('position',[origin(1)+185 origin(2)+168-75 35 35],...
    'style','togglebutton','string','','cdata',imread('matlab.bmp','bmp'),'userdata','mat',...
    'callback',@datablob_callback,'backgroundcolor',handles.colors.togglebuttonbg,'tag','datablob');
tabbutton=uicontrol('position',[origin(1)+50 origin(2)+168-75 35 35],...
    'style','togglebutton','string','','cdata',imread('tab.bmp','bmp'),'userdata','tab',...
    'callback',@datablob_callback,'backgroundcolor',handles.colors.togglebuttonbg,'value',1,'tag','datablob');
csvbutton=uicontrol('position',[origin(1)+95 origin(2)+168-75 35 35],...
    'style','togglebutton','string','','cdata',imread('csv.bmp','bmp'),'userdata','csv',...
    'callback',@datablob_callback,'backgroundcolor',handles.colors.togglebuttonbg,'tag','datablob');


selectedonlycheck=uicontrol('position',[origin(1)+35 origin(2)+50 220 20],...
    'style','checkbox','string','Only export selected rows','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',0,'callback',[],...
    'tag','selectedonly');


exportdatabutton=uicontrol('position',[origin(1)+30 origin(2)+10 220 30],...
    'style','pushbutton','string','Export Data Set','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',12,'horizontalalignment','left','fontweight','bold','callback',@raveexportdata);

handles.dataexportblobs=[xlsbutton matbutton tabbutton csvbutton];

formatinfo=uicontrol('position',[origin(1)+25 origin(2)+144-75 220 16],...
    'style','text','string','Tab Delimited','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',8,'horizontalalignment','center','tag','formatinfo');

scaletext=uicontrol('position',[49+50+4.6*handles.lastexport.scale  origin(2)+250 50 30],...
    'style','text','string',{[num2str(10*handles.lastexport.scale),' %'];[num2str(round(10*handles.lastexport.scale*.96)) 'dpi']},'backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

scaler=uicontrol('position',[origin(1)+25 origin(2)+305 150 20],...
    'style','text','string','Exported Size:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');


%scale slider:
[jslider hslider]=javacomponent('javax.swing.JSlider',[origin(1)+40 origin(2)+280 190 30]);
jslider.setPaintTicks(1)

jslider.setSnapToTicks(0)
jslider.setMinimum(2)
jslider.setMaximum(40)
jslider.setMajorTickSpacing(2);
%jslider.setMinorTickSpacing(1);
jslider.setValue(handles.lastexport.scale);
set(jslider,'background',java.awt.Color.white)
set(jslider,'StateChangedCallback',{@jslider_Callback,scaletext});

handles.javacomps=jslider;





%% group items and finish up
handles.tabobjects=[graphtext datatext destinationtext layouttext filebutton printbutton pptbutton wordbutton...
    destinationinfo oneinone allinone allinonetext exportgraphbutton formattext xlsbutton matbutton tabbutton csvbutton...
    formatinfo exportdatabutton nosmooth linesmooth mayasmooth aatext scaletext scaler hslider selectedonlycheck];

handles.activetab='export';

guidata(src,handles);
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function graphblob_callback(src,eventdata)
handles=guidata(src);

set(handles.mainaxes,'buttondownfcn',@ravedragview)
set(handles.explorer,'buttondownfcn',@ravemoveview)

destinationinfo=findobj(handles.tabobjects,'tag','destinationinfo');

set(handles.graphexportblobs,'value',0)
set(src,'value',1)

uicontrol(handles.dummyobj);

%set export desination string
handles.exportdestination=get(src,'userdata');

switch get(src,'userdata')
    case 'file'
        set(destinationinfo,'string','Save as Image')
    case 'print'
        set(destinationinfo,'string','Print')
    case 'ppt'
        set(destinationinfo,'string','Export to Powerpoint')
    case 'word'
        set(destinationinfo,'string','Export to Word')
end

guidata(gcf,handles)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function layout_callback(src,eventdata)
handles=guidata(src);

set(handles.mainaxes,'buttondownfcn',@ravedragview)
set(handles.explorer,'buttondownfcn',@ravemoveview)

allinone=findobj(handles.tabobjects,'tag','allinone');
oneinone=findobj(handles.tabobjects,'tag','oneinone');

set([allinone oneinone],'value',0)
set(src,'value',1)
uicontrol(handles.dummyobj)

if src==allinone
    set(handles.infobar,'string','Click and drag in workspace to draw a region of interest.')
    set(handles.mainaxes,'buttondownfcn',{@raveplaceroi,handles})
    set(handles.explorer,'buttondownfcn',{@raveplaceroi,handles})
%     buttons=findobj(handles.tabobjects,'type','uicontrol');
%     set(buttons,'enable','inactive');
else
    set(handles.infobar,'string','');
end

function raveplaceroi(src,eventdata,handles) %#ok<INUSL>
%check that they clicked on exploreraxes or mainaxes:
if ~any([gco==handles.explorer,gco==handles.mainaxes])
    set(handles.mainaxes,'buttondownfcn',@ravedragview)
    set(handles.explorer,'buttondownfcn',@ravemoveview)
    return
end

set(handles.fig,'windowbuttondownfcn',[]);
buttons=findobj(handles.tabobjects,'type','uicontrol');
set(buttons,'enable','on');



%clicked=get(handles.mainaxes,'currentpoint');
clicked=get(gco,'currentpoint');

xclick=clicked(1,1,1);
yclick=clicked(1,2,1);
handles.temp.startx=xclick;
handles.temp.starty=yclick;

snapsize=1;

xclick=snapsize*floor(xclick/snapsize);
yclick=snapsize*floor(yclick/snapsize);


delete(handles.roi);
handles.roi=[];
guidata(src,handles);


handles.roi(1)=rectangle('position',[xclick yclick handles.prefs.gridsize handles.prefs.gridsize],'linewidth',2,'edgecolor',[0 1 .5],...
    'parent',handles.mainaxes,'hittest','on','buttondownfcn','','facecolor','none');
handles.roi(2)=rectangle('position',[xclick yclick handles.prefs.gridsize handles.prefs.gridsize],'linewidth',2,'edgecolor',[0 1 .5],...
    'parent',handles.explorer,'hittest','on','buttondownfcn','');

%allow dragging
set(handles.fig,'WindowButtonMotionFcn',@ravesizeroi);
set(handles.fig,'WindowButtonUpFcn',@ravesizeroidone);
set(handles.fig,'windowbuttondownfcn',[]);

guidata(src,handles);

function ravesizeroi(src,eventdata) %#ok<INUSD>

handles=guidata(src);

finish=get(gco,'currentpoint');
starty=handles.temp.starty;
startx=handles.temp.startx;
endx=finish(1,1,1);
endy=finish(1,2,1);

snapsize=1;

pos1=snapsize*floor(min(startx,endx)/snapsize);
pos3=snapsize*ceil(max(startx,endx)/snapsize)-pos1;
pos2=snapsize*floor(min(starty,endy)/snapsize);
pos4=snapsize*ceil(max(starty,endy)/snapsize)-pos2;

set(handles.roi,'position',[pos1 pos2 pos3 pos4]);

function ravesizeroidone(src,eventdata) %#ok<INUSD>
handles=guidata(src);
%stop dragging
set(handles.fig,'WindowButtonMotionFcn',[]);
set(handles.fig,'WindowButtonUpFcn',[]);
set(handles.mainaxes,'buttondownfcn',@ravedragview);
set(handles.explorer,'buttondownfcn',@ravemoveview);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function datablob_callback(src,eventdata)
handles=guidata(src);

set(handles.mainaxes,'buttondownfcn',@ravedragview)
set(handles.explorer,'buttondownfcn',@ravemoveview)

formatinfo=findobj(handles.tabobjects,'tag','formatinfo');

set(handles.dataexportblobs,'value',0)
set(src,'value',1)

uicontrol(handles.dummyobj);

%set export desination string
handles.exportformat=get(src,'userdata');

switch get(src,'userdata')
    case 'xls'
        set(formatinfo,'string','Excel Workbook')
    case 'mat'
        set(formatinfo,'string','Matlab (*.mat)')
    case 'tab'
        set(formatinfo,'string','Tab Delimited')
    case 'csv'
        set(formatinfo,'string','Comma Seperated Values')
end


guidata(gcf,handles)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function raveexportgraph(src,eventdata)
handles=guidata(src);
%legdata=get(handles.fakerax(handles.activeindex),'userdata');
udata=get(handles.fakeax(handles.activeindex),'userdata');

set(handles.mainaxes,'buttondownfcn',@ravedragview)
set(handles.explorer,'buttondownfcn',@ravemoveview)

%nosmooth=findobj(handles.tabobjects,'tag','nosmooth');
linesmooth=findobj(handles.tabobjects,'tag','linesmooth');
mayasmooth=findobj(handles.tabobjects,'tag','mayasmooth');
allinone=findobj(handles.tabobjects,'tag','allinone');


%set(handles.AX,'units','pixels')
%set(handles.fakeAX,'units','pixels')


if ~get(allinone,'value') %export single graph
    ai=handles.activeindex;
    position=get(handles.allblockers(ai),'position'); 

else %export all graphs in the roi.
    position=get(handles.roi(1),'position'); 
    if isempty(handles.roi);return;end
    ai=[];
    roipos=get(handles.roi(1),'pos');
    for a=1:numel(handles.allgraphs);
        if ~ishandle(handles.allgraphs(a));continue;end
        thispos=get(handles.allgraphs(a),'position');
        if thispos(1)>=roipos(1) && thispos(1) <= roipos(1)+roipos(3) && thispos(2)>=roipos(2) && thispos(2)<=roipos(2)+roipos(4)
            ai=union(ai,a);
        end
    end
end


numgraphs=numel(ai);

for f=1:numgraphs
    if ~isempty(handles.subax{ai(f)})
        numsubgraphs(f)=numel(handles.subax{ai(f)}); %#ok<AGROW>
    else
        numsubgraphs(f)=0; %#ok<AGROW>
    end
    
    if ~isempty(handles.statsax{ai(f)})
        numstatsgraphs(f)=numel(handles.statsax{ai(f)}); %#ok<AGROW>
    else
        numstatsgraphs(f)=0; %#ok<AGROW>
    end
    
    
end


%make a window for each graph:
%savewindow=zeros(numgraphs,1);

savewindow=figure('name','Saving... DO NOT CLOSE OR MINIMIZE THIS WINDOW','units','pixels','position',[50 50 position(3) position(4)],...
    'color',[1 1 1],'colormap',get(handles.fig,'colormap'),'visible','on',...
    'renderer','painters','inverthardcopy','off','numbertitle','off');



graphpos=get(handles.allgraphs(ai),'position');
if numgraphs==1
    graphpos={graphpos}; %make into a cell if it's not already.
end

offset=cell(numgraphs,1);
for g=1:numgraphs
    offset{g}=graphpos{g}-position;
end

    
for f=1:numgraphs
    legdata=get(handles.fakerax(ai(f)),'userdata');
    
    
    % set(savewindow,'position',[50 50 position{1}(3) position{1}(4)])
    %compare figure size to the size you told it to be. for large graphs
    %figure will be smaller.
    
    ratio=get(savewindow,'position')./position;
    ratio=min(ratio(3:4));

    %copy graphs to new window:
    SfakerAX=copyobj(handles.fakerax(ai(f)),savewindow);
    SfakeAX=copyobj(handles.fakeax(ai(f)),savewindow);
    SAX=copyobj(handles.allgraphs(ai(f)),savewindow);
    SsubfakerAX=copyobj(handles.subfakerax{ai(f)},savewindow); %NOTE: need to make ALL graphs expand these arrays or this will give error for reference greater than index
    SsubfakeAX=copyobj(handles.subfakeax{ai(f)},savewindow);
    SsubAX=copyobj(handles.subax{ai(f)},savewindow);
    SstatsAX=copyobj(handles.statsax{ai(f)},savewindow);
    Slegends=copyobj(legdata,savewindow);
    
    thistype=get(handles.allgraphs(ai(f)),'userdata');
    if strcmp(thistype,'table');continue;end
    
    set(SfakerAX,'position',[offset{f}(1) offset{f}(2) graphpos{f}(3) graphpos{f}(4)]*ratio);
    set(SfakeAX,'position',[offset{f}(1) offset{f}(2) graphpos{f}(3) graphpos{f}(4)]*ratio);
    set(SAX,'position',[offset{f}(1) offset{f}(2) graphpos{f}(3) graphpos{f}(4)]*ratio);
    
    for y=1:numel(Slegends) %for each legend:
        legpos=get(legdata(y),'position');
        legoffset=legpos-position;
        set(Slegends(y),'position',[legoffset(1) legoffset(2) legpos(3) legpos(4)]*ratio)
        
        %if legend is outside bounds of savewindow, expand window to
        %encompass legend
        tttt=get(Slegends(y),'title');
        if ~isempty(get(tttt,'string'));
            set(tttt,'ver','middle');
            legendextent=[legoffset(1) legoffset(2) legoffset(1)+legpos(3) legoffset(2)+legpos(4)+10]*ratio;
        else
            legendextent=[legoffset(1) legoffset(2) legoffset(1)+legpos(3) legoffset(2)+legpos(4)]*ratio;
        end

        figpos=get(savewindow,'position');
        
        legpos=getpixelposition(Slegends(y));
        
        if legpos(1)<=0
            legpos(1)=5;
            figpos(3)=figpos(3)+legpos(3)+15;
            allaxes=setdiff(get(savewindow,'children'),Slegends);
            for t=1:numel(allaxes)
                thispos=get(allaxes(t),'position');
                thispos(1)=thispos(1)+legpos(3)+10;
                set(allaxes(t),'position',thispos)
            end
        end
        
        if legpos(2)<=0
            legpos(2)=5;
            figpos(4)=figpos(4)+legpos(4)+15;
            allaxes=setdiff(get(savewindow,'children'),Slegends);
            for t=1:numel(allaxes)
                thispos=get(allaxes(t),'position');
                thispos(2)=thispos(2)+legpos(4)+10;
                set(allaxes(t),'position',thispos)
            end
        end
        
        set(savewindow,'position',figpos)
        set(Slegends(y),'position',legpos)
        legpos=getpixelposition(Slegends(y));
        
        if figpos(3)<=legendextent(3)
            figpos(3)=legendextent(3)+5;
        end
        
        if figpos(4)<=legendextent(4)
            figpos(4)=legendextent(4)+5;
        end
        
        set(savewindow,'position',figpos);
        
        
    end
    
    
    if numsubgraphs(f)~=0
        subgraphpos=get(handles.subax{ai(f)},'position');
        if iscell(subgraphpos);subgraphpos=cell2mat(subgraphpos);end;
        %subgraphpos=cell2mat(get(handles.subax{handles.activeindex(f)},'position'));
        
        if iscell(position)
            suboffset=bsxfun(@minus,subgraphpos,position{:});
        else
            suboffset=bsxfun(@minus,subgraphpos,position);
        end
            
        
        newpos=[suboffset(:,1) suboffset(:,2) subgraphpos(:,3) subgraphpos(:,4)];
        
        for g=1:numsubgraphs(f)
            
                set(SsubfakerAX(g),'position',newpos(g,:));
                set(SsubfakeAX(g),'position',newpos(g,:));
            
            set(SsubAX(g),'position',newpos(g,:));
        end
    end
    
    if numstatsgraphs(f)~=0
        statsgraphpos=get(handles.statsax{ai(f)},'position');
        if iscell(statsgraphpos);statsgraphpos=cell2mat(statsgraphpos);end;
        %statsgraphpos=cell2mat(get(handles.statsax{handles.activeindex(f)},'position'));
        
        if iscell(position)
            statsoffset=bsxfun(@minus,statsgraphpos,position{:});
        else
            statsoffset=bsxfun(@minus,statsgraphpos,position);
        end
        
        newpos=[statsoffset(:,1) statsoffset(:,2) statsgraphpos(:,3) statsgraphpos(:,4)];
        
        for g=1:numstatsgraphs(f)
            set(SstatsAX(g),'position',newpos(g,:));
        end
    end
    
    
end


switch handles.exportdestination
    case 'print' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        set(savewindow,'paperpositionmode','auto')
        set(savewindow,'inverthardcopy','off')
        printdlg(savewindow)
        close(savewindow)
        
        
        
    case 'file' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        if get(mayasmooth,'value')
            filetypez={'*.bmp','Windows Bitmap (*.bmp)';...
                '*.tif','TIFF (*.tif)';...
                '*.png','Portable Network Graphics (*.png)';...
                '*.jpg','JPEG (*.jpg)';...
                '*.*',  'All Files (*.*)'};
        else
            filetypez={'*.fig','MATLAB Figure (*.fig)';
                '*.tif','TIFF (*.tif)';...
                '*.emf','Enhanced Metafile (*.emf)';...
                '*.bmp','Windows Bitmap (*.bmp)';...
                '*.png','Portable Network Graphics (*.png)';...
                '*.pdf','Portable Document Format (*.pdf)';...
                '*.jpg','JPEG (*.jpg)';...
                '*.eps','Encapsulated PostScript (*.eps)';...
                '*.ai','Adobe Illustrator (*.ai)';...
                '*.*',  'All Files (*.*)'};
        end
        
        [name, path,index] = uiputfile(filetypez,'Save as...'); %#ok<NASGU>
        
        set(savewindow,'paperpositionmode','auto')
        set(savewindow,'inverthardcopy','off')
        
        if get(linesmooth,'value') %do smoothing.
            linez=findobj(savewindow,'type','line','marker','none');
            set(linez,'LineSmoothing','on');
        end
        
        if name~=0 %if user didn't cancel:
            if get(mayasmooth,'value') %do MYAA:
                
                if udata.format.bg<=2
                    set(SfakeAX,'xcolor',[0 0 0],'ycolor',[0 0 0],'zcolor',[0 0 0])
                    set(SfakerAX,'xcolor',[0 0 0],'ycolor',[0 0 0],'zcolor',[0 0 0])
                    set(SsubfakeAX,'xcolor',[0 0 0],'ycolor',[0 0 0],'zcolor',[0 0 0])
                    set(SsubfakerAX,'xcolor',[0 0 0],'ycolor',[0 0 0],'zcolor',[0 0 0])
                else
                    set(SfakeAX,'xcolor',[1 1 1],'ycolor',[1 1 1],'zcolor',[1 1 1])
                    set(SfakerAX,'xcolor',[1 1 1],'ycolor',[1 1 1],'zcolor',[1 1 1])
                    set(SsubfakeAX,'xcolor',[1 1 1],'ycolor',[1 1 1],'zcolor',[1 1 1])
                    set(SsubfakerAX,'xcolor',[1 1 1],'ycolor',[1 1 1],'zcolor',[1 1 1])
                end
                
                set(savewindow,'visible','on');
                
                scalefactor=handles.javacomps.getValue*0.1;
                
                [figyer,drawring]=ravemyaa([3 3./scalefactor],'standard'); %opens a new figure
                set(savewindow,'visible','off'); %we'll close it later.
                close(figyer); %myaa window
                
                imwrite(drawring,[path name],name(end-2:end));
                
            else %do non-maya export with saveas.
                
                switch name(end-2:end)
                    case 'bmp'
                        set(savewindow,'visible','on');
                        formatstring='-dbmp';
                        renderer='-zbuffer';
                    case 'tif'
                        formatstring='-dtiff';
                        renderer='-zbuffer';
                    case 'png'
                        formatstring='-dpng';
                        renderer='-zbuffer';
                    case 'jpg'
                        formatstring='-djpeg';
                        renderer='-zbuffer';
                    case 'emf'
                        formatstring='-dmeta';
                        renderer='-painters';
                    case 'eps'
                        formatstring='-depsc';
                        renderer='-painters';
                    case '.ai'
                        formatstring='-dill';
                        renderer='-painters';
                    case 'pdf'
                        %use FEX print_pdf function for maximum quality. REQUIRES GHOSTSCRIPT EXE TO BE ON SYSTEM PATH.
                        % see:  http://www.mathworks.com/support/solutions/data/1-15ZLK.html?solution=1-15ZLK
                        %see: http://www.proschematic.com/manual/Printing.html
                        didntwork=raveprint_pdf([path name], savewindow);
                        
                        if didntwork %if no ghostscript, use regular print command.
                            set(handles.infobar,'string','Tip: For publication quality PDFs, put gswin32c.exe on your system path.')
                            formatstring='-dpdf';
                            renderer='-painters';
                        else
                            close(savewindow)
                            return
                        end
                        
                    case 'fig'
                        set(savewindow,'name',name,'visible','on');
                        saveas(savewindow,[path name]);
                        close(savewindow)
                        return
                end
                
                figpos=get(savewindow,'position');
                resolution=get(0,'ScreenPixelsPerInch')/ratio;
                set(savewindow,'papersize',figpos(3:4)/resolution,'paperposition',[0 0 figpos(3:4)/resolution]);
                
                scalefactor=handles.javacomps.getValue*0.1;
                
                rez=num2str(round(resolution*scalefactor));
                if strcmp(formatstring,'-dmeta') || strcmp(formatstring,'-dpdf') %override scaling for vector output. use factor of 2 to fix marker scaling issues.
                    rez=num2str(round(resolution*2));
                end
                print(savewindow,[path name],formatstring,['-r',num2str(rez)],renderer)
            end
        end
        close(savewindow)
        ravefinish %make done sound.
        
        
    case 'ppt' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
    case 'word' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
end


function aa_callback(src,ev) %manage radio buttons.
handles=guidata(src);

set(handles.mainaxes,'buttondownfcn',@ravedragview)
set(handles.explorer,'buttondownfcn',@ravemoveview)

nosmooth=findobj(handles.tabobjects,'tag','nosmooth');
linesmooth=findobj(handles.tabobjects,'tag','linesmooth');
mayasmooth=findobj(handles.tabobjects,'tag','mayasmooth');

set([nosmooth linesmooth mayasmooth],'value',0)
set(src,'value',1)


function jslider_Callback(src,ev,scaletext) %#ok<INUSL>
pos=get(scaletext,'position');
set(scaletext,'string',{[num2str(src.getValue*10),' %'];[num2str(round(src.getValue*10*.96)),'dpi']},'position',[49+50+4.6*src.getValue pos(2:4)]);

handles=guidata(scaletext);
handles.lastexport.scale=src.getValue;

set(handles.mainaxes,'buttondownfcn',@ravedragview)
set(handles.explorer,'buttondownfcn',@ravemoveview)

guidata(scaletext,handles);



function raveexportdata(src,ev)
%export selected dataset to a file.
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

set(handles.mainaxes,'buttondownfcn',@ravedragview)
set(handles.explorer,'buttondownfcn',@ravemoveview)

formattype=findobj(handles.tabobjects,'tag','datablob','value',1);
selectedonly=findobj(handles.tabobjects,'tag','selectedonly');


d=udata.view.dataset;
allnames=handles.allnames{d};


if get(selectedonly,'value')
    alldata=handles.alldata{d}(handles.isselected{d}{udata.view.analysis},:);
else
    alldata=handles.alldata{d};
end


thisformat=get(formattype,'userdata');


switch thisformat
    case 'xls'
        [name path]=uiputfile('*.xls','Choose a destination','rave_data.xls');
        if name==0;return;end;
        dest=fullfile(path,name);
        xlswrite(dest, allnames,1,'A1');
        xlswrite(dest, alldata,1,'A2');
        
    case 'mat'
        [name path]=uiputfile('*.mat','Choose a destination','rave_data.mat');
        if name==0;return;end;
        dest=fullfile(path,name);
        data=alldata;
        column_names=allnames;
        
        save(dest,'data','column_names');
        
        
    case 'tab'
        [name path]=uiputfile('*.txt','Choose a destination','rave_data.txt');
        if name==0;return;end;
        dest=fullfile(path,name);
        
        if name==0;return;end;
        
        fid=fopen(dest,'wt');
        for t=1:numel(allnames)-1
            fprintf(fid,'%s\t',allnames{t});
        end
        fprintf(fid,'%s\n',allnames{end});
        fclose(fid);
        dlmwrite(dest,alldata,'-append','delimiter','\t','newline','pc','precision',8);
        
        
        
        
    case 'csv'
        [name path]=uiputfile('*.csv','Choose a destination','rave_data.csv');
        if name==0;return;end;
        dest=fullfile(path,name);
        
        fid=fopen(dest,'wt');
        for t=1:numel(allnames)-1
            fprintf(fid,'%s,',allnames{t});
        end
        fprintf(fid,'%s\n',allnames{end});
        fclose(fid);
        dlmwrite(dest,alldata,'-append','delimiter',',','newline','pc');
        
end

ravefinish

