function optimizetab(src,eventdata)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

ravechangetab(src,[]);
handles=guidata(src);

%move front tabs to front if necessary
if handles.fronttabsfront==1 %move swap fronttabs and backtabs
    backpos=get(handles.backtabaxes,'position');
    frontpos=get(handles.fronttabaxes,'position');
    set(handles.fronttabaxes,'position',backpos)
    set(handles.backtabaxes,'position',frontpos)
    [x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,1,0); %#ok<NASGU>
    set(handles.fronttablines,'xdata',x,'ydata',y)
    set(handles.fronttabaxes,'xcolor',handles.colors.frame,'ycolor',handles.colors.frame);
    set(handles.backtabaxes,'xcolor',handles.colors.tabbg,'ycolor',handles.colors.tabbg);
end
handles.fronttabsfront=0;

set(handles.infobar,'string','');

handles.temp.settings=cell(0);
handles.temp.filenames=cell(0);

delete(handles.tabobjects)

%raise this tab to top
[x,y,v]=calculatetabprofile(handles.backtabprofile,.1,3,1); %#ok<NASGU>
set(handles.backtablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');

udata=get(handles.fakeax(handles.activeindex),'userdata');
if ~isempty(udata)
    d=udata.view.dataset;
    a=udata.view.analysis;
else
    d=1;
    a=1;
end

%% make methods list:

optkeywords=fieldnames(handles.optimizerinfo);

for g=1:numel(optkeywords);
    optclass{g}=handles.optimizerinfo.(optkeywords{g}).class;
    longname{g}=handles.optimizerinfo.(optkeywords{g}).longname;
end

methodstring={'<Select algorithm from list>','SINGLE-OBJECTIVE',longname{strcmp(optclass,'single')},'MULTI-OBJECTIVE',longname{strcmp(optclass,'multi')},'RANKING',longname{strcmp(optclass,'ranking')}};
keywordstring={'','',optkeywords{strcmp(optclass,'single')},'',optkeywords{strcmp(optclass,'multi')},'',optkeywords{strcmp(optclass,'ranking')}};

%% text headers
projecttext=text(.15-.10317,.97*1.0317-.0317,'Project','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
algorithmtext=text(.15-.10317,.85*1.0317-.0317,'Algorithm','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
constraintstext=text(.15-.10317,.57*1.0317-.0317,'Constraints','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
parameterstext=text(.15-.10317,.41*1.0317-.0317,'Settings','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
resultstext=text(.15-.10317,.186*1.0317-.0317,'Results','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);


%% uicontrols

analysistext=text(.2-.10317,.895*1.0317-.0317,'Analysis:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
datasettext=text(.2-.10317,.935*1.0317-.0317,'Data Set:','horizontalalignment','left',...
    'backgroundcolor','none',...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');
analysismenu=uicontrol('position',[origin(1)+95 origin(2)+530+13 160 30],...
    'style','popupmenu','string','Default','callback',@analysismenu_callback,...
    'backgroundcolor',[1 1 1],'tag','analysismenu','value',a);
datasetmenu=uicontrol('position',[origin(1)+95 origin(2)+556+13 160 30],...
    'style','popupmenu','string',handles.datasetnames,'callback',@datasetmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','datasetmenu','value',d);

methodmenu=uicontrol('position',[origin(1)+40 origin(2)+464+22 215 30],...
    'style','popupmenu','string',methodstring,...
    'callback',@methodmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','methodmenu','userdata',keywordstring);

targetstext=text(.20-.10317,.77*1.0317-.0317,'Select Objective(s):','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',10,...
    'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');


responsevars=handles.allnames{d};

objectivemenu=uicontrol('position',[origin(1)+40 origin(2)+370+20 215 80],...
    'style','listbox','string',responsevars,...
    'callback',@objectivemenu_callback,...
    'backgroundcolor',[1 1 1],'tag','objectivemenu','max',100);


editinputsbutton=uicontrol('position',[origin(1)+30 origin(2)+345+20 100 22],...
    'style','pushbutton','string','Edit Input Ranges',...
    'callback',@editinputs_callback,...
    'backgroundcolor',handles.colors.buttonbg,'tag','edittargets');

edittargetsbutton=uicontrol('position',[origin(1)+140 origin(2)+345+20 110 22],...
    'style','pushbutton','string','Edit Objective Goals',...
    'callback',@edittargets_callback,...
    'backgroundcolor',handles.colors.buttonbg,'tag','edittargets');

if isempty(handles.constrainttext{d}{a})
    conststring='No constraints have been created yet.';
else
    conststring=[cellstr('') handles.constrainttext{d}{a}];
end

constraintmenu=uicontrol('position',[origin(1)+40 origin(2)+240+20 215 80],...
    'style','listbox','string',conststring,...
    'callback',@constraintmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','constraintmenu','max',100);

dat = {'',''};
cnames = {'Parameter','Value'};


jtable=javax.swing.JTable(dat,cnames); %initialize a table
j=javax.swing.JScrollPane(jtable); %initialize a scrollpane that contains the table
jtable.setFillsViewportHeight(true); %make table fill the scrollpane
[jscrollpane mtable]=javacomponent(j,[origin(1)+40 origin(2)+120+22+3 215 90],handles.fig); %place scrollpane on your gui

%% Setting column widths:
jtable.getTableHeader().setReorderingAllowed(false) %turn off draggable columns
jtable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF)%turn off auto resizing
colwidths=[100 100]; %some column widths

for c=1:2 %set the widths:
    jtable.getColumnModel().getColumn(c-1).setPreferredWidth(colwidths(c))
end

handles.tabflexgrid=jtable;
set(mtable,'tag','mtable');


%startbutton:
startbutton=uicontrol('position',[origin(1)+160-4 origin(2)+114 100 26],...
    'style','pushbutton','string','Start','backgroundcolor',handles.colors.buttonbg,...
    'callback',@startbutton_callback,'enable','off','tag','startbutton','fontsize',12,...
    'fontweight','bold','foregroundcolor',[0 .8 0],'enable','off');

batchbutton=uicontrol('position',[origin(1)+40 origin(2)+114 100 26],...
    'style','pushbutton','string','Batch...','backgroundcolor',handles.colors.buttonbg,...
    'callback',@batchbutton_callback,'enable','off','tag','batchbutton','fontsize',12,...
    'fontweight','bold','foregroundcolor',[0 .8 0],'enable','on');




%% results:

txtcheck=uicontrol('position',[origin(1)+140 origin(2)+80-8+18 100 22],...
    'style','checkbox','string','txt me!','callback','',...
    'backgroundcolor',[1 1 1],'tag','txtcheck','value',0,'fontsize',9);

drawcheck=uicontrol('position',[origin(1)+30 origin(2)+80-8 226 22],...
    'style','checkbox','string','Animate Progress on Selected Graph','callback',@drawcheck_callback,...
    'backgroundcolor',[1 1 1],'tag','drawcheck','value',0,'fontsize',9);

anicheck=uicontrol('position',[origin(1)+40 origin(2)+60-8 120 22],...
    'style','checkbox','string','Save Animation','callback','',...
    'backgroundcolor',[1 1 1],'tag','anicheck','value',0,'fontsize',9,'enable','off');

anisettings=uicontrol('position',[origin(1)+160 origin(2)+60-8 80 22],...
    'style','pushbutton','string','Settings','callback',@anicheck_callback,...
    'backgroundcolor',[1 1 1],'tag','anisettings','value',0,'fontsize',9,'backgroundcolor',handles.colors.buttonbg,'enable','off');

appenddata=uicontrol('position',[origin(1)+30 origin(2)+27 200 22],...
    'style','pushbutton','string','Append final population to dataset','callback',@appenddata_callback,...
    'backgroundcolor',[1 1 1],'tag','appenddata','value',0,'fontsize',9,'backgroundcolor',handles.colors.buttonbg);

exportdata=uicontrol('position',[origin(1)+30 origin(2)+02 200 22],...
    'style','pushbutton','string','Export final population to file','callback',@exportdata_callback,...
    'backgroundcolor',[1 1 1],'tag','makenewdataset','value',0,'fontsize',9,'backgroundcolor',handles.colors.buttonbg);

handles.tabobjects=[projecttext algorithmtext parameterstext constraintstext resultstext methodmenu targetstext objectivemenu...
    analysistext analysismenu datasettext datasetmenu constraintmenu edittargetsbutton...
    drawcheck anicheck anisettings appenddata exportdata startbutton editinputsbutton txtcheck mtable batchbutton];


guidata(src,handles);





function methodmenu_callback(src,ev)
handles=guidata(src);

keywordstring=get(src,'userdata');
thismethod=keywordstring{get(src,'value')};

startbutton=findobj(handles.tabobjects,'tag','startbutton');
batchbutton=findobj(handles.tabobjects,'tag','batchbutton');
jtable=handles.tabflexgrid;
mtable=findobj(handles.tabobjects,'tag','mtable');
objectivemenu=findobj(handles.tabobjects,'tag','objectivemenu');
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
d=get(datasetmenu,'value');
%%

if isempty(thismethod)
    set(handles.infobar,'string','This is a category. Choose again.');
    return
else
    set(handles.infobar,'string','');
end

set([startbutton batchbutton],'enable','on');

%do selected method:
allsettings=handles.optimizerinfo.(thismethod);

%limit selection of objectives based on optimizer class:
thisclass=allsettings.class;
if strcmp(thisclass,'single')
    set(objectivemenu,'max',1,'value',1)
else
    set(objectivemenu,'max',100)
end

%remove standard fields:
allsettings=rmfield(allsettings,{'longname','class'});

settingnames={};
settingvalues={};
isastring=false(0);

deezfields=fieldnames(allsettings);
for d=1:numel(deezfields);
    thisvalue=handles.optimizerinfo.(thismethod).(deezfields{d});
    if ~iscell(thisvalue);continue;end
    if numel(thisvalue)~=2;continue;end
    if ~ischar(thisvalue{1});continue;end
    
    settingnames{end+1}=thisvalue{1};
    settingvalues{end+1}=thisvalue{2};
    
    if ischar(thisvalue{2})
        isastring(end+1)=true;
    else
        isastring(end+1)=false;
    end
    
end

%convert numerical setting values to text:
for g=1:numel(settingvalues)
    if ~isastring(g)
        settingvalues{g}=num2str(settingvalues{g});
    end
end

%update table:
jscrollpane=get(mtable,'javapeer');

if isempty(settingnames)
    settingnames={''};
    settingvalues={''};
end

data=[settingnames' settingvalues'];
cnames = {'Parameter','Value'};
jtable=javax.swing.JTable(data,cnames); %initialize a table
jscrollpane.setViewportView(jtable)

handles.tabflexgrid=jtable;


nrows=numel(settingnames);
% settingstable.rows=nrows+1;
% settingstable.col=0;
for r=1:nrows
    jtable.setValueAt(settingnames{r},r-1,0);
    %     settingstable.row=r;
    %     settingstable.text=settingnames{r};
end

settingstable.col=1;
for r=1:nrows
    settingstable.row=r;
    settingstable.text=settingvalues{r};
end

handles.temp.settingnames=deezfields;
handles.temp.isastring=isastring;

guidata(handles.fig,handles);


function objectivemenu_callback(src,ev)
handles=guidata(src);
set(handles.infobar,'string','')

function constraintmenu_callback(src,ev)
handles=guidata(src);
set(handles.infobar,'string','')

function datasetmenu_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');


drawcheck=findobj(handles.tabobjects,'tag','drawcheck');
anicheck=findobj(handles.tabobjects,'tag','drawcheck');
if isempty(udata) || udata.view.dataset~=get(src,'value')
    set(drawcheck,'value',0,'enable','off')
    set(anicheck,'value',0,'enable','off');
else
    set(drawcheck,'enable','on')
    set(anicheck,'enable','on');
end


function analysismenu_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

drawcheck=findobj(handles.tabobjects,'tag','drawcheck');
anicheck=findobj(handles.tabobjects,'tag','drawcheck');
if ~isempty(udata) || udata.view.analysis~=get(src,'value')
    set(drawcheck,'value',0,'enable','off')
    set(anicheck,'value',0,'enable','off');
else
    set(drawcheck,'enable','on')
    set(anicheck,'enable','on');
end


function editinputs_callback(src,ev)
handles=guidata(src);
set(0,'userdata',get(handles.fig,'position'));

datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
analysismenu=findobj(handles.tabobjects,'tag','analysismenu');

d=get(datasetmenu,'value');
a=get(analysismenu,'value');

hhh=ravechangerangesgui(d,a,handles);
uiwait(hhh);
newhandles=get(0,'userdata');
if ~isstruct(newhandles);return;end
handles=newhandles;
guidata(handles.fig,handles);

set(0,'userdata',[]);



function edittargets_callback(src,ev)
handles=guidata(src);
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
d=get(datasetmenu,'value');

hhh=ravedefinetargetsgui(src,[]);
uiwait(hhh)

newranges=get(0,'userdata');
if isempty(newranges);return;end

handles.targetvalues{d}=newranges;
guidata(handles.fig,handles);

set(0,'userdata',[]);

function drawcheck_callback(src,ev)

handles=guidata(src);

anicheck=findobj(handles.tabobjects,'tag','anicheck');
anisettings=findobj(handles.tabobjects,'tag','anisettings');

if get(src,'value')
    set([anicheck anisettings],'enable','on')
else
    set([anicheck anisettings],'enable','off','value',0)
end

function appenddata_callback(src,ev)
handles=guidata(src);
%prompt for options:
set(0,'userdata',get(handles.fig,'position'));

hhh=appenddatagui(handles.fig,[]); %note, MUST call with handles.fig
uiwait(hhh);

options=get(0,'userdata');
if ~isstruct(options);return;end

set(0,'userdata',[]);

datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
d=get(datasetmenu,'value');

if isfield(handles.temp,'finalpop')
    if isempty(handles.temp.finalpop)
        return
    end
else
    return
end

alldata=[handles.temp.finalpop];

%evaluate population for all responses:
index=find(handles.isfunction{d});

outs=raveevaluateregression(src,d,index,alldata);
alldata(:,index)=outs;

nrows=size(alldata,1);
newdata=alldata;

if strcmp(options.method,'append')
    handles.alldata{d}=[handles.alldata{d};newdata];
    handles.alltext{d}=[handles.alltext{d};cell(size(newdata))];
    
    for a=1:numel(handles.analysisnames)
        handles.rowcolors{d}{a}=[handles.rowcolors{d}{a};options.color*ones(nrows,1)];
        handles.isvisible{d}{a}=[handles.isvisible{d}{a};true(nrows,1)];
        if options.selectexisting
            oldselect=true(size(handles.alldata{d},1),1);
        else
            oldselect=handles.isselected{d}{a};
        end
        
        if options.selectnew
            newselect=true(size(alldata,1),1);
        else
            newselect=false(size(alldata,1),1);
        end
        
        handles.isselected{d}{a}=[oldselect;newselect];
        
        
    end
else %replace data
    handles.alldata{d}=newdata;
    handles.alltext{d}=cell(size(newdata));
    
    for a=1:numel(handles.analysisnames)
        handles.rowcolors{d}{a}=options.color*ones(nrows,1);
        handles.isvisible{d}{a}=true(nrows,1);
        if options.selectnew
            newselect=true(size(alldata,1),1);
        else
            newselect=false(size(alldata,1),1);
        end
        
        handles.isselected{d}{a}=newselect;
    end
end



guidata(handles.fig,handles);
raveupdatedataset(handles.fig,[],'addrows',d);

function exportdata_callback(src,ev,settings)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

isbatch=~isempty(handles.temp.settings);

datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
d=get(datasetmenu,'value');

if isfield(handles.temp,'finalpop')
    if isempty(handles.temp.finalpop)
        return
    end
else
    return
end

%prompt for file name:
if ~isbatch
    filename=makefilename(src,ev,[]);
    [filename path]=uiputfile('*.txt','Choose a destination',filename);
    if filename==0;return;end
    batchpath='';
else
    filename=handles.temp.filename;
    path=cd;
    batchpath=handles.temp.batchname{1};
end

dest=fullfile(path,batchpath,filename);
if ~isdir(fullfile(path,batchpath))
    mkdir(fullfile(path,batchpath));
end
%make string of names:

allnames=handles.allnames{udata.view.dataset};
alldata=[handles.temp.finalpop];

%evaluate population for all responses:
index=find(handles.isfunction{d});

outs=raveevaluateregression(src,d,index,alldata);
alldata(:,index)=outs;


fid=fopen(dest,'wt');
for t=1:numel(allnames)-1
    fprintf(fid,'%s\t',allnames{t});
end
fprintf(fid,'%s\n',allnames{end});


fclose(fid);

dlmwrite(dest,alldata,'-append','delimiter','\t','newline','pc');

ravefinish;


function startbutton_callback(src,ev)
handles=guidata(src);

if isempty(handles.temp.settings) %single run
    
    settings=create_settings(src,ev);
    do_optimizer(src,ev,settings);
    
else %batch mode
    handles.temp.batchname=inputdlg('Name this batch. Results will be saved in a folder with this name.');
    guidata(src,handles);
    for j=1:numel(handles.temp.settings)
        handles.temp.filename=handles.temp.filenames{j};
        guidata(src,handles);
        settings=handles.temp.settings{j};
        do_optimizer(src,ev,settings);
        exportdata_callback(src,ev);
    end
end


%try %#ok<TRYNC>
txtcheck=findobj(handles.tabobjects,'tag','txtcheck');
if get(txtcheck,'value')
    ravesendtxtmessage(handles.fig,[],'Your optimizer has finished.');
end

%end

if isjava(handles.trayIcon)
    trayIconClasses = handles.trayIcon.getClass.getClasses;
    MessageTypes = trayIconClasses(1).getEnumConstants;
    handles.trayIcon.displayMessage('RAVE:','Your optimizer has finished.',MessageTypes(3));
end



function batchbutton_callback(src,ev)
handles=guidata(src);
settings=create_settings(src,ev);

if ~isempty(settings)
    handles.temp.filenames{end+1}=makefilename(src,ev,settings);
    handles.temp.settings{end+1}=settings;
    set(handles.infobar,'string',['1 optimizer run added to batch. ',num2str(numel(handles.temp.settings)),' total in batch.'])
end

guidata(src,handles);

function settings=create_settings(src,ev)
%create settings structure for optimizers.
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
startbutton=findobj(handles.tabobjects,'tag','startbutton');
jtable=handles.tabflexgrid;
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
analysismenu=findobj(handles.tabobjects,'tag','analysismenu');
methodmenu=findobj(handles.tabobjects,'tag','methodmenu');
objectivemenu=findobj(handles.tabobjects,'tag','objectivemenu');
constraintmenu=findobj(handles.tabobjects,'tag','constraintmenu');
drawcheck=findobj(handles.tabobjects,'tag','drawcheck');
anicheck=findobj(handles.tabobjects,'tag','anicheck');

allmethods=get(methodmenu,'userdata');
thismethod=allmethods{get(methodmenu,'value')};

%load default settings:
allsettings=handles.optimizerinfo.(thismethod);
settings=rmfield(allsettings,{'longname','class'});

d=get(datasetmenu,'value');
a=get(analysismenu,'value');

allvars=handles.allnames{d};
%respvars=get(objectivemenu,'string');
%thisvar=respvars(get(objectivemenu,'value'));

thisvar=get(objectivemenu,'value');

%objindex=ismember(allvars,thisvar);

settings.src=handles.fig;
settings.method=thismethod;
settings.d=d;
settings.a=a;
settings.x=thisvar;

switch handles.activetype
    case 'scatter'
        plotvars=[udata.view.x udata.view.y];
    case 'scatter3D'
        plotvars=[udata.view.x udata.view.y udata.view.z];
    case 'scattermatrix'
        plotvars=[udata.view.x];
    otherwise
        plotvars=[];
end

settings.plotprogress=get(drawcheck,'value'); %draw progress on current graph (scatterplot)
settings.animation=get(anicheck,'value'); %logical 1=export animaiton;
if isempty(plotvars);settings.animation=0;end %cant animate if no plotvars.
settings.plotvars=plotvars;

settings.targets=handles.targetvalues{d}(thisvar);
settings.activeconstraints=get(constraintmenu,'value')-1;
origin=get(handles.tabaxes,'position');
settings.startbutton=src; %handle to startbutton
settings.stopbuttonpos=[origin(1)+40 origin(2)+114 100 26]; %position to make stop button, or empty to not make stop button

%get variables for these functions:
indexx=handles.funvars{d}{settings.x(1)};
for k=2:numel(settings.x)
    indexx=union(indexx,handles.funvars{d}{settings.x(k)});
end
settings.funvars=indexx;

settings.funvars=find(~handles.isfunction{d});



% do the same thing for the plot functions:

if ~isempty(plotvars)
    plotindexx=handles.funvars{d}{settings.plotvars(1)};
    for k=2:numel(settings.plotvars)
        plotindexx=union(plotindexx,handles.funvars{d}{settings.plotvars(k)});
    end
    settings.plotinputs=plotindexx;
else
    settings.plotinputs=[];
end

settingnames=handles.temp.settingnames;
isastring=handles.temp.isastring;

for g=1:numel(settingnames);
    thisvalue=jtable.getValueAt(g-1,1);
    if ~isastring(g)
        if ~isempty(thisvalue)
            thisvalue=str2num(thisvalue); %#ok<ST2NM> %note: use str2num, it is more robust than str2double.
        else
            thisvalue=[];
        end
    end
    settings.(settingnames{g})=thisvalue;
end

function do_optimizer(src,ev,settings)
%runs the optimizer specified by the settings structure
handles=guidata(src);

thismethod=settings.method;

%call optimization function:
eval(['rave',thismethod,'(src,[],settings);'])

try %#ok<TRYNC>
    txtcheck=findobj(handles.tabobjects,'tag','txtcheck');
    if get(txtcheck,'value')
        ravesendtxtmessage(handles.fig,[],'Your optimizer has finished.');
    end
    
end


function filename=makefilename(src,ev,settings)
%make default file name for saving the results of this optimizer
handles=guidata(src);


if isempty(settings) %if not called from batch mode...
    
    methods=get(findobj(handles.tabobjects,'tag','methodmenu'),'string');
    settings.method=methods{get(findobj(handles.tabobjects,'tag','methodmenu'),'value')};
end

switch settings.method
    case 'nsgaii'
        allobjs=get(findobj(handles.tabobjects,'tag','objectivemenu'),'string');
        objs=allobjs(get(findobj(handles.tabobjects,'tag','objectivemenu'),'value'));
        
        for o=1:numel(objs)
            objs{o}=[objs{o},', '];
        end
        
        objs=objs';
        objs=cell2mat(objs);
        
        filename=objs;
        filename=filename(:)';
        
        jtable=handles.tabflexgrid;
        pop=jtable.getValueAt(0,1);
        gens=jtable.getValueAt(1,1);
        filename=['NSGA-II ',filename,pop,'p ',gens,'g.txt'];
        
    otherwise
        filename='optimizer_results.txt';
        
end





