function raveloadstate(varargin)
%This function is used to load a saved rave state by clicking the load
%button in the rave gui, or by double clicking it in windows.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

%THIS FUNCTION HAS BEEN SUPERSEDED BY RAVEOPENSTATE

if nargin==1
    %only one input, was double clicked in windows.
    fullname=varargin{1};
    
else
    %two inputs
    src=varargin{1};
    eventdata=varargin{2};
    
    [name,path,index]=uigetfile({'*.rve','RAVE Project (*.rve)';'*.*','All Files (*.*)'},'Choose state to load:');
    if name==0;return;end
    fullname=fullfile(path,name);
    
    %load([path name],'-mat');
    
    
end




movefile(fullname,[fullname(1:end-3),'fig']);
newf=openfig([fullname(1:end-3),'fig']);
movefile([fullname(1:end-3),'fig'],[fullname(1:end-3),'rve'])

handles=guidata(newf);
handles.fig=newf;


%resize to fill screen
set(handles.fig,'units','pixels')


% if handles.prefs.ravemonitor==1 || size(get(0,'monitorpositions'),1)==1
%     screensize=get(0,'screensize');
% else %handles.prefs.ravemonitor=index of monitor to use.
%     screensize=get(0,'monitorpositions');
%     offset=max(screensize(:,4));
%     screensize=screensize(handles.prefs.ravemonitor,:);
%     screensize(2)=offset-screensize(4);
%     screensize(3)=screensize(3)-screensize(1);
% end



load raveprefs

outerwidth=raveprefs.preferredposition(3);
outerheight=raveprefs.preferredposition(4);

% 
% 
% switch raveprefs.os
%     case 'vista'
%         startmenuheight=30;
%     case 'classic'
%         startmenuheight=28;
%     case 'xp'
%         startmenuheight=31;
%     case 'windows7'
%         startmenuheight=50;
% end
% 
% %dont allow for taskbar on 2nd monitor:
% if handles.prefs.ravemonitor~=1;startmenuheight=0;end




%size for current theme:
%set(handles.fig,'position',[1 startmenuheight+1 outerwidth-1 outerheight-startmenuheight-19])
%set(handles.fig,'outerposition',[screensize(1)+1 screensize(2)+startmenuheight outerwidth outerheight-startmenuheight-1])
set(handles.fig,'position',raveprefs.preferredposition);

innerposition=get(handles.fig,'position');

% switch raveprefs.os
%     case 'vista'
%         innerposition=innerposition+[-5 -5 5 10];
%     case 'classic'
%         innerposition=innerposition+[-2 -2 9 4];
%     case 'windows7'
%         innerposition=innerposition+[-5 -5 5 0];
%     case 'xp'
%         innerposition=innerposition+[-5 -5 5 10]; %not really sure what these values should be...don't use xp.
% end

handles.width=innerposition(3);
handles.height=innerposition(4);

%pos=get(handles.fig,'position');
%splashpos=get(splashax,'position');

% switch raveprefs.os
%     case 'classic'
%         set(handles.fig,'visible','on','windowscrollwheelfcn',@wheelscroll,'position',pos+[-4 -3 8 8]+handles.prefs.nudge)
%         % set(splashax,'position',splashpos+[0 -2 0 0])
%     case {'vista','windows7'}
%         set(handles.fig,'windowscrollwheelfcn',@wheelscroll,'position',pos+[0 0 0 0]+handles.prefs.nudge)
%     otherwise
%         set(handles.fig,'visible','on','windowscrollwheelfcn',@wheelscroll,'position',pos+[-4 -3 8 8]+handles.prefs.nudge)
% end

set(handles.fig,'visible','on','windowscrollwheelfcn',@wheelscroll)


%make tray icon:
sysTray = java.awt.SystemTray.getSystemTray;
if (sysTray.isSupported)
    raveroot=which('rave');
    raveroot=raveroot(1:end-6);    
    myIcon = fullfile(raveroot,'images\tinyrave.png');
    iconImage = java.awt.Toolkit.getDefaultToolkit.createImage(myIcon);
    handles.trayIcon = java.awt.TrayIcon(iconImage, 'initial tooltip');
    handles.trayIcon.setToolTip('RAVE is running.');
    sysTray.add(handles.trayIcon)
else
    handles.trayIcon=[];
end


%restore scroll functions:

jvscroll=findjobj(handles.vscroll);
jhscroll=findjobj(handles.hscroll);
try %#ok<TRYNC>
    jvscroll.MouseDraggedCallback = {@vertscroll,handles.fig};
    jhscroll.MouseDraggedCallback = {@horizscroll,handles.fig};
end


%delete old table and make a new one:
delete(handles.maintable.handle);

%% make main data table
guidata(handles.fig,handles);
ravecreatemaintable(handles.fig,[]);
handles=guidata(handles.fig);

%depending on which table method i'm currently using, invisible is either "off" or "0"
set(handles.maintable.handle,'visible','off');

%create table models for data sets:
a=handles.maintable.a;
for d=1:numel(handles.alldata);
   handles.maintable.models{d}=ravecreatemaintablemodel(handles.fig,[],d,a);    
end

guidata(handles.fig,handles);

%fix table visiblity:
showhideworkspace(handles.fig,[]);

%refresh managetab since javabuttons dont save:
managetab(handles.fig,[])


